@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Implementation of Drawable (Graphic objects)
 //
@@ -8,6 +8,7 @@
 #define MAGICK_IMPLEMENTATION
 #define MAGICK_DRAWABLE_IMPLEMENTATION
 
+#include "Magick++/Include.h"
 #include <math.h>
 #include <string>
 
@@ -91,7 +92,7 @@ Magick::Drawable& Magick::Drawable::operator= (const Magick::Drawable& original_
 }
 
 // Operator to invoke contained object
-void Magick::Drawable::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::Drawable::operator()( MagickLib::DrawContext context_ ) const
 {
   if(dp)
     dp->operator()( context_ );
@@ -170,7 +171,7 @@ Magick::VPath& Magick::VPath::operator= (const Magick::VPath& original_ )
 }
 
 // Operator to invoke contained object
-void Magick::VPath::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::VPath::operator()( MagickLib::DrawContext context_ ) const
 {
   if(dp)
     dp->operator()( context_ );
@@ -230,7 +231,7 @@ Magick::DrawableAffine::DrawableAffine( void )
 Magick::DrawableAffine::~DrawableAffine( void )
 {
 }
-void Magick::DrawableAffine::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableAffine::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawAffine( context_, &_affine );
 }
@@ -243,7 +244,7 @@ Magick::DrawableBase* Magick::DrawableAffine::copy() const
 Magick::DrawableArc::~DrawableArc( void )
 {
 }
-void Magick::DrawableArc::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableArc::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawArc( context_, _startX, _startY, _endX, _endY, _startDegrees, _endDegrees );
 }
@@ -269,7 +270,7 @@ Magick::DrawableBezier::DrawableBezier( const Magick::DrawableBezier& original_
 Magick::DrawableBezier::~DrawableBezier( void )
 {
 }
-void Magick::DrawableBezier::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableBezier::operator()( MagickLib::DrawContext context_ ) const
 {
   size_t num_coords = _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
@@ -301,7 +302,7 @@ Magick::DrawableBase* Magick::DrawableBezier::copy() const
 Magick::DrawablePopClipPath::~DrawablePopClipPath ( void )
 {
 }
-void Magick::DrawablePopClipPath::operator() ( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawablePopClipPath::operator() ( MagickLib::DrawContext context_ ) const
 {
   DrawPopClipPath( context_ );
   DrawPopDefs(context_);
@@ -325,7 +326,7 @@ Magick::DrawablePushClipPath::~DrawablePushClipPath( void )
 {
 }
 void Magick::DrawablePushClipPath::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPushDefs(context_);
   DrawPushClipPath( context_, _id.c_str());
@@ -349,7 +350,7 @@ Magick::DrawableClipPath::DrawableClipPath ( const Magick::DrawableClipPath& ori
 Magick::DrawableClipPath::~DrawableClipPath( void )
 {
 }
-void Magick::DrawableClipPath::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableClipPath::operator()( MagickLib::DrawContext context_ ) const
 {
 	DrawSetClipPath( context_, _id.c_str());
 }
@@ -362,7 +363,7 @@ Magick::DrawableBase* Magick::DrawableClipPath::copy() const
 Magick::DrawableCircle::~DrawableCircle ( void )
 {
 }
-void Magick::DrawableCircle::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableCircle::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawCircle( context_, _originX, _originY, _perimX, _perimY );
 }
@@ -375,7 +376,7 @@ Magick::DrawableBase* Magick::DrawableCircle::copy() const
 Magick::DrawableColor::~DrawableColor( void )
 {
 }
-void Magick::DrawableColor::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableColor::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawColor( context_, _x, _y, _paintMethod );
 }
@@ -525,7 +526,7 @@ std::string Magick::DrawableCompositeImage::magick( void )
 }
 
 void Magick::DrawableCompositeImage::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawComposite( context_, _composition, _x, _y, _width, _height,
                  _image->constImage() );
@@ -541,7 +542,7 @@ Magick::DrawableEllipse::~DrawableEllipse( void )
 {
 }
 void Magick::DrawableEllipse::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawEllipse( context_, _originX, _originY, _radiusX, _radiusY,
                _arcStart, _arcEnd );
@@ -565,10 +566,13 @@ Magick::DrawableFillColor::~DrawableFillColor( void )
 {
 }
 void Magick::DrawableFillColor::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   PixelPacket color = static_cast<PixelPacket>(_color);
-  DrawSetFillColor( context_, &color );
+  PixelWand *pixel_wand=NewPixelWand();
+  PixelSetQuantumColor(pixel_wand,&color);
+  DrawSetFillColor(context_,pixel_wand);
+  DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableFillColor::copy() const
 {
@@ -580,7 +584,7 @@ Magick::DrawableFillRule::~DrawableFillRule ( void )
 {
 }
 void Magick::DrawableFillRule::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetFillRule( context_, _fillRule );
 }
@@ -594,7 +598,7 @@ Magick::DrawableFillOpacity::~DrawableFillOpacity ( void )
 {
 }
 void Magick::DrawableFillOpacity::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetFillOpacity( context_, _opacity );
 }
@@ -634,7 +638,7 @@ Magick::DrawableFont::DrawableFont ( const Magick::DrawableFont& original_ )
 Magick::DrawableFont::~DrawableFont ( void )
 {
 }
-void Magick::DrawableFont::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableFont::operator()( MagickLib::DrawContext context_ ) const
 {
   // font
   if(_font.length())
@@ -667,7 +671,7 @@ Magick::DrawableGravity::~DrawableGravity ( void )
 {
 }
 void Magick::DrawableGravity::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetGravity( context_, _gravity );
 }
@@ -680,7 +684,7 @@ Magick::DrawableBase* Magick::DrawableGravity::copy() const
 Magick::DrawableLine::~DrawableLine ( void )
 {
 }
-void Magick::DrawableLine::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableLine::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawLine( context_, _startX, _startY, _endX, _endY );
 }
@@ -693,7 +697,7 @@ Magick::DrawableBase* Magick::DrawableLine::copy() const
 Magick::DrawableMatte::~DrawableMatte ( void )
 {
 }
-void Magick::DrawableMatte::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableMatte::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawMatte( context_, _x, _y, _paintMethod );
 }
@@ -714,7 +718,7 @@ Magick::DrawablePath::DrawablePath ( const Magick::DrawablePath& original_ )
 Magick::DrawablePath::~DrawablePath ( void )
 {
 }
-void Magick::DrawablePath::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawablePath::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawPathStart( context_ );
 
@@ -733,7 +737,7 @@ Magick::DrawableBase* Magick::DrawablePath::copy() const
 Magick::DrawablePoint::~DrawablePoint ( void )
 {
 }
-void Magick::DrawablePoint::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawablePoint::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawPoint( context_, _x, _y );
 }
@@ -747,7 +751,7 @@ Magick::DrawablePointSize::~DrawablePointSize ( void )
 {
 }
 void Magick::DrawablePointSize::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetFontSize( context_, _pointSize );
 }
@@ -770,7 +774,7 @@ Magick::DrawablePolygon::~DrawablePolygon ( void )
 {
 }
 void Magick::DrawablePolygon::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   size_t num_coords = _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
@@ -809,7 +813,7 @@ Magick::DrawablePolyline::~DrawablePolyline ( void )
 {
 }
 void Magick::DrawablePolyline::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   size_t num_coords = _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
@@ -838,7 +842,7 @@ Magick::DrawablePopGraphicContext::~DrawablePopGraphicContext ( void )
 {
 }
 void Magick::DrawablePopGraphicContext::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPopGraphicContext( context_ );
 }
@@ -852,7 +856,7 @@ Magick::DrawablePushGraphicContext::~DrawablePushGraphicContext ( void )
 {
 }
 void Magick::DrawablePushGraphicContext::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPushGraphicContext( context_ );
 }
@@ -866,7 +870,7 @@ Magick::DrawablePopPattern::~DrawablePopPattern ( void )
 {
 }
 void Magick::DrawablePopPattern::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPopPattern( context_ );
 }
@@ -899,7 +903,7 @@ Magick::DrawablePushPattern::~DrawablePushPattern ( void )
 {
 }
 void Magick::DrawablePushPattern::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPushPattern( context_, _id.c_str(), _x, _y, _width, _height );
 }
@@ -913,7 +917,7 @@ Magick::DrawableRectangle::~DrawableRectangle ( void )
 {
 }
 void Magick::DrawableRectangle::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawRectangle( context_, _upperLeftX, _upperLeftY,
                  _lowerRightX, _lowerRightY );
@@ -928,7 +932,7 @@ Magick::DrawableRotation::~DrawableRotation ( void )
 {
 }
 void Magick::DrawableRotation::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawRotate( context_, _angle );
 }
@@ -942,7 +946,7 @@ Magick::DrawableRoundRectangle::~DrawableRoundRectangle ( void )
 {
 }
 void Magick::DrawableRoundRectangle::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawRoundRectangle( context_, _centerX,_centerY, _width,_hight,
                       _cornerWidth, _cornerHeight);
@@ -957,7 +961,7 @@ Magick::DrawableScaling::~DrawableScaling ( void )
 {
 }
 void Magick::DrawableScaling::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawScale( context_, _x, _y );
 }
@@ -971,7 +975,7 @@ Magick::DrawableSkewX::~DrawableSkewX ( void )
 {
 }
 void Magick::DrawableSkewX::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSkewX( context_, _angle );
 }
@@ -984,7 +988,7 @@ Magick::DrawableBase* Magick::DrawableSkewX::copy() const
 Magick::DrawableSkewY::~DrawableSkewY ( void )
 {
 }
-void Magick::DrawableSkewY::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::DrawableSkewY::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawSkewY( context_, _angle );
 }
@@ -1031,7 +1035,7 @@ Magick::DrawableDashArray& Magick::DrawableDashArray::operator=
   return *this;
 }
 void Magick::DrawableDashArray::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeDashArray( context_, _size, _dasharray );
 }
@@ -1041,7 +1045,7 @@ Magick::DrawableBase* Magick::DrawableDashArray::copy() const
 }
 void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_dasharray));
+  _dasharray=(double *) RelinquishMagickMemory(_dasharray);
 
   if(dasharray_)
     {
@@ -1055,7 +1059,7 @@ void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
       _size = n;
 
       // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMemory((n+1)*sizeof(double)));
+      _dasharray=static_cast<double*>(AcquireMagickMemory((n+1)*sizeof(double)));
       // Copy elements
       {
         double *q = _dasharray;
@@ -1070,7 +1074,7 @@ void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
 // code to the const double* version.
 void Magick::DrawableDashArray::dasharray( const unsigned int* dasharray_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_dasharray));
+  _dasharray=(double *) RelinquishMagickMemory(_dasharray);
 
   if(dasharray_)
     {
@@ -1084,7 +1088,7 @@ void Magick::DrawableDashArray::dasharray( const unsigned int* dasharray_ )
       _size = n;
 
       // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMemory((n+1)*sizeof(double)));
+      _dasharray=static_cast<double*>(AcquireMagickMemory((n+1)*sizeof(double)));
       // Copy elements
       {
         double *q = _dasharray;
@@ -1101,7 +1105,7 @@ Magick::DrawableDashOffset::~DrawableDashOffset ( void )
 {
 }
 void Magick::DrawableDashOffset::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeDashOffset( context_, _offset );
 }
@@ -1115,7 +1119,7 @@ Magick::DrawableStrokeLineCap::~DrawableStrokeLineCap ( void )
 {
 }
 void Magick::DrawableStrokeLineCap::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeLineCap( context_, _linecap );
 }
@@ -1129,7 +1133,7 @@ Magick::DrawableStrokeLineJoin::~DrawableStrokeLineJoin ( void )
 {
 }
 void Magick::DrawableStrokeLineJoin::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeLineJoin( context_, _linejoin );
 }
@@ -1143,7 +1147,7 @@ Magick::DrawableMiterLimit::~DrawableMiterLimit ( void )
 {
 }
 void Magick::DrawableMiterLimit::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeMiterLimit( context_, _miterlimit );
 }
@@ -1157,7 +1161,7 @@ Magick::DrawableStrokeAntialias::~DrawableStrokeAntialias ( void )
 {
 }
 void Magick::DrawableStrokeAntialias::operator()
-( MagickLib::DrawingWand *context_ ) const
+( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeAntialias( context_, static_cast<int>(_flag) );
 }
@@ -1181,10 +1185,13 @@ Magick::DrawableStrokeColor::~DrawableStrokeColor ( void )
 {
 }
 void Magick::DrawableStrokeColor::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   PixelPacket color = static_cast<PixelPacket>(_color);
-  DrawSetStrokeColor( context_, &color );
+  PixelWand *pixel_wand=NewPixelWand();
+  PixelSetQuantumColor(pixel_wand,&color);
+  DrawSetStrokeColor(context_,pixel_wand);
+  DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableStrokeColor::copy() const
 {
@@ -1196,7 +1203,7 @@ Magick::DrawableStrokeOpacity::~DrawableStrokeOpacity ( void )
 {
 }
 void Magick::DrawableStrokeOpacity::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeOpacity( context_, _opacity );
 }
@@ -1210,7 +1217,7 @@ Magick::DrawableStrokeWidth::~DrawableStrokeWidth ( void )
 {
 }
 void Magick::DrawableStrokeWidth::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetStrokeWidth( context_, _width );
 }
@@ -1247,7 +1254,7 @@ Magick::DrawableText::~DrawableText ( void )
 {
 }
 void Magick::DrawableText::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetTextEncoding( context_, _encoding.c_str() );
   DrawAnnotation( context_, _x, _y,
@@ -1271,7 +1278,7 @@ Magick::DrawableTextAntialias::~DrawableTextAntialias ( void )
 {
 }
 void Magick::DrawableTextAntialias::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetTextAntialias( context_, static_cast<int>(_flag) );
 }
@@ -1295,7 +1302,7 @@ Magick::DrawableTextDecoration::~DrawableTextDecoration( void )
 {
 }
 void Magick::DrawableTextDecoration::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetTextDecoration( context_, _decoration );
 }
@@ -1319,10 +1326,13 @@ Magick::DrawableTextUnderColor::~DrawableTextUnderColor ( void )
 {
 }
 void Magick::DrawableTextUnderColor::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   PixelPacket color = static_cast<PixelPacket>(_color);
-  DrawSetTextUnderColor( context_, &color );
+  PixelWand *pixel_wand=NewPixelWand();
+  PixelSetQuantumColor(pixel_wand,&color);
+  DrawSetTextUnderColor(context_,pixel_wand);
+  DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableTextUnderColor::copy() const
 {
@@ -1334,7 +1344,7 @@ Magick::DrawableTranslation::~DrawableTranslation ( void )
 {
 }
 void Magick::DrawableTranslation::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawTranslate( context_, _x, _y );
 }
@@ -1348,7 +1358,7 @@ Magick::DrawableViewbox::~DrawableViewbox ( void )
 {
 }
 void Magick::DrawableViewbox::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawSetViewbox( context_, _x1, _y1, _x2, _y2 );
 }
@@ -1450,7 +1460,7 @@ Magick::PathArcAbs::PathArcAbs ( const Magick::PathArcAbs& original_ )
 Magick::PathArcAbs::~PathArcAbs ( void )
 {
 }
-void Magick::PathArcAbs::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathArcAbs::operator()( MagickLib::DrawContext context_ ) const
 {
   for( PathArcArgsList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1480,7 +1490,7 @@ Magick::PathArcRel::PathArcRel ( const Magick::PathArcRel& original_ )
 Magick::PathArcRel::~PathArcRel ( void )
 {
 }
-void Magick::PathArcRel::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathArcRel::operator()( MagickLib::DrawContext context_ ) const
 {
   for( PathArcArgsList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1501,7 +1511,7 @@ Magick::VPathBase* Magick::PathArcRel::copy() const
 Magick::PathClosePath::~PathClosePath ( void )
 {
 }
-void Magick::PathClosePath::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathClosePath::operator()( MagickLib::DrawContext context_ ) const
 {
   DrawPathClose( context_ );
 }
@@ -1597,7 +1607,7 @@ Magick::PathCurvetoAbs::~PathCurvetoAbs ( void )
 {
 }
 void Magick::PathCurvetoAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( PathCurveToArgsList::const_iterator p = _args.begin();
        p != _args.end(); p++ )
@@ -1627,7 +1637,7 @@ Magick::PathCurvetoRel::~PathCurvetoRel ( void )
 {
 }
 void Magick::PathCurvetoRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( PathCurveToArgsList::const_iterator p = _args.begin();
        p != _args.end(); p++ )
@@ -1659,7 +1669,7 @@ Magick::PathSmoothCurvetoAbs::~PathSmoothCurvetoAbs ( void )
 {
 }
 void Magick::PathSmoothCurvetoAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1694,7 +1704,7 @@ Magick::PathSmoothCurvetoRel::~PathSmoothCurvetoRel ( void )
 {
 }
 void Magick::PathSmoothCurvetoRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1801,7 +1811,7 @@ Magick::PathQuadraticCurvetoAbs::~PathQuadraticCurvetoAbs ( void )
 {
 }
 void Magick::PathQuadraticCurvetoAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( PathQuadraticCurvetoArgsList::const_iterator p = _args.begin();
        p != _args.end(); p++ )
@@ -1833,7 +1843,7 @@ Magick::PathQuadraticCurvetoRel::~PathQuadraticCurvetoRel ( void )
 {
 }
 void Magick::PathQuadraticCurvetoRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( PathQuadraticCurvetoArgsList::const_iterator p = _args.begin();
        p != _args.end(); p++ )
@@ -1865,7 +1875,7 @@ Magick::PathSmoothQuadraticCurvetoAbs::~PathSmoothQuadraticCurvetoAbs ( void )
 {
 }
 void Magick::PathSmoothQuadraticCurvetoAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1896,7 +1906,7 @@ Magick::PathSmoothQuadraticCurvetoRel::~PathSmoothQuadraticCurvetoRel ( void )
 {
 }
 void Magick::PathSmoothQuadraticCurvetoRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1927,7 +1937,7 @@ Magick::PathLinetoAbs::PathLinetoAbs ( const Magick::PathLinetoAbs& original_ )
 Magick::PathLinetoAbs::~PathLinetoAbs ( void )
 {
 }
-void Magick::PathLinetoAbs::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathLinetoAbs::operator()( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1954,7 +1964,7 @@ Magick::PathLinetoRel::PathLinetoRel ( const Magick::PathLinetoRel& original_ )
 Magick::PathLinetoRel::~PathLinetoRel ( void )
 {
 }
-void Magick::PathLinetoRel::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathLinetoRel::operator()( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -1975,7 +1985,7 @@ Magick::PathLinetoHorizontalAbs::~PathLinetoHorizontalAbs ( void )
 {
 }
 void Magick::PathLinetoHorizontalAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPathLineToHorizontalAbsolute( context_, _x );
 }
@@ -1987,7 +1997,7 @@ Magick::PathLinetoHorizontalRel::~PathLinetoHorizontalRel ( void )
 {
 }
 void Magick::PathLinetoHorizontalRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPathLineToHorizontalRelative( context_, _x );
 }
@@ -2003,7 +2013,7 @@ Magick::PathLinetoVerticalAbs::~PathLinetoVerticalAbs ( void )
 {
 }
 void Magick::PathLinetoVerticalAbs::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPathLineToVerticalAbsolute( context_, _y );
 }
@@ -2015,7 +2025,7 @@ Magick::PathLinetoVerticalRel::~PathLinetoVerticalRel ( void )
 {
 }
 void Magick::PathLinetoVerticalRel::operator()
-  ( MagickLib::DrawingWand *context_ ) const
+  ( MagickLib::DrawContext context_ ) const
 {
   DrawPathLineToVerticalRelative( context_, _y );
 }
@@ -2043,7 +2053,7 @@ Magick::PathMovetoAbs::PathMovetoAbs ( const Magick::PathMovetoAbs& original_ )
 Magick::PathMovetoAbs::~PathMovetoAbs ( void )
 {
 }
-void Magick::PathMovetoAbs::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathMovetoAbs::operator()( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
@@ -2070,7 +2080,7 @@ Magick::PathMovetoRel::PathMovetoRel ( const Magick::PathMovetoRel& original_ )
 Magick::PathMovetoRel::~PathMovetoRel ( void )
 {
 }
-void Magick::PathMovetoRel::operator()( MagickLib::DrawingWand *context_ ) const
+void Magick::PathMovetoRel::operator()( MagickLib::DrawContext context_ ) const
 {
   for( CoordinateList::const_iterator p = _coordinates.begin();
        p != _coordinates.end(); p++ )
