@@ -2,7 +2,7 @@
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
 //
-// Inclusion of ImageMagick headers (with namespace magic)
+// Inclusion of GraphicsMagick headers (with namespace magic)
 
 #ifndef Magick_Include_header
 #define Magick_Include_header
@@ -42,28 +42,31 @@
 
 //
 // Include ImageMagick headers into namespace "MagickLib". If
-// MAGICK_IMPLEMENTATION is defined, include ImageMagick development
+// MAGICK_IMPLEMENTATION is defined, include GraphicsMagick development
 // headers.  This scheme minimizes the possibility of conflict with
 // user code.
 //
 namespace MagickLib
 {
 #include <magick/api.h>
+#include <wand/magick_wand.h>
 #undef inline // Remove possible definition from config.h
 
 #undef class
 }
 
 //
-// Provide appropriate DLL imports/exports for Visual C++ and MinGW builds
+// Provide appropriate DLL imports/exports for Visual C++,
+// Borland C++Builder and MinGW builds
 //
-#if defined(WIN32) && !defined(__CYGWIN__)
-# define MagickWIN32
+#if defined(WIN32) && !defined(__CYGWIN__) && !defined(__MINGW32__)
+# define MagickCplusPlusDLLSupported
 #endif
-#if defined(MagickWIN32)
+#if defined(MagickCplusPlusDLLSupported)
 #  if defined(_MT) && defined(_DLL) && !defined(_LIB)
 #    define MagickDLLBuild
 #    pragma warning( disable: 4273 )      /* Disable the stupid dll linkage warnings */
+#    pragma warning( disable: 4251 )
 #    if !defined(MAGICK_IMPLEMENTATION)
 #      define MagickDLLDecl __declspec(dllimport)
 #      define MagickDLLDeclExtern extern __declspec(dllimport)
@@ -71,8 +74,14 @@ namespace MagickLib
 #        pragma message( "Magick++ lib DLL import" )
 #      endif
 #    else
-#      define MagickDLLDecl __declspec(dllexport)
-#      define MagickDLLDeclExtern extern __declspec(dllexport)
+#      if defined(__BORLANDC__)
+#        define MagickDLLDecl __declspec(dllexport)
+#        define MagickDLLDeclExtern __declspec(dllexport)
+#        pragma message( "BCBMagick++ lib DLL export" )
+#      else
+#        define MagickDLLDecl __declspec(dllexport)
+#        define MagickDLLDeclExtern extern __declspec(dllexport)
+#      endif
 #      if defined(_VISUALC_)
 #        pragma message( "Magick++ lib DLL export" )
 #      endif
@@ -89,8 +98,9 @@ namespace MagickLib
 #  define MagickDLLDeclExtern
 #endif
 
+
 //
-// Import ImageMagick symbols and types which are used as part of the
+// Import GraphicsMagick symbols and types which are used as part of the
 // Magick++ API definition into namespace "Magick".
 //
 namespace Magick
@@ -169,6 +179,10 @@ namespace Magick
   using MagickLib::ThresholdCompositeOp;
   using MagickLib::UndefinedCompositeOp;
   using MagickLib::XorCompositeOp;
+  using MagickLib::CopyCyanCompositeOp;
+  using MagickLib::CopyMagentaCompositeOp;
+  using MagickLib::CopyYellowCompositeOp;
+  using MagickLib::CopyBlackCompositeOp;
   
   // Compression algorithms
   using MagickLib::CompressionType;
@@ -194,6 +208,22 @@ namespace Magick
   using MagickLib::LSBEndian;
   using MagickLib::MSBEndian;
 
+  // Evaluate options
+  using MagickLib::UndefinedEvaluateOperator;
+  using MagickLib::AddEvaluateOperator;
+  using MagickLib::AndEvaluateOperator;
+  using MagickLib::DivideEvaluateOperator;
+  using MagickLib::LeftShiftEvaluateOperator;
+  using MagickLib::MaxEvaluateOperator;
+  using MagickLib::MinEvaluateOperator;
+  using MagickLib::MultiplyEvaluateOperator;
+  using MagickLib::OrEvaluateOperator;
+  using MagickLib::RightShiftEvaluateOperator;
+  using MagickLib::SetEvaluateOperator;
+  using MagickLib::SubtractEvaluateOperator;
+  using MagickLib::XorEvaluateOperator;
+  using MagickLib::MagickEvaluateOperator;
+
   // Fill rules
   using MagickLib::FillRule;
   using MagickLib::UndefinedRule;
@@ -394,25 +424,30 @@ namespace Magick
 
 #if defined(MAGICK_IMPLEMENTATION)
   //
-  // ImageMagick symbols used in implementation code
+  // GraphicsMagick symbols used in implementation code
   //
   using MagickLib::AcquireCacheView;
   using MagickLib::AcquireImagePixels;
-  using MagickLib::AcquireMemory;
+  using MagickLib::AcquireMagickMemory;
+  using MagickLib::AcquireString;
+	using MagickLib::AcquireStringInfo;
   using MagickLib::AdaptiveThresholdImage;
   using MagickLib::AddNoiseImage;
   using MagickLib::AffineMatrix;
+  using MagickLib::AffineTransformImage;
   using MagickLib::AllocateImage;
   using MagickLib::AnnotateImage;
   using MagickLib::AspectValue;
   using MagickLib::Base64Decode;
   using MagickLib::Base64Encode;
   using MagickLib::BlobError;
+  using MagickLib::BlobFatalError;
   using MagickLib::BlobToImage;
   using MagickLib::BlobWarning;
   using MagickLib::BlurImage;
   using MagickLib::BorderImage;
   using MagickLib::CacheError;
+  using MagickLib::CacheFatalError;
   using MagickLib::CacheWarning;
   using MagickLib::ChannelImage;
   using MagickLib::CharcoalImage;
@@ -422,26 +457,38 @@ namespace Magick
   using MagickLib::CloneImageInfo;
   using MagickLib::CloneQuantizeInfo;
   using MagickLib::CloseCacheView;
+  using MagickLib::CoderError;
+  using MagickLib::CoderFatalError;
+  using MagickLib::CoderWarning;
   using MagickLib::ColorFloodfillImage;
   using MagickLib::ColorizeImage;
+  using MagickLib::ColorPacket;
   using MagickLib::CompositeImage;
+  using MagickLib::ConfigureError;
+  using MagickLib::ConfigureFatalError;
+  using MagickLib::ConfigureWarning;
   using MagickLib::ConstituteImage;
   using MagickLib::ContrastImage;
   using MagickLib::ConvolveImage;
   using MagickLib::CorruptImageError;
+  using MagickLib::CorruptImageFatalError;
   using MagickLib::CorruptImageWarning;
   using MagickLib::CropImage;
   using MagickLib::CycleColormapImage;
   using MagickLib::DelegateError;
+  using MagickLib::DelegateFatalError;
   using MagickLib::DelegateWarning;
   using MagickLib::DeleteMagickRegistry;
   using MagickLib::DespeckleImage;
   using MagickLib::DestroyDrawInfo;
+  using MagickLib::DestroyDrawingWand;
   using MagickLib::DestroyExceptionInfo;
   using MagickLib::DestroyImageInfo;
   using MagickLib::DestroyImageList;
   using MagickLib::DestroyMagick;
+  using MagickLib::DestroyPixelWand;
   using MagickLib::DestroyQuantizeInfo;
+  using MagickLib::DestroyStringInfo;
   using MagickLib::DispatchImage;
   using MagickLib::DisplayImages;
   using MagickLib::DrawAffine;
@@ -453,11 +500,12 @@ namespace Magick
   using MagickLib::DrawColor;
   using MagickLib::DrawComment;
   using MagickLib::DrawComposite;
-  using MagickLib::DrawingWand;
-  using MagickLib::DestroyDrawingWand;
   using MagickLib::DrawEllipse;
+  using MagickLib::DrawError;
+  using MagickLib::DrawFatalError;
   using MagickLib::DrawImage;
   using MagickLib::DrawInfo;
+  using MagickLib::DrawingWand;
   using MagickLib::DrawLine;
   using MagickLib::DrawMatte;
   using MagickLib::DrawPathClose;
@@ -501,7 +549,6 @@ namespace Magick
   using MagickLib::DrawSetClipRule;
   using MagickLib::DrawSetClipUnits;
   using MagickLib::DrawSetFillColor;
-  using MagickLib::DrawSetFillColorString;
   using MagickLib::DrawSetFillOpacity;
   using MagickLib::DrawSetFillPatternURL;
   using MagickLib::DrawSetFillRule;
@@ -514,7 +561,6 @@ namespace Magick
   using MagickLib::DrawSetGravity;
   using MagickLib::DrawSetStrokeAntialias;
   using MagickLib::DrawSetStrokeColor;
-  using MagickLib::DrawSetStrokeColorString;
   using MagickLib::DrawSetStrokeDashArray;
   using MagickLib::DrawSetStrokeDashOffset;
   using MagickLib::DrawSetStrokeLineCap;
@@ -527,18 +573,22 @@ namespace Magick
   using MagickLib::DrawSetTextDecoration;
   using MagickLib::DrawSetTextEncoding;
   using MagickLib::DrawSetTextUnderColor;
-  using MagickLib::DrawSetTextUnderColorString;
   using MagickLib::DrawSetViewbox;
   using MagickLib::DrawSkewX;
   using MagickLib::DrawSkewY;
   using MagickLib::DrawTranslate;
+  using MagickLib::DrawWarning;
   using MagickLib::EdgeImage;
   using MagickLib::EmbossImage;
   using MagickLib::EnhanceImage;
+  using MagickLib::ExecuteModuleProcess;
   using MagickLib::EqualizeImage;
+  using MagickLib::EvaluateImage;
+  using MagickLib::EvaluateImageChannel;
   using MagickLib::ExceptionInfo;
   using MagickLib::ExceptionType;
   using MagickLib::FileOpenError;
+  using MagickLib::FileOpenFatalError;
   using MagickLib::FileOpenWarning;
   using MagickLib::FlattenImages;
   using MagickLib::FlipImage;
@@ -551,16 +601,21 @@ namespace Magick
   using MagickLib::GaussianBlurImage;
   using MagickLib::GetCacheView;
   using MagickLib::GetCacheViewIndexes;
+  using MagickLib::GetImageClipMask;
+  using MagickLib::GetImageChannelExtrema;
+  using MagickLib::GetImageChannelMean;
   using MagickLib::GetColorTuple;
   using MagickLib::GetDrawInfo;
   using MagickLib::GetExceptionInfo;
   using MagickLib::GetGeometry;
   using MagickLib::GetImageAttribute;
-  using MagickLib::GetImageAttribute;
   using MagickLib::GetImageBoundingBox;
+  using MagickLib::GetImageChannelDepth;
   using MagickLib::GetImageDepth;
   using MagickLib::GetImageInfo;
+  using MagickLib::GetImageOption;
   using MagickLib::GetImagePixels;
+  using MagickLib::GetImageProfile;
   using MagickLib::GetImageQuantizeError;
   using MagickLib::GetImageType;
   using MagickLib::GetIndexes;
@@ -578,17 +633,19 @@ namespace Magick
   using MagickLib::HeightValue;
   using MagickLib::IdentityAffine;
   using MagickLib::ImageAttribute;
+  using MagickLib::ImageError;
+  using MagickLib::ImageFatalError;
   using MagickLib::ImageInfo;
   using MagickLib::ImageInfoRegistryType;
   using MagickLib::ImageRegistryType;
   using MagickLib::ImageToBlob;
+  using MagickLib::ImageWarning;
   using MagickLib::ImplodeImage;
   using MagickLib::IsEventLogging;
   using MagickLib::IsGeometry;
   using MagickLib::IsImagesEqual;
   using MagickLib::IsSubimage;
   using MagickLib::LessValue;
-  using MagickLib::LiberateMemory;
   using MagickLib::LocaleCompare;
   using MagickLib::LogMagickEvent;
   using MagickLib::MagickInfo;
@@ -599,20 +656,32 @@ namespace Magick
   using MagickLib::MedianFilterImage;
   using MagickLib::MinifyImage;
   using MagickLib::MissingDelegateError;
+  using MagickLib::MissingDelegateFatalError;
   using MagickLib::MissingDelegateWarning;
   using MagickLib::ModulateImage;
+  using MagickLib::ModuleError;
+  using MagickLib::ModuleFatalError;
+  using MagickLib::ModuleWarning;
+  using MagickLib::MonitorError;
+  using MagickLib::MonitorFatalError;
+  using MagickLib::MonitorWarning;
   using MagickLib::MontageInfo;
   using MagickLib::NegateImage;
+  using MagickLib::NewPixelWand;
+  using MagickLib::NoValue;
   using MagickLib::NoiseType;
   using MagickLib::NormalizeImage;
   using MagickLib::OilPaintImage;
   using MagickLib::OpaqueImage;
   using MagickLib::OpenCacheView;
   using MagickLib::OptionError;
+  using MagickLib::OptionFatalError;
   using MagickLib::OptionWarning;
   using MagickLib::PercentValue;
   using MagickLib::PingBlob;
   using MagickLib::PingImage;
+  using MagickLib::PixelSetQuantumColor;
+	using MagickLib::PixelWand;
   using MagickLib::PointInfo;
   using MagickLib::PopImagePixels;
   using MagickLib::ProfileImage;
@@ -623,14 +692,20 @@ namespace Magick
   using MagickLib::QueryColorDatabase;
   using MagickLib::RGBTransformImage;
   using MagickLib::RaiseImage;
-  using MagickLib::ReacquireMemory;
   using MagickLib::ReadImage;
   using MagickLib::RectangleInfo;
   using MagickLib::RectangleInfo;
   using MagickLib::ReduceNoiseImage;
   using MagickLib::RegisterMagickInfo;
+  using MagickLib::RegistryError;
+  using MagickLib::RegistryFatalError;
   using MagickLib::RegistryType;
+  using MagickLib::RegistryWarning;
+  using MagickLib::RelinquishMagickMemory;
+  using MagickLib::RemoveImageOption;
+  using MagickLib::ResizeMagickMemory;
   using MagickLib::ResourceLimitError;
+  using MagickLib::ResourceLimitFatalError;
   using MagickLib::ResourceLimitWarning;
   using MagickLib::RollImage;
   using MagickLib::RotateImage;
@@ -642,15 +717,19 @@ namespace Magick
   using MagickLib::SetClientName;
   using MagickLib::SetImage;
   using MagickLib::SetImageAttribute;
+  using MagickLib::SetImageChannelDepth;
   using MagickLib::SetImageClipMask;
   using MagickLib::SetImageDepth;
   using MagickLib::SetImageInfo;
   using MagickLib::SetImageOpacity;
+  using MagickLib::SetImageOption;
   using MagickLib::SetImagePixels;
+  using MagickLib::SetImageProfile;
   using MagickLib::SetImageType;
   using MagickLib::SetLogEventMask;
   using MagickLib::SetMagickInfo;
   using MagickLib::SetMagickRegistry;
+  using MagickLib::SetStringInfoDatum;
   using MagickLib::ShadeImage;
   using MagickLib::SharpenImage;
   using MagickLib::ShaveImage;
@@ -661,6 +740,10 @@ namespace Magick
   using MagickLib::SpreadImage;
   using MagickLib::SteganoImage;
   using MagickLib::StereoImage;
+  using MagickLib::StreamError;
+  using MagickLib::StreamFatalError;
+  using MagickLib::StreamWarning;
+  using MagickLib::StringInfo;
   using MagickLib::SwirlImage;
   using MagickLib::SyncCacheView;
   using MagickLib::SyncImage;
@@ -672,6 +755,9 @@ namespace Magick
   using MagickLib::TransformImage;
   using MagickLib::TransformRGBImage;
   using MagickLib::TransparentImage;
+  using MagickLib::TypeError;
+  using MagickLib::TypeFatalError;
+  using MagickLib::TypeWarning;
   using MagickLib::UndefinedException;
   using MagickLib::UndefinedRegistryType;
   using MagickLib::UnregisterMagickInfo;
@@ -682,6 +768,7 @@ namespace Magick
   using MagickLib::WriteImage;
   using MagickLib::XNegative;
   using MagickLib::XServerError;
+  using MagickLib::XServerFatalError;
   using MagickLib::XServerWarning;
   using MagickLib::XValue;
   using MagickLib::YNegative;
