@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Definition and implementation of template functions for using
 // Magick::Image with STL containers.
@@ -9,10 +9,13 @@
 #ifndef Magick_STL_header
 #define Magick_STL_header
 
+#include "Magick++/Include.h"
 #include <algorithm>
 #include <functional>
 #include <iterator>
-#include "Magick++/Include.h"
+#include <map>
+#include <utility>
+
 #include "Magick++/CoderInfo.h"
 #include "Magick++/Drawable.h"
 #include "Magick++/Exception.h"
@@ -70,6 +73,18 @@ namespace Magick
     NoiseType _noiseType;
   };
 
+  // Transform image by specified affine (or free transform) matrix.
+  class MagickDLLDecl affineTransformImage : public std::unary_function<Image&,void>
+  {
+  public:
+    affineTransformImage( const DrawableAffine &affine_ );
+
+    void operator()( Image &image_ ) const;
+
+  private:
+    DrawableAffine _affine;
+  };
+
   // Annotate image (draw text on image)
   class MagickDLLDecl annotateImage : public std::unary_function<Image&,void>
   {
@@ -1203,6 +1218,19 @@ namespace Magick
     Color        _color;
   };
 
+  // Composition operator to be used when composition is implicitly used
+  // (such as for image flattening).
+  class MagickDLLDecl composeImage : public std::unary_function<Image&,void>
+  {
+  public:
+    composeImage( const CompositeOperator compose_ );
+                                                                                
+    void operator()( Image &image_ ) const;
+                                                                                
+  private:
+    CompositeOperator _compose;
+  };
+
   // Compression type
   class MagickDLLDecl compressTypeImage : public std::unary_function<Image&,void>
   {
@@ -1688,7 +1716,7 @@ namespace Magick
 
   // Insert images in image list into existing container (appending to container)
   // The images should not be deleted since only the image ownership is passed.
-  // The options copied into the object.
+  // The options are copied into the object.
   template <class Container>
   void insertImages( Container *sequence_,
 		     MagickLib::Image* images_ ) {
@@ -1766,16 +1794,30 @@ namespace Magick
 
   // Merge a sequence of images.
   // This is useful for GIF animation sequences that have page
-  // offsets and disposal methods.
-  // The images are modified in-place. No pointer re-assignments are performed.
-  template <class InputIterator>
-  void coalesceImages( InputIterator first_,
-		       InputIterator last_ ) {
+  // offsets and disposal methods. A container to contain
+  // the updated image sequence is passed via the coalescedImages_
+  // option.
+  template <class InputIterator, class Container >
+  void coalesceImages( Container *coalescedImages_,
+                       InputIterator first_,
+                       InputIterator last_ ) {
     MagickLib::ExceptionInfo exceptionInfo;
     MagickLib::GetExceptionInfo( &exceptionInfo );
+
+    // Build image list
     linkImages( first_, last_ );
-    MagickLib::CoalesceImages( first_->image(), &exceptionInfo );
+    MagickLib::Image* images = MagickLib::CoalesceImages( first_->image(),
+                                                          &exceptionInfo);
+    // Unlink image list
     unlinkImages( first_, last_ );
+
+    // Ensure container is empty
+    coalescedImages_->clear();
+
+    // Move images to container
+    insertImages( coalescedImages_, images );
+
+    // Report any error
     throwException( exceptionInfo );
   }
 
@@ -1795,26 +1837,34 @@ namespace Magick
                       CoderInfo::MatchType isMultiFrame_ = CoderInfo::AnyMatch
                       ) {
     // Obtain first entry in MagickInfo list
+    unsigned long number_formats;
     MagickLib::ExceptionInfo exceptionInfo;
     MagickLib::GetExceptionInfo( &exceptionInfo );
-    const MagickLib::MagickInfo *magickInfo =
-      MagickLib::GetMagickInfo( "*", &exceptionInfo );
-    throwException( exceptionInfo );
-    if( !magickInfo )
-      throwExceptionExplicit(MagickLib::MissingDelegateError,
-                             "Coder list not returned!", 0 );
+    char **coder_list =
+      MagickLib::GetMagickList( "*", &number_formats );
+    if( !coder_list )
+      {
+        throwException( exceptionInfo );
+        throwExceptionExplicit(MagickLib::MissingDelegateError,
+                             "Coder array not returned!", 0 );
+      }
 
     // Clear out container
     container_->clear();
 
-    for ( ; magickInfo != 0; magickInfo=(const MagickLib::MagickInfo *) magickInfo->next)
+    for ( int i=0; i < number_formats; i++)
       {
+        const MagickLib::MagickInfo *magick_info =
+          MagickLib::GetMagickInfo( coder_list[i], &exceptionInfo );
+        coder_list[i]=(char *)
+          MagickLib::RelinquishMagickMemory( coder_list[i] );
+
         // Skip stealth coders
-        if ( magickInfo->stealth )
+        if ( magick_info->stealth )
           continue;
 
         try {
-          CoderInfo coderInfo( magickInfo );
+          CoderInfo coderInfo( magick_info->name );
 
           // Test isReadable_
           if ( isReadable_ != CoderInfo::AnyMatch &&
@@ -1837,14 +1887,86 @@ namespace Magick
           // Append matches to container
           container_->push_back( coderInfo );
         }
-        // Intentionally ignore missing delegate errors
-        catch ( Magick::ErrorMissingDelegate )
+        // Intentionally ignore missing module errors
+        catch ( Magick::ErrorModule )
           {
             continue;
           }
       }
+    coder_list=(char **) MagickLib::RelinquishMagickMemory( coder_list );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
+  //
+  // Fill container with color histogram.
+  // Entries are of type "std::pair<Color,unsigned long>".  Use the pair
+  // "first" member to access the Color and the "second" member to access
+  // the number of times the color occurs in the image.
+  //
+  // For example:
+  //
+  //  Using <map>:
+  //
+  //  Image image("image.miff");
+  //  map<Color,unsigned long> histogram;
+  //  colorHistogram( &histogram, image );
+  //  std::map<Color,unsigned long>::const_iterator p=histogram.begin();
+  //  while (p != histogram.end())
+  //    {
+  //      cout << setw(10) << (int)p->second << ": ("
+  //           << setw(quantum_width) << (int)p->first.redQuantum() << ","
+  //           << setw(quantum_width) << (int)p->first.greenQuantum() << ","
+  //           << setw(quantum_width) << (int)p->first.blueQuantum() << ")"
+  //           << endl;
+  //      p++;
+  //    }
+  //
+  //  Using <vector>:
+  //
+  //  Image image("image.miff");
+  //  std::vector<std::pair<Color,unsigned long> > histogram;
+  //  colorHistogram( &histogram, image );
+  //  std::vector<std::pair<Color,unsigned long> >::const_iterator p=histogram.begin();
+  //  while (p != histogram.end())
+  //    {
+  //      cout << setw(10) << (int)p->second << ": ("
+  //           << setw(quantum_width) << (int)p->first.redQuantum() << ","
+  //           << setw(quantum_width) << (int)p->first.greenQuantum() << ","
+  //           << setw(quantum_width) << (int)p->first.blueQuantum() << ")"
+  //           << endl;
+  //      p++;
+  //    }
+
+  template <class Container >
+  void colorHistogram( Container *histogram_, const Image image)
+  {
+    MagickLib::ExceptionInfo exceptionInfo;
+    MagickLib::GetExceptionInfo( &exceptionInfo );
+
+    // Obtain histogram array
+    unsigned long colors;
+    MagickLib::ColorPacket *histogram_array = 
+      MagickLib::GetImageHistogram( image.constImage(), &colors, &exceptionInfo );
+    throwException( exceptionInfo );
+
+    // Clear out container
+    histogram_->clear();
+
+    // Transfer histogram array to container
+    for ( unsigned long i=0; i < colors; i++)
+      {
+        histogram_->insert(histogram_->end(),std::pair<const Color,unsigned long>
+                           ( Color(histogram_array[i].pixel.red,
+                                   histogram_array[i].pixel.green,
+                                   histogram_array[i].pixel.blue),
+                                   histogram_array[i].count) );
+      }
+    
+    // Deallocate histogram array
+    histogram_array=(MagickLib::ColorPacket *)
+      MagickLib::RelinquishMagickMemory(histogram_array);
+  }
+                      
   // Break down an image sequence into constituent parts.  This is
   // useful for creating GIF or MNG animation sequences.
   template <class InputIterator, class Container >
@@ -1963,7 +2085,7 @@ namespace Magick
 		      const Montage &montageOpts_ ) {
 
     MagickLib::MontageInfo* montageInfo =
-      static_cast<MagickLib::MontageInfo*>(MagickLib::AcquireMemory(sizeof(MagickLib::MontageInfo)));
+      static_cast<MagickLib::MontageInfo*>(MagickLib::AcquireMagickMemory(sizeof(MagickLib::MontageInfo)));
 
     // Update montage options with those set in montageOpts_
     montageOpts_.updateMontageInfo( *montageInfo );
@@ -2088,6 +2210,7 @@ namespace Magick
   }
 
   // Read images into existing container (appending to container)
+  // FIXME: need a way to specify options like size, depth, and density.
   template <class Container>
   void readImages( Container *sequence_,
 		   const std::string &imageSpec_ ) {
@@ -2122,31 +2245,21 @@ namespace Magick
 		    const std::string &imageSpec_,
 		    bool adjoin_ = true ) {
 
-    // Save original image settings
-    std::string origFileName = first_->fileName();
-    std::string origMagick = first_->magick();
-    bool origAdjoin = first_->adjoin();
-
-    first_->fileName( imageSpec_ );
     first_->adjoin( adjoin_ );
 
     MagickLib::ExceptionInfo exceptionInfo;
     MagickLib::GetExceptionInfo( &exceptionInfo );
 
     linkImages( first_, last_ );
-    int errorStat = MagickLib::WriteImage( first_->imageInfo(),
-					   first_->image() );
+    int errorStat = MagickLib::WriteImages( first_->imageInfo(),
+                                            first_->image(),
+                                            imageSpec_.c_str(),
+                                            &exceptionInfo );
     unlinkImages( first_, last_ );
 
-    // Restore original image settings
-    first_->fileName( origFileName );
-    first_->magick( origMagick );
-    first_->adjoin( origAdjoin );
-
     if ( errorStat != false )
       return;
 
-    MagickLib::GetImageException( first_->image(), &exceptionInfo );
     throwException( exceptionInfo );
   }
   // Write images to BLOB
@@ -2155,9 +2268,6 @@ namespace Magick
 		    InputIterator last_,
 		    Blob *blob_,
 		    bool adjoin_ = true) {
-    // Save original image settings
-    std::string origMagick = first_->magick();
-    bool origAdjoin = first_->adjoin();
 
     first_->adjoin( adjoin_ );
 
@@ -2170,19 +2280,13 @@ namespace Magick
 					 first_->image(),
 					 &length,
 					 &exceptionInfo);
-    blob_->updateNoCopy( data, length );
+    blob_->updateNoCopy( data, length, Magick::Blob::MallocAllocator );
 
     unlinkImages( first_, last_ );
 
-    // Restore original image settings
-    first_->magick( origMagick );
-    first_->adjoin( origAdjoin );
-
     throwException( exceptionInfo );
   }
 
-
-
 } // namespace Magick
 
 #endif // Magick_STL_header
