@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Implementation of Options
 //
@@ -9,6 +9,7 @@
 
 #define MAGICK_IMPLEMENTATION
 
+#include "Magick++/Include.h"
 #include <string>
 #include <string.h>
 #include <stdlib.h>
@@ -17,13 +18,11 @@
 #include "Magick++/Functions.h"
 #include "Magick++/Exception.h"
 
-#define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
-
 // Constructor
 Magick::Options::Options( void )
-  : _imageInfo(static_cast<ImageInfo*>(AcquireMemory(sizeof(ImageInfo)))),
-    _quantizeInfo(static_cast<QuantizeInfo*>(AcquireMemory(sizeof(QuantizeInfo)))),
-    _drawInfo(static_cast<DrawInfo*>(AcquireMemory( sizeof(DrawInfo))))
+  : _imageInfo(static_cast<ImageInfo*>(AcquireMagickMemory(sizeof(ImageInfo)))),
+    _quantizeInfo(static_cast<QuantizeInfo*>(AcquireMagickMemory(sizeof(QuantizeInfo)))),
+    _drawInfo(static_cast<DrawInfo*>(AcquireMagickMemory( sizeof(DrawInfo))))
 {
   // Initialize image info with defaults
   GetImageInfo( _imageInfo );
@@ -108,7 +107,7 @@ Magick::Color Magick::Options::backgroundColor ( void ) const
 void Magick::Options::backgroundTexture ( const std::string &backgroundTexture_ )
 {
   if ( backgroundTexture_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->texture) );
+    _imageInfo->texture=(char *) RelinquishMagickMemory(_imageInfo->texture);
   else
     Magick::CloneString( &_imageInfo->texture, backgroundTexture_ );
 }
@@ -182,7 +181,7 @@ bool Magick::Options::debug ( void ) const
 void Magick::Options::density ( const Magick::Geometry &density_ )
 {
   if ( !density_.isValid() )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->density) );
+    _imageInfo->density=(char *) RelinquishMagickMemory(_imageInfo->density);
   else
     Magick::CloneString( &_imageInfo->density, density_ );
 }
@@ -275,8 +274,8 @@ void Magick::Options::font ( const std::string &font_ )
 {
   if ( font_.length() == 0 )
     {
-      LiberateMemory( reinterpret_cast<void**>(&_imageInfo->font) );
-      LiberateMemory( reinterpret_cast<void**>(&_drawInfo->font) );
+      _imageInfo->font=(char *) RelinquishMagickMemory(_imageInfo->font);
+      _drawInfo->font=(char *) RelinquishMagickMemory(_drawInfo->font);
     }
   else
     {
@@ -368,7 +367,7 @@ bool Magick::Options::monochrome ( void ) const
 void Magick::Options::page ( const Magick::Geometry &pageSize_ )
 {
   if ( !pageSize_.isValid() )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->page) );
+    _imageInfo->page=(char *) RelinquishMagickMemory(_imageInfo->page);
   else
     Magick::CloneString( &_imageInfo->page, pageSize_ );
 }
@@ -437,7 +436,7 @@ Magick::ResolutionType Magick::Options::resolutionUnits ( void ) const
 
 void Magick::Options::size ( const Geometry &geometry_ )
 {
-  LiberateMemory( reinterpret_cast<void**>(&_imageInfo->size) );
+  _imageInfo->size=(char *) RelinquishMagickMemory(_imageInfo->size);
 
   if ( geometry_.isValid() )
     Magick::CloneString( &_imageInfo->size, geometry_ );
@@ -471,8 +470,8 @@ Magick::Color Magick::Options::strokeColor ( void ) const
 
 void Magick::Options::strokeDashArray ( const double* strokeDashArray_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_drawInfo->dash_pattern));
-	_drawInfo->dash_pattern = NULL;
+  _drawInfo->dash_pattern=(double *)
+    RelinquishMagickMemory(_drawInfo->dash_pattern);
 
   if(strokeDashArray_)
     {
@@ -481,7 +480,7 @@ void Magick::Options::strokeDashArray ( const double* strokeDashArray_ )
       for (x=0; strokeDashArray_[x]; x++);
       // Allocate elements
       _drawInfo->dash_pattern =
-        static_cast<double*>(AcquireMemory((x+1)*sizeof(double)));
+        static_cast<double*>(AcquireMagickMemory((x+1)*sizeof(double)));
       // Copy elements
       memcpy(_drawInfo->dash_pattern,strokeDashArray_,
              (x+1)*sizeof(double));
@@ -605,7 +604,7 @@ std::string Magick::Options::textEncoding ( void ) const
 void Magick::Options::tileName ( const std::string &tileName_ )
 {
   if ( tileName_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->tile) );
+    _imageInfo->tile=(char *) RelinquishMagickMemory(_imageInfo->tile);
   else
     Magick::CloneString( &_imageInfo->tile, tileName_ );
 }
@@ -768,7 +767,7 @@ bool Magick::Options::verbose ( void ) const
 void Magick::Options::view ( const std::string &view_ )
 {
   if ( view_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->view) );
+    _imageInfo->view=(char *) RelinquishMagickMemory(_imageInfo->view);
   else
     Magick::CloneString( &_imageInfo->view, view_ );
 }
@@ -783,7 +782,7 @@ std::string Magick::Options::view ( void ) const
 void Magick::Options::x11Display ( const std::string &display_ )
 {
   if ( display_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->server_name) );
+    _imageInfo->server_name=(char *) RelinquishMagickMemory(_imageInfo->server_name);
   else
     Magick::CloneString( &_imageInfo->server_name, display_ );
 }
