@@ -1,15 +1,14 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Tests for setting/getting Magick::Image attributes
 //
 
+#include <Magick++.h>
 #include <string>
 #include <iostream>
 
-#include <Magick++.h>
-
 using namespace std;
 
 using namespace Magick;
@@ -448,10 +447,10 @@ int main( int /*argc*/, char ** argv)
     //
     // classType
     //
-    if ( image.classType() != PseudoClass )
+    if ( image.classType() != DirectClass )
       {
 	++failures;
-	cout << "Line: " << __LINE__ << ", classType is not PseudoClass" << endl;
+	cout << "Line: " << __LINE__ << ", classType is not DirectClass" << endl;
       }
 
     //
@@ -474,28 +473,6 @@ int main( int /*argc*/, char ** argv)
       }
     image.colorFuzz( 0 );
 
-    //
-    // colorMap
-    //
-    // Test default (first color is image canvas base color)
-    if ( image.colorMap(0) != canvasColor )
-      {
-	++failures;
-	cout << "Line: " << __LINE__ << ", colorMap(0) ("
-	     << string(image.colorMap(0))
-	     << ") is not canvas base color " << endl;
-      }
-
-    // Test setting/getting colorMap
-    image.colorMap(0,"#00AAFF");
-    if ( image.colorMap(0) != Color("#00AAFF") )
-      {
-	++failures;
-	cout << "Line: " << __LINE__
-             << ", failed to set/get colorMap(0)" << endl;
-      }
-    image.colorMap(0,canvasColor);
-
     //
     // columns
     //
@@ -589,16 +566,65 @@ int main( int /*argc*/, char ** argv)
 
     }
 
+    //
+    // Format specific defines
+    //
+    if (image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
+      }
+
+    image.defineSet("foo","bar",true);
+    if (!image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports not set."
+             << endl;
+      }
+
+    image.defineSet("foo","bar",false);
+    if (image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
+      }
+
+    image.defineValue("foo","bar","value");
+    std::string value = image.defineValue("foo","bar");
+    if (image.defineValue("foo","bar") != "value")
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports value \""
+             << value << "\""
+             << endl;
+      }
+
+    image.defineSet("foo","bar",false);
+    if (image.defineSet("foo","bar"))
+      {
+        ++failures;
+        cout << "Line: " << __LINE__
+             << ", define for foo:bar incorrectly reports set."
+             << endl;
+      }
+
     //
     // depth
     //
-    if ( image.depth() != 8 )
+    if ( image.depth() != QuantumDepth )
       {
 	++failures;
 	cout << "Line: " << __LINE__
              << ", depth ("
              << image.depth()
-             << ") is not equal to 8" << endl;
+             << ") is not equal to " << QuantumDepth << endl;
       }
 
     //
@@ -607,12 +633,18 @@ int main( int /*argc*/, char ** argv)
     {
       // Since this is not a montage image, simply verify error report
       bool caughtException = false;
+      cout << "Testing throwing and catching exceptions. A program crash or a message" << endl
+           << "that the exception was not caught indicates a test failure.  A properly" << endl
+           << "formatted exception message indicates success:" << endl;
       try
 	{
-	  image.directory();
+	  //image.directory();
+          Magick::Image bad_image("foo");
 	}
-      catch ( Exception )
+      catch ( Exception exception_)
 	{
+          cout << "Caught exception, good!:" << endl
+               << "  \"" << exception_.what() << "\"" << endl;
 	  caughtException = true;
 	}
       if ( caughtException != true )
@@ -881,11 +913,11 @@ int main( int /*argc*/, char ** argv)
     // matte
     //
     // Test default
-    if ( image.matte() != false )
+    if ( image.matte() == false )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", matte default is not false as expected" << endl;
+             << ", matte default is not true as expected" << endl;
       }
 
     // Test set/get
@@ -1079,11 +1111,11 @@ int main( int /*argc*/, char ** argv)
     // quality
     //
     // Test default
-    if ( image.quality() != 75 )
+    if ( image.quality() != 0 )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", quality default is not 75 as expected" << endl;
+             << ", quality default is not 0 as expected" << endl;
       }
 
     // Test set/get
@@ -1093,7 +1125,7 @@ int main( int /*argc*/, char ** argv)
 	++failures;
 	cout << "Line: " << __LINE__ << ", quality set/get failed" << endl;
       }
-    image.quality(75);
+    image.quality(0);
 
     //
     // quantizeColors
@@ -1250,14 +1282,15 @@ int main( int /*argc*/, char ** argv)
     // signature
     //
 
-    if ( image.signature() != "ec00214ca0fd535c2a625b466ab7af8411d0990db801c6d3adc1e412c39986d6" &&
-	 image.signature() != "ffed55ed476d32fde2b1b85606f3152534317b0cc4a04d3c23325dc34110212d" )
+    if ( image.signature() != "849077c0eb67c77cf3b06ff2419cc2b11dea5ff7d8f95466914f23731d1f1a68" &&
+	 image.signature() != "7fe11319c31622db67434ab155cd733afbb391e5dbc2657d08893ebea19ac6dc" &&
+         image.signature() != "56f66a46543e65e7f78022c1c5cb3bd76b82b987b0daaf5cb33fab12caf0563f")
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", signature ("
 	     << image.signature()
 	     << ") is incorrect" << endl;
-	// image.display();
+	image.display();
       }
 
     //
@@ -1449,7 +1482,6 @@ int main( int /*argc*/, char ** argv)
              << ", yResolution default (" << image.yResolution()
              << ") is not zero as expected" << endl;
       }
-
   }
   catch( Exception &error_ )
     {
