@@ -9,8 +9,7 @@
 %                            A   A  R  R     T                                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                Read/Write PFS: 1st Publisher Image Format.                  %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -51,17 +50,19 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 
 static void InsertRow(unsigned char *p,int y,Image *image)
 {
-int bit; long x;
+unsigned long bit; long x;
 register PixelPacket *q;
 IndexPacket index;
 register IndexPacket *indexes;
@@ -79,7 +80,7 @@ register IndexPacket *indexes;
     {
     for (bit=0; bit < 8; bit++)
        {
-       index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+       index=(IndexPacket) (((*p) & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
        indexes[x+bit]=index;
        *q++=image->colormap[index];
        }
@@ -87,18 +88,18 @@ register IndexPacket *indexes;
     }
        if ((image->columns % 8) != 0)
      {
-     for (bit=0; bit < (long) (image->columns % 8); bit++)
+     for (bit=0; bit < (image->columns % 8); bit++)
          {
-         index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+         index=(IndexPacket) (((*p) & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
          indexes[x+bit]=index;
          *q++=image->colormap[index];
          }
      p++;
      }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
      break;
 /*            if (image->previous == (Image *) NULL)
-     if (QuantumTick(y,image->rows))
+     if (QuantumTick(y,image->rows) != 0)
        ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
       break;
       }
@@ -118,7 +119,7 @@ register IndexPacket *indexes;
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadARTImage reads an ART X image file and returns it.  It
+%  ReadARTImage() reads an ART X image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -142,23 +143,28 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image *image;
   int i;
-  unsigned width,height,dummy;
+  unsigned long width,height,dummy;
   long ldblk;
   unsigned char *BImgBuff=NULL;
   unsigned char k;
-  unsigned int status;
+  MagickBooleanType status;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read ART image.
   */
@@ -169,41 +175,35 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   ldblk=(long) ((width+7) / 8);
   k=(unsigned char) ((-ldblk) & 0x01);
-
-   if (GetBlobSize(image)!=(8+((long)ldblk+k)*height))
-     ThrowReaderException(CorruptImageError,"NotAnARTImageFile",image);
-
-
+  if (GetBlobSize(image) != (MagickSizeType) (8+(ldblk+(long) k)*height))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
  image->columns=width;
  image->rows=height;
  image->depth=1;
  image->colors=1l << image->depth;
- 
+
 /* printf("ART header checked OK %d,%d\n",image->colors,image->depth); */
 
- if (!AllocateImageColormap(image,image->colors)) goto NoMemory;
+ if (AllocateImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
 
 /* ----- Load RLE compressed raster ----- */
- BImgBuff=(unsigned char *) AcquireMemory(ldblk);  /*Ldblk was set in the check phase*/
+ BImgBuff=(unsigned char *) AcquireMagickMemory((size_t) ldblk);  /*Ldblk was set in the check phase*/
  if (BImgBuff==NULL)
 NoMemory:
-  ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-
+  ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
  for (i=0;i< (int) height;i++)
       {
-      (void) ReadBlob(image,ldblk,(char *)BImgBuff);
-      (void) ReadBlob(image,k,(char *)&dummy);
+      (void) ReadBlob(image,(size_t) ldblk,BImgBuff);
+      (void) ReadBlob(image,(size_t) k,(unsigned char *) &dummy);
       InsertRow(BImgBuff,i,image);
       }
-  if (BImgBuff!=NULL) LiberateMemory((void **) &BImgBuff);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
-
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -216,7 +216,7 @@ NoMemory:
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterARTImage adds attributes for the ART image format to
+%  RegisterARTImage() adds attributes for the ART image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -241,7 +241,7 @@ ModuleExport void RegisterARTImage(void)
     };
 
   entry=SetMagickInfo("ART");
-  entry->decoder=(DecoderHandler) ReadARTImage;
+  entry->decoder=(DecoderHandler *) ReadARTImage;
   entry->description=AcquireString("PFS: 1st Publisher");
   entry->module=AcquireString("ART");
   entry->note=AcquireString(ARTNote);
@@ -259,7 +259,7 @@ ModuleExport void RegisterARTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterARTImage removes format registrations made by the
+%  UnregisterARTImage() removes format registrations made by the
 %  ART module from the list of supported formats.
 %
 %  The format of the UnregisterARTImage method is:
