@@ -10,39 +10,26 @@
 %               CCCC  A   A  P        T    IIIII   OOO   N   N                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                             Read Text Caption.                              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,13 +40,16 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/annotate.h"
 #include "magick/blob.h"
+#include "magick/draw.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
-#include "magick/render.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,7 +62,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadCAPTIONImage reads a CAPTION image file and returns it.  It
+%  ReadCAPTIONImage() reads a CAPTION image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -106,6 +96,9 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   Image
     *image;
 
+  MagickBooleanType
+    status;
+
   register char
     *p,
     *q;
@@ -116,50 +109,53 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   TypeMetric
     metrics;
 
-  unsigned int
-    status;
-
   /*
     Initialize Image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if (image->columns == 0)
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (*image_info->filename != '@')
-    caption=AllocateString(image_info->filename);
+    caption=AcquireString(image_info->filename);
   else
     {
-      unsigned long
+      size_t
         length;
 
       /*
         Read caption.
       */
-      (void) strncpy(image->filename,image_info->filename+1,MaxTextExtent-2);
+      (void) CopyMagickString(image->filename,image_info->filename+1,
+        MaxTextExtent-2);
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-      if (status == False)
-        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        {
+          DestroyImageList(image);
+          return((Image *) NULL);
+        }
       length=MaxTextExtent;
-      caption=(char *) AcquireMemory(length);
+      caption=(char *) AcquireMagickMemory(length);
       p=caption;
       if (caption != (char *) NULL)
         while (ReadBlobString(image,p) != (char *) NULL)
         {
           p+=strlen(p);
-          if ((p-caption+MaxTextExtent+1) < (long) length)
+          if ((size_t) (p-caption+MaxTextExtent+1) < length)
             continue;
           length<<=1;
-          ReacquireMemory((void **) &caption,length);
+          caption=(char *) ResizeMagickMemory(caption,length);
           if (caption == (char *) NULL)
             break;
           p=caption+strlen(caption);
         }
       if (caption == (char *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       CloseBlob(image);
     }
   /*
@@ -167,7 +163,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   */
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->fill=image_info->pen;
-  draw_info->text=AllocateString(caption);
+  draw_info->text=AcquireString(caption);
   p=caption;
   q=draw_info->text;
   for (i=0; *p != '\0'; p++)
@@ -175,11 +171,11 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     *q++=(*p);
     *q='\0';
     status=GetTypeMetrics(image,draw_info,&metrics);
-    if (status == False)
-      ThrowReaderException(TypeError,"UnableToGetTypeMetrics",image);
-    if ((metrics.width+metrics.max_advance/2) < image->columns)
+    if (status == MagickFalse)
+      ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
+    if ((metrics.width+metrics.max_advance/2) < (double) image->columns)
       continue;
-    for (p--; !isspace((int) *p) && (p > caption); p--);
+    for (p--; (isspace((int) ((unsigned char) *p)) == 0) && (p > caption); p--);
     *p++='\n';
     q=draw_info->text;
     i++;
@@ -191,12 +187,13 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
     Draw caption.
   */
   (void) strcpy(draw_info->text,caption);
-  FormatString(geometry,"+%g+%g",metrics.max_advance/4,metrics.ascent);
-  draw_info->geometry=AllocateString(geometry);
+  (void) FormatMagickString(geometry,MaxTextExtent,"+%g+%g",
+    metrics.max_advance/4,metrics.ascent);
+  draw_info->geometry=AcquireString(geometry);
   (void) AnnotateImage(image,draw_info);
   DestroyDrawInfo(draw_info);
-  LiberateMemory((void **) &caption);
-  return(image);
+  caption=(char *) RelinquishMagickMemory(caption);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -210,7 +207,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterCAPTIONImage adds attributes for the CAPTION image format to
+%  RegisterCAPTIONImage() adds attributes for the CAPTION image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -228,8 +225,8 @@ ModuleExport void RegisterCAPTIONImage(void)
     *entry;
 
   entry=SetMagickInfo("CAPTION");
-  entry->decoder=(DecoderHandler) ReadCAPTIONImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadCAPTIONImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Image caption");
   entry->module=AcquireString("CAPTION");
   (void) RegisterMagickInfo(entry);
@@ -246,7 +243,7 @@ ModuleExport void RegisterCAPTIONImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterCAPTIONImage removes format registrations made by the
+%  UnregisterCAPTIONImage() removes format registrations made by the
 %  CAPTION module from the list of supported formats.
 %
 %  The format of the UnregisterCAPTIONImage method is:
