@@ -10,39 +10,26 @@
 %                         CCCC  M   M    Y    K   K                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write RAW CMYK Image Format.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,22 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteCMYKImage(const ImageInfo *,Image *);
 
 /*
@@ -78,7 +70,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadCMYKImage reads an image of raw red, green, and blue samples and
+%  ReadCMYKImage() reads an image of raw red, green, and blue samples and
 %  returns it.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -108,6 +100,9 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     i,
     x;
@@ -115,49 +110,54 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   register PixelPacket
     *q;
 
-  size_t
+  ssize_t
     count;
 
+  size_t
+    packet_size;
+
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
-    packet_size;
+    width;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open image file.
       */
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-      if (status == False)
-        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        {
+          DestroyImageList(image);
+          return((Image *) NULL);
+        }
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
   /*
     Allocate memory for a scanline.
   */
-  packet_size=image->depth > 8 ? 8 : 4;
+  packet_size=(size_t) (image->depth > 8 ? 8 : 4);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
     {
-      image->matte=True;
-      packet_size=image->depth > 8 ? 10 : 8;
+      image->matte=MagickTrue;
+      packet_size=(size_t) (image->depth > 8 ? 10 : 8);
     }
   scanline=(unsigned char *)
-    AcquireMemory(packet_size*image->extract_info.width);
+    AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -175,7 +175,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       Convert raster image to pixel packets.
     */
     image->colorspace=CMYKColorspace;
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     switch (image_info->interlace)
@@ -191,23 +191,24 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          if (!image->matte)
+          if (image->matte == MagickFalse)
             (void) PushImagePixels(image,CMYKQuantum,scanline+x);
           else
             (void) PushImagePixels(image,CMYKAQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
@@ -216,13 +217,14 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         /*
           Line interlacing:  CCC...MMM...YYY...KKK...CCC...MMM...YYY...KKK...
         */
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
@@ -233,21 +235,21 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           (void) PushImagePixels(image,YellowQuantum,scanline+x);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
           (void) PushImagePixels(image,BlackQuantum,scanline+x);
-          if (image->matte)
+          if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
               (void) PushImagePixels(image,AlphaQuantum,scanline+x);
             }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
@@ -264,40 +266,47 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             AppendImageFormat("C",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         i=0;
-        span=image->rows*(image->matte ? 5 : 4);
+        span=image->rows*(image->matte != MagickFalse ? 5 : 4);
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,CyanQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("M",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
@@ -308,24 +317,27 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,MagentaQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("Y",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
@@ -336,24 +348,27 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,YellowQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("K",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
@@ -364,18 +379,18 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,BlackQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-        if (image->matte)
+        if (image->matte != MagickFalse)
           {
             /*
               Read matte channel.
@@ -385,8 +400,11 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-                if (status == False)
-                  ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+                if (status == MagickFalse)
+                  {
+                    DestroyImageList(image);
+                    return((Image *) NULL);
+                  }
               }
             for (y=0; y < image->extract_info.y; y++)
               (void) ReadBlob(image,packet_size*image->extract_info.width,
@@ -399,27 +417,31 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               if (q == (PixelPacket *) NULL)
                 break;
               (void) PushImagePixels(image,AlphaQuantum,scanline+x);
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span))
-                  if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
-                    break;
+                if (QuantumTick(i,span) != 0)
+                  {
+                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    if (status == MagickFalse)
+                      break;
+                  }
               i++;
             }
-            count=image->extract_info.height-image->rows-image->extract_info.y;
-            for (i=0; i < (long) count; i++)
+            width=image->extract_info.height-image->rows-image->extract_info.y;
+            for (i=0; i < (long) width; i++)
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
         break;
       }
     }
-    if (EOFBlob(image))
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -446,15 +468,13 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (count != 0);
-  LiberateMemory((void **) &scanline);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -468,7 +488,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterCMYKImage adds attributes for the CMYK image format to
+%  RegisterCMYKImage() adds attributes for the CMYK image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -486,17 +506,17 @@ ModuleExport void RegisterCMYKImage(void)
     *entry;
 
   entry=SetMagickInfo("CMYK");
-  entry->decoder=(DecoderHandler) ReadCMYKImage;
-  entry->encoder=(EncoderHandler) WriteCMYKImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadCMYKImage;
+  entry->encoder=(EncoderHandler *) WriteCMYKImage;
+  entry->raw=MagickTrue;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, and black samples");
   entry->module=AcquireString("CMYK");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CMYKA");
-  entry->decoder=(DecoderHandler) ReadCMYKImage;
-  entry->encoder=(EncoderHandler) WriteCMYKImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadCMYKImage;
+  entry->encoder=(EncoderHandler *) WriteCMYKImage;
+  entry->raw=MagickTrue;
   entry->description=
     AcquireString("Raw cyan, magenta, yellow, black, and opacity samples");
   entry->module=AcquireString("CMYK");
@@ -514,7 +534,7 @@ ModuleExport void RegisterCMYKImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterCMYKImage removes format registrations made by the
+%  UnregisterCMYKImage() removes format registrations made by the
 %  CMYK module from the list of supported formats.
 %
 %  The format of the UnregisterCMYKImage method is:
@@ -539,16 +559,16 @@ ModuleExport void UnregisterCMYKImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteCMYKImage writes an image to a file in red, green, and blue
+%  WriteCMYKImage() writes an image to a file in red, green, and blue
 %  rasterfile format.
 %
 %  The format of the WriteCMYKImage method is:
 %
-%      unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteCMYKImage return True if the image is written.
+%    o status: Method WriteCMYKImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -558,20 +578,24 @@ ModuleExport void UnregisterCMYKImage(void)
 %
 %
 */
-static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  long
     y;
 
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
+  size_t
+    packet_size;
+
   unsigned char
     *pixels;
 
   unsigned int
-    packet_size,
-    scene,
     status;
 
   /*
@@ -581,20 +605,22 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  packet_size=image->depth > 8 ? 8 : 4;
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  packet_size=(size_t) (image->depth > 8 ? 8 : 4);
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
-    packet_size=image->depth > 8 ? 10 : 8;
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
+    packet_size=(size_t) (image->depth > 8 ? 10 : 8);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open output image file.
       */
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      if (status == False)
-        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        return(status);
     }
   scene=0;
   do
@@ -604,7 +630,7 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,CMYKColorspace);
     if (LocaleCompare(image_info->magick,"CMYKA") == 0)
-      if (!image->matte)
+      if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
     switch (image_info->interlace)
     {
@@ -630,9 +656,13 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -647,21 +677,25 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,CyanQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,MagentaQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,YellowQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,BlackQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"CMYKA") == 0)
             {
               (void) PopImagePixels(image,AlphaQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
+            }
+          if (QuantumTick(y,image->rows) != 0)
+            {
+              status=MagickMonitor(SaveImageTag,y,image->rows,
+                &image->exception);
+              if (status == MagickFalse)
+                break;
             }
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-              break;
         }
         break;
       }
@@ -676,8 +710,8 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("C",image->filename);
             status=
               OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -685,7 +719,7 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,CyanQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -693,10 +727,10 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("M",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
-        if (!MagickMonitor(SaveImageTag,100,400,&image->exception))
+        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == MagickFalse)
           break;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -704,7 +738,7 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,MagentaQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -712,10 +746,10 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("Y",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
-        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
           break;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -723,7 +757,7 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,YellowQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -731,10 +765,10 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("K",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
-        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
           break;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -742,11 +776,11 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,BlackQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"CMYKA") == 0)
           {
-            if (!MagickMonitor(SaveImageTag,300,400,&image->exception))
+            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == MagickFalse)
               break;
             if (image_info->interlace == PartitionInterlace)
               {
@@ -754,8 +788,8 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,WriteBinaryBlobMode,
                   &image->exception);
-                if (status == False)
-                  ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+                if (status == MagickFalse)
+                  return(status);
               }
             for (y=0; y < (long) image->rows; y++)
             {
@@ -764,12 +798,13 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
               if (p == (const PixelPacket *) NULL)
                 break;
               (void) PopImagePixels(image,AlphaQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-        if (!MagickMonitor(SaveImageTag,400,400,&image->exception))
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
+        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == MagickFalse)
           break;
         break;
       }
@@ -779,13 +814,10 @@ static unsigned int WriteCMYKImage(const ImageInfo *image_info,Image *image)
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  LiberateMemory((void **) &pixels);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
