@@ -2,15 +2,14 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%                             CCC   U   U  TTTTT                              %
-%                            C   C  U   U    T                                %
-%                            C      U   U    T                                %
-%                            C   C  U   U    T                                %
-%                             CCC    UUU     T                                %
+%                              CCC  U   U  TTTTT                              %
+%                             C     U   U    T                                %
+%                             C     U   U    T                                %
+%                             C     U   U    T                                %
+%                              CCC   UUU     T                                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                         Read DR Halo Image Format.                          %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -50,11 +49,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 typedef struct
 {
@@ -80,14 +83,15 @@ typedef struct
 } CUTPalHeader;
 
 
-static void InsertRow(int depth,unsigned char *p,long y,Image *image)
+static void InsertRow(long depth,unsigned char *p,long y,Image *image)
 {
-  int bit; long x;
+  unsigned long bit; long x;
   register PixelPacket *q;
   IndexPacket index;
   register IndexPacket *indexes;
 
 
+  index=0;
   switch (depth)
     {
     case 1:  /* Convert bitmap scanline. */
@@ -100,7 +104,7 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
           {
             for (bit=0; bit < 8; bit++)
               {
-                index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+                index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
                 indexes[x+bit]=index;
                 *q++=image->colormap[index];
               }
@@ -108,18 +112,18 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
           }
         if ((image->columns % 8) != 0)
           {
-            for (bit=0; bit < (long) (image->columns % 8); bit++)
+            for (bit=0; bit < (image->columns % 8); bit++)
               {
-                index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+                index=(IndexPacket) ((((*p) & (0x80 >> bit)) != 0) ? 0x01 : 0x00);
                 indexes[x+bit]=index;
                 *q++=image->colormap[index];
               }
             p++;
           }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         /*            if (image->previous == (Image *) NULL)
-                      if (QuantumTick(y,image->rows))
+                      if (QuantumTick(y,image->rows) != 0)
                       ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
@@ -131,56 +135,49 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
         indexes=GetIndexes(image);
         for (x=0; x < ((long) image->columns-1); x+=2)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
             indexes[x]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 4) & 0x3);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
             indexes[x]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 2) & 0x3);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
             indexes[x]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p) & 0x3);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p) & 0x3);
             indexes[x+1]=index;
             *q++=image->colormap[index];
             p++;
           }
         if ((image->columns % 4) != 0)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 6) & 0x3);
             indexes[x]=index;
             *q++=image->colormap[index];
             if ((image->columns % 4) >= 1)
 
               {
-                index=(IndexPacket) ((*p >> 4) & 0x3);
-                ConstrainColormapIndex(image,index);
+                index=ConstrainColormapIndex(image,(*p >> 4) & 0x3);
                 indexes[x]=index;
                 *q++=image->colormap[index];
                 if ((image->columns % 4) >= 2)
 
                   {
-                    index=(IndexPacket) ((*p >> 2) & 0x3);
-                    ConstrainColormapIndex(image,index);
+                    index=ConstrainColormapIndex(image,(*p >> 2) & 0x3);
                     indexes[x]=index;
                     *q++=image->colormap[index];
                   }
               }
             p++;
           }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows))
+                   if (QuantumTick(y,image->rows) != 0)
                    ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
-      
+
     case 4:  /* Convert PseudoColor scanline. */
       {
         q=SetImagePixels(image,0,y,image->columns,1);
@@ -189,28 +186,25 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
         indexes=GetIndexes(image);
         for (x=0; x < ((long) image->columns-1); x+=2)
           {
-            index=(IndexPacket) ((*p >> 4) & 0xf);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
             indexes[x]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p) & 0xf);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p) & 0xf);
             indexes[x+1]=index;
             *q++=image->colormap[index];
             p++;
           }
         if ((image->columns % 2) != 0)
           {
-            index=(IndexPacket) ((*p >> 4) & 0xf);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
             indexes[x]=index;
             *q++=image->colormap[index];
             p++;
           }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         /*         if (image->previous == (Image *) NULL)
-                   if (QuantumTick(y,image->rows))
+                   if (QuantumTick(y,image->rows) != 0)
                    ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
         break;
       }
@@ -222,16 +216,15 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
 
         for (x=0; x < (long) image->columns; x++)
           {
-            index=(IndexPacket) (*p);
-            ConstrainColormapIndex(image,index);
+            index=ConstrainColormapIndex(image,*p);
             indexes[x]=index;
             *q++=image->colormap[index];
             p++;
           }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         /*           if (image->previous == (Image *) NULL)
-                     if (QuantumTick(y,image->rows))
+                     if (QuantumTick(y,image->rows) != 0)
                      ProgressMonitor(LoadImageText,image->rows-y-1,image->rows);*/
       }
       break;
@@ -241,26 +234,26 @@ static void InsertRow(int depth,unsigned char *p,long y,Image *image)
 
 static int GetCutColors(Image *image)
 {
-  int MaxColor,x,y;
+  long MaxColor,x,y;
   PixelPacket *q;
   int ScaleCharToQuantum16;
 
   /*This procedure computes number of colors in Grayed R[i]=G[i]=B[i] image*/
-  ScaleCharToQuantum16=ScaleCharToQuantum(16);
+  ScaleCharToQuantum16=(int) ScaleCharToQuantum(16);
   MaxColor=0;
-  for (y=0; y < (long)image->rows; y++)  
+  for (y=0; y < (long)image->rows; y++)
     {
-      q=SetImagePixels(image,0,y,image->columns,1);  
-      for (x=0; x < (long)image->columns; x++)  
-        {       
-          if(MaxColor<q->red) MaxColor=q->red;
-          if(MaxColor>=ScaleCharToQuantum16) return(255);  
-          q++;  
-        }  
+      q=SetImagePixels(image,0,y,image->columns,1);
+      for (x=0; x < (long)image->columns; x++)
+        {
+          if(MaxColor<(int) q->red) MaxColor=(int) q->red;
+          if(MaxColor>=ScaleCharToQuantum16) return(255);
+          q++;
+        }
     }
-  if(MaxColor<(int) ScaleCharToQuantum(2)) MaxColor=2;    
-  else if(MaxColor<(int) ScaleCharToQuantum(16)) MaxColor=16;    
-  return(MaxColor);
+  if(MaxColor<(int) ScaleCharToQuantum(2)) MaxColor=2;
+  else if(MaxColor<(int) ScaleCharToQuantum(16)) MaxColor=16;
+  return((int) MaxColor);
 }
 
 /*
@@ -274,7 +267,7 @@ static int GetCutColors(Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadCUTImage reads an CUT X image file and returns it.  It
+%  ReadCUTImage() reads an CUT X image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -298,12 +291,12 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image *image,*palette;
   ImageInfo *clone_info;
-  unsigned int status;
+  MagickBooleanType status;
   unsigned long EncodedByte;
   unsigned char RunCount,RunValue,RunCountMasked;
   CUTHeader  Header;
   CUTPalHeader PalHeader;
-  int depth;
+  long depth;
   long i,j;
   long ldblk;
   unsigned char *BImgBuff=NULL,*ptrB;
@@ -314,12 +307,17 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read CUT image.
   */
@@ -330,24 +328,24 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Header.Reserved=ReadBlobLSBShort(image);
 
   if (Header.Width==0 || Header.Height==0 || Header.Reserved!=0)
-    CUT_KO:  ThrowReaderException(CorruptImageError,"NotACUTImageFile",image);
+    CUT_KO:  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
 
   /*---This code checks first line of image---*/
   EncodedByte=ReadBlobLSBShort(image);
-  RunCount=ReadBlobByte(image);
+  RunCount=(unsigned char) ReadBlobByte(image);
   RunCountMasked=RunCount & 0x7F;
   ldblk=0;
-  while(RunCountMasked!=0)  /*end of line?*/
+  while((int) RunCountMasked!=0)  /*end of line?*/
     {
       i=1;
-      if(RunCount<0x80) i=RunCountMasked;
+      if((int) RunCount<0x80) i=(long) RunCountMasked;
       (void) SeekBlob(image,TellBlob(image)+i,SEEK_SET);
-      if(EOFBlob(image)) goto CUT_KO;  /*wrong data*/
+      if(EOFBlob(image) != MagickFalse) goto CUT_KO;  /*wrong data*/
       EncodedByte-=i+1;
-      ldblk+=RunCountMasked;
+      ldblk+=(long) RunCountMasked;
 
-      RunCount=ReadBlobByte(image);
-      if(EOFBlob(image))  goto CUT_KO;  /*wrong data: unexpected eof in line*/
+      RunCount=(unsigned char) ReadBlobByte(image);
+      if(EOFBlob(image) != MagickFalse)  goto CUT_KO;  /*wrong data: unexpected eof in line*/
       RunCountMasked=RunCount & 0x7F;
     }
   if(EncodedByte!=1) goto CUT_KO;  /*wrong data: size incorrect*/
@@ -361,37 +359,37 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=Header.Width;
   image->rows=Header.Height;
   image->depth=8;
-  image->colors=1l >> i;
+  image->colors=1UL << (unsigned long) i;
+
 
- 
   /* ----- Do something with palette ----- */
   if ((clone_info=CloneImageInfo(image_info)) == NULL) goto NoPalette;
- 
- 
+
+
   i=(long) strlen(clone_info->filename);
   j=i;
   while(--i>0)
     {
-      if(clone_info->filename[i]=='.') 
+      if(clone_info->filename[i]=='.')
         {
           break;
         }
       if(clone_info->filename[i]=='/' || clone_info->filename[i]=='\\' ||
-         clone_info->filename[i]==':' ) 
+         clone_info->filename[i]==':' )
         {
           i=j;
           break;
         }
     }
-  
+
   (void) strcpy(clone_info->filename+i,".PAL");
   if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL)
     {
       (void) strcpy(clone_info->filename+i,".pal");
       if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL)
         {
-          clone_info->filename[i]=0;
-          if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL) 
+          clone_info->filename[i]='\0';
+          if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL)
             {
               DestroyImageInfo(clone_info);
               clone_info=NULL;
@@ -402,39 +400,39 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   if( (palette=AllocateImage(clone_info))==NULL ) goto NoPalette;
   status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-    ErasePalette:     
+    ErasePalette:
       DestroyImage(palette);
       palette=NULL;
       goto NoPalette;
     }
- 
-     
+
+
   if(palette!=NULL)
     {
-      (void) ReadBlob(palette,2,PalHeader.FileId);
-      if(strncmp(PalHeader.FileId,"AH",2)) goto ErasePalette;
+      (void) ReadBlob(palette,2,(unsigned char *) PalHeader.FileId);
+      if(strncmp(PalHeader.FileId,"AH",2) != 0) goto ErasePalette;
       PalHeader.Version=ReadBlobLSBShort(palette);
       PalHeader.Size=ReadBlobLSBShort(palette);
-      PalHeader.FileType=ReadBlobByte(palette);
-      PalHeader.SubType=ReadBlobByte(palette);
+      PalHeader.FileType=(char) ReadBlobByte(palette);
+      PalHeader.SubType=(char) ReadBlobByte(palette);
       PalHeader.BoardID=ReadBlobLSBShort(palette);
       PalHeader.GraphicsMode=ReadBlobLSBShort(palette);
       PalHeader.MaxIndex=ReadBlobLSBShort(palette);
       PalHeader.MaxRed=ReadBlobLSBShort(palette);
       PalHeader.MaxGreen=ReadBlobLSBShort(palette);
       PalHeader.MaxBlue=ReadBlobLSBShort(palette);
-      (void) ReadBlob(palette,20,PalHeader.PaletteId);
-   
+      (void) ReadBlob(palette,20,(unsigned char *) PalHeader.PaletteId);
+
       if(PalHeader.MaxIndex<1) goto ErasePalette;
       image->colors=PalHeader.MaxIndex+1;
-      if (!AllocateImageColormap(image,image->colors)) goto NoMemory;
-   
+      if (AllocateImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
+
       if(PalHeader.MaxRed==0) PalHeader.MaxRed=MaxRGB;  /*avoid division by 0*/
       if(PalHeader.MaxGreen==0) PalHeader.MaxGreen=MaxRGB;
       if(PalHeader.MaxBlue==0) PalHeader.MaxBlue=MaxRGB;
-   
+
       for(i=0;i<=(int) PalHeader.MaxIndex;i++)
         {      /*this may be wrong- I don't know why is palette such strange*/
           j=(long) TellBlob(palette);
@@ -444,40 +442,40 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
               (void) SeekBlob(palette,j,SEEK_SET);
             }
           image->colormap[i].red=ReadBlobLSBShort(palette);
-          if(MaxRGB!=PalHeader.MaxRed) 
+          if(MaxRGB!=(Quantum) PalHeader.MaxRed)
             {
               image->colormap[i].red=(Quantum)
                 (((double)image->colormap[i].red*MaxRGB+(PalHeader.MaxRed>>1))/PalHeader.MaxRed+0.5);
             }
           image->colormap[i].green=ReadBlobLSBShort(palette);
-          if(MaxRGB!=PalHeader.MaxGreen) 
+          if(MaxRGB!=(Quantum) PalHeader.MaxGreen)
             {
               image->colormap[i].green=(Quantum)
                 (((double)image->colormap[i].green*MaxRGB+(PalHeader.MaxGreen>>1))/PalHeader.MaxGreen+0.5);
             }
-          image->colormap[i].blue=ReadBlobLSBShort(palette);       
-          if(MaxRGB!=PalHeader.MaxBlue)  
+          image->colormap[i].blue=ReadBlobLSBShort(palette);
+          if(MaxRGB!=(Quantum) PalHeader.MaxBlue)
             {
               image->colormap[i].blue=(Quantum)
                 (((double)image->colormap[i].blue*MaxRGB+(PalHeader.MaxBlue>>1))/PalHeader.MaxBlue+0.5);
             }
-         
+
         }
     }
 
-   
+
 
  NoPalette:
   if(palette==NULL)
-    { 
-   
+    {
+
       image->colors=256;
-      if (!AllocateImageColormap(image,image->colors))
+      if (AllocateImageColormap(image,image->colors) == MagickFalse)
         {
         NoMemory:
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
-            }     
-   
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+            }
+
       for (i=0; i < (long)image->colors; i++)
         {
           image->colormap[i].red=ScaleCharToQuantum(i);
@@ -486,9 +484,9 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
 
-           
+
   /* ----- Load RLE compressed raster ----- */
-  BImgBuff=(unsigned char *) AcquireMemory(ldblk);  /*Ldblk was set in the check phase*/
+  BImgBuff=(unsigned char *) AcquireMagickMemory((size_t) ldblk);  /*Ldblk was set in the check phase*/
   if(BImgBuff==NULL) goto NoMemory;
 
   (void) SeekBlob(image,6 /*sizeof(Header)*/,SEEK_SET);
@@ -499,34 +497,34 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ptrB=BImgBuff;
       j=ldblk;
 
-      RunCount=ReadBlobByte(image);
+      RunCount=(unsigned char) ReadBlobByte(image);
       RunCountMasked=RunCount & 0x7F;
 
-      while(RunCountMasked!=0)    
+      while((int) RunCountMasked!=0)
         {
-          if(RunCountMasked>j)
+          if((long) RunCountMasked>j)
             {    /*Wrong Data*/
               RunCountMasked=(unsigned char) j;
-              if(j==0) 
+              if(j==0)
                 {
                   break;
                 }
             }
 
-          if(RunCount>0x80)
+          if((int) RunCount>0x80)
             {
-              RunValue=ReadBlobByte(image);
-              (void) memset(ptrB,RunValue,RunCountMasked);
+              RunValue=(unsigned char) ReadBlobByte(image);
+              (void) ResetMagickMemory(ptrB,(int) RunValue,(size_t) RunCountMasked);
             }
           else {
-            (void) ReadBlob(image,RunCountMasked,ptrB);
+            (void) ReadBlob(image,(size_t) RunCountMasked,ptrB);
           }
-         
-          ptrB+=RunCountMasked;
-          j-=RunCountMasked;     
-    
-          if(EOFBlob(image)) goto Finish;  /* wrong data: unexpected eof in line */
-          RunCount=ReadBlobByte(image);
+
+          ptrB+=(int) RunCountMasked;
+          j-=(int) RunCountMasked;
+
+          if (EOFBlob(image) != MagickFalse) goto Finish;  /* wrong data: unexpected eof in line */
+          RunCount=(unsigned char) ReadBlobByte(image);
           RunCountMasked=RunCount & 0x7F;
         }
 
@@ -538,7 +536,8 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   if(palette==NULL)
     {    /*attempt to detect binary (black&white) images*/
-      if((image->storage_class == PseudoClass) && IsGrayImage(image,&image->exception))
+      if ((image->storage_class == PseudoClass) &&
+          (IsGrayImage(image,&image->exception) != MagickFalse))
         {
           if(GetCutColors(image)==2)
             {
@@ -551,33 +550,35 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   if(image->colormap[i].green!=sample) goto Finish;
                   if(image->colormap[i].blue!=sample) goto Finish;
                 }
-     
+
               image->colormap[1].red=image->colormap[1].green=image->colormap[1].blue=MaxRGB;
-              for (i=0; i < (long)image->rows; i++)  
+              for (i=0; i < (long)image->rows; i++)
                 {
-                  q=SetImagePixels(image,0,i,image->columns,1);  
-                  for (j=0; j < (long)image->columns; j++)  
-                    {       
+                  q=SetImagePixels(image,0,i,image->columns,1);
+                  for (j=0; j < (long)image->columns; j++)
+                    {
                       if(q->red==ScaleCharToQuantum(1))
                         {
                           q->red=q->green=q->blue=MaxRGB;
                         }
-                      q++;  
-                    }  
-                  if (!SyncImagePixels(image)) goto Finish;
+                      q++;
+                    }
+                  if (SyncImagePixels(image) == MagickFalse) goto Finish;
                 }
             }
-        }        
-    } 
+        }
+    }
 
  Finish:
-  if(BImgBuff!=NULL) free(BImgBuff);
+  if(BImgBuff!=NULL)
+    BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
   if(palette!=NULL) DestroyImage(palette);
   if(clone_info!=NULL) DestroyImageInfo(clone_info);
-  if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile",image);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
-  return(image);       
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -591,7 +592,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterCUTImage adds attributes for the CUT image format to
+%  RegisterCUTImage() adds attributes for the CUT image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -609,8 +610,8 @@ ModuleExport void RegisterCUTImage(void)
     *entry;
 
   entry=SetMagickInfo("CUT");
-  entry->decoder=(DecoderHandler) ReadCUTImage;
-  entry->seekable_stream=True;
+  entry->decoder=(DecoderHandler *) ReadCUTImage;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("DR Halo");
   entry->module=AcquireString("CUT");
   (void) RegisterMagickInfo(entry);
@@ -627,7 +628,7 @@ ModuleExport void RegisterCUTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterCUTImage removes format registrations made by the
+%  UnregisterCUTImage() removes format registrations made by the
 %  CUT module from the list of supported formats.
 %
 %  The format of the UnregisterCUTImage method is:
