@@ -10,39 +10,26 @@
 %                            DDDD   P      SSSSS                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read Postscript Using the Display Postscript System.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,15 +41,23 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/client.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 #include "magick/xwindow.h"
 #if defined(HasDPS)
+#include <DPS/dpsXclient.h>
+#include <DPS/dpsXpreview.h>
+#endif
 
+#if defined(HasDPS)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -74,7 +69,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadDPSImage reads a Adobe Postscript image file and returns it.  It
+%  ReadDPSImage() reads a Adobe Postscript image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -96,9 +91,6 @@
 */
 static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  char
-    *client_name;
-
   Display
     *display;
 
@@ -161,6 +153,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   display=XOpenDisplay(image_info->server_name);
@@ -175,22 +169,20 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     return((Image *) NULL);
   /*
     Get user defaults from X resource database.
   */
-  client_name=SetClientName((char *) NULL);
-  resource_database=XGetResourceDatabase(display,client_name);
-  XGetResourceInfo(resource_database,client_name,&resource_info);
+  resource_database=XGetResourceDatabase(display,GetClientName());
+  XGetResourceInfo(resource_database,GetClientName(),&resource_info);
   /*
     Allocate standard colormap.
   */
   map_info=XAllocStandardColormap();
   visual_info=(XVisualInfo *) NULL;
   if (map_info == (XStandardColormap *) NULL)
-    ThrowReaderException(ResourceLimitError,"UnableToCreateStandardColormap",
-      image)
+    ThrowReaderException(ResourceLimitError,"UnableToCreateStandardColormap")
   else
     {
       /*
@@ -215,8 +207,9 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   pixels_per_point=XDPSPixelsPerPoint(screen);
   if ((image->x_resolution != 0.0) && (image->y_resolution != 0.0))
     pixels_per_point=Min(image->x_resolution,image->y_resolution)/72.0;
-  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,image->blob->file,
-    visual_info->depth,pixels_per_point,&pixmap,&bits_per_pixel,&page);
+  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,
+    GetBlobFileHandle(image),visual_info->depth,pixels_per_point,&pixmap,
+    &bits_per_pixel,&page);
   if ((status == dps_status_failure) || (status == dps_status_no_extension))
     {
       DestroyImage(image);
@@ -228,8 +221,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Rasterize the file into the pixmap.
   */
   status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
-    image->blob->file,bits_per_pixel.height,visual_info->depth,&page,-page.x,
-    -page.y,pixels_per_point,True,False,True,&sans);
+    GetBlobFileHandle(image),bits_per_pixel.height,visual_info->depth,&page,
+    -page.x,-page.y,pixels_per_point,MagickTrue,MagickFalse,MagickTrue,&sans);
   if (status != dps_status_success)
     {
       DestroyImage(image);
@@ -253,7 +246,8 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Get the colormap colors.
   */
-  colors=(XColor *) AcquireMemory(visual_info->colormap_size*sizeof(XColor));
+  colors=(XColor *)
+    AcquireMagickMemory(visual_info->colormap_size*sizeof(XColor));
   if (colors == (XColor *) NULL)
     {
       DestroyImage(image);
@@ -313,10 +307,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->storage_class=PseudoClass;
   image->columns=dps_image->width;
   image->rows=dps_image->height;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   switch (image->storage_class)
   {
@@ -380,10 +374,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleShortToQuantum(colors[index].blue);
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          if (QuantumTick(y,image->rows) != 0)
+            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
               break;
         }
       else
@@ -403,10 +397,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          if (QuantumTick(y,image->rows) != 0)
+            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
               break;
         }
       break;
@@ -416,10 +410,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create colormap.
       */
-      if (!AllocateImageColormap(image,visual_info->colormap_size))
+      if (AllocateImageColormap(image,visual_info->colormap_size) == MagickFalse)
         {
           DestroyImage(image);
-          LiberateMemory((void **) &colors);
+          colors=(XColor *) RelinquishMagickMemory(colors);
           XDestroyImage(dps_image);
           XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
             (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
@@ -428,8 +422,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (i=0; i < (long) image->colors; i++)
       {
         image->colormap[colors[i].pixel].red=ScaleShortToQuantum(colors[i].red);
-        image->colormap[colors[i].pixel].green=ScaleShortToQuantum(colors[i].green);
-        image->colormap[colors[i].pixel].blue=ScaleShortToQuantum(colors[i].blue);
+        image->colormap[colors[i].pixel].green=
+          ScaleShortToQuantum(colors[i].green);
+        image->colormap[colors[i].pixel].blue=
+          ScaleShortToQuantum(colors[i].blue);
       }
       /*
         Convert X image to PseudoClass packets.
@@ -442,29 +438,29 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
           indexes[x]=(unsigned short) XGetPixel(dps_image,x,y);
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
       }
       break;
     }
   }
-  LiberateMemory((void **) &colors);
+  colors=(XColor *) RelinquishMagickMemory(colors);
   XDestroyImage(dps_image);
   if (image->storage_class == PseudoClass)
-    SyncImage(image);
+    (void) SyncImage(image);
   /*
     Rasterize matte image.
   */
-  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,image->blob->file,1,
-    pixels_per_point,&pixmap,&bits_per_pixel,&page);
+  status=XDPSCreatePixmapForEPSF((DPSContext) NULL,screen,
+    GetBlobFileHandle(image),1,pixels_per_point,&pixmap,&bits_per_pixel,&page);
   if ((status != dps_status_failure) && (status != dps_status_no_extension))
     {
       status=XDPSImageFileIntoDrawable((DPSContext) NULL,screen,pixmap,
-        image->blob->file,bits_per_pixel.height,1,&page,-page.x,
-        -page.y,pixels_per_point,True,True,True,&sans);
+        GetBlobFileHandle(image),bits_per_pixel.height,1,&page,-page.x,
+        -page.y,pixels_per_point,MagickTrue,MagickTrue,MagickTrue,&sans);
       if (status == dps_status_success)
         {
           XImage
@@ -479,7 +475,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (matte_image != (XImage *) NULL)
             {
               image->storage_class=DirectClass;
-              image->matte=True;
+              image->matte=MagickTrue;
               for (y=0; y < (long) image->rows; y++)
               {
                 q=SetImagePixels(image,0,y,image->columns,1);
@@ -488,11 +484,11 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 for (x=0; x < (long) image->columns; x++)
                 {
                   q->opacity=OpaqueOpacity;
-                  if (!XGetPixel(matte_image,x,y))
+                  if (XGetPixel(matte_image,x,y) == 0)
                     q->opacity=TransparentOpacity;
                   q++;
                 }
-                if (!SyncImagePixels(image))
+                if (SyncImagePixels(image) == MagickFalse)
                   break;
               }
               XDestroyImage(matte_image);
@@ -505,14 +501,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   XFreeResources(display,visual_info,map_info,(XPixelInfo *) NULL,
     (XFontStruct *) NULL,&resource_info,(XWindowInfo *) NULL);
   CloseBlob(image);
-  return(image);
-}
-#else
-static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  (void) ThrowException(exception,MissingDelegateError,
-    "DPSLibraryIsNotAvailable",image_info->filename);
-  return((Image *) NULL);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -527,7 +516,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterDPSImage adds attributes for the Display Postscript image
+%  RegisterDPSImage() adds attributes for the Display Postscript image
 %  format to the list of supported formats.  The attributes include the image
 %  format tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -545,8 +534,10 @@ ModuleExport void RegisterDPSImage(void)
     *entry;
 
   entry=SetMagickInfo("DPS");
-  entry->decoder=(DecoderHandler) ReadDPSImage;
-  entry->blob_support=False;
+#if defined(HasDPS)
+  entry->decoder=(DecoderHandler *) ReadDPSImage;
+#endif
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("Display Postscript Interpreter");
   entry->module=AcquireString("DPS");
   (void) RegisterMagickInfo(entry);
@@ -563,7 +554,7 @@ ModuleExport void RegisterDPSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterDPSImage removes format registrations made by the
+%  UnregisterDPSImage() removes format registrations made by the
 %  DPS module from the list of supported formats.
 %
 %  The format of the UnregisterDPSImage method is:
