@@ -12,36 +12,23 @@
 %                                                                             %
 %                  Read Windows Enahanced Metafile Format.                    %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                              Bill Radcliffe                                 %
 %                                   2001                                      %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */
@@ -51,12 +38,6 @@
  */
 
 #include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/error.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/static.h"
-#include "magick/utility.h"
 #if defined(HasWINGDI32)
 #  if defined(__CYGWIN__)
 #    include <windows.h>
@@ -66,6 +47,15 @@
 #  endif
 #endif
 
+#include "magick/blob.h"
+#include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -77,16 +67,16 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsEMF returns True if the image format type, identified by the
+%  IsEMF() returns MagickTrue if the image format type, identified by the
 %  magick string, is a Microsoft Windows Enhanced MetaFile (EMF) file.
 %
 %  The format of the ReadEMFImage method is:
 %
-%      unsigned int IsEMF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF returns True if the image format type is EMF.
+%    o status:  Method IsEMF() returns MagickTrue if the image format type is EMF.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -95,13 +85,13 @@
 %
 %
 */
-static unsigned int IsEMF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 {
   if (length < 48)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+40,"\040\105\115\106\000\000\001\000",8) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -115,16 +105,16 @@ static unsigned int IsEMF(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsWMF returns True if the image format type, identified by the
+%  IsWMF() returns MagickTrue if the image format type, identified by the
 %  magick string, is a Windows MetaFile (WMF) file.
 %
 %  The format of the ReadEMFImage method is:
 %
-%      unsigned int IsEMF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsEMF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsEMF returns True if the image format type is WMF.
+%    o status:  Method IsEMF() returns MagickTrue if the image format type is WMF.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -133,15 +123,15 @@ static unsigned int IsEMF(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int IsWMF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\327\315\306\232",4) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\001\000\011\000",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -155,7 +145,7 @@ static unsigned int IsWMF(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadEMFImage reads an Microsoft Windows Enhanced MetaFile (EMF) or
+%  ReadEMFImage() reads an Microsoft Windows Enhanced MetaFile (EMF) or
 %  Windows MetaFile (WMF) file using the Windows API and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
@@ -249,7 +239,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
           DeleteMetaFile(hOld);
           return((HENHMETAFILE) NULL);
         }
-      pBits=(LPBYTE) AcquireMemory(dwSize);
+      pBits=(LPBYTE) AcquireMagickMemory(dwSize);
       if (pBits == (LPBYTE) NULL)
         {
           DeleteMetaFile(hOld);
@@ -257,7 +247,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
         }
       if (GetMetaFileBitsEx(hOld,dwSize,pBits) == 0)
         {
-          LiberateMemory((void **) &pBits);
+          pBits=(char *) RelinquishMagickMemory(pBits);
           DeleteMetaFile(hOld);
           return((HENHMETAFILE) NULL);
         }
@@ -272,7 +262,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
       hTemp=SetWinMetaFileBits(dwSize,pBits,hDC,&mp);
       ReleaseDC(NULL,hDC);
       DeleteMetaFile(hOld);
-      LiberateMemory((void **) &pBits);
+      pBits=(char *) RelinquishMagickMemory(pBits);
       GetEnhMetaFileHeader(hTemp,sizeof(ENHMETAHEADER),&emfh);
       *width=emfh.rclFrame.right-emfh.rclFrame.left;
       *height=emfh.rclFrame.bottom-emfh.rclFrame.top;
@@ -286,12 +276,12 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
   if (hFile == INVALID_HANDLE_VALUE)
     return(NULL);
   dwSize=GetFileSize(hFile,NULL);
-  pBits=(LPBYTE) AcquireMemory(dwSize);
+  pBits=(LPBYTE) AcquireMagickMemory(dwSize);
   ReadFile(hFile,pBits,dwSize,&dwSize,NULL);
   CloseHandle(hFile);
   if (((PAPMHEADER) pBits)->dwKey != 0x9ac6cdd7l)
     {
-      LiberateMemory((void **) &pBits);
+      pBits=(char *) RelinquishMagickMemory(pBits);
       return((HENHMETAFILE) NULL);
     }
   /*
@@ -308,7 +298,7 @@ static HENHMETAFILE ReadEnhMetaFile(const char *szFileName,long *width,
   hDC=GetDC(NULL);
   hTemp=SetWinMetaFileBits(dwSize,&(pBits[sizeof(APMHEADER)]),hDC,&mp);
   ReleaseDC(NULL,hDC);
-  LiberateMemory((void **) &pBits);
+  pBits=(char *) RelinquishMagickMemory(pBits);
   return(hTemp);
 }
 
@@ -353,8 +343,8 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
 
   image=AllocateImage(image_info);
   hemf=ReadEnhMetaFile(image_info->filename,&width,&height);
-  if (!hemf)
-    ThrowReaderException(CorruptImageError,"NotAnEMFImageFile",image);
+  if (hemf == 0)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if ((image->columns == 0) || (image->rows == 0))
     {
       double
@@ -373,8 +363,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           if (image->units == PixelsPerCentimeterResolution)
             x_resolution*=CENTIMETERS_INCH;
         }
-      image->rows=ceil((height*CENTIMETERS_INCH)/1000*y_resolution);
-      image->columns=ceil((width*CENTIMETERS_INCH)/1000*x_resolution);
+      image->rows=(unsigned long)
+        ((height*CENTIMETERS_INCH)/1000*y_resolution+0.5);
+      image->columns=(unsigned long)
+        ((width*CENTIMETERS_INCH)/1000*x_resolution+0.5);
     }
   if (image_info->size != (char *) NULL)
     {
@@ -398,20 +390,20 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       register char
         *p;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       geometry=GetPageGeometry(image_info->page);
       p=strchr(geometry,'>');
-      if (!p)
+      if (p == (char *) NULL)
         {
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned int)
+            image->columns=(unsigned long)
               ((image->columns*image->x_resolution)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned int)
+            image->rows=(unsigned long)
               ((image->rows*image->y_resolution)+0.5);
         }
       else
@@ -420,21 +412,21 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
           flags=ParseMetaGeometry(geometry,&sans,&sans,&image->columns,
             &image->rows);
           if (image->x_resolution != 0.0)
-            image->columns=(unsigned int)
+            image->columns=(unsigned long)
               (((image->columns*image->x_resolution)/72.0)+0.5);
           if (image->y_resolution != 0.0)
-            image->rows=(unsigned int)
+            image->rows=(unsigned long)
               (((image->rows*image->y_resolution)/72.0)+0.5);
         }
-      LiberateMemory((void **) &geometry);
+      geometry=(char *) RelinquishMagickMemory(geometry);
     }
   hDC=GetDC(NULL);
-  if (!hDC)
-    ThrowReaderException(ResourceLimitError,"UnableToCreateADC",image);
+  if (hDC == 0)
+    ThrowReaderException(ResourceLimitError,"UnableToCreateADC");
   /*
     Initialize the bitmap header info.
   */
-  (void) memset(&DIBinfo,0,sizeof(BITMAPINFO));
+  (void) ResetMagickMemory(&DIBinfo,0,sizeof(BITMAPINFO));
   DIBinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
   DIBinfo.bmiHeader.biWidth=image->columns;
   DIBinfo.bmiHeader.biHeight=(-1)*image->rows;
@@ -444,20 +436,20 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   hBitmap=
     CreateDIBSection(hDC,&DIBinfo,DIB_RGB_COLORS,(void **) &ppBits,NULL,0);
   ReleaseDC(NULL,hDC);
-  if (!hBitmap)
-    ThrowReaderException(ResourceLimitError,"UnableToCreateBitmap",image);
+  if (hBitmap == 0)
+    ThrowReaderException(ResourceLimitError,"UnableToCreateBitmap");
   hDC=CreateCompatibleDC(NULL);
-  if (!hDC)
+  if (hDC == 0)
     {
       DeleteObject(hBitmap);
-      ThrowReaderException(ResourceLimitError,"UnableToCreateADC",image);
+      ThrowReaderException(ResourceLimitError,"UnableToCreateADC");
     }
   hOldBitmap=(HBITMAP) SelectObject(hDC,hBitmap);
-  if (!hOldBitmap)
+  if (hOldBitmap == 0)
     {
       DeleteDC(hDC);
       DeleteObject(hBitmap);
-      ThrowReaderException(ResourceLimitError,"UnableToCreateBitmap",image);
+      ThrowReaderException(ResourceLimitError,"UnableToCreateBitmap");
     }
   /*
     Initialize the bitmap to the image background color.
@@ -496,7 +488,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
       pBits++;
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
   }
   if (hemf)
@@ -504,7 +496,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   SelectObject(hDC,hOldBitmap);
   DeleteDC(hDC);
   DeleteObject(hBitmap);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 #endif /* HasWINGDI32 */
 
@@ -519,7 +511,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterEMFImage adds attributes for the EMF image format to
+%  RegisterEMFImage() adds attributes for the EMF image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -533,25 +525,28 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
 */
 ModuleExport void RegisterEMFImage(void)
 {
-#if defined(HasWINGDI32)
   MagickInfo
     *entry;
 
-  entry = SetMagickInfo("EMF");
-  entry->decoder = ReadEMFImage;
-  entry->description = AcquireString("Windows WIN32 API rendered Enhanced Meta File");
-  entry->magick=(MagickHandler) IsEMF;
-  entry->blob_support = False;
-  entry->module = AcquireString("WMF");
+  entry=SetMagickInfo("EMF");
+#if defined(HasWINGDI32)
+  entry->decoder=ReadEMFImage;
+#endif
+  entry->description=
+    AcquireString("Windows WIN32 API rendered Enhanced Meta File");
+  entry->magick=(MagickHandler *) IsEMF;
+  entry->blob_support=MagickFalse;
+  entry->module=AcquireString("WMF");
   (void) RegisterMagickInfo(entry);
-  entry = SetMagickInfo("WMFWIN32");
-  entry->decoder = ReadEMFImage;
-  entry->description = AcquireString("Windows WIN32 API rendered Meta File");
-  entry->magick=(MagickHandler) IsWMF;
-  entry->blob_support = False;
-  entry->module = AcquireString("WMFWIN32");
+  entry=SetMagickInfo("WMFWIN32");
+#if defined(HasWINGDI32)
+  entry->decoder=ReadEMFImage;
+#endif
+  entry->description=AcquireString("Windows WIN32 API rendered Meta File");
+  entry->magick=(MagickHandler *) IsWMF;
+  entry->blob_support=MagickFalse;
+  entry->module=AcquireString("WMFWIN32");
   (void) RegisterMagickInfo(entry);
-#endif /* HasWINGDI32 */
 }
 
 /*
@@ -565,7 +560,7 @@ ModuleExport void RegisterEMFImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterEMFImage removes format registrations made by the
+%  UnregisterEMFImage() removes format registrations made by the
 %  EMF module from the list of supported formats.
 %
 %  The format of the UnregisterEMFImage method is:
@@ -575,8 +570,6 @@ ModuleExport void RegisterEMFImage(void)
 */
 ModuleExport void UnregisterEMFImage(void)
 {
-#if defined(HasWINGDI32)
   (void) UnregisterMagickInfo("EMF");
   (void) UnregisterMagickInfo("WMFWIN32");
-#endif /* HasWINGDI32 */
 }
