@@ -10,39 +10,26 @@
 %                            F      P      X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write FlashPIX Image Format.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,25 +42,34 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 #if defined(HasFPX)
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
 #include <fpxlib.h>
 #else
 #include "Fpxlib.h"
 #endif
 #endif
 
+#if defined(HasFPX)
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFPXImage(const ImageInfo *,Image *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -86,16 +82,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsFPX returns True if the image format type, identified by the
+%  IsFPX() returns MagickTrue if the image format type, identified by the
 %  magick string, is FPX.
 %
 %  The format of the IsFPX method is:
 %
-%      unsigned int IsFPX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFPX returns True if the image format type is FPX.
+%    o status:  Method IsFPX() returns MagickTrue if the image format type is FPX.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -104,13 +100,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFPX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsFPX(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\320\317\021\340",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 #if defined(HasFPX)
@@ -125,7 +121,7 @@ static unsigned int IsFPX(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFPXImage reads a FlashPix image file and returns it.  It
+%  ReadFPXImage() reads a FlashPix image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.  This method was contributed by BillR@corbis.com.
 %
@@ -177,6 +173,9 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -196,13 +195,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *scanline;
 
-  unsigned int
-    status,
-    scene;
-
   unsigned long
     height,
     memory_limit,
+    scene,
     tile_width,
     tile_height,
     width;
@@ -212,25 +208,30 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
   fpx_status=FPX_InitSystem();
   if (fpx_status != FPX_OK)
-    ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary",image);
+    ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary",image);
+      ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
     }
   tile_width=64;
   tile_height=64;
@@ -250,12 +251,20 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status == FPX_LOW_MEMORY_ERROR)
     {
       FPX_ClearSystem();
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+      ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
+        image->filename);
+      DestroyImageList(image);
+      return(MagickFalse);
+    }
+  if (colorspace.numberOfComponents == 0)
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
     }
   if (image_info->view == (char *) NULL)
     {
@@ -268,7 +277,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       aspect_ratio=(float) width/height;
       fpx_status=FPX_GetImageResultAspectRatio(flashpix,&aspect_ratio);
       if (fpx_status != FPX_OK)
-        ThrowReaderException(DelegateError,"UnableToReadAspectRatio",image);
+        ThrowReaderException(DelegateError,"UnableToReadAspectRatio");
       if (width != (unsigned long) ((aspect_ratio*height)+0.5))
         Swap(width,height);
     }
@@ -276,7 +285,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(DelegateError,"UnableToReadSummaryInfo",image);
+      ThrowReaderException(DelegateError,"UnableToReadSummaryInfo");
     }
   if (summary_info.title_valid)
     if ((summary_info.title.length != 0) &&
@@ -288,18 +297,17 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image label.
         */
-        label=(char *) AcquireMemory(summary_info.title.length+1);
+        label=(char *) AcquireMagickMemory(summary_info.title.length+1);
         if (label == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-              image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         (void) strncpy(label,(char *) summary_info.title.ptr,
           summary_info.title.length);
         label[summary_info.title.length]='\0';
         (void) SetImageAttribute(image,"label",label);
-        LiberateMemory((void **) &label);
+        label=(char *) RelinquishMagickMemory(label);
       }
   if (summary_info.comments_valid)
     if ((summary_info.comments.length != 0) &&
@@ -311,18 +319,17 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image comment.
         */
-        comments=(char *) AcquireMemory(summary_info.comments.length+1);
+        comments=(char *) AcquireMagickMemory(summary_info.comments.length+1);
         if (comments == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-              image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         (void) strncpy(comments,(char *) summary_info.comments.ptr,
           summary_info.comments.length);
         comments[summary_info.comments.length]='\0';
         (void) SetImageAttribute(image,"comment",comments);
-        LiberateMemory((void **) &comments);
+        comments=(char *) RelinquishMagickMemory(comments);
       }
   /*
     Determine resolution by scene specification.
@@ -349,35 +356,34 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=width;
   image->rows=height;
   if ((colorspace.numberOfComponents % 2) == 0)
-    image->matte=True;
+    image->matte=MagickTrue;
   if (colorspace.numberOfComponents == 1)
     {
       /*
         Create linear colormap.
       */
-      if (!AllocateImageColormap(image,MaxColormapSize))
+      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
         {
           FPX_ClearSystem();
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Allocate memory for the image and pixel buffer.
   */
-  scanline=(unsigned char *) AcquireMemory(colorspace.numberOfComponents*
+  scanline=(unsigned char *) AcquireMagickMemory(colorspace.numberOfComponents*
     image->columns*(tile_height+1));
   if (scanline == (unsigned char *) NULL)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
@@ -428,11 +434,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (long) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            LiberateMemory((void **) &scanline);
+            scanline=(char *) RelinquishMagickMemory(scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-              image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
       }
     /*
@@ -458,7 +463,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q->green=index;
           q->blue=index;
         }
-      if (image->matte)
+      if (image->matte != MagickFalse)
         q->opacity=ScaleCharToQuantum(255-*a);
       q++;
       r+=red_component->columnStride;
@@ -466,23 +471,16 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
-  LiberateMemory((void **) &scanline);
+  scanline=(char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  return(image);
-}
-#else
-static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  (void) ThrowException(exception,MissingDelegateError,
-    "FPXLibraryIsNotAvailable",image_info->filename);
-  return((Image *) NULL);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -497,7 +495,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFPXImage adds attributes for the FPX image format to
+%  RegisterFPXImage() adds attributes for the FPX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -515,11 +513,13 @@ ModuleExport void RegisterFPXImage(void)
     *entry;
 
   entry=SetMagickInfo("FPX");
-  entry->decoder=(DecoderHandler) ReadFPXImage;
-  entry->encoder=(EncoderHandler) WriteFPXImage;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
-  entry->magick=(MagickHandler) IsFPX;
+#if defined(HasFPX)
+  entry->decoder=(DecoderHandler *) ReadFPXImage;
+  entry->encoder=(EncoderHandler *) WriteFPXImage;
+#endif
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->magick=(MagickHandler *) IsFPX;
   entry->description=AcquireString("FlashPix Format");
   entry->module=AcquireString("FPX");
   (void) RegisterMagickInfo(entry);
@@ -536,7 +536,7 @@ ModuleExport void RegisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFPXImage removes format registrations made by the
+%  UnregisterFPXImage() removes format registrations made by the
 %  FPX module from the list of supported formats.
 %
 %  The format of the UnregisterFPXImage method is:
@@ -561,16 +561,16 @@ ModuleExport void UnregisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteFPXImage writes an image in the FlashPix image format.  This
+%  WriteFPXImage() writes an image in the FlashPix image format.  This
 %  method was contributed by BillR@corbis.com.
 %
 %  The format of the WriteFPXImage method is:
 %
-%      unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o fpx_status: Method WriteFPXImage return True if the image is written.
+%    o fpx_status: Method WriteFPXImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -762,7 +762,7 @@ static void SetSaturation(double saturation,FPXColorTwistMatrix *color_twist)
   *color_twist=result;
 }
 
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 {
   FPXBackground
     background_color;
@@ -801,15 +801,15 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     i;
 
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     memory_limit,
     tile_height,
@@ -822,9 +822,12 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
+  CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
@@ -833,11 +836,11 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(DelegateError,"UnableToInitializeFPXLibrary",image);
+    ThrowWriterException(DelegateError,"UnableToInitializeFPXLibrary");
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
-  if (image->matte)
+  if (image->matte != MagickFalse)
     colorspace.numberOfComponents=4;
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
@@ -850,7 +853,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   background_color.color3_value=0;
   background_color.color4_value=0;
   compression=NONE;
-  if (image_info->compression == JPEGCompression)
+  if (image->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
   {
 #if defined(macintosh)
@@ -866,53 +869,52 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       compression,&flashpix);
   }
   if (fpx_status != FPX_OK)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
-  if (image_info->compression == JPEGCompression)
+    return(status);
+  if (compression == JPEG_UNSPECIFIED)
     {
       /*
         Initialize the compression by quality for the entire image.
       */
-      fpx_status=
-        FPX_SetJPEGCompression(flashpix,(unsigned short) (image_info->quality));
+      fpx_status=FPX_SetJPEGCompression(flashpix,(unsigned short)
+        image_info->quality == 0 ? 75 : image_info->quality);
       if (fpx_status != FPX_OK)
-        ThrowWriterException(DelegateError,"UnableToSetJPEGLevel",image);
+        ThrowWriterException(DelegateError,"UnableToSetJPEGLevel");
     }
   /*
     Set image summary info.
   */
-  summary_info.title_valid=False;
-  summary_info.subject_valid=False;
-  summary_info.author_valid=False;
-  summary_info.comments_valid=False;
-  summary_info.keywords_valid=False;
-  summary_info.OLEtemplate_valid=False;
-  summary_info.last_author_valid=False;
-  summary_info.rev_number_valid=False;
-  summary_info.edit_time_valid=False;
-  summary_info.last_printed_valid=False;
-  summary_info.create_dtm_valid=False;
-  summary_info.last_save_dtm_valid=False;
-  summary_info.page_count_valid=False;
-  summary_info.word_count_valid=False;
-  summary_info.char_count_valid=False;
-  summary_info.thumbnail_valid=False;
-  summary_info.appname_valid=False;
-  summary_info.security_valid=False;
+  summary_info.title_valid=MagickFalse;
+  summary_info.subject_valid=MagickFalse;
+  summary_info.author_valid=MagickFalse;
+  summary_info.comments_valid=MagickFalse;
+  summary_info.keywords_valid=MagickFalse;
+  summary_info.OLEtemplate_valid=MagickFalse;
+  summary_info.last_author_valid=MagickFalse;
+  summary_info.rev_number_valid=MagickFalse;
+  summary_info.edit_time_valid=MagickFalse;
+  summary_info.last_printed_valid=MagickFalse;
+  summary_info.create_dtm_valid=MagickFalse;
+  summary_info.last_save_dtm_valid=MagickFalse;
+  summary_info.page_count_valid=MagickFalse;
+  summary_info.word_count_valid=MagickFalse;
+  summary_info.char_count_valid=MagickFalse;
+  summary_info.thumbnail_valid=MagickFalse;
+  summary_info.appname_valid=MagickFalse;
+  summary_info.security_valid=MagickFalse;
   label=GetImageAttribute(image,"label");
   if (label != (ImageAttribute *) NULL)
     {
       /*
         Note image label.
       */
-      summary_info.title_valid=True;
+      summary_info.title_valid=MagickTrue;
       summary_info.title.length=strlen(label->value);
       summary_info.title.ptr=(unsigned char *)
-        AcquireMemory(strlen(label->value)+1);
-      if (summary_info.title.ptr != (unsigned char *) NULL)
-        (void) strncpy((char *) summary_info.title.ptr,label->value,
-          MaxTextExtent-1);
-      else
-        ThrowWriterException(DelegateError,"UnableToSetImageTitle",image);
+        AcquireMagickMemory(strlen(label->value)+1);
+      if (summary_info.title.ptr == (unsigned char *) NULL)
+        ThrowWriterException(DelegateError,"UnableToSetImageTitle");
+      (void) CopyMagickString((char *) summary_info.title.ptr,label->value,
+        MaxTextExtent);
     }
   comment=GetImageAttribute(image,"comment");
   if (comment != (ImageAttribute *) NULL)
@@ -920,28 +922,27 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       /*
         Note image comment.
       */
-      summary_info.comments_valid=True;
+      summary_info.comments_valid=MagickTrue;
       summary_info.comments.length=strlen(comment->value);
       summary_info.comments.ptr=(unsigned char *)
-        AcquireMemory(strlen(comment->value)+1);
-      if (summary_info.comments.ptr != (unsigned char *) NULL)
-        (void) strcpy((char *) summary_info.comments.ptr,comment->value);
-      else
-        ThrowWriterException(DelegateError,"UnableToSetImageComment",image);
+        AcquireMagickMemory(strlen(comment->value)+1);
+      if (summary_info.comments.ptr == (unsigned char *) NULL)
+        ThrowWriterException(DelegateError,"UnableToSetImageComment");
+      (void) strcpy((char *) summary_info.comments.ptr,comment->value);
     }
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(DelegateError,"UnableToSetSummaryInfo",image);
+    ThrowWriterException(DelegateError,"UnableToSetSummaryInfo");
   /*
     Allocate pixels.
   */
   pixels=(unsigned char *)
-    AcquireMemory(colorspace.numberOfComponents*image->columns);
+    AcquireMagickMemory(colorspace.numberOfComponents*image->columns);
   if (pixels == (unsigned char *) NULL)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
@@ -967,20 +968,20 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   */
   for (y=0; y < (long) image->rows; y++)
   {
-    if (!AcquireImagePixels(image,0,y,image->columns,1,&image->exception))
+    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == MagickFalse)
       break;
     if (fpx_info.numberOfComponents == 1)
       (void) PopImagePixels(image,GrayQuantum,pixels);
     else
-      if (!image->matte)
+      if (image->matte == MagickFalse)
         (void) PopImagePixels(image,RGBQuantum,pixels);
       else
         (void) PopImagePixels(image,RGBAQuantum,pixels);
     fpx_status=FPX_WriteImageLine(flashpix,&fpx_info);
     if (fpx_status != FPX_OK)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
         break;
   }
   if (image_info->view != (char *) NULL)
@@ -1003,7 +1004,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       FPXROI
         view_rect;
 
-      unsigned int
+      MagickBooleanType
         affine_valid,
         aspect_ratio_valid,
         color_twist_valid,
@@ -1015,7 +1016,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         Initialize default viewing parameters.
       */
       contrast=1.0;
-      contrast_valid=False;
+      contrast_valid=MagickFalse;
       color_twist.byy=1.0;
       color_twist.byc1=0.0;
       color_twist.byc2=0.0;
@@ -1032,16 +1033,16 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       color_twist.dummy5_zero=0.0;
       color_twist.dummy6_zero=0.0;
       color_twist.dummy7_one=1.0;
-      color_twist_valid=False;
+      color_twist_valid=MagickFalse;
       sharpen=0.0;
-      sharpen_valid=False;
+      sharpen_valid=MagickFalse;
       aspect_ratio=(double) image->columns/image->rows;
-      aspect_ratio_valid=False;
+      aspect_ratio_valid=MagickFalse;
       view_rect.left=(float) 0.1;
       view_rect.width=aspect_ratio-0.2;
       view_rect.top=(float) 0.1;
       view_rect.height=(float) 0.8; /* 1.0-0.2 */
-      view_rect_valid=False;
+      view_rect_valid=MagickFalse;
       affine.a11=1.0;
       affine.a12=0.0;
       affine.a13=0.0;
@@ -1058,7 +1059,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       affine.a42=0.0;
       affine.a43=0.0;
       affine.a44=1.0;
-      affine_valid=False;
+      affine_valid=MagickFalse;
       if (0)
         {
           /*
@@ -1067,57 +1068,48 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
           SetBrightness(0.5,&color_twist);
           SetSaturation(0.5,&color_twist);
           SetColorBalance(0.5,1.0,1.0,&color_twist);
-          color_twist_valid=True;
+          color_twist_valid=MagickTrue;
         }
       if (affine_valid)
         {
           fpx_status=FPX_SetImageAffineMatrix(flashpix,&affine);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetAffineMatrix",image);
+            ThrowWriterException(DelegateError,"UnableToSetAffineMatrix");
         }
       if (aspect_ratio_valid)
         {
           fpx_status=FPX_SetImageResultAspectRatio(flashpix,&aspect_ratio);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetAspectRatio",image);
+            ThrowWriterException(DelegateError,"UnableToSetAspectRatio");
         }
       if (color_twist_valid)
         {
           fpx_status=FPX_SetImageColorTwistMatrix(flashpix,&color_twist);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetColorTwist",image);
+            ThrowWriterException(DelegateError,"UnableToSetColorTwist");
         }
       if (contrast_valid)
         {
           fpx_status=FPX_SetImageContrastAdjustment(flashpix,&contrast);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetContrast",image);
+            ThrowWriterException(DelegateError,"UnableToSetContrast");
         }
       if (sharpen_valid)
         {
           fpx_status=FPX_SetImageFilteringValue(flashpix,&sharpen);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetFilteringValue",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetFilteringValue");
         }
       if (view_rect_valid)
         {
           fpx_status=FPX_SetImageROI(flashpix,&view_rect);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateError,"UnableToSetRegionOfInterest",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetRegionOfInterest");
         }
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  LiberateMemory((void **) &pixels);
-  CloseBlob(image);
-  return(True);
-}
-#else
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateError,"FPXLibraryIsNotAvailable",
-    image->filename)
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  return(MagickTrue);
 }
 #endif
