@@ -10,39 +10,26 @@
 %            GGG   R  R   A   A  DDDD   IIIII  EEEEE  N   N    T              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                   Read An Image Filled Using Gradient.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,11 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 #include "magick/studio.h"
 
 /*
@@ -72,7 +63,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadGRADIENTImage creates a gradient image and initializes it to
+%  ReadGRADIENTImage creates a gradient image and initializes it to
 %  the color range as specified by the filename.  It allocates the memory
 %  necessary for the new Image structure and returns a pointer to the new
 %  image.
@@ -112,30 +103,32 @@ static Image *ReadGRADIENTImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-  (void) strncpy(colorname,image_info->filename,MaxTextExtent-1);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
+  (void) CopyMagickString(colorname,image_info->filename,MaxTextExtent);
   (void) sscanf(image_info->filename,"%[^-]",colorname);
-  if (!QueryColorDatabase(colorname,&start_color,exception))
+  if (QueryColorDatabase(colorname,&start_color,exception) == MagickFalse)
     {
       DestroyImage(image);
       return((Image *) NULL);
     }
   (void) strcpy(colorname,"white");
-  if (PixelIntensityToQuantum(&start_color) > (0.5*MaxRGB))
+  if (PixelIntensityToQuantum(&start_color) > (MaxRGB/2))
     (void) strcpy(colorname,"black");
   (void) sscanf(image_info->filename,"%*[^-]-%s",colorname);
-  if (!QueryColorDatabase(colorname,&stop_color,exception))
+  if (QueryColorDatabase(colorname,&stop_color,exception) == MagickFalse)
     {
       DestroyImage(image);
       return((Image *) NULL);
     }
   (void) GradientImage(image,&start_color,&stop_color);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -149,7 +142,7 @@ static Image *ReadGRADIENTImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterGRADIENTImage adds attributes for the GRADIENT image format
+%  RegisterGRADIENTImage() adds attributes for the GRADIENT image format
 %  to the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -167,9 +160,9 @@ ModuleExport void RegisterGRADIENTImage(void)
     *entry;
 
   entry=SetMagickInfo("GRADIENT");
-  entry->decoder=(DecoderHandler) ReadGRADIENTImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadGRADIENTImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Gradual passing from one shade to another");
   entry->module=AcquireString("GRADIENT");
   (void) RegisterMagickInfo(entry);
@@ -186,7 +179,7 @@ ModuleExport void RegisterGRADIENTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterGRADIENTImage removes format registrations made by the
+%  UnregisterGRADIENTImage() removes format registrations made by the
 %  GRADIENT module from the list of supported formats.
 %
 %  The format of the UnregisterGRADIENTImage method is:
