@@ -10,39 +10,26 @@
 %                         GGG   R  R   A   A    Y                             %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write RAW Gray Image Format.                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,21 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteGRAYImage(const ImageInfo *,Image *);
 
 /*
@@ -78,7 +69,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadGRAYImage reads an image of raw grayscale samples and returns
+%  ReadGRAYImage() reads an image of raw grayscale samples and returns
 %  it.  It allocates the memory necessary for the new Image structure and
 %   returns a pointer to the new image.
 %
@@ -109,6 +100,9 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     j,
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     i,
     x;
@@ -116,40 +110,46 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   register PixelPacket
     *q;
 
-  size_t
+  ssize_t
     count;
 
+  size_t
+    packet_size;
+
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
-    packet_size;
+    width;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   /*
     Allocate memory for a scanline.
   */
-  packet_size=image->depth > 8 ? 2 : 1;
-  scanline=(unsigned char *) AcquireMemory(packet_size*image->extract_info.width);
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  scanline=(unsigned char *)
+    AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -166,9 +166,9 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     /*
       Convert raster image to pixel packets.
     */
-    if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if (AllocateImageColormap(image,1 << image->depth) == MagickFalse)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     for (y=0; y < image->extract_info.y; y++)
@@ -181,19 +181,19 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
       if (q == (PixelPacket *) NULL)
         break;
       (void) PushImagePixels(image,GrayQuantum,scanline+x);
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
     }
-    count=image->extract_info.height-image->rows-image->extract_info.y;
-    for (j=0; j < (long) count; j++)
+    width=image->extract_info.height-image->rows-image->extract_info.y;
+    for (j=0; j < (long) width; j++)
       (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-    if (EOFBlob(image))
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -218,15 +218,13 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (count != 0);
-  LiberateMemory((void **) &scanline);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -240,7 +238,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterGRAYImage adds attributes for the GRAY image format to
+%  RegisterGRAYImage() adds attributes for the GRAY image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -258,76 +256,12 @@ ModuleExport void RegisterGRAYImage(void)
     *entry;
 
   entry=SetMagickInfo("GRAY");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadGRAYImage;
+  entry->encoder=(EncoderHandler *) WriteGRAYImage;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Raw gray samples");
   entry->module=AcquireString("GRAY");
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("R");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw red samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("C");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw cyan samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("G");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw green samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("M");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw magenta samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("B");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw blue samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("Y");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw yellow samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("O");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw opacity samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("K");
-  entry->decoder=(DecoderHandler) ReadGRAYImage;
-  entry->encoder=(EncoderHandler) WriteGRAYImage;
-  entry->stealth=True;
-  entry->raw=True;
-  entry->description=AcquireString("Raw black samples");
-  entry->module=AcquireString("GRAY");
-  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -341,7 +275,7 @@ ModuleExport void RegisterGRAYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterGRAYImage removes format registrations made by the
+%  UnregisterGRAYImage() removes format registrations made by the
 %  GRAY module from the list of supported formats.
 %
 %  The format of the UnregisterGRAYImage method is:
@@ -352,14 +286,6 @@ ModuleExport void RegisterGRAYImage(void)
 ModuleExport void UnregisterGRAYImage(void)
 {
   (void) UnregisterMagickInfo("GRAY");
-  (void) UnregisterMagickInfo("R");
-  (void) UnregisterMagickInfo("C");
-  (void) UnregisterMagickInfo("G");
-  (void) UnregisterMagickInfo("M");
-  (void) UnregisterMagickInfo("B");
-  (void) UnregisterMagickInfo("Y");
-  (void) UnregisterMagickInfo("O");
-  (void) UnregisterMagickInfo("K");
 }
 
 /*
@@ -373,16 +299,16 @@ ModuleExport void UnregisterGRAYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteGRAYImage writes an image to a file as gray scale intensity
+%  WriteGRAYImage() writes an image to a file as gray scale intensity
 %  values.
 %
 %  The format of the WriteGRAYImage method is:
 %
-%      unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteGRAYImage return True if the image is written.
+%    o status: Method WriteGRAYImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -392,22 +318,26 @@ ModuleExport void UnregisterGRAYImage(void)
 %
 %
 */
-static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
+  size_t
+    packet_size;
+
   unsigned char
     *scanline;
 
-  unsigned int
-    packet_size,
-    scene,
-    status;
-
   /*
     Open output image file.
   */
@@ -415,9 +345,11 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   /*
     Convert image to gray scale PseudoColor class.
   */
@@ -428,10 +360,11 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
       Allocate memory for scanline.
     */
     (void) SetImageColorspace(image,RGBColorspace);
-    packet_size=image->depth > 8 ? 2: 1;
-    scanline=(unsigned char *) AcquireMemory(packet_size*image->columns);
+    packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+    scanline=(unsigned char *)
+      AcquireMagickMemory(packet_size*image->columns);
     if (scanline == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert MIFF to GRAY raster scanline.
     */
@@ -443,22 +376,22 @@ static unsigned int WriteGRAYImage(const ImageInfo *image_info,Image *image)
       (void) PopImagePixels(image,GrayQuantum,scanline);
       (void) WriteBlob(image,packet_size*image->columns,scanline);
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
     }
-    LiberateMemory((void **) &scanline);
+    scanline=(unsigned char *) RelinquishMagickMemory(scanline);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
