@@ -10,39 +10,26 @@
 %       H   H  IIIII  SSSSS    T     OOO    GGG   R  R   A   A  M   M         %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                          Write A Histogram Image.                           %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,17 +42,22 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/resource.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteHISTOGRAMImage(const ImageInfo *,Image *);
 
 /*
@@ -79,7 +71,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterHISTOGRAMImage adds attributes for the Histogram image format
+%  RegisterHISTOGRAMImage() adds attributes for the Histogram image format
 %  to the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -97,8 +89,8 @@ ModuleExport void RegisterHISTOGRAMImage(void)
     *entry;
 
   entry=SetMagickInfo("HISTOGRAM");
-  entry->encoder=(EncoderHandler) WriteHISTOGRAMImage;
-  entry->adjoin=False;
+  entry->encoder=(EncoderHandler *) WriteHISTOGRAMImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Histogram of the image");
   entry->module=AcquireString("HISTOGRAM");
   (void) RegisterMagickInfo(entry);
@@ -115,7 +107,7 @@ ModuleExport void RegisterHISTOGRAMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterHISTOGRAMImage removes format registrations made by the
+%  UnregisterHISTOGRAMImage() removes format registrations made by the
 %  HISTOGRAM module from the list of supported formats.
 %
 %  The format of the UnregisterHISTOGRAMImage method is:
@@ -139,7 +131,7 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteHISTOGRAMImage writes an image to a file in Histogram format.
+%  WriteHISTOGRAMImage() writes an image to a file in Histogram format.
 %  The image shows a histogram of the color (or gray) values in the image.  The
 %  image consists of three overlaid histograms:  a red one for the red channel,
 %  a green one for the green channel, and a blue one for the blue channel.  The
@@ -151,12 +143,12 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %
 %  The format of the WriteHISTOGRAMImage method is:
 %
-%      unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
+%      MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
 %        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteHISTOGRAMImage return True if the image is written.
+%    o status: Method WriteHISTOGRAMImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -166,7 +158,7 @@ ModuleExport void UnregisterHISTOGRAMImage(void)
 %
 %
 */
-static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
+static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   Image *image)
 {
 #define HistogramDensity  "256x200"
@@ -174,9 +166,6 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
   char
     filename[MaxTextExtent];
 
-  double
-    scale;
-
   FILE
     *file;
 
@@ -186,15 +175,19 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
   int
     unique_file;
 
-  long
-    *blue,
-    *green,
-    maximum,
-    *red;
-
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickPixelPacket
+    *histogram;
+
+  MagickRealType
+    maximum,
+    scale;
+
   RectangleInfo
     geometry;
 
@@ -208,10 +201,7 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
     *q,
     *r;
 
-  unsigned int
-    status;
-
-  unsigned long
+  size_t
     length;
 
   /*
@@ -221,38 +211,35 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  SetImageDepth(image,image->depth);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  (void) SetImageDepth(image,image->depth);
   SetGeometry(image,&geometry);
   if (image_info->density == (char *) NULL)
-    (void) ParseMetaGeometry(HistogramDensity,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
+    (void) ParseAbsoluteGeometry(HistogramDensity,&geometry);
   else
-    (void) ParseMetaGeometry(image_info->density,&geometry.x,&geometry.y,
-      &geometry.width,&geometry.height);
-  histogram_image=CloneImage(image,geometry.width,geometry.height,True,
+    (void) ParseAbsoluteGeometry(image_info->density,&geometry);
+  histogram_image=CloneImage(image,geometry.width,geometry.height,MagickTrue,
     &image->exception);
   if (histogram_image == (Image *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
-  histogram_image->matte=True;
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  histogram_image->storage_class=DirectClass;
   /*
     Allocate histogram count arrays.
   */
-  length=Max(ScaleQuantumToChar(MaxRGB)+1,histogram_image->columns);
-  red=(long *) AcquireMemory(length*sizeof(long));
-  green=(long *) AcquireMemory(length*sizeof(long));
-  blue=(long *) AcquireMemory(length*sizeof(long));
-  if ((red == (long *) NULL) || (green == (long *) NULL) ||
-      (blue == (long *) NULL))
+  length=(size_t) Max((unsigned long) ScaleQuantumToChar(MaxRGB)+1,
+    histogram_image->columns);
+  histogram=(MagickPixelPacket *)
+    AcquireMagickMemory(length*sizeof(MagickPixelPacket));
+  if (histogram == (MagickPixelPacket *) NULL)
     {
       DestroyImage(histogram_image);
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  memset(red,0,length*sizeof(long));
-  memset(green,0,length*sizeof(long));
-  memset(blue,0,length*sizeof(long));
   /*
     Initialize histogram count arrays.
   */
+  (void) ResetMagickMemory(histogram,0,length*sizeof(MagickPixelPacket));
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -260,36 +247,27 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      red[ScaleQuantumToChar(p->red)]++;
-      green[ScaleQuantumToChar(p->green)]++;
-      blue[ScaleQuantumToChar(p->blue)]++;
+      histogram[ScaleQuantumToChar(p->red)].red++;
+      histogram[ScaleQuantumToChar(p->green)].green++;
+      histogram[ScaleQuantumToChar(p->blue)].blue++;
       p++;
     }
   }
-  maximum=0;
+  maximum=histogram[0].red;
   for (x=0; x < (long) histogram_image->columns; x++)
   {
-    if (maximum < red[x])
-      maximum=red[x];
-    if (maximum < green[x])
-      maximum=green[x];
-    if (maximum < blue[x])
-      maximum=blue[x];
+    if (maximum < histogram[x].red)
+      maximum=histogram[x].red;
+    if (maximum < histogram[x].green)
+      maximum=histogram[x].green;
+    if (maximum < histogram[x].blue)
+      maximum=histogram[x].blue;
   }
-  for (x=0; x < (long) histogram_image->columns; x++)
-  {
-    if (red[x] > maximum)
-      red[x]=maximum;
-    if (green[x] > maximum)
-      green[x]=maximum;
-    if (blue[x] > maximum)
-      blue[x]=maximum;
-  }
-  scale=(double) histogram_image->rows/maximum;
+  scale=(MagickRealType) histogram_image->rows/maximum;
   /*
     Initialize histogram image.
   */
-  (void) QueryColorDatabase("black",&histogram_image->background_color,
+  (void) QueryColorDatabase("#000000",&histogram_image->background_color,
     &image->exception);
   SetImage(histogram_image,OpaqueOpacity);
   for (x=0; x < (long) histogram_image->columns; x++)
@@ -297,43 +275,41 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
     q=GetImagePixels(histogram_image,x,0,1,histogram_image->rows);
     if (q == (PixelPacket *) NULL)
       break;
-    y=(long) (histogram_image->rows-(long) (scale*red[x]));
+    y=(long) (histogram_image->rows-scale*histogram[x].red+0.5);
     r=q+y;
     for ( ; y < (long) histogram_image->rows; y++)
     {
       r->red=MaxRGB;
       r++;
     }
-    y=(long) (histogram_image->rows-(long) (scale*green[x]));
+    y=(long) (histogram_image->rows-scale*histogram[x].green+0.5);
     r=q+y;
     for ( ; y < (long) histogram_image->rows; y++)
     {
       r->green=MaxRGB;
       r++;
     }
-    y=(long) (histogram_image->rows-(long) (scale*blue[x]));
+    y=(long) (histogram_image->rows-scale*histogram[x].blue+0.5);
     r=q+y;
     for ( ; y < (long) histogram_image->rows; y++)
     {
       r->blue=MaxRGB;
       r++;
     }
-    if (!SyncImagePixels(histogram_image))
+    if (SyncImagePixels(histogram_image) == MagickFalse)
       break;
-    if (QuantumTick(x,histogram_image->columns))
+    if (QuantumTick(x,(long) histogram_image->columns))
       {
         status=MagickMonitor(SaveImageTag,x,histogram_image->columns,
           &image->exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   }
   /*
-    Free memory resources.
+    Free resources.
   */
-  LiberateMemory((void **) &blue);
-  LiberateMemory((void **) &green);
-  LiberateMemory((void **) &red);
+  histogram=(MagickPixelPacket *) RelinquishMagickMemory(histogram);
   file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(filename);
   if (unique_file != -1)
@@ -348,15 +324,16 @@ static unsigned int WriteHISTOGRAMImage(const ImageInfo *image_info,
       */
       (void) GetNumberColors(image,file,&image->exception);
       (void) fclose(file);
-      FormatString(command,"@%.1024s",filename);
+      (void) FormatMagickString(command,MaxTextExtent,"@%s",filename);
       (void) SetImageAttribute(histogram_image,"comment",command);
     }
-  (void) LiberateUniqueFileResource(filename);
+  (void) RelinquishUniqueFileResource(filename);
   /*
     Write Histogram image as MIFF.
   */
-  (void) strncpy(filename,histogram_image->filename,MaxTextExtent-1);
-  (void) FormatString(histogram_image->filename,"miff:%.1024s",filename);
+  (void) CopyMagickString(filename,histogram_image->filename,MaxTextExtent);
+  (void) FormatMagickString(histogram_image->filename,MaxTextExtent,
+    "miff:%s",filename);
   status=WriteImage(image_info,histogram_image);
   DestroyImage(histogram_image);
   return(status);
