@@ -10,39 +10,26 @@
 %                        JJJ    BBBB   IIIII   GGG                            %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                       Read/Write JBIG Image Format.                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,23 +41,31 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/nt_feature.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+#if defined(HasJBIG)
+#include "jbig.h"
+#endif
 
 /*
   Forward declarations.
 */
 #if defined(HasJBIG)
-static unsigned int
+static MagickBooleanType
   WriteJBIGImage(const ImageInfo *,Image *);
 #endif
 
 #if defined(HasJBIG)
-#include "jbig.h"
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -82,7 +77,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadJBIGImage reads a JBIG image file and returns it.  It
+%  ReadJBIGImage() reads a JBIG image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -133,7 +128,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   struct jbg_dec_state
@@ -141,9 +136,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
 
   unsigned char
     bit,
-    *buffer;
-
-  unsigned int
+    *buffer,
     byte;
 
   /*
@@ -151,12 +144,17 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize JBIG toolkit.
   */
@@ -171,19 +169,22 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   /*
     Read JBIG file.
   */
-  buffer=(unsigned char *) AcquireMemory(MaxBufferSize);
+  buffer=(unsigned char *) AcquireMagickMemory(MaxBufferSize);
   if (buffer == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   status=JBG_EAGAIN;
   do
   {
-    length=(long) ReadBlob(image,MaxBufferSize,(char *) buffer);
+    length=(long) ReadBlob(image,MaxBufferSize,buffer);
     if (length == 0)
       break;
     p=buffer;
     count=0;
     while ((length > 0) && ((status == JBG_EAGAIN) || (status == JBG_EOK)))
     {
+      unsigned int
+        count;
+
       status=jbg_dec_in(&jbig_info,p,length,&count);
       p+=count;
       length-=count;
@@ -194,10 +195,10 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   */
   image->columns=jbg_dec_getwidth(&jbig_info);
   image->rows=jbg_dec_getheight(&jbig_info);
-  if (!AllocateImageColormap(image,2))
+  if (AllocateImageColormap(image,2) == MagickFalse)
     {
-      LiberateMemory((void **) &buffer);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
+      buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   image->colormap[0].red=0;
   image->colormap[0].green=0;
@@ -207,10 +208,10 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   image->colormap[1].blue=MaxRGB;
   image->x_resolution=300;
   image->y_resolution=300;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Convert X bitmap image to pixel packets.
@@ -236,19 +237,19 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
       indexes[x]=index;
       *q++=image->colormap[index];
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
   /*
     Free scale resource.
   */
   jbg_dec_free(&jbig_info);
-  LiberateMemory((void **) &buffer);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -263,7 +264,7 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterJBIGImage adds attributes for the JBIG image format to
+%  RegisterJBIGImage() adds attributes for the JBIG image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -287,14 +288,14 @@ ModuleExport void RegisterJBIGImage(void)
 
   *version='\0';
 #if defined(JBG_VERSION)
-  (void) strncpy(version,JBG_VERSION,MaxTextExtent-1);
+  (void) CopyMagickString(version,JBG_VERSION,MaxTextExtent);
 #endif
   entry=SetMagickInfo("BIE");
 #if defined(HasJBIG)
-  entry->decoder=(DecoderHandler) ReadJBIGImage;
-  entry->encoder=(EncoderHandler) WriteJBIGImage;
+  entry->decoder=(DecoderHandler *) ReadJBIGImage;
+  entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
-  entry->adjoin=False;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
     entry->version=AcquireString(version);
@@ -302,8 +303,8 @@ ModuleExport void RegisterJBIGImage(void)
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBG");
 #if defined(HasJBIG)
-  entry->decoder=(DecoderHandler) ReadJBIGImage;
-  entry->encoder=(EncoderHandler) WriteJBIGImage;
+  entry->decoder=(DecoderHandler *) ReadJBIGImage;
+  entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
@@ -312,8 +313,8 @@ ModuleExport void RegisterJBIGImage(void)
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBIG");
 #if defined(HasJBIG)
-  entry->decoder=(DecoderHandler) ReadJBIGImage;
-  entry->encoder=(EncoderHandler) WriteJBIGImage;
+  entry->decoder=(DecoderHandler *) ReadJBIGImage;
+  entry->encoder=(EncoderHandler *) WriteJBIGImage;
 #endif
   entry->description=AcquireString(JBIGDescription);
   if (*version != '\0')
@@ -333,7 +334,7 @@ ModuleExport void RegisterJBIGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterJBIGImage removes format registrations made by the
+%  UnregisterJBIGImage() removes format registrations made by the
 %  JBIG module from the list of supported formats.
 %
 %  The format of the UnregisterJBIGImage method is:
@@ -360,15 +361,15 @@ ModuleExport void UnregisterJBIGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteJBIGImage writes an image in the JBIG encoded image format.
+%  WriteJBIGImage() writes an image in the JBIG encoded image format.
 %
 %  The format of the WriteJBIGImage method is:
 %
-%      unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteJBIGImage return True if the image is written.
+%    o status: Method WriteJBIGImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -388,11 +389,17 @@ static void JBIGEncode(unsigned char *pixels,size_t length,void *data)
   (void) WriteBlob(image,length,pixels);
 }
 
-static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
@@ -414,12 +421,8 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     *pixels,
     polarity;
 
-  unsigned int
-    status;
-
   unsigned long
-    number_packets,
-    scene;
+    number_packets;
 
   /*
     Open image file.
@@ -428,9 +431,11 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   scene=0;
   do
   {
@@ -439,9 +444,9 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     number_packets=((image->columns+7) >> 3)*image->rows;
-    pixels=(unsigned char *) AcquireMemory(number_packets);
+    pixels=(unsigned char *) AcquireMagickMemory(number_packets);
     if (pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert pixels to a bitmap.
     */
@@ -474,9 +479,12 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
        }
       if (bit != 0)
         *q++=byte << (8-bit);
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-          break;
+      if (QuantumTick(y,image->rows) != 0)
+        {
+          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          if (status == MagickFalse)
+            break;
+        }
     }
     /*
       Initialize JBIG info structure.
@@ -502,15 +510,20 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
             GeometryInfo
               geometry_info;
 
-            unsigned int
+            MagickStatusType
               flags;
 
             flags=ParseGeometry(image_info->density,&geometry_info);
             x_resolution=geometry_info.rho;
             y_resolution=geometry_info.sigma;
-            if (!(flags & SigmaValue))
+            if ((flags & SigmaValue) == 0)
               y_resolution=x_resolution;
           }
+        if (image->units == PixelsPerCentimeterResolution)
+          {
+            x_resolution*=2.54;
+            y_resolution*=2.54;
+          }
         (void) jbg_enc_lrlmax(&jbig_info,x_resolution,y_resolution);
       }
     (void) jbg_enc_lrange(&jbig_info,-1,-1);
@@ -521,19 +534,16 @@ static unsigned int WriteJBIGImage(const ImageInfo *image_info,Image *image)
     */
     jbg_enc_out(&jbig_info);
     jbg_enc_free(&jbig_info);
-    LiberateMemory((void **) &pixels);
+    pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
 #endif
