@@ -10,39 +10,26 @@
 %                            M   M  A   A  P                                  %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                  Read/Write Image Colormaps As An Image File.               %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,16 +41,21 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteMAPImage(const ImageInfo *,Image *);
 
 /*
@@ -77,7 +69,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadMAPImage reads an image of raw RGB colormap and colormap index
+%  ReadMAPImage() reads an image of raw RGB colormap and colormap index
 %  bytes and returns it.  It allocates the memory necessary for the new Image
 %  structure and returns a pointer to the new image.
 %
@@ -108,6 +100,9 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -123,44 +118,50 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
+  size_t
+    packet_size;
+
   unsigned char
     *colormap,
     *pixels;
 
-  unsigned int
-    packet_size,
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize image structure.
   */
   image->storage_class=PseudoClass;
-  if (!AllocateImageColormap(image,image->offset ? image->offset : 256))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  packet_size=image->depth > 8 ? 2 : 1;
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
-  packet_size=image->colors > 256 ? 6 : 3;
-  colormap=(unsigned char *) AcquireMemory(packet_size*image->colors);
+  status=AllocateImageColormap(image,(unsigned long)
+    (image->offset != 0 ? image->offset : 256));
+  if (status == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+  packet_size=(size_t) (image->colors > 256 ? 6 : 3);
+  colormap=(unsigned char *) AcquireMagickMemory(packet_size*image->colors);
   if ((pixels == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Read image colormap.
   */
-  (void) ReadBlob(image,packet_size*image->colors,(char *) colormap);
+  (void) ReadBlob(image,packet_size*image->colors,colormap);
   p=colormap;
   if (image->depth <= 8)
     for (i=0; i < (long) image->colors; i++)
@@ -172,23 +173,23 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     for (i=0; i < (long) image->colors; i++)
     {
-      image->colormap[i].red=(*p++ << 8);
+      image->colormap[i].red=(Quantum) (*p++ << 8);
       image->colormap[i].red|=(*p++);
-      image->colormap[i].green=(*p++ << 8);
+      image->colormap[i].green=(Quantum) (*p++ << 8);
       image->colormap[i].green|=(*p++);
-      image->colormap[i].blue=(*p++ << 8);
+      image->colormap[i].blue=(Quantum) (*p++ << 8);
       image->colormap[i].blue|=(*p++);
     }
-  LiberateMemory((void **) &colormap);
-  if (image_info->ping)
+  colormap=(unsigned char *) RelinquishMagickMemory(colormap);
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Read image pixels.
   */
-  packet_size=image->depth > 8 ? 2 : 1;
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
   for (y=0; y < (long) image->rows; y++)
   {
     p=pixels;
@@ -196,24 +197,28 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (q == (PixelPacket *) NULL)
       break;
     indexes=GetIndexes(image);
-    (void) ReadBlob(image,packet_size*image->columns,(char *) pixels);
+    (void) ReadBlob(image,(size_t) packet_size*image->columns,pixels);
     for (x=0; x < (long) image->columns; x++)
     {
-      index=ConstrainColormapIndex(image,*p++);
+      index=ConstrainColormapIndex(image,*p);
+      p++;
       if (image->colors > 256)
-        index=ConstrainColormapIndex(image,(index << 8)+(*p++));
-      indexes[x]=index;
+        {
+          index=ConstrainColormapIndex(image,((int) (index << 8)+(int) (*p)));
+          p++;
+        }
+      indexes[x]=(IndexPacket) index;
       *q++=image->colormap[index];
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
   }
-  LiberateMemory((void **) &pixels);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -227,7 +232,7 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterMAPImage adds attributes for the MAP image format to
+%  RegisterMAPImage() adds attributes for the MAP image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -245,10 +250,10 @@ ModuleExport void RegisterMAPImage(void)
     *entry;
 
   entry=SetMagickInfo("MAP");
-  entry->decoder=(DecoderHandler) ReadMAPImage;
-  entry->encoder=(EncoderHandler) WriteMAPImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadMAPImage;
+  entry->encoder=(EncoderHandler *) WriteMAPImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Colormap intensities and indices");
   entry->module=AcquireString("MAP");
   (void) RegisterMagickInfo(entry);
@@ -265,7 +270,7 @@ ModuleExport void RegisterMAPImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterMAPImage removes format registrations made by the
+%  UnregisterMAPImage() removes format registrations made by the
 %  MAP module from the list of supported formats.
 %
 %  The format of the UnregisterMAPImage method is:
@@ -289,16 +294,16 @@ ModuleExport void UnregisterMAPImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteMAPImage writes an image to a file as red, green, and blue
+%  WriteMAPImage() writes an image to a file as red, green, and blue
 %  colormap bytes followed by the colormap indexes.
 %
 %  The format of the WriteMAPImage method is:
 %
-%      unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteMAPImage return True if the image is written.
+%    o status: Method WriteMAPImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -308,11 +313,14 @@ ModuleExport void UnregisterMAPImage(void)
 %
 %
 */
-static unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -326,16 +334,13 @@ static unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
+  size_t
+    packet_size;
+
   unsigned char
     *colormap,
     *pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    packet_size;
-
   /*
     Open output image file.
   */
@@ -343,22 +348,24 @@ static unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate colormap.
   */
-  if (!IsPaletteImage(image,&image->exception))
-    SetImageType(image,PaletteType);
-  packet_size=image->depth > 8 ? 2 : 1;
-  pixels=(unsigned char *) AcquireMemory(image->columns*packet_size);
-  packet_size=image->colors > 256 ? 6 : 3;
-  colormap=(unsigned char *) AcquireMemory(packet_size*image->colors);
+  if (IsPaletteImage(image,&image->exception) == MagickFalse)
+    (void) SetImageType(image,PaletteType);
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
+  packet_size=(size_t) (image->colors > 256 ? 6 : 3);
+  colormap=(unsigned char *) AcquireMagickMemory(packet_size*image->colors);
   if ((pixels == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Write colormap to file.
   */
@@ -366,22 +373,22 @@ static unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
   if (image->depth <= 8)
     for (i=0; i < (long) image->colors; i++)
     {
-      *q++=image->colormap[i].red;
-      *q++=image->colormap[i].green;
-      *q++=image->colormap[i].blue;
+      *q++=(unsigned char) image->colormap[i].red;
+      *q++=(unsigned char) image->colormap[i].green;
+      *q++=(unsigned char) image->colormap[i].blue;
     }
   else
     for (i=0; i < (long) image->colors; i++)
     {
-      *q++=image->colormap[i].red >> 8;
-      *q++=image->colormap[i].red & 0xff;
-      *q++=image->colormap[i].green >> 8;
-      *q++=image->colormap[i].green & 0xff;
-      *q++=image->colormap[i].blue >> 8;
-      *q++=image->colormap[i].blue & 0xff;
+      *q++=(unsigned char) (image->colormap[i].red >> 8);
+      *q++=(unsigned char) image->colormap[i].red;
+      *q++=(unsigned char) (image->colormap[i].green >> 8);
+      *q++=(unsigned char) image->colormap[i].green;
+      *q++=(unsigned char) (image->colormap[i].blue >> 8);
+      *q++=(unsigned char) image->colormap[i].blue;
     }
-  (void) WriteBlob(image,packet_size*image->colors,(char *) colormap);
-  LiberateMemory((void **) &colormap);
+  (void) WriteBlob(image,packet_size*image->colors,colormap);
+  colormap=(unsigned char *) RelinquishMagickMemory(colormap);
   /*
     Write image pixels to file.
   */
@@ -395,12 +402,12 @@ static unsigned int WriteMAPImage(const ImageInfo *image_info,Image *image)
     for (x=0; x < (long) image->columns; x++)
     {
       if (image->colors > 256)
-        *q++=indexes[x] >> 8;
-      *q++=indexes[x];
+        *q++=(unsigned char) (indexes[x] >> 8);
+      *q++=(unsigned char) indexes[x];
     }
-    (void) WriteBlob(image,q-pixels,(char *) pixels);
+    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
   }
-  LiberateMemory((void **) &pixels);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(status);
 }
