@@ -2,15 +2,14 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%               M   M   AAA   TTTTT  L       AAA   BBBB                       %
-%               MM MM  A   A    T    L      A   A  B   B                      %
-%               M M M  AAAAA    T    L      AAAAA  BBBB                       %
-%               M   M  A   A    T    L      A   A  B   B                      %
-%               M   M  A   A    T    LLLLL  A   A  BBBB                       %
+%                  M   M   AAA   TTTTT  L       AAA   BBBB                    %
+%                  MM MM  A   A    T    L      A   A  B   B                   %
+%                  M M M  AAAAA    T    L      AAAAA  BBBB                    %
+%                  M   M  A   A    T    L      A   A  B   B                   %
+%                  M   M  A   A    T    LLLLL  A   A  BBBB                    %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                        Read MATLAB Image Format.                            %
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
@@ -50,16 +49,23 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color_private.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/shear.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+#include "magick/transform.h"
 
 typedef struct
 {
   char identific[125];
   char idx[3];
+  unsigned short Version;
+  unsigned short Magic;    
   unsigned long unknown0;
   unsigned long ObjectSize;
   unsigned long unknown1;
@@ -72,7 +78,8 @@ typedef struct
 
   unsigned long SizeX;
   unsigned long SizeY;
-  unsigned short Flag1;
+  unsigned long Flag1;
+  unsigned short NameComprSize;
   unsigned short NameFlag;
 }
 MATHeader;
@@ -101,7 +108,7 @@ static void InsertRow(unsigned char *p, int y, Image * image)
           *q++ = image->colormap[index];
           p++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
       return;
@@ -119,7 +126,7 @@ static void InsertRow(unsigned char *p, int y, Image * image)
           p += 2;
           q++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
         return;
       }
@@ -147,7 +154,7 @@ static void InsertFloatRow(double *p, int y, Image * image, double Min, double M
     p++;
     q++;
   }
-  if (!SyncImagePixels(image))
+  if (SyncImagePixels(image) == MagickFalse)
     return;
   return;
 }
@@ -196,7 +203,7 @@ static void InsertComplexFloatRow(double *p, int y, Image * image, double Min,
     p++;
     q++;
   }
-  if (!SyncImagePixels(image))
+  if (SyncImagePixels(image) == MagickFalse)
     return;
   return;
 }
@@ -213,6 +220,18 @@ static void ReadBlobWordLSB(Image * I, size_t len, unsigned short *data)
     (void) SeekBlob(I, len, SEEK_CUR);
 }
 
+/*This function reads one block of unsigned shortS*/
+static void ReadBlobWordMSB(Image * I, size_t len, unsigned short *data)
+{
+  while (len >= 2)
+  {
+    *data++ = ReadBlobMSBShort(I);
+    len -= 2;
+  }
+  if (len > 0)
+    (void) SeekBlob(I, len, SEEK_CUR);
+}
+
 static double ReadBlobLSBdouble(Image * image)
 {
   typedef union
@@ -227,9 +246,10 @@ static double ReadBlobLSBdouble(Image * image)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if (ReadBlob(image, 8, (unsigned char *) &buffer) == 0)
-    return (0.0);
+    return(0.0);
   if (*(char *) &lsb_first == 1)
     return (buffer.d);
 
@@ -258,6 +278,52 @@ static void ReadBlobDoublesLSB(Image * I, size_t len, double *data)
   if (len > 0)
     (void) SeekBlob(I, len, SEEK_CUR);
 }
+
+static double ReadBlobMSBdouble(Image * image)
+{
+  typedef union
+  {
+    double d;
+    char chars[8];
+  }
+  dbl;
+  static unsigned long lsb_first = 1;
+  dbl buffer;
+  char c;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+
+  if (ReadBlob(image, 8, (unsigned char *) &buffer) == 0)
+    return (0.0);
+  if (!(*(char *) &lsb_first == 1))
+        return (buffer.d);
+
+  c = buffer.chars[0];
+  buffer.chars[0] = buffer.chars[7];
+  buffer.chars[7] = c;
+  c = buffer.chars[1];
+  buffer.chars[1] = buffer.chars[6];
+  buffer.chars[6] = c;
+  c = buffer.chars[2];
+  buffer.chars[2] = buffer.chars[5];
+  buffer.chars[5] = c;
+  c = buffer.chars[3];
+  buffer.chars[3] = buffer.chars[4];
+  buffer.chars[4] = c;
+  return (buffer.d);
+}
+
+static void ReadBlobDoublesMSB(Image * I, size_t len, double *data)
+{
+  while (len >= 8)
+  {
+    *data++ = ReadBlobMSBdouble(I);
+    len -= sizeof(double);
+  }
+  if (len > 0)
+    (void) SeekBlob(I, len, SEEK_CUR);
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -270,13 +336,13 @@ static void ReadBlobDoublesLSB(Image * I, size_t len, double *data)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadMATImage reads an MAT X image file and returns it.  It
+%  ReadMATImage() reads an MAT X image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
 %  The format of the ReadMATImage method is:
 %
-%      Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%    Image *ReadMATImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -294,13 +360,14 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
 {
   Image *image,
    *rotated_image;
-  unsigned int status;
+  MagickBooleanType status;
   MATHeader MATLAB_HDR;
   unsigned long size;
-  ExtendedSignedIntegralType filepos;
+  MagickOffsetType filepos;
   unsigned long CellType;
   int i,
-    x;
+      x,
+      lsb = 1;
   long ldblk;
   unsigned char *BImgBuff = NULL;
   double Min,
@@ -313,13 +380,16 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   image = AllocateImage(image_info);
 
   status = OpenBlob(image_info, image, ReadBinaryBlobMode, exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError, "UnableToOpenFile", image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return(MagickFalse);
+    }
   /*
      Read MATLAB image.
    */
-  (void) ReadBlob(image, 125, &MATLAB_HDR.identific);
-  (void) ReadBlob(image, 3, &MATLAB_HDR.idx);
+  (void) ReadBlob(image, 125, (unsigned char *) &MATLAB_HDR.identific);
+  (void) ReadBlob(image, 3, (unsigned char *) &MATLAB_HDR.idx);
   MATLAB_HDR.unknown0 = ReadBlobLSBLong(image);
   MATLAB_HDR.ObjectSize = ReadBlobLSBLong(image);
   MATLAB_HDR.unknown1 = ReadBlobLSBLong(image);
@@ -334,15 +404,13 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   MATLAB_HDR.NameFlag = ReadBlobLSBShort(image);
 
   if (strncmp(MATLAB_HDR.identific, "MATLAB", 6))
-  MATLAB_KO:ThrowReaderException(CorruptImageError, "NotAMATLABImageFile",
-                         image);
+  MATLAB_KO:ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   if (strncmp(MATLAB_HDR.idx, "\1IM", 3))
     goto MATLAB_KO;
   if (MATLAB_HDR.unknown0 != 0x0E)
     goto MATLAB_KO;
   if (MATLAB_HDR.DimFlag != 8)
-    ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported",
-                         image);
+    ThrowReaderException(CoderError, "MultidimensionalMatricesAreNotSupported");
 
   /*printf("MATLAB_HDR.StructureFlag %ld\n",MATLAB_HDR.StructureFlag); */
   if (MATLAB_HDR.StructureFlag != 6 && MATLAB_HDR.StructureFlag != 0x806)
@@ -351,7 +419,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   switch (MATLAB_HDR.NameFlag)
   {
     case 0:
-      (void) ReadBlob(image, 4, &size); /*Object name string */
+      (void) ReadBlob(image, 4, (unsigned char *) &size); /*Object name string */
       size = 4 * (long) ((size + 3 + 1) / 4);
       (void) SeekBlob(image, size, SEEK_CUR);
       break;
@@ -359,7 +427,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
     case 2:
     case 3:
     case 4:
-      (void) ReadBlob(image, 4, &size); /*Object name string */
+      (void) ReadBlob(image, 4, (unsigned char *) &size); /*Object name string */
       break;
     default:
       goto MATLAB_KO;
@@ -367,7 +435,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
 
   CellType = ReadBlobLSBLong(image);    /*Additional object type */
   /*fprintf(stdout,"Cell type:%ld\n",CellType); */
-  (void) ReadBlob(image, 4, &size);     /*data size */
+  (void) ReadBlob(image, 4, (unsigned char *) &size);     /*data size */
 
   switch (CellType)
   {
@@ -386,14 +454,15 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
     case 9:
       image->depth = Min(QuantumDepth,32);        /*double type cell */
       if (sizeof(double) != 8)
-        ThrowReaderException(CoderError, "IncompatibleSizeOfDouble", image);
+        ThrowReaderException(CoderError, "IncompatibleSizeOfDouble");
       if (MATLAB_HDR.StructureFlag == 0x806)
       {                         /*complex double type cell */
       }
       ldblk = (long) (8 * MATLAB_HDR.SizeX);
       break;
     default:
-  ThrowReaderException(CoderError, "UnsupportedCellTypeInTheMatrix", image)}
+    ThrowReaderException(CoderError, "UnsupportedCellTypeInTheMatrix");
+  }
 
   image->columns = MATLAB_HDR.SizeX;
   image->rows = MATLAB_HDR.SizeY;
@@ -406,10 +475,10 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   if (CellType == 2)
   {
     image->colors = 256;
-    if (!AllocateImageColormap(image, image->colors))
+    if (AllocateImageColormap(image, image->colors) == MagickFalse)
     {
-  NoMemory:ThrowReaderException(ResourceLimitError, "MemoryAllocationFailed",
-                           image)}
+  NoMemory:ThrowReaderException(ResourceLimitError, "MemoryAllocationFailed");
+    }
 
     for (i = 0; i < (long) image->colors; i++)
     {
@@ -420,7 +489,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   }
 
   /* ----- Load raster data ----- */
-  BImgBuff = (unsigned char *) AcquireMemory(ldblk);    /*Ldblk was set in the check phase */
+  BImgBuff = (unsigned char *) AcquireMagickMemory(ldblk);    /*Ldblk was set in the check phase */
   if (BImgBuff == NULL)
     goto NoMemory;
 
@@ -431,7 +500,11 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
     filepos = TellBlob(image);
     for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
     {
-      ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+      if(lsb) {
+          ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+      } else {
+          ReadBlobDoublesMSB(image, ldblk, (double *) BImgBuff);
+      }
       dblrow = (double *) BImgBuff;
       if (i == 0)
       {
@@ -455,15 +528,23 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
     switch (CellType)
     {
       case 4:
-        ReadBlobWordLSB(image, ldblk, (unsigned short *) BImgBuff);
-        InsertRow(BImgBuff, i, image);
-        break;
+          if(lsb) {
+              ReadBlobWordLSB(image, ldblk, (unsigned short *) BImgBuff);
+          } else {
+              ReadBlobWordMSB(image, ldblk, (unsigned short *) BImgBuff);
+          }
+          InsertRow(BImgBuff, i, image);
+          break;
       case 9:
-        ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
-        InsertFloatRow((double *) BImgBuff, i, image, Min, Max);
-        break;
+          if(lsb) {
+              ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);      
+          } else {
+              ReadBlobDoublesMSB(image, ldblk, (double *) BImgBuff);      
+          }
+          InsertFloatRow((double *) BImgBuff, i, image, Min, Max);
+          break;
       default:
-        (void) ReadBlob(image, ldblk, (char *) BImgBuff);
+        (void) ReadBlob(image, ldblk, BImgBuff);
         InsertRow(BImgBuff, i, image);
     }
   }
@@ -471,12 +552,16 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
   /*Read complex part of numbers here */
   if (MATLAB_HDR.StructureFlag == 0x806)
   {
-    if (CellType == 9)          /*Find Min and Max Values for complex parts of floats */
+    if (CellType == 9) /*Find Min and Max Values for complex parts of floats */
     {
       filepos = TellBlob(image);
       for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
       {
-        ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+        if(lsb) {
+            ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+        } else {
+            ReadBlobDoublesMSB(image, ldblk, (double *) BImgBuff);
+        }
         dblrow = (double *) BImgBuff;
         if (i == 0)
         {
@@ -495,7 +580,11 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
 
       for (i = 0; i < (long) MATLAB_HDR.SizeY; i++)
       {
-        ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+        if(lsb) {
+            ReadBlobDoublesLSB(image, ldblk, (double *) BImgBuff);
+        } else {
+            ReadBlobDoublesMSB(image, ldblk, (double *) BImgBuff);
+        }            
         InsertComplexFloatRow((double *) BImgBuff, i, image, Min, Max);
       }
     }
@@ -503,7 +592,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
 
   if (BImgBuff != NULL)
   {
-    LiberateMemory((void **) &BImgBuff);
+    BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
     BImgBuff = NULL;
   }
   CloseBlob(image);
@@ -534,7 +623,7 @@ static Image *ReadMATImage(const ImageInfo * image_info, ExceptionInfo * excepti
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterMATImage adds attributes for the MAT image format to
+%  RegisterMATImage() adds attributes for the MAT image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -550,11 +639,11 @@ ModuleExport void RegisterMATImage(void)
 {
   MagickInfo * entry;
 
-  entry = SetMagickInfo("MAT");
-  entry->decoder = (DecoderHandler) ReadMATImage;
-  entry->seekable_stream = True;
-  entry->description = AcquireString("MATLAB image format");
-  entry->module = AcquireString("MAT");
+  entry=SetMagickInfo("MAT");
+  entry->decoder=(DecoderHandler *) ReadMATImage;
+  entry->seekable_stream=MagickTrue;
+  entry->description=AcquireString("MATLAB image format");
+  entry->module=AcquireString("MAT");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -569,7 +658,7 @@ ModuleExport void RegisterMATImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterMATImage removes format registrations made by the
+%  UnregisterMATImage() removes format registrations made by the
 %  MAT module from the list of supported formats.
 %
 %  The format of the UnregisterMATImage method is:
