@@ -10,39 +10,26 @@
 %                     M   M  A   A    T      T    EEEEE                       %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Write Matte Channel To MIFF File.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,16 +41,18 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteMATTEImage(const ImageInfo *,Image *);
 
 /*
@@ -77,7 +66,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterMATTEImage adds attributes for the MATTE image format to
+%  RegisterMATTEImage() adds attributes for the MATTE image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -95,8 +84,7 @@ ModuleExport void RegisterMATTEImage(void)
     *entry;
 
   entry=SetMagickInfo("MATTE");
-  entry->encoder=(EncoderHandler) WriteMATTEImage;
-  entry->raw=True;
+  entry->encoder=(EncoderHandler *) WriteMATTEImage;
   entry->description=AcquireString("MATTE format");
   entry->module=AcquireString("MATTE");
   (void) RegisterMagickInfo(entry);
@@ -113,7 +101,7 @@ ModuleExport void RegisterMATTEImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterMATTEImage removes format registrations made by the
+%  UnregisterMATTEImage() removes format registrations made by the
 %  MATTE module from the list of supported formats.
 %
 %  The format of the UnregisterMATTEImage method is:
@@ -137,26 +125,24 @@ ModuleExport void UnregisterMATTEImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function WriteMATTEImage writes an image of matte bytes to a file.  It
+%  Function WriteMATTEImage() writes an image of matte bytes to a file.  It
 %  consists of data from the matte component of the image [0..255].
 %
 %  The format of the WriteMATTEImage method is:
 %
-%      unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Function WriteMATTEImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMATTEImage(const ImageInfo *image_info,
+  Image *image)
 {
   Image
     *matte_image;
@@ -164,6 +150,9 @@ static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -173,16 +162,14 @@ static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
-
-  if (!image->matte)
-    ThrowWriterException(CoderError,"ImageDoesNotHaveAMatteChannel",image);
+  if (image->matte == MagickFalse)
+    ThrowWriterException(CoderError,"ImageDoesNotHaveAMatteChannel");
   matte_image=
-    CloneImage(image,image->columns,image->rows,True,&image->exception);
+    CloneImage(image,image->columns,image->rows,MagickTrue,&image->exception);
   if (matte_image == (Image *) NULL)
-    return(False);
-  SetImageType(matte_image,TrueColorType);
+    return(MagickFalse);
+  (void) SetImageType(matte_image,TrueColorType);
+  matte_image->matte=MagickFalse;
   /*
     Convert image to matte pixels.
   */
@@ -201,14 +188,18 @@ static unsigned int WriteMATTEImage(const ImageInfo *image_info,Image *image)
       p++;
       q++;
     }
-    if (!SyncImagePixels(matte_image))
+    if (SyncImagePixels(matte_image) == MagickFalse)
       break;
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-          break;
+      if (QuantumTick(y,image->rows) != 0)
+        {
+          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          if (status == MagickFalse)
+            break;
+        }
   }
-  (void) FormatString(matte_image->filename,"MIFF:%.1024s",image->filename);
+  (void) FormatMagickString(matte_image->filename,MaxTextExtent,
+    "MIFF:%s",image->filename);
   status=WriteImage(image_info,matte_image);
   DestroyImage(matte_image);
   return(status);
