@@ -10,39 +10,26 @@
 %                        M   M  P      EEEEE   GGGG                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                          Write MPEG Image Format.                           %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,19 +40,25 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/error.h"
+#include "magick/constitute.h"
 #include "magick/delegate.h"
+#include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
-#include "magick/resource.h"
+#include "magick/memory_.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteMPEGImage(const ImageInfo *image_info,Image *image);
 
 /*
@@ -79,16 +72,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsMPEG returns True if the image format type, identified by the
+%  IsMPEG() returns MagickTrue if the image format type, identified by the
 %  magick string, is MPEG.
 %
 %  The format of the IsMPEG method is:
 %
-%      unsigned int IsMPEG(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsMPEG(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsMPEG returns True if the image format type is MPEG.
+%    o status:  Method IsMPEG() returns MagickTrue if the image format type is MPEG.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -97,13 +90,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsMPEG(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsMPEG(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\000\000\001\263",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -117,7 +110,7 @@ static unsigned int IsMPEG(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadMPEGImage reads an binary file in the MPEG video stream format
+%  ReadMPEGImage() reads an binary file in the MPEG video stream format
 %  and returns it.  It allocates the memory necessary for the new Image
 %  structure and returns a pointer to the new image.  This method differs from
 %  the other decoder methods in that only the Photoshop resource (MPEG)
@@ -148,64 +141,70 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
     *images;
 
   ImageInfo
-    *clone_info;
+    *read_info;
+
+  MagickBooleanType
+    status;
 
   register long
     i;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   CloseBlob(image);
   /*
     Convert MPEG to PPM with delegate.
   */
   image=AllocateImage(image_info);
-  clone_info=CloneImageInfo(image_info);
-  (void) InvokeDelegate(clone_info,image,"mpeg-decode",(char *) NULL,exception);
+  read_info=CloneImageInfo(image_info);
+  (void) InvokeDelegate(read_info,image,"mpeg-decode",(char *) NULL,exception);
   DestroyImage(image);
   /*
     Read PPM files.
   */
   images=NewImageList();
-  for (i=(long) clone_info->scene; ; i++)
+  for (i=(long) read_info->scene; ; i++)
   {
-    FormatString(clone_info->filename,"%.1024s%ld.ppm",clone_info->unique,i);
-    if (!IsAccessible(clone_info->filename))
+    (void) FormatMagickString(read_info->filename,MaxTextExtent,
+      "%s%ld.ppm",read_info->unique,i);
+    if (IsAccessible(read_info->filename) == MagickFalse)
       break;
-    image=ReadImage(clone_info,exception);
-    (void) remove(clone_info->filename);
+    image=ReadImage(read_info,exception);
     if (image == (Image *) NULL)
       break;
     (void) strcpy(image->magick,image_info->magick);
-    image->scene=i;
+    image->scene=(unsigned long) i;
     AppendImageToList(&images,image);
-    if (clone_info->number_scenes != 0)
-      if (i >= (long) (clone_info->scene+clone_info->number_scenes-1))
+    if (read_info->number_scenes != 0)
+      if (i >= (long) (read_info->scene+read_info->number_scenes-1))
         break;
   }
   /*
     Free resources.
   */
-  for (i++; ; i++)
+  for (i=0; ; i++)
   {
-    FormatString(clone_info->filename,"%.1024s%ld.ppm",clone_info->unique,i);
-    if (IsAccessible(clone_info->filename) == False)
+    (void) FormatMagickString(read_info->filename,MaxTextExtent,
+      "%s%ld.ppm",read_info->unique,i);
+    if (IsAccessible(read_info->filename) == MagickFalse)
       break;
-    (void) remove(clone_info->filename);
+    (void) remove(read_info->filename);
   }
-  DestroyImageInfo(clone_info);
+  DestroyImageInfo(read_info);
   return(images);
 }
 
@@ -220,7 +219,7 @@ static Image *ReadMPEGImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterMPEGImage adds attributes for the MPEG image format to
+%  RegisterMPEGImage() adds attributes for the MPEG image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -238,26 +237,26 @@ ModuleExport void RegisterMPEGImage(void)
     *entry;
 
   entry=SetMagickInfo("MPEG");
-  entry->decoder=(DecoderHandler) ReadMPEGImage;
-  entry->encoder=(EncoderHandler) WriteMPEGImage;
-  entry->magick=(MagickHandler) IsMPEG;
-  entry->blob_support=False;
+  entry->decoder=(DecoderHandler *) ReadMPEGImage;
+  entry->encoder=(EncoderHandler *) WriteMPEGImage;
+  entry->magick=(MagickHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPG");
-  entry->decoder=(DecoderHandler) ReadMPEGImage;
-  entry->encoder=(EncoderHandler) WriteMPEGImage;
-  entry->magick=(MagickHandler) IsMPEG;
-  entry->blob_support=False;
+  entry->decoder=(DecoderHandler *) ReadMPEGImage;
+  entry->encoder=(EncoderHandler *) WriteMPEGImage;
+  entry->magick=(MagickHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M2V");
-  entry->decoder=(DecoderHandler) ReadMPEGImage;
-  entry->encoder=(EncoderHandler) WriteMPEGImage;
-  entry->magick=(MagickHandler) IsMPEG;
-  entry->blob_support=False;
+  entry->decoder=(DecoderHandler *) ReadMPEGImage;
+  entry->encoder=(EncoderHandler *) WriteMPEGImage;
+  entry->magick=(MagickHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
   entry->description=AcquireString("MPEG Video Stream");
   entry->module=AcquireString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -274,7 +273,7 @@ ModuleExport void RegisterMPEGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterMPEGImage removes format registrations made by the
+%  UnregisterMPEGImage() removes format registrations made by the
 %  BIM module from the list of supported formats.
 %
 %  The format of the UnregisterBIMImage method is:
@@ -284,7 +283,9 @@ ModuleExport void RegisterMPEGImage(void)
 */
 ModuleExport void UnregisterMPEGImage(void)
 {
+  (void) UnregisterMagickInfo("M2V");
   (void) UnregisterMagickInfo("MPEG");
+  (void) UnregisterMagickInfo("MPG");
 }
 
 /*
@@ -298,17 +299,17 @@ ModuleExport void UnregisterMPEGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteMPEGImage writes an image to a file in MPEG video stream format.
+%  WriteMPEGImage() writes an image to a file in MPEG video stream format.
 %  Lawrence Livermore National Laboratory (LLNL) contributed code to adjust
 %  the MPEG parameters to correspond to the compression quality setting.
 %
 %  The format of the WriteMPEGImage method is:
 %
-%      unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteMPEGImage return True if the image is written.
+%    o status: Method WriteMPEGImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -319,7 +320,7 @@ ModuleExport void UnregisterMPEGImage(void)
 %
 */
 
-static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
+static MagickBooleanType WriteMPEGParameterFiles(const ImageInfo *image_info,
   Image *image,const char *basename)
 {
   char
@@ -336,12 +337,18 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
     quant,
     vertical_factor;
 
+  MagickBooleanType
+    mpeg;
+
   register Image
     *p;
 
   register long
     i;
 
+  ssize_t
+    count;
+
   static int
     q_matrix[]=
     {
@@ -355,36 +362,30 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
       27, 29, 35, 38, 46, 56, 69, 83
     };
 
-  unsigned int
-    mpeg;
-
-  unsigned long
-    count;
-
   /*
     Write parameter file (see mpeg2encode documentation for details).
   */
   file=fopen(basename,"w");
   if (file == (FILE *) NULL)
-    return(False);
+    return(MagickFalse);
   (void) fprintf(file,"MPEG\n");  /* comment */
-  (void) fprintf(file,"%.1024s.%%d\n",image->filename); /* source frame file */
+  (void) fprintf(file,"%s.%%d\n",image->filename); /* source frame file */
   (void) fprintf(file,"-\n");  /* reconstructed frame file */
-  if (image_info->quality == DefaultCompressionQuality)
+  if (image_info->quality == UndefinedCompressionQuality)
     (void) fprintf(file,"-\n");  /* default intra quant matrix */
   else
     {
       /*
         Write intra quant matrix file.
       */
-      FormatString(filename,"%.1024s.iqm",basename);
+      (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",basename);
       (void) fprintf(file,"%s\n",filename);
       parameter_file=fopen(filename,"w");
       if (parameter_file == (FILE *) NULL)
-        return(False);
-      if (image_info->quality >= DefaultCompressionQuality)
+        return(MagickFalse);
+      if (image_info->quality >= 75)
         {
-          q=Max((image_info->quality-DefaultCompressionQuality)*2.0,1.0);
+          q=(double) Max((image_info->quality-75)*2,1);
           for (i=0; i < 64; i++)
           {
             quant=(long) Min(Max(q_matrix[i]/q,1.0),255.0);
@@ -395,7 +396,7 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
         }
       else
         {
-          q=Max((DefaultCompressionQuality-image_info->quality)/8.0,1.0);
+          q=(double) Max((75-image_info->quality)/8,1);
           for (i=0; i < 64; i++)
           {
             quant=(long) Min(Max(q*q_matrix[i]+0.5,1.0),255.0);
@@ -406,18 +407,18 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
         }
       (void) fclose(parameter_file);
     }
-  if (image_info->quality == DefaultCompressionQuality)
+  if (image_info->quality == UndefinedCompressionQuality)
     (void) fprintf(file,"-\n");  /* default non intra quant matrix */
   else
     {
       /*
         Write non intra quant matrix file.
       */
-      FormatString(filename,"%.1024s.niq",basename);
+      (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",basename);
       (void) fprintf(file,"%s\n",filename);
       parameter_file=fopen(filename,"w");
       if (parameter_file == (FILE *) NULL)
-        return(False);
+        return(MagickFalse);
       q=Min(Max(66.0-(2*image_info->quality)/3.0,1.0),255);
       for (i=0; i < 64; i++)
       {
@@ -427,36 +428,37 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
       }
       (void) fclose(parameter_file);
     }
-  (void) fprintf(file,"%.1024s.log\n",basename);  /* statistics log */
+  (void) fprintf(file,"%s.log\n",basename);  /* statistics log */
   (void) fprintf(file,"1\n");  /* input picture file format */
   count=0;
   for (p=image; p != (Image *) NULL; p=p->next)
     count+=Max((p->delay+1)/3,1);
-  (void) fprintf(file,"%lu\n",count); /* number of frames */
+  (void) fprintf(file,"%lu\n",(unsigned long) count); /* number of frames */
   (void) fprintf(file,"0\n");  /* number of first frame */
   (void) fprintf(file,"00:00:00:00\n");  /* timecode of first frame */
-  mpeg=LocaleCompare(image_info->magick,"M2V") != 0;
+  mpeg=(MagickBooleanType) (LocaleCompare(image_info->magick,"M2V") != 0);
   if (image_info->quality > 98)
     (void) fprintf(file,"1\n");
   else
-    (void) fprintf(file,"%d\n",mpeg ? 12 : 15);
+    (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 12 : 15);
   if (image_info->quality > 98)
     (void) fprintf(file,"1\n");
   else
     (void) fprintf(file,"3\n");
-  (void) fprintf(file,"%d\n",mpeg ? 1 : 0);  /* ISO/IEC 11172-2 stream */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* ISO/IEC 11172-2 stream */
   (void) fprintf(file,"0\n");  /* select frame picture coding */
-  (void) fprintf(file,"%lu\n",image->columns+(image->columns & 0x01 ? 1 : 0));
-  (void) fprintf(file,"%lu\n",image->rows+(image->rows & 0x01 ? 1 : 0));
-  (void) fprintf(file,"%d\n",mpeg ? 8 : 2);  /* aspect ratio */
-  (void) fprintf(file,"%d\n",mpeg ? 3 : 5);  /* frame rate code */
-  (void) fprintf(file,"%.1f\n",mpeg ? 1152000.0 : 5000000.0);  /* bit rate */
-  (void) fprintf(file,"%d\n",mpeg ? 20 : 112);  /* vbv buffer size */
+  (void) fprintf(file,"%lu\n",image->columns+
+    ((image->columns & 0x01) != 0 ? 1 : 0));
+  (void) fprintf(file,"%lu\n",image->rows+((image->rows & 0x01) != 0 ? 1 : 0));
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 8 : 2);  /* aspect ratio */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 3 : 5);  /* frame rate code */
+  (void) fprintf(file,"%g\n",mpeg != MagickFalse ? 1152000.0 : 5000000.0);  /* bit rate */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 20 : 112);  /* vbv buffer size */
   (void) fprintf(file,"0\n");  /* low delay */
-  (void) fprintf(file,"%d\n",mpeg ? 1 : 0);  /* constrained parameter */
-  (void) fprintf(file,"%d\n",mpeg ? 4 : 1);  /* profile ID */
-  (void) fprintf(file,"%d\n",mpeg ? 8 : 4);  /* level ID */
-  (void) fprintf(file,"%d\n",mpeg ? 1 : 0);  /* progressive sequence */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* constrained parameter */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 4 : 1);  /* profile ID */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 8 : 4);  /* level ID */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* progressive sequence */
   vertical_factor=2;
   if (image_info->sampling_factor != (char *) NULL)
     {
@@ -466,20 +468,20 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
       long
         horizontal_factor;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
       horizontal_factor=(long) geometry_info.rho;
       vertical_factor=(long) geometry_info.sigma;
-      if (!(flags & SigmaValue))
+      if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
-      if (mpeg)
+      if (mpeg != MagickFalse)
         {
           if ((horizontal_factor != 2) || (vertical_factor != 2))
             {
               (void) fclose(file);
-              return(False);
+              return(MagickFalse);
             }
         }
       else
@@ -487,25 +489,28 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
             ((vertical_factor != 1) && (vertical_factor != 2)))
           {
             (void) fclose(file);
-            return(False);
+            return(MagickFalse);
           }
     }
-  (void) fprintf(file,"%d\n",vertical_factor==2 ? 1 : 2); /* chroma format */
-  (void) fprintf(file,"%d\n",mpeg ? 1 : 2);  /* video format */
+  (void) fprintf(file,"%d\n",vertical_factor == 2 ? 1 : 2); /* chroma format */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 2);  /* video format */
   (void) fprintf(file,"5\n");  /* color primaries */
   (void) fprintf(file,"5\n");  /* transfer characteristics */
-  (void) fprintf(file,"%d\n",mpeg ? 5 : 4);  /* matrix coefficients */
-  (void) fprintf(file,"%lu\n",image->columns+(image->columns & 0x01 ? 1 : 0));
-  (void) fprintf(file,"%lu\n",image->rows+(image->rows & 0x01 ? 1 : 0));
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 5 : 4);  /* matrix coefficients */
+  (void) fprintf(file,"%lu\n",image->columns+
+    ((image->columns & 0x01) != 0 ? 1 : 0));
+  (void) fprintf(file,"%lu\n",image->rows+((image->rows & 0x01) != 0 ? 1 : 0));
   (void) fprintf(file,"0\n");  /* intra dc precision */
-  (void) fprintf(file,"%d\n",mpeg ? 0 : 1);  /* top field */
-  (void) fprintf(file,"%d %d %d\n",mpeg ? 1 : 0,mpeg ? 1 : 0, mpeg ? 1 : 0);
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 0 : 1);  /* top field */
+  (void) fprintf(file,"%d %d %d\n",mpeg != MagickFalse ? 1 : 0,
+    mpeg != MagickFalse ? 1 : 0, mpeg != MagickFalse ? 1 : 0);
   (void) fprintf(file,"0 0 0\n");  /* concealment motion vector */
-  (void) fprintf(file,"%d %d %d\n",mpeg ? 0 : 1,mpeg ? 0 : 1,mpeg ? 0 : 1);
-  (void) fprintf(file,"%d 0 0\n",mpeg ? 0 : 1);  /* intra vlc format */
+  (void) fprintf(file,"%d %d %d\n",mpeg != MagickFalse ? 0 : 1,
+    mpeg != MagickFalse ? 0 : 1,mpeg != MagickFalse ? 0 : 1);
+  (void) fprintf(file,"%d 0 0\n",mpeg != MagickFalse ? 0 : 1);  /* intra vlc format */
   (void) fprintf(file,"0 0 0\n");  /* alternate scan */
   (void) fprintf(file,"0\n");  /* repeat first field */
-  (void) fprintf(file,"%d\n",mpeg ? 1 : 0);  /* progressive frame */
+  (void) fprintf(file,"%d\n",mpeg != MagickFalse ? 1 : 0);  /* progressive frame */
   (void) fprintf(file,"0\n");  /* intra slice refresh period */
   (void) fprintf(file,"0\n");  /* reaction parameter */
   (void) fprintf(file,"0\n");  /* initial average activity */
@@ -521,10 +526,10 @@ static unsigned int WriteMPEGParameterFiles(const ImageInfo *image_info,
   (void) fprintf(file,"1 1 7 7\n");
   (void) fprintf(file,"1 1 3 3\n");
   (void) fclose(file);
-  return(True);
+  return(MagickTrue);
 }
 
-static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,Image *image)
 {
   char
     basename[MaxTextExtent],
@@ -535,11 +540,14 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
     *next_image;
 
   ImageInfo
-    *clone_info;
+    *write_info;
 
   int
     file;
 
+  MagickBooleanType
+    status;
+
   register Image
     *p;
 
@@ -552,10 +560,6 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   unsigned char
     *blob;
 
-  unsigned int
-    logging,
-    status;
-
   unsigned long
     count,
     scene;
@@ -567,10 +571,11 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  logging=LogMagickEvent(CoderEvent,GetMagickModule(),"enter");
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   CloseBlob(image);
   /*
     Determine if the sequence of images have identical page info.
@@ -590,7 +595,7 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
     {
       coalesce_image=CoalesceImages(image,&image->exception);
       if (coalesce_image == (Image *) NULL)
-        return(False);
+        return(MagickFalse);
     }
   /*
     Write YUV files.
@@ -598,25 +603,27 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   file=AcquireUniqueFileResource(basename);
   if (file != -1)
     (void) close(file);
-  FormatString(coalesce_image->filename,basename);
-  clone_info=CloneImageInfo(image_info);
-  status=WriteMPEGParameterFiles(clone_info,coalesce_image,basename);
-  if (status == False)
+  (void) FormatMagickString(coalesce_image->filename,MaxTextExtent,basename);
+  write_info=CloneImageInfo(image_info);
+  status=WriteMPEGParameterFiles(write_info,coalesce_image,basename);
+  if (status == MagickFalse)
     {
       if (coalesce_image != image)
         DestroyImage(coalesce_image);
       (void) remove(basename);
-      if (image_info->quality != DefaultCompressionQuality)
+      if (image_info->quality != 75)
         {
-          FormatString(filename,"%.1024s.iqm",basename);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",
+            basename);
           (void) remove(filename);
-          FormatString(filename,"%.1024s.niq",basename);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",
+            basename);
           (void) remove(filename);
         }
-      ThrowWriterException(CoderError,"UnableToWriteMPEGParameters",image)
+      ThrowWriterException(CoderError,"UnableToWriteMPEGParameters");
     }
   count=0;
-  clone_info->interlace=PlaneInterlace;
+  write_info->interlace=PlaneInterlace;
   for (p=coalesce_image; p != (Image *) NULL; p=p->next)
   {
     char
@@ -629,7 +636,7 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
     {
       p->scene=count;
       count++;
-      status=False;
+      status=MagickFalse;
       switch (i)
       {
         case 0:
@@ -637,52 +644,58 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
           Image
             *frame;
 
-          FormatString(p->filename,"%.1024s.%lu.yuv",basename,p->scene);
-          FormatString(filename,"%.1024s.%lu.yuv",basename,p->scene);
-          FormatString(previous_image,"%.1024s.%lu.yuv",basename,p->scene);
-          frame=CloneImage(p,0,0,True,&p->exception);
+          (void) FormatMagickString(p->filename,MaxTextExtent,"%s.%lu.yuv",
+            basename,p->scene);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s.%lu.yuv",
+            basename,p->scene);
+          (void) FormatMagickString(previous_image,MaxTextExtent,
+            "%s.%lu.yuv",basename,p->scene);
+          frame=CloneImage(p,0,0,MagickTrue,&p->exception);
           if (frame == (Image *) NULL)
             break;
-          status=WriteImage(clone_info,frame);
+          status=WriteImage(write_info,frame);
           DestroyImage(frame);
           break;
         }
         case 1:
+        {
           blob=(unsigned char *)
             FileToBlob(previous_image,&length,&image->exception);
+        }
         default:
         {
-          FormatString(filename,"%.1024s.%lu.yuv",basename,p->scene);
+          (void) FormatMagickString(filename,MaxTextExtent,"%s.%lu.yuv",
+            basename,p->scene);
           if (length > 0)
             status=BlobToFile(filename,blob,length,&image->exception);
           break;
         }
       }
-      if (logging)
+      if (image->debug != MagickFalse)
         {
-          if (status)
+          if (status != MagickFalse)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "%lu. Wrote YUV file for scene %lu:",i,p->scene);
           else
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "%lu. Failed to write YUV file for scene %lu:",i,p->scene);
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%.1024s",
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"%s",
             filename);
         }
     }
     p->scene=scene;
     if (blob != (unsigned char *) NULL)
-      LiberateMemory((void **) &blob);
-    if (status == False)
+      blob=(unsigned char *) RelinquishMagickMemory(blob);
+    if (status == MagickFalse)
       break;
   }
   /*
     Convert YUV to MPEG.
   */
-  (void) strncpy(coalesce_image->filename,basename,MaxTextExtent-1);
-  status=InvokeDelegate(clone_info,coalesce_image,(char *) NULL,"mpeg-encode",
+  (void) CopyMagickString(coalesce_image->filename,basename,MaxTextExtent);
+  status=InvokeDelegate(write_info,coalesce_image,(char *) NULL,"mpeg-encode",
     &image->exception);
-  DestroyImageInfo(clone_info);
+  DestroyImageInfo(write_info);
   /*
     Free resources.
   */
@@ -691,21 +704,22 @@ static unsigned int WriteMPEGImage(const ImageInfo *image_info,Image *image)
   {
     for (i=0; i < (long) Max((p->delay+1)/3,1); i++)
     {
-      FormatString(p->filename,"%.1024s.%lu.yuv",basename,count++);
+      (void) FormatMagickString(p->filename,MaxTextExtent,"%s.%lu.yuv",
+        basename,count++);
       (void) remove(p->filename);
     }
-    (void) strncpy(p->filename,image_info->filename,MaxTextExtent-1);
+    (void) CopyMagickString(p->filename,image_info->filename,MaxTextExtent);
   }
-  (void) LiberateUniqueFileResource(basename);
-  FormatString(filename,"%.1024s.iqm",basename);
+  (void) RelinquishUniqueFileResource(basename);
+  (void) FormatMagickString(filename,MaxTextExtent,"%s.iqm",basename);
   (void) remove(filename);
-  FormatString(filename,"%.1024s.niq",basename);
+  (void) FormatMagickString(filename,MaxTextExtent,"%s.niq",basename);
   (void) remove(filename);
-  FormatString(filename,"%.1024s.log",basename);
+  (void) FormatMagickString(filename,MaxTextExtent,"%s.log",basename);
   (void) remove(filename);
   if (coalesce_image != image)
     DestroyImage(coalesce_image);
-  if (logging)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),"exit");
   return(status);
 }
