@@ -9,39 +9,27 @@
 %                            M   M   V V   G   G                              %
 %                            M   M    V     GGG                               %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
 %                                                                             %
+%                 Read/Write Magick Vector Graphics Metafiles.                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 April 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +42,19 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/draw.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
-#include "magick/render.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteMVGImage(const ImageInfo *,Image *);
 
 /*
@@ -78,16 +68,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsMVG returns True if the image format type, identified by the
+%  IsMVG() returns MagickTrue if the image format type, identified by the
 %  magick string, is MVG.
 %
 %  The format of the IsMVG method is:
 %
-%      unsigned int IsMVG(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsMVG(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsMVG returns True if the image format type is MVG.
+%    o status:  Method IsMVG() returns MagickTrue if the image format type is MVG.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -96,13 +86,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsMVG(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsMVG(const unsigned char *magick,const size_t length)
 {
   if (length < 20)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"push graphic-context",20) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -116,7 +106,7 @@ static unsigned int IsMVG(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadMVGImage creates a gradient image and initializes it to
+%  ReadMVGImage creates a gradient image and initializes it to
 %  the X server color range as specified by the filename.  It allocates the
 %  memory necessary for the new Image structure and returns a pointer to the
 %  new image.
@@ -147,23 +137,31 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
+  MagickBooleanType
+    status;
+
   size_t
     length;
 
-  unsigned int
-    status;
+  unsigned char
+    *primitive;
 
   /*
     Open image.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   if ((image->columns == 0) || (image->rows == 0))
     {
       char
@@ -185,21 +183,28 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
           continue;
         (void) sscanf(p,"viewbox %lf %lf %lf %lf",&bounds.x1,&bounds.y1,
           &bounds.x2,&bounds.y2);
-        image->columns=(unsigned long) (bounds.x2-bounds.x1+0.5);
-        image->rows=(unsigned long) (bounds.y2-bounds.y1+0.5);
+        image->columns=(unsigned long) ((bounds.x2-bounds.x1)+0.5);
+        image->rows=(unsigned long) ((bounds.y2-bounds.y1)+0.5);
         break;
       }
     }
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  draw_info->affine.sx=
+    image->x_resolution == 0.0 ? 1.0 : image->x_resolution/72.0;
+  draw_info->affine.sy=
+    image->y_resolution == 0.0 ? 1.0 : image->y_resolution/72.0;
+  image->columns=(unsigned long) (draw_info->affine.sx*image->columns);
+  image->rows=(unsigned long) (draw_info->affine.sy*image->rows);
+  SetImage(image,OpaqueOpacity);
   /*
     Render drawing.
   */
-  SetImage(image,OpaqueOpacity);
-  draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->fill=image_info->pen;
-  if (GetBlobStreamType(image) == BlobStream)
-    draw_info->primitive=AllocateString((char *) GetBlobStreamData(image));
+  primitive=GetBlobStreamData(image);
+  if (primitive != (unsigned char) NULL)
+    draw_info->primitive=AcquireString((char *) primitive);
   else
     draw_info->primitive=(char *) FileToBlob(image->filename,&length,exception);
   if (draw_info->primitive == (char *) NULL)
@@ -207,7 +212,7 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) DrawImage(image,draw_info);
   DestroyDrawInfo(draw_info);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -221,7 +226,7 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterMVGImage adds attributes for the MVG image format
+%  RegisterMVGImage() adds attributes for the MVG image format
 %  to the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -239,11 +244,11 @@ ModuleExport void RegisterMVGImage(void)
     *entry;
 
   entry=SetMagickInfo("MVG");
-  entry->decoder=(DecoderHandler) ReadMVGImage;
-  entry->encoder=(EncoderHandler) WriteMVGImage;
-  entry->magick=(MagickHandler) IsMVG;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->decoder=(DecoderHandler *) ReadMVGImage;
+  entry->encoder=(EncoderHandler *) WriteMVGImage;
+  entry->magick=(MagickHandler *) IsMVG;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("Magick Vector Graphics");
   entry->module=AcquireString("MVG");
   (void) RegisterMagickInfo(entry);
@@ -260,7 +265,7 @@ ModuleExport void RegisterMVGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterMVGImage removes format registrations made by the
+%  UnregisterMVGImage() removes format registrations made by the
 %  MVG module from the list of supported formats.
 %
 %  The format of the UnregisterMVGImage method is:
@@ -284,15 +289,15 @@ ModuleExport void UnregisterMVGImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteMVGImage writes an image to a file in MVG image format.
+%  WriteMVGImage() writes an image to a file in MVG image format.
 %
 %  The format of the WriteMVGImage method is:
 %
-%      unsigned int WriteMVGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteMVGImage return True if the image is written.
+%    o status: Method WriteMVGImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -302,12 +307,12 @@ ModuleExport void UnregisterMVGImage(void)
 %
 %
 */
-static unsigned int WriteMVGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image)
 {
   const ImageAttribute
     *attribute;
 
-  unsigned int
+  MagickBooleanType
     status;
 
   /*
@@ -317,13 +322,16 @@ static unsigned int WriteMVGImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   attribute=GetImageAttribute(image,"[MVG]");
   if (attribute == (ImageAttribute *) NULL)
-    ThrowWriterException(OptionError,"NoImageVectorGraphics",image);
+    ThrowWriterException(OptionError,"NoImageVectorGraphics");
   status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
-  (void) WriteBlob(image,strlen(attribute->value),attribute->value);
+  if (status == MagickFalse)
+    return(status);
+  (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
+    attribute->value);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
