@@ -10,39 +10,26 @@
 %                            P       CCCC  DDDD                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write Photo CD Image Format.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,19 +41,31 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/client.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/decorate.h"
 #include "magick/error.h"
-#include "magick/command.h"
 #include "magick/gem.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/mogrify.h"
 #include "magick/monitor.h"
+#include "magick/montage.h"
+#include "magick/resize.h"
+#include "magick/shear.h"
 #include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePCDImage(const ImageInfo *,Image *);
 
 /*
@@ -80,18 +79,18 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method DecodeImage recovers the Huffman encoded luminance and chrominance
+%  DecodeImage recovers the Huffman encoded luminance and chrominance
 %  deltas.
 %
 %  The format of the DecodeImage method is:
 %
-%      static unsigned int DecodeImage(Image *image,unsigned char *luma,
+%      static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
 %        unsigned char *chroma1,unsigned char *chroma2)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the deltas are
-%      recovered without error, otherwise False.
+%    o status:  Method DecodeImage returns MagickTrue if all the deltas are
+%      recovered without error, otherwise MagickFalse.
 %
 %    o image: The address of a structure of type Image.
 %
@@ -107,7 +106,7 @@ static unsigned int
 %
 %
 */
-static unsigned int DecodeImage(Image *image,unsigned char *luma,
+static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   unsigned char *chroma1,unsigned char *chroma2)
 {
 #define IsSync  ((sum & 0xffffff00) == 0xfffffe00)
@@ -120,14 +119,14 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   { \
     if (p >= (buffer+0x800)) \
       { \
-        (void) ReadBlob(image,0x800,(char *) buffer); \
+        (void) ReadBlob(image,0x800,buffer); \
         p=buffer; \
       } \
     sum|=((unsigned int) (*p) << (24-bits)); \
     bits+=8; \
     p++; \
   } \
-  if (EOFBlob(image)) \
+  if (EOFBlob(image) != MagickFalse) \
     break; \
 }
 
@@ -137,7 +136,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
       length,
       sequence;
 
-    unsigned int
+    MagickStatusType
       mask;
 
     unsigned char
@@ -148,6 +147,9 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     count,
     quantum;
 
+  MagickBooleanType
+    status;
+
   PCDTable
     *pcd_table[3];
 
@@ -179,13 +181,16 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     Initialize Huffman tables.
   */
   assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(luma != (unsigned char *) NULL);
   assert(chroma1 != (unsigned char *) NULL);
   assert(chroma2 != (unsigned char *) NULL);
-  buffer=(unsigned char *) AcquireMemory(0x800);
+  buffer=(unsigned char *) AcquireMagickMemory(0x800);
   if (buffer == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      (char *) NULL);
+      image->filename);
   sum=0;
   bits=32;
   p=buffer+0x800;
@@ -193,12 +198,12 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
-    pcd_table[i]=(PCDTable *) AcquireMemory(length*sizeof(PCDTable));
+    pcd_table[i]=(PCDTable *) AcquireMagickMemory(length*sizeof(PCDTable));
     if (pcd_table[i] == (PCDTable *) NULL)
       {
-        LiberateMemory((void **) &buffer);
+        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-          (char *) NULL)
+          image->filename);
       }
     r=pcd_table[i];
     for (j=0; j < (long) length; j++)
@@ -207,14 +212,14 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
       r->length=(sum & 0xff)+1;
       if (r->length > 16)
         {
-          LiberateMemory((void **) &buffer);
-          return(False);
+          buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+          return(MagickFalse);
         }
       PCDGetBits(16);
       r->sequence=(sum & 0xffff) << 16;
       PCDGetBits(8);
       r->key=(unsigned char) (sum & 0xff);
-      r->mask=(~((((unsigned int) 1) << (32-r->length))-1));
+      r->mask=(~((((MagickBooleanType) 1) << (32-r->length))-1));
       r++;
     }
     pcd_length[i]=length;
@@ -228,7 +233,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     PCDGetBits(16);
   while ((sum & 0x00fff000) != 0x00fff000)
     PCDGetBits(8);
-  while (!IsSync)
+  while (IsSync == 0)
     PCDGetBits(1);
   /*
     Recover the Huffman encoded luminance and chrominance deltas.
@@ -240,7 +245,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
   q=luma;
   for ( ; ; )
   {
-    if (IsSync)
+    if (IsSync != 0)
       {
         /*
           Determine plane and row number.
@@ -276,14 +281,18 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
-            ThrowBinaryException(CorruptImageError,"CorruptPCDImage",
-              image->filename)
+            ThrowBinaryException(CorruptImageError,"CorruptImage",
+              image->filename);
           }
         }
         length=pcd_length[plane];
         if (QuantumTick(row,image->rows))
-          if (!MagickMonitor(DecodeImageTag,row,image->rows,&image->exception))
-            break;
+          {
+            status=MagickMonitor(DecodeImageTag,row,image->rows,
+              &image->exception);
+            if (status == MagickFalse)
+              break;
+          }
         continue;
       }
     /*
@@ -292,13 +301,14 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     r=pcd_table[plane];
     for (i=0; ((i < (long) length) && ((sum & r->mask) != r->sequence)); i++)
       r++;
-    if ((row > image->rows) || (r == (PCDTable *) NULL))
+    if ((row > image->rows) || (r == (PCDTable *) NULL) ||
+        ((size_t) (q-luma) > (image->columns*image->rows)))
       {
-        (void) ThrowException(&image->exception,CorruptImageWarning,"SkipToSyncByte",
-          image->filename);
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          CorruptImageWarning,"SkipToSyncByte",image->filename);
         while ((sum & 0x00fff000) != 0x00fff000)
           PCDGetBits(8);
-        while (!IsSync)
+        while (IsSync == 0)
           PCDGetBits(1);
         continue;
       }
@@ -312,12 +322,12 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
     count--;
   }
   /*
-    Free memory.
+    Free resources.
   */
   for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
-    LiberateMemory((void **) &pcd_table[i]);
-  LiberateMemory((void **) &buffer);
-  return(True);
+    pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  return(MagickTrue);
 }
 
 /*
@@ -331,16 +341,16 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsPCD returns True if the image format type, identified by the
+%  IsPCD() returns MagickTrue if the image format type, identified by the
 %  magick string, is PCD.
 %
 %  The format of the IsPCD method is:
 %
-%      unsigned int IsPCD(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPCD returns True if the image format type is PCD.
+%    o status:  Method IsPCD() returns MagickTrue if the image format type is PCD.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -349,13 +359,13 @@ static unsigned int DecodeImage(Image *image,unsigned char *luma,
 %
 %
 */
-static unsigned int IsPCD(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPCD(const unsigned char *magick,const size_t length)
 {
   if (length < 2052)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick+2048,"PCD_",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -369,7 +379,7 @@ static unsigned int IsPCD(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPCDImage reads a Photo CD image file and returns it.  It
+%  ReadPCDImage() reads a Photo CD image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.  Much of the PCD decoder was derived from
 %  the program hpcdtoppm(1) by Hadmut Danisch.
@@ -401,7 +411,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
     *montage_image;
 
   ImageInfo
-    *clone_info;
+    *mogrify_info;
 
   MontageInfo
     *montage_info;
@@ -409,21 +419,22 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   /*
     Create image tiles.
   */
-  clone_info=CloneImageInfo(image_info);
-  commands[0]=SetClientName((char *) NULL);
+  mogrify_info=CloneImageInfo(image_info);
+  commands[0]=(char *) GetClientName();
   commands[1]=(char *) "-label";
   commands[2]=(char *) DefaultTileLabel;
-  (void) MogrifyImages(clone_info,3,commands,&image);
-  DestroyImageInfo(clone_info);
+  (void) MogrifyImages(mogrify_info,3,commands,&image);
+  DestroyImageInfo(mogrify_info);
   /*
     Create the PCD Overview image.
   */
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
-  (void) strncpy(montage_info->filename,image_info->filename,MaxTextExtent-1);
+  (void) CopyMagickString(montage_info->filename,image_info->filename,
+    MaxTextExtent);
   montage_image=MontageImages(image,montage_info,exception);
   DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   DestroyImage(image);
   return(montage_image);
 }
@@ -436,10 +447,16 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     x;
 
-  ExtendedSignedIntegralType
-    number_pixels,
+  MagickBooleanType
+    overview,
+    status;
+
+  MagickOffsetType
     offset;
 
+  MagickSizeType
+    number_pixels;
+
   register long
     i,
     y;
@@ -461,14 +478,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *header,
     *luma;
 
-  unsigned int
-    overview,
-    rotate,
-    status;
-
   unsigned long
     height,
     number_images,
+    rotate,
     scene,
     width;
 
@@ -477,26 +490,31 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Determine if this is a PCD file.
   */
-  header=(unsigned char *) AcquireMemory(3*0x800);
+  header=(unsigned char *) AcquireMagickMemory(3*0x800);
   if (header == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  count=ReadBlob(image,3*0x800,(char *) header);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,3*0x800,header);
   overview=LocaleNCompare((char *) header,"PCD_OPA",7) == 0;
   if ((count == 0) ||
       ((LocaleNCompare((char *) header+0x800,"PCD",3) != 0) && !overview))
-    ThrowReaderException(CorruptImageError,"NotAPCDImageFile",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   rotate=header[0x0e02] & 0x03;
   number_images=(header[10] << 8) | header[11];
-  LiberateMemory((void **) &header);
+  header=(unsigned char *) RelinquishMagickMemory(header);
   /*
     Determine resolution by scene specification.
   */
@@ -539,15 +557,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate luma and chroma memory.
   */
-  number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+  number_pixels=(MagickSizeType) image->columns*image->rows;
   if (number_pixels != (size_t) number_pixels)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  chroma1=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
-  chroma2=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
-  luma=(unsigned char *) AcquireMemory(image->columns*image->rows+1);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  chroma1=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
+  chroma2=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
+  luma=(unsigned char *) AcquireMagickMemory(image->columns*image->rows+1);
   if ((chroma1 == (unsigned char *) NULL) ||
       (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Advance to image data.
   */
@@ -579,8 +597,10 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (j=1; j <= (long) number_images; j++)
       {
         handler=SetMonitorHandler((MonitorHandler) NULL);
-        FormatString(image->filename,"images/img%04ld.pcd",j);
-        FormatString(image->magick_filename,"images/img%04ld.pcd",j);
+        (void) FormatMagickString(image->filename,MaxTextExtent,
+          "images/img%04ld.pcd",j);
+        (void) FormatMagickString(image->magick_filename,MaxTextExtent,
+          "images/img%04ld.pcd",j);
         image->scene=j;
         image->columns=width;
         image->rows=height;
@@ -590,13 +610,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         c2=chroma2;
         for (y=0; y < (long) height; y+=2)
         {
-          (void) ReadBlob(image,width,(char *) yy);
+          (void) ReadBlob(image,width,yy);
           yy+=image->columns;
-          (void) ReadBlob(image,width,(char *) yy);
+          (void) ReadBlob(image,width,yy);
           yy+=image->columns;
-          (void) ReadBlob(image,width >> 1,(char *) c1);
+          (void) ReadBlob(image,width >> 1,c1);
           c1+=image->columns;
-          (void) ReadBlob(image,width >> 1,(char *) c2);
+          (void) ReadBlob(image,width >> 1,c2);
           c2+=image->columns;
         }
         Upsample(image->columns >> 1,image->rows >> 1,image->columns,chroma1);
@@ -619,13 +639,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleCharToQuantum(*c2++);
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
         }
+        image->colorspace=YCCColorspace;
         if (LocaleCompare(image_info->magick,"PCDS") == 0)
-          (void) TransformRGBImage(image,sRGBColorspace);
-        else
-          (void) TransformRGBImage(image,YCCColorspace);
+          image->colorspace=sRGBColorspace;
+        SetImageColorspace(image,RGBColorspace);
         if (j < (long) number_images)
           {
             /*
@@ -640,12 +660,12 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             image=SyncNextImageInList(image);
           }
         (void) SetMonitorHandler(handler);
-        if (!MagickMonitor(LoadImageTag,j-1,number_images,exception))
+        if (MagickMonitor(LoadImageTag,j-1,number_images,exception) == MagickFalse)
           break;
       }
-      LiberateMemory((void **) &chroma2);
-      LiberateMemory((void **) &chroma1);
-      LiberateMemory((void **) &luma);
+      chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
+      chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
+      luma=(unsigned char *) RelinquishMagickMemory(luma);
       while (image->previous != (Image *) NULL)
         image=image->previous;
       overview_image=OverviewImage(image_info,image,exception);
@@ -659,13 +679,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   c2=chroma2;
   for (y=0; y < (long) height; y+=2)
   {
-    (void) ReadBlob(image,width,(char *) yy);
+    (void) ReadBlob(image,width,yy);
     yy+=image->columns;
-    (void) ReadBlob(image,width,(char *) yy);
+    (void) ReadBlob(image,width,yy);
     yy+=image->columns;
-    (void) ReadBlob(image,width >> 1,(char *) c1);
+    (void) ReadBlob(image,width >> 1,c1);
     c1+=image->columns;
-    (void) ReadBlob(image,width >> 1,(char *) c2);
+    (void) ReadBlob(image,width >> 1,c2);
     c2+=image->columns;
   }
   if (scene >= 4)
@@ -692,7 +712,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           offset=TellBlob(image)/0x800+12;
           (void) SeekBlob(image,offset*0x800,SEEK_SET);
           status=DecodeImage(image,luma,chroma1,chroma2);
-          if ((scene >= 6) && status)
+          if ((scene >= 6) && (status != MagickFalse))
             {
               /*
                 Recover luminance deltas for 6144x4096 image (vaporware).
@@ -724,25 +744,25 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       q->blue=ScaleCharToQuantum(*c2++);
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
-  LiberateMemory((void **) &chroma2);
-  LiberateMemory((void **) &chroma1);
-  LiberateMemory((void **) &luma);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
+  chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
+  luma=(unsigned char *) RelinquishMagickMemory(luma);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  if (!image_info->ping)
+  if (image_info->ping == MagickFalse)
     if ((rotate == 1) || (rotate == 3))
       {
         double
           degrees;
-  
+
         Image
           *rotate_image;
 
@@ -769,11 +789,11 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->chromaticity.white_point.x=0.3127f;
   image->chromaticity.white_point.y=0.3290f;
   image->gamma=1.000f/2.200f;
+  image->colorspace=YCCColorspace;
   if (LocaleCompare(image_info->magick,"PCDS") == 0)
-    (void) TransformRGBImage(image,sRGBColorspace);
-  else
-    (void) TransformRGBImage(image,YCCColorspace);
-  return(image);
+    image->colorspace=sRGBColorspace;
+  SetImageColorspace(image,RGBColorspace);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -787,7 +807,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPCDImage adds attributes for the PCD image format to
+%  RegisterPCDImage() adds attributes for the PCD image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -805,17 +825,17 @@ ModuleExport void RegisterPCDImage(void)
     *entry;
 
   entry=SetMagickInfo("PCD");
-  entry->decoder=(DecoderHandler) ReadPCDImage;
-  entry->encoder=(EncoderHandler) WritePCDImage;
-  entry->magick=(MagickHandler) IsPCD;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadPCDImage;
+  entry->encoder=(EncoderHandler *) WritePCDImage;
+  entry->magick=(MagickHandler *) IsPCD;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Photo CD");
   entry->module=AcquireString("PCD");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCDS");
-  entry->decoder=(DecoderHandler) ReadPCDImage;
-  entry->encoder=(EncoderHandler) WritePCDImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadPCDImage;
+  entry->encoder=(EncoderHandler *) WritePCDImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Photo CD");
   entry->module=AcquireString("PCD");
   (void) RegisterMagickInfo(entry);
@@ -832,7 +852,7 @@ ModuleExport void RegisterPCDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPCDImage removes format registrations made by the
+%  UnregisterPCDImage() removes format registrations made by the
 %  PCD module from the list of supported formats.
 %
 %  The format of the UnregisterPCDImage method is:
@@ -857,16 +877,16 @@ ModuleExport void UnregisterPCDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WritePCDImage writes an image in the Photo CD encoded image
+%  WritePCDImage() writes an image in the Photo CD encoded image
 %  format.
 %
 %  The format of the WritePCDImage method is:
 %
-%      unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePCDImage return True if the image is written.
+%    o status: Method WritePCDImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -877,7 +897,7 @@ ModuleExport void UnregisterPCDImage(void)
 %
 */
 
-static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
+static MagickBooleanType WritePCDTile(const ImageInfo *image_info,Image *image,
   char *page_geometry,char *tile_geometry)
 {
   GeometryInfo
@@ -890,6 +910,12 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickStatusType
+    flags;
+
   RectangleInfo
     geometry;
 
@@ -901,9 +927,6 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
     i,
     x;
 
-  unsigned int
-    flags;
-
   /*
     Scale image to tile size.
   */
@@ -917,11 +940,11 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
   tile_image=ResizeImage(image,geometry.width,geometry.height,TriangleFilter,
     1.0,&image->exception);
   if (tile_image == (Image *) NULL)
-    return(False);
+    return(MagickFalse);
   flags=ParseGeometry(page_geometry,&geometry_info);
   geometry.width=(unsigned long) geometry_info.rho;
   geometry.height=(unsigned long) geometry_info.sigma;
-  if (!(flags & SigmaValue))
+  if ((flags & SigmaValue) == 0)
     geometry.height=geometry.width;
   if ((tile_image->columns != geometry.width) ||
       (tile_image->rows != geometry.height))
@@ -939,16 +962,16 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
       border_info.height=(geometry.height-tile_image->rows+1) >> 1;
       bordered_image=BorderImage(tile_image,&border_info,&image->exception);
       if (bordered_image == (Image *) NULL)
-        return(False);
+        return(MagickFalse);
       DestroyImage(tile_image);
       tile_image=bordered_image;
     }
   (void) TransformImage(&tile_image,(char *) NULL,tile_geometry);
-  (void) TransformRGBImage(tile_image,YCCColorspace);
+  (void) SetImageColorspace(tile_image,YCCColorspace);
   downsample_image=ResizeImage(tile_image,tile_image->columns/2,
     tile_image->rows/2,TriangleFilter,1.0,&image->exception);
   if (downsample_image == (Image *) NULL)
-    return(False);
+    return(MagickFalse);
   /*
     Write tile to PCD file.
   */
@@ -982,31 +1005,36 @@ static unsigned int WritePCDTile(const ImageInfo *image_info,Image *image,
       q++;
     }
     if (QuantumTick(y,tile_image->rows))
-      if (!MagickMonitor(SaveImageTag,y,tile_image->rows,&image->exception))
-        break;
+      {
+        status=MagickMonitor(SaveImageTag,y,tile_image->rows,&image->exception);
+        if (status == MagickFalse)
+          break;
+      }
   }
   for (i=0; i < 0x800; i++)
     (void) WriteBlobByte(image,'\0');
   DestroyImage(downsample_image);
   DestroyImage(tile_image);
-  return(True);
+  return(MagickTrue);
 }
 
-static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
 {
   Image
     *pcd_image;
 
+  MagickBooleanType
+    status;
+
   register long
     i;
 
-  unsigned int
-    status;
-
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   pcd_image=image;
   if (image->columns < image->rows)
     {
@@ -1018,15 +1046,15 @@ static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
       */
       rotate_image=RotateImage(image,90.0,&image->exception);
       if (rotate_image == (Image *) NULL)
-        return(False);
+        return(MagickFalse);
       pcd_image=rotate_image;
     }
   /*
     Open output image file.
   */
   status=OpenBlob(image_info,pcd_image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",pcd_image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(pcd_image,RGBColorspace);
   /*
     Write PCD image header.
@@ -1051,7 +1079,7 @@ static unsigned int WritePCDImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(pcd_image,0x01);
   for (i=0; i < 1944; i++)
     (void) WriteBlobByte(pcd_image,'\0');
-  (void) WriteBlob(pcd_image,7,"PCD_IPI");
+  (void) WriteBlob(pcd_image,7,(unsigned char *) "PCD_IPI");
   (void) WriteBlobByte(pcd_image,0x06);
   for (i=0; i < 1530; i++)
     (void) WriteBlobByte(pcd_image,'\0');
