@@ -10,39 +10,26 @@
 %                            P       CCCC  LLLLL                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                        Write HP PCL Printer Format.                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,16 +42,22 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePCLImage(const ImageInfo *,Image *);
 
 /*
@@ -78,16 +71,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsPCL returns True if the image format type, identified by the
+%  IsPCL() returns MagickTrue if the image format type, identified by the
 %  magick string, is PCL.
 %
 %  The format of the IsPCL method is:
 %
-%      unsigned int IsPCL(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPCL(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPCL returns True if the image format type is PCL.
+%    o status:  Method IsPCL() returns MagickTrue if the image format type is PCL.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -96,15 +89,15 @@ static unsigned int
 %
 %
 */
-static unsigned int IsPCL(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPCL(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\033E\033",3) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\033E\033&",4) == 0)
-    return(False);
-  return(False);
+    return(MagickFalse);
+  return(MagickFalse);
 }
 
 /*
@@ -118,7 +111,7 @@ static unsigned int IsPCL(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPCLImage adds attributes for the PCL image format to
+%  RegisterPCLImage() adds attributes for the PCL image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -136,9 +129,9 @@ ModuleExport void RegisterPCLImage(void)
     *entry;
 
   entry=SetMagickInfo("PCL");
-  entry->encoder=(EncoderHandler) WritePCLImage;
-  entry->magick=(MagickHandler) IsPCL;
-  entry->adjoin=False;
+  entry->encoder=(EncoderHandler *) WritePCLImage;
+  entry->magick=(MagickHandler *) IsPCL;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Page Control Language");
   entry->module=AcquireString("PCL");
   (void) RegisterMagickInfo(entry);
@@ -155,7 +148,7 @@ ModuleExport void RegisterPCLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPCLImage removes format registrations made by the
+%  UnregisterPCLImage() removes format registrations made by the
 %  PCL module from the list of supported formats.
 %
 %  The format of the UnregisterPCLImage method is:
@@ -179,16 +172,16 @@ ModuleExport void UnregisterPCLImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WritePCLImage writes an image in the Page Control Language encoded
+%  WritePCLImage() writes an image in the Page Control Language encoded
 %  image format.
 %
 %  The format of the WritePCLImage method is:
 %
-%      unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePCLImage return True if the image is written.
+%    o status: Method WritePCLImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -199,7 +192,7 @@ ModuleExport void UnregisterPCLImage(void)
 %
 %
 */
-static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
 {
   char
     buffer[MaxTextExtent];
@@ -207,6 +200,9 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -219,9 +215,6 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
-  unsigned int
-    status;
-
   unsigned long
     density;
 
@@ -232,16 +225,17 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize the printer.
   */
   (void) WriteBlobString(image,"\033E");  /* printer reset */
   (void) WriteBlobString(image,"\033&l0E");  /* top margin 0 */
-  (void) WriteBlobString(image,buffer);
   density=75;
   if (image_info->density != (char *) NULL)
     {
@@ -249,10 +243,13 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
         geometry;
 
       (void) ParseGeometry(image_info->density,&geometry);
-      density=geometry.rho;
+      density=(unsigned long) geometry.rho;
     }
   if (IsMonochromeImage(image,&image->exception))
     {
+      MagickBooleanType
+        polarity;
+
       register unsigned char
         bit,
         byte;
@@ -260,11 +257,16 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
       /*
         Write PCL monochrome image.
       */
-      FormatString(buffer,"\033*t%ldR",density);  /* set resolution */
+      polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+      if (image->colors == 2)
+        polarity=PixelIntensityToQuantum(&image->colormap[0]) <
+          PixelIntensityToQuantum(&image->colormap[1]);
+      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image,"\033*r1A");  /* start graphics */
       (void) WriteBlobString(image,"\033*b0M");  /* no compression */
-      FormatString(buffer,"\033*b%luW",(image->columns+7)/8);
+      (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+        (image->columns+7)/8);
       (void) WriteBlobString(image,buffer);
       for (y=0; y < (long) image->rows; y++)
       {
@@ -277,7 +279,7 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
         for (x=0; x < (long) image->columns; x++)
         {
           byte<<=1;
-          byte|=indexes[x] ? 0x01 : 0x00;
+          byte|=indexes[x] == polarity ? 0x00 : 0x01;
           bit++;
           if (bit == 8)
             {
@@ -290,12 +292,16 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobByte(image,byte << (8-bit));
         if (y < (long) image->rows)
           {
-            FormatString(buffer,"\033*b%luW",(image->columns+7)/8);
+            (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+              (image->columns+7)/8);
             (void) WriteBlobString(image,buffer);
           }
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
       (void) WriteBlobString(image,"\033*rB");  /* end graphics */
     }
@@ -311,21 +317,22 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
         Allocate pixel buffers.
       */
       pixels=(unsigned char *)
-        AcquireMemory(3*image->columns*sizeof(unsigned char));
+        AcquireMagickMemory(3*image->columns*sizeof(unsigned char));
       if (pixels == (unsigned char *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       /*
         Write PCL color image.
       */
       (void) WriteBlobString(image,"\033*r3F");  /* set presentation mode */
-      FormatString(buffer,"\033*t%luR",density);  /* set resolution */
+      (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%luR",density);  /* set resolution */
       (void) WriteBlobString(image,buffer);
-      FormatString(buffer,"\033*r%luT",image->rows);
+      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luT",image->rows);
       (void) WriteBlobString(image,buffer);
-      FormatString(buffer,"\033*r%luS",image->columns);
+      (void) FormatMagickString(buffer,MaxTextExtent,"\033*r%luS",
+        image->columns);
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image,"\033*v6W");  /* set color mode */
-      (void) WriteBlob(image,6,color_mode);
+      (void) WriteBlob(image,6,(unsigned char *) color_mode);
       (void) WriteBlobString(image,"\033*r1A");  /* start raster graphics */
       (void) WriteBlobString(image,"\033*b0Y");  /* set y offset */
       (void) WriteBlobString(image,"\033*b0M");  /* no compression */
@@ -342,17 +349,21 @@ static unsigned int WritePCLImage(const ImageInfo *image_info,Image *image)
           *q++=ScaleQuantumToChar(p->blue);
           p++;
         }
-        FormatString(buffer,"\033*b%luW",3*image->columns);  /* send row */
+        (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
+          3*image->columns);  /* send row */
         (void) WriteBlobString(image,buffer);
         (void) WriteBlob(image,3*image->columns,pixels);
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
       (void) WriteBlobString(image,"\033*r0C");  /* end graphics */
-      LiberateMemory((void **) &pixels);
+      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     }
   (void) WriteBlobString(image,"\033E");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
