@@ -10,39 +10,26 @@
 %                            P       CCCC  X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                Read/Write ZSoft IBM PC Paintbrush Image Format.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Typedef declarations.
@@ -72,7 +64,7 @@ typedef struct _PCXInfo
     encoding,
     bits_per_pixel;
 
-  short int
+  short
     left,
     top,
     right,
@@ -84,7 +76,7 @@ typedef struct _PCXInfo
     reserved,
     planes;
 
-  short int
+  short
     bytes_per_line,
     palette_info;
 
@@ -95,7 +87,7 @@ typedef struct _PCXInfo
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePCXImage(const ImageInfo *,Image *);
 
 /*
@@ -109,16 +101,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsDCX returns True if the image format type, identified by the
+%  IsDCX() returns MagickTrue if the image format type, identified by the
 %  magick string, is DCX.
 %
 %  The format of the IsDCX method is:
 %
-%      unsigned int IsDCX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsDCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsDCX returns True if the image format type is DCX.
+%    o status:  Method IsDCX() returns MagickTrue if the image format type is DCX.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -127,13 +119,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsDCX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsDCX(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\261\150\336\72",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -147,16 +139,16 @@ static unsigned int IsDCX(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsPCX returns True if the image format type, identified by the
+%  IsPCX() returns MagickTrue if the image format type, identified by the
 %  magick string, is PCX.
 %
 %  The format of the IsPCX method is:
 %
-%      unsigned int IsPCX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPCX returns True if the image format type is PCX.
+%    o status:  Method IsPCX() returns MagickTrue if the image format type is PCX.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -165,15 +157,15 @@ static unsigned int IsDCX(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int IsPCX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\012\002",2) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\012\005",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -187,7 +179,7 @@ static unsigned int IsPCX(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPCXImage reads a ZSoft IBM PC Paintbrush file and returns it.
+%  ReadPCXImage() reads a ZSoft IBM PC Paintbrush file and returns it.
 %  It allocates the memory necessary for the new Image structure and returns
 %  a pointer to the new image.
 %
@@ -212,9 +204,6 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  ExtendedSignedIntegralType
-    *page_table;
-
   int
     bits,
     id,
@@ -223,6 +212,12 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    *page_table;
+
   PCXInfo
     pcx_info;
 
@@ -242,7 +237,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *p,
     *r;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
@@ -251,9 +246,6 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *pcx_pixels,
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
     pcx_packets;
 
@@ -262,16 +254,21 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Determine if this is a PCX file.
   */
-  page_table=(ExtendedSignedIntegralType *) NULL;
+  page_table=(MagickOffsetType *) NULL;
   if (LocaleCompare(image_info->magick,"DCX") == 0)
     {
       unsigned long
@@ -282,21 +279,22 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       magic=ReadBlobLSBLong(image);
       if (magic != 987654321)
-        ThrowReaderException(CorruptImageError,"NotADCXImageFile",image);
-      page_table=(ExtendedSignedIntegralType *)
-         AcquireMemory(1024*sizeof(ExtendedSignedIntegralType));
-      if (page_table == (ExtendedSignedIntegralType *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      page_table=(MagickOffsetType *)
+         AcquireMagickMemory(1024*sizeof(MagickOffsetType));
+      if (page_table == (MagickOffsetType *) NULL)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (id=0; id < 1024; id++)
       {
-        page_table[id]=(ExtendedSignedIntegralType) ReadBlobLSBLong(image);
+        page_table[id]=(MagickOffsetType) ReadBlobLSBLong(image);
         if (page_table[id] == 0)
           break;
       }
     }
-  if (page_table != (ExtendedSignedIntegralType *) NULL)
-    (void) SeekBlob(image,(ExtendedSignedIntegralType) page_table[0],SEEK_SET);
-  count=ReadBlob(image,1,(char *) &pcx_info.identifier);
+  if (page_table != (MagickOffsetType *) NULL)
+    (void) SeekBlob(image,(MagickOffsetType) page_table[0],SEEK_SET);
+  pcx_colormap=(unsigned char *) NULL;
+  count=ReadBlob(image,1,&pcx_info.identifier);
   for (id=1; id < 1024; id++)
   {
     /*
@@ -304,7 +302,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     pcx_info.version=ReadBlobByte(image);
     if ((count == 0) || (pcx_info.identifier != 0x0a))
-      ThrowReaderException(CorruptImageError,"NotAPCXImageFile",image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     pcx_info.encoding=ReadBlobByte(image);
     pcx_info.bits_per_pixel=ReadBlobByte(image);
     pcx_info.left=ReadBlobLSBShort(image);
@@ -323,18 +321,18 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->x_resolution=pcx_info.horizontal_resolution;
     image->y_resolution=pcx_info.vertical_resolution;
     image->colors=16;
-    pcx_colormap=(unsigned char *) AcquireMemory(3*256);
+    pcx_colormap=(unsigned char *) AcquireMagickMemory(3*256);
     if (pcx_colormap == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    (void) ReadBlob(image,3*image->colors,(char *) pcx_colormap);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    (void) ReadBlob(image,3*image->colors,pcx_colormap);
     pcx_info.reserved=ReadBlobByte(image);
     pcx_info.planes=ReadBlobByte(image);
     if ((pcx_info.bits_per_pixel != 8) || (pcx_info.planes == 1))
       if ((pcx_info.version == 3) || (pcx_info.version == 5) ||
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
         image->colors=1 << (pcx_info.bits_per_pixel*pcx_info.planes);
-    if (!AllocateImageColormap(image,image->colors))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    if (AllocateImageColormap(image,image->colors) == MagickFalse)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((pcx_info.bits_per_pixel >= 8) && (pcx_info.planes != 1))
       image->storage_class=DirectClass;
     p=pcx_colormap;
@@ -348,19 +346,19 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     pcx_info.palette_info=ReadBlobLSBShort(image);
     for (i=0; i < 58; i++)
       (void) ReadBlobByte(image);
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
       Read image data.
     */
     pcx_packets=image->rows*pcx_info.bytes_per_line*pcx_info.planes;
-    pcx_pixels=(unsigned char *) AcquireMemory(pcx_packets);
-    scanline=(unsigned char *) AcquireMemory(
-      Max(image->columns,pcx_info.bytes_per_line)*pcx_info.planes);
+    pcx_pixels=(unsigned char *) AcquireMagickMemory(pcx_packets);
+    scanline=(unsigned char *) AcquireMagickMemory(
+      Max((long) image->columns,pcx_info.bytes_per_line)*pcx_info.planes);
     if ((pcx_pixels == (unsigned char *) NULL) ||
         (scanline == (unsigned char *) NULL))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Uncompress image data.
     */
@@ -401,8 +399,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             Initialize image colormap.
           */
           if (image->colors > 256)
-            ThrowReaderException(CorruptImageError,"ColormapExceeds256Colors",
-              image);
+            ThrowReaderException(CorruptImageError,"ColormapExceeds256Colors");
           if ((pcx_info.bits_per_pixel*pcx_info.planes) == 1)
             {
               /*
@@ -422,7 +419,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   256 color images have their color map at the end of the file.
                 */
                 pcx_info.colormap_signature=ReadBlobByte(image);
-                (void) ReadBlob(image,3*image->colors,(char *) pcx_colormap);
+                (void) ReadBlob(image,3*image->colors,pcx_colormap);
                 p=pcx_colormap;
                 for (i=0; i < (long) image->colors; i++)
                 {
@@ -431,7 +428,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   image->colormap[i].blue=ScaleCharToQuantum(*p++);
                 }
             }
-          LiberateMemory((void **) &pcx_colormap);
+          pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
         }
     /*
       Convert PCX raster image to pixel packets.
@@ -551,7 +548,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
             case 8:
             {
-              (void) memcpy(r,p,image->columns);
+              (void) CopyMagickMemory(r,p,image->columns);
               break;
             }
             default:
@@ -570,25 +567,27 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->red=ScaleCharToQuantum(*r++);
             q->green=ScaleCharToQuantum(*r++);
             q->blue=ScaleCharToQuantum(*r++);
-            if (image->matte)
+            if (image->matte != MagickFalse)
               q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*r++));
           }
         q++;
       }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
     }
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
-    LiberateMemory((void **) &scanline);
-    LiberateMemory((void **) &pcx_pixels);
-    if (EOFBlob(image))
+      (void) SyncImage(image);
+    scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+    if (pcx_colormap != (unsigned char *) NULL)
+      pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
+    pcx_pixels=(unsigned char *) RelinquishMagickMemory(pcx_pixels);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -598,12 +597,12 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (page_table == (ExtendedSignedIntegralType *) NULL)
+    if (page_table == (MagickOffsetType *) NULL)
       break;
     if (page_table[id] == 0)
       break;
-    (void) SeekBlob(image,(ExtendedSignedIntegralType) page_table[id],SEEK_SET);
-    count=ReadBlob(image,1,(char *) &pcx_info.identifier);
+    (void) SeekBlob(image,(MagickOffsetType) page_table[id],SEEK_SET);
+    count=ReadBlob(image,1,&pcx_info.identifier);
     if ((count != 0) && (pcx_info.identifier == 0x0a))
       {
         /*
@@ -618,16 +617,14 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   }
-  if (page_table != (ExtendedSignedIntegralType *) NULL)
-    LiberateMemory((void **) &page_table);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  if (page_table != (MagickOffsetType *) NULL)
+    page_table=(MagickOffsetType *) RelinquishMagickMemory(page_table);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -641,7 +638,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPCXImage adds attributes for the PCX image format to
+%  RegisterPCXImage() adds attributes for the PCX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -659,19 +656,19 @@ ModuleExport void RegisterPCXImage(void)
     *entry;
 
   entry=SetMagickInfo("DCX");
-  entry->decoder=(DecoderHandler) ReadPCXImage;
-  entry->encoder=(EncoderHandler) WritePCXImage;
-  entry->seekable_stream=True;
-  entry->magick=(MagickHandler) IsDCX;
+  entry->decoder=(DecoderHandler *) ReadPCXImage;
+  entry->encoder=(EncoderHandler *) WritePCXImage;
+  entry->seekable_stream=MagickTrue;
+  entry->magick=(MagickHandler *) IsDCX;
   entry->description=AcquireString("ZSoft IBM PC multi-page Paintbrush");
   entry->module=AcquireString("PCX");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCX");
-  entry->decoder=(DecoderHandler) ReadPCXImage;
-  entry->encoder=(EncoderHandler) WritePCXImage;
-  entry->magick=(MagickHandler) IsPCX;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->decoder=(DecoderHandler *) ReadPCXImage;
+  entry->encoder=(EncoderHandler *) WritePCXImage;
+  entry->magick=(MagickHandler *) IsPCX;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("ZSoft IBM PC Paintbrush");
   entry->module=AcquireString("PCX");
   (void) RegisterMagickInfo(entry);
@@ -688,7 +685,7 @@ ModuleExport void RegisterPCXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPCXImage removes format registrations made by the
+%  UnregisterPCXImage() removes format registrations made by the
 %  PCX module from the list of supported formats.
 %
 %  The format of the UnregisterPCXImage method is:
@@ -713,16 +710,16 @@ ModuleExport void UnregisterPCXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WritePCXImage writes an image in the ZSoft IBM PC Paintbrush file
+%  WritePCXImage() writes an image in the ZSoft IBM PC Paintbrush file
 %  format.
 %
 %  The format of the WritePCXImage method is:
 %
-%      unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePCXImage return True if the image is written.
+%    o status: Method WritePCXImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -732,7 +729,7 @@ ModuleExport void UnregisterPCXImage(void)
 %
 %
 */
-static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
@@ -746,6 +743,13 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   register IndexPacket
     *indexes;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    *page_table,
+    scene;
+
   register long
     i,
     x;
@@ -763,15 +767,6 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     *pcx_pixels,
     previous;
 
-  unsigned int
-    status;
-
-  ExtendedSignedIntegralType
-    *page_table;
-
-  unsigned long
-    scene;
-
   /*
     Open output image file.
   */
@@ -779,28 +774,31 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
-  page_table=(ExtendedSignedIntegralType *) NULL;
-  if (image_info->adjoin)
+  page_table=(MagickOffsetType *) NULL;
+  if ((LocaleCompare(image_info->magick,"DCX") == 0) ||
+      ((image->next != (Image *) NULL) && image_info->adjoin))
     {
       /*
         Write the DCX page table.
       */
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
-      page_table=(ExtendedSignedIntegralType *)
-        AcquireMemory(1024*sizeof(ExtendedSignedIntegralType));
-      if (page_table == (ExtendedSignedIntegralType *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      page_table=(MagickOffsetType *)
+        AcquireMagickMemory(1024*sizeof(MagickOffsetType));
+      if (page_table == (MagickOffsetType *) NULL)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (scene=0; scene < 1024; scene++)
         (void) WriteBlobLSBLong(image,0x00000000L);
     }
   scene=0;
   do
   {
-    if (page_table != (ExtendedSignedIntegralType *) NULL)
+    if (page_table != (MagickOffsetType *) NULL)
       page_table[scene]=TellBlob(image);
     /*
       Initialize PCX raster file header.
@@ -833,7 +831,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     if (image->storage_class == DirectClass)
       {
         pcx_info.planes=3;
-        if (image->matte)
+        if (image->matte != MagickFalse)
           pcx_info.planes++;
       }
     pcx_info.bytes_per_line=(short)
@@ -847,19 +845,20 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,pcx_info.version);
     (void) WriteBlobByte(image,pcx_info.encoding);
     (void) WriteBlobByte(image,pcx_info.bits_per_pixel);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.left);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.top);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.right);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.bottom);
-    (void) WriteBlobLSBShort(image,(unsigned int)
+    (void) WriteBlobLSBShort(image,(unsigned short) pcx_info.left);
+    (void) WriteBlobLSBShort(image,(unsigned short) pcx_info.top);
+    (void) WriteBlobLSBShort(image,(unsigned short) pcx_info.right);
+    (void) WriteBlobLSBShort(image,(unsigned short) pcx_info.bottom);
+    (void) WriteBlobLSBShort(image,(unsigned short)
       pcx_info.horizontal_resolution);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.vertical_resolution);
+    (void) WriteBlobLSBShort(image,(unsigned short)
+      pcx_info.vertical_resolution);
     /*
       Dump colormap to file.
     */
-    pcx_colormap=(unsigned char *) AcquireMemory(3*256);
+    pcx_colormap=(unsigned char *) AcquireMagickMemory(3*256);
     if (pcx_colormap == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (i=0; i < (3*256); i++)
       pcx_colormap[i]=0;
     q=pcx_colormap;
@@ -870,17 +869,17 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
         *q++=ScaleQuantumToChar(image->colormap[i].green);
         *q++=ScaleQuantumToChar(image->colormap[i].blue);
       }
-    (void) WriteBlob(image,3*16,(char *) pcx_colormap);
+    (void) WriteBlob(image,3*16,pcx_colormap);
     (void) WriteBlobByte(image,pcx_info.reserved);
     (void) WriteBlobByte(image,pcx_info.planes);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.bytes_per_line);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.palette_info);
+    (void) WriteBlobLSBShort(image,pcx_info.bytes_per_line);
+    (void) WriteBlobLSBShort(image,pcx_info.palette_info);
     for (i=0; i < 58; i++)
       (void) WriteBlobByte(image,'\0');
     length=image->rows*pcx_info.bytes_per_line*pcx_info.planes;
-    pcx_pixels=(unsigned char *) AcquireMemory(length);
+    pcx_pixels=(unsigned char *) AcquireMagickMemory(length);
     if (pcx_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     q=pcx_pixels;
     if (image->storage_class == DirectClass)
       {
@@ -924,9 +923,13 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
               p++;
             }
           }
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-              break;
+          if (QuantumTick(y,image->rows) != 0)
+            {
+              status=MagickMonitor(SaveImageTag,y,image->rows,
+                &image->exception);
+              if (status == MagickFalse)
+                break;
+            }
         }
       }
     else
@@ -941,9 +944,13 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           for (x=0; x < (long) image->columns; x++)
             *q++=indexes[x];
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       else
         {
@@ -985,9 +992,13 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             if (bit != 0)
               *q++=byte << (8-bit);
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                  break;
+              if (QuantumTick(y,image->rows) != 0)
+                {
+                  status=MagickMonitor(SaveImageTag,y,image->rows,
+                    &image->exception);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         }
     /*
@@ -1023,29 +1034,29 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlobByte(image,count);
           }
         (void) WriteBlobByte(image,previous);
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
     }
     (void) WriteBlobByte(image,pcx_info.colormap_signature);
-    (void) WriteBlob(image,3*256,(char *) pcx_colormap);
-    LiberateMemory((void **) &pcx_pixels);
-    LiberateMemory((void **) &pcx_colormap);
+    (void) WriteBlob(image,3*256,pcx_colormap);
+    pcx_pixels=(unsigned char *) RelinquishMagickMemory(pcx_pixels);
+    pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
     if (scene >= 1023)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
-  if (page_table != (ExtendedSignedIntegralType *) NULL)
+  } while (image_info->adjoin != MagickFalse);
+  if (page_table != (MagickOffsetType *) NULL)
     {
       /*
         Write the DCX page table.
@@ -1055,10 +1066,12 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
       for (i=0; i <= (long) scene; i++)
         (void) WriteBlobLSBLong(image,(unsigned long) page_table[i]);
-      LiberateMemory((void **) &page_table);
+      page_table=(MagickOffsetType *)
+        RelinquishMagickMemory(page_table);
     }
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToWriteFile",image);
+  if (status == MagickFalse)
+    (void) ThrowMagickException(&image->exception,GetMagickModule(),
+      FileOpenError,"UnableToWriteFile",image->filename,strerror(errno));
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
