@@ -10,39 +10,26 @@
 %                            P      DDDD   BBBB                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%               Read/Write Palm Database ImageViewer Image Format.            %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,12 +42,17 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Typedef declarations.
@@ -121,7 +113,7 @@ typedef struct _PDBImage
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePDBImage(const ImageInfo *,Image *);
 
 /*
@@ -135,18 +127,18 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method DecodeImage unpacks the packed image pixels into runlength-encoded
+%  DecodeImage unpacks the packed image pixels into runlength-encoded
 %  pixel packets.
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(Image *image,unsigned char *pixels,
+%      MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
 %        const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the pixels are
-%      uncompressed without error, otherwise False.
+%    o status:  Method DecodeImage returns MagickTrue if all the pixels are
+%      uncompressed without error, otherwise MagickFalse.
 %
 %    o image: The address of a structure of type Image.
 %
@@ -155,11 +147,10 @@ static unsigned int
 %
 %
 */
-static unsigned int DecodeImage(Image *image,unsigned char *pixels,
+static MagickBooleanType DecodeImage(Image *image,unsigned char *pixels,
   const size_t length)
 {
-  int
-    count,
+  long
     pixel;
 
   register long
@@ -168,6 +159,9 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
   register unsigned char
     *p;
 
+  ssize_t
+    count;
+
   p=pixels;
   while (p < (pixels+length))
   {
@@ -184,7 +178,7 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
     for (i=0; i < count; i++)
       *p++=(unsigned char) pixel;
   }
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -198,16 +192,16 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsPDB returns True if the image format type, identified by the
+%  IsPDB() returns MagickTrue if the image format type, identified by the
 %  magick string, is PDB.
 %
 %  The format of the ReadPDBImage method is:
 %
-%      unsigned int IsPDB(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPDB returns True if the image format type is PDB.
+%    o status:  Method IsPDB() returns MagickTrue if the image format type is PDB.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -216,13 +210,13 @@ static unsigned int DecodeImage(Image *image,unsigned char *pixels,
 %
 %
 */
-static unsigned int IsPDB(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPDB(const unsigned char *magick,const size_t length)
 {
   if (length < 68)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+60,"vIMGView",8) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -236,7 +230,7 @@ static unsigned int IsPDB(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPDBImage reads an Pilot image file and returns it.  It
+%  ReadPDBImage() reads an Pilot image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -272,6 +266,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     offset,
     y;
 
+  MagickBooleanType
+    status;
+
   PDBImage
     pdb_image;
 
@@ -296,9 +293,6 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     packets;
@@ -308,16 +302,21 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Determine if this is a PDB image file.
   */
-  count=ReadBlob(image,32,pdb_info.name);
+  count=ReadBlob(image,32,(unsigned char *) pdb_info.name);
   pdb_info.attributes=ReadBlobMSBShort(image);
   pdb_info.version=ReadBlobMSBShort(image);
   pdb_info.create_time=ReadBlobMSBLong(image);
@@ -326,25 +325,25 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   pdb_info.modify_number=ReadBlobMSBLong(image);
   pdb_info.application_info=ReadBlobMSBLong(image);
   pdb_info.sort_info=ReadBlobMSBLong(image);
-  (void) ReadBlob(image,4,pdb_info.type);
-  (void) ReadBlob(image,4,pdb_info.id);
+  (void) ReadBlob(image,4,(unsigned char *) pdb_info.type);
+  (void) ReadBlob(image,4,(unsigned char *) pdb_info.id);
   if ((count == 0) || (memcmp(pdb_info.type,"vIMG",4) != 0) ||
       (memcmp(pdb_info.id,"View",4) != 0))
-    ThrowReaderException(CoderError,"NotASupportedImageFile",image);
+    ThrowReaderException(CoderError,"ImproperImageHeader");
   pdb_info.seed=ReadBlobMSBLong(image);
   pdb_info.next_record=ReadBlobMSBLong(image);
   pdb_info.number_records=ReadBlobMSBShort(image);
   if (pdb_info.next_record != 0)
-    ThrowReaderException(CoderError,"MultipleRecordListNotSupported",image);
+    ThrowReaderException(CoderError,"MultipleRecordListNotSupported");
   /*
     Read record header.
   */
   offset=(long) ReadBlobMSBLong(image);
-  (void) ReadBlob(image,3,tag);
+  (void) ReadBlob(image,3,(unsigned char *) tag);
   record_type=ReadBlobByte(image);
   if (((record_type != 0x00) && (record_type != 0x01)) ||
       (memcmp(tag,"\x40\x6f\x80",3) != 0))
-    ThrowReaderException(CorruptImageError,"CorruptPDBImage",image);
+    ThrowReaderException(CorruptImageError,"CorruptImage");
   if ((offset-TellBlob(image)) == 6)
     {
       (void) ReadBlobByte(image);
@@ -353,11 +352,11 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (pdb_info.number_records > 1)
     {
       offset=(long) ReadBlobMSBLong(image);
-      (void) ReadBlob(image,3,tag);
+      (void) ReadBlob(image,3,(unsigned char *) tag);
       record_type=ReadBlobByte(image);
       if (((record_type != 0x00) && (record_type != 0x01)) ||
           (memcmp(tag,"\x40\x6f\x80",3) != 0))
-        ThrowReaderException(CorruptImageError,"CorruptPDBImage",image);
+        ThrowReaderException(CorruptImageError,"CorruptImage");
       if ((offset-TellBlob(image)) == 6)
         {
           (void) ReadBlobByte(image);
@@ -367,7 +366,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read image header.
   */
-  (void) ReadBlob(image,32,pdb_image.name);
+  (void) ReadBlob(image,32,(unsigned char *) pdb_image.name);
   pdb_image.version=ReadBlobByte(image);
   pdb_image.type=ReadBlobByte(image);
   pdb_image.reserved_1=ReadBlobMSBLong(image);
@@ -387,23 +386,23 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->depth=8;
   image->storage_class=PseudoClass;
   bits_per_pixel=pdb_image.type == 0 ? 2 : pdb_image.type == 2 ? 4 : 1;
-  if (!AllocateImageColormap(image,1 << bits_per_pixel))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  if (image_info->ping)
+  if (AllocateImageColormap(image,1 << bits_per_pixel) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   packets=(bits_per_pixel*image->columns/8)*image->rows;
-  pixels=(unsigned char *) AcquireMemory(packets+256);
+  pixels=(unsigned char *) AcquireMagickMemory(packets+256);
   if (pixels == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   switch (pdb_image.version)
   {
     case 0:
     {
       image->compression=NoCompression;
-      (void) ReadBlob(image,packets,(char *) pixels);
+      (void) ReadBlob(image,packets,pixels);
       break;
     }
     case 1:
@@ -413,8 +412,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
     default:
-      ThrowReaderException(CorruptImageWarning,"UnrecognizedCompressionType",
-        image)
+      ThrowReaderException(CorruptImageError,"UnrecognizedCompressionType");
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -443,10 +441,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           p++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
       }
       break;
@@ -478,10 +476,10 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *q++=image->colormap[index];
           p++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
       }
       break;
@@ -507,20 +505,20 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           *q++=image->colormap[index];
           p++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (QuantumTick(y,image->rows) != 0)
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
             break;
       }
       break;
     }
     default:
-      ThrowReaderException(CorruptImageError,"NotAPDBImageFile",image)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   }
-  LiberateMemory((void **) &pixels);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   if ((offset-TellBlob(image)) == 0)
     {
@@ -533,7 +531,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       register char
         *p;
 
-      unsigned int
+      unsigned long
         length;
 
       /*
@@ -541,7 +539,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       c=ReadBlobByte(image);
       length=MaxTextExtent;
-      comment=(char *) AcquireMemory(length+1);
+      comment=(char *) AcquireMagickMemory(length+1);
       p=comment;
       if (comment != (char *) NULL)
         for ( ; c != EOF; p++)
@@ -550,7 +548,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               length<<=1;
               length+=MaxTextExtent;
-              ReacquireMemory((void **) &comment,length+1);
+              comment=(char *) ResizeMagickMemory(comment,length+1);
               if (comment == (char *) NULL)
                 break;
               p=comment+strlen(comment);
@@ -560,12 +558,12 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           c=ReadBlobByte(image);
         }
       if (comment == (char *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       (void) SetImageAttribute(image,"comment",comment);
-      LiberateMemory((void **) &comment);
+      comment=(char *) RelinquishMagickMemory(comment);
     }
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -579,7 +577,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPDBImage adds attributes for the PDB image format to
+%  RegisterPDBImage() adds attributes for the PDB image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -597,9 +595,9 @@ ModuleExport void RegisterPDBImage(void)
     *entry;
 
   entry=SetMagickInfo("PDB");
-  entry->decoder=(DecoderHandler) ReadPDBImage;
-  entry->encoder=(EncoderHandler) WritePDBImage;
-  entry->magick=(MagickHandler) IsPDB;
+  entry->decoder=(DecoderHandler *) ReadPDBImage;
+  entry->encoder=(EncoderHandler *) WritePDBImage;
+  entry->magick=(MagickHandler *) IsPDB;
   entry->description=AcquireString("Palm Database ImageViewer Format");
   entry->module=AcquireString("PDB");
   (void) RegisterMagickInfo(entry);
@@ -616,7 +614,7 @@ ModuleExport void RegisterPDBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPDBImage removes format registrations made by the
+%  UnregisterPDBImage() removes format registrations made by the
 %  PDB module from the list of supported formats.
 %
 %  The format of the UnregisterPDBImage method is:
@@ -640,15 +638,15 @@ ModuleExport void UnregisterPDBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WritePDBImage writes an image
+%  WritePDBImage() writes an image
 %
 %  The format of the WritePDBImage method is:
 %
-%      unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePDBImage return True if the image is written.
+%    o status: Method WritePDBImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -660,11 +658,11 @@ ModuleExport void UnregisterPDBImage(void)
 */
 
 static unsigned char *EncodeRLE(unsigned char *destination,
-  unsigned char *source,unsigned int literal,unsigned int repeat)
+  unsigned char *source,MagickBooleanType literal,MagickBooleanType repeat)
 {
   if (literal > 0)
     *destination++=literal-1;
-  memcpy(destination,source,literal);
+  CopyMagickMemory(destination,source,literal);
   destination+=literal;
   if (repeat > 0)
     {
@@ -674,7 +672,7 @@ static unsigned char *EncodeRLE(unsigned char *destination,
   return(destination);
 }
 
-static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 {
   int
     bits;
@@ -682,6 +680,9 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PDBImage
     pdb_image;
 
@@ -691,19 +692,18 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   register long
     x;
 
+  size_t
+    packet_size;
+
   unsigned char
     *buffer,
     *p,
     *q,
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     literal,
-    packet_size,
     packets,
     repeat;
 
@@ -717,17 +717,19 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
-  bits_per_pixel=image_info->depth;
+  bits_per_pixel=image->depth;
   if (GetImageType(image,&image->exception) == BilevelType)
     bits_per_pixel=1;
   if ((bits_per_pixel != 1) && (bits_per_pixel != 2))
     bits_per_pixel=4;
-  memset(pdb_info.name,0,32);
-  (void) strncpy(pdb_info.name,image_info->filename,32);
+  (void) ResetMagickMemory(pdb_info.name,0,32);
+  (void) CopyMagickString(pdb_info.name,image_info->filename,32);
   pdb_info.attributes=0;
   pdb_info.version=0;
   pdb_info.create_time=time(NULL);
@@ -736,13 +738,13 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   pdb_info.modify_number=0;
   pdb_info.application_info=0;
   pdb_info.sort_info=0;
-  memcpy(pdb_info.type,"vIMG",4);
-  memcpy(pdb_info.id,"View",4);
+  CopyMagickMemory(pdb_info.type,"vIMG",4);
+  CopyMagickMemory(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
   comment=GetImageAttribute(image,"comment");
   pdb_info.number_records=(comment == (ImageAttribute *) NULL ? 1 : 2);
-  (void) WriteBlob(image,32,pdb_info.name);
+  (void) WriteBlob(image,32,(unsigned char *) pdb_info.name);
   (void) WriteBlobMSBShort(image,pdb_info.attributes);
   (void) WriteBlobMSBShort(image,pdb_info.version);
   (void) WriteBlobMSBLong(image,pdb_info.create_time);
@@ -751,12 +753,12 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBLong(image,pdb_info.modify_number);
   (void) WriteBlobMSBLong(image,pdb_info.application_info);
   (void) WriteBlobMSBLong(image,pdb_info.sort_info);
-  (void) WriteBlob(image,4,pdb_info.type);
-  (void) WriteBlob(image,4,pdb_info.id);
+  (void) WriteBlob(image,4,(unsigned char *) pdb_info.type);
+  (void) WriteBlob(image,4,(unsigned char *) pdb_info.id);
   (void) WriteBlobMSBLong(image,pdb_info.seed);
   (void) WriteBlobMSBLong(image,pdb_info.next_record);
   (void) WriteBlobMSBShort(image,pdb_info.number_records);
-  (void) strncpy(pdb_image.name,pdb_info.name,32);
+  (void) CopyMagickString(pdb_image.name,pdb_info.name,32);
   pdb_image.version=1;  /* RLE Compressed */
   switch (bits_per_pixel)
   {
@@ -776,16 +778,16 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_image.width=(short) (16*(image->columns/16+1));
   pdb_image.height=(short) image->rows;
   packets=(bits_per_pixel*image->columns/8)*image->rows;
-  p=(unsigned char *) AcquireMemory(packets+packets);
+  p=(unsigned char *) AcquireMagickMemory(2*packets);
   if (p == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
-  buffer=(unsigned char *) AcquireMemory(256);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  buffer=(unsigned char *) AcquireMagickMemory(256);
   if (buffer == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
-  packet_size=image->depth > 8 ? 2: 1;
-  scanline=(unsigned char *) AcquireMemory(packet_size*image->columns);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  packet_size=(size_t) (image->depth > 8 ? 2: 1);
+  scanline=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
@@ -797,7 +799,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   buffer[0]=0x00;
   for (y=0; y < (long) image->rows; y++)
   {
-    if (!AcquireImagePixels(image,0,y,image->columns,1,&image->exception))
+    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == MagickFalse)
       break;
     (void) PopImagePixels(image,GrayQuantum,scanline);
     for (x=0; x < pdb_image.width; x++)
@@ -839,7 +841,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
               if (0x7f < literal)
                 {
                   q=EncodeRLE(q,buffer,(literal < 0x80 ? literal : 0x80),0);
-                  CloneMemory(buffer,buffer+literal+repeat,0x80);
+                  CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
                   literal-=0x80;
                 }
             }
@@ -847,13 +849,13 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
         buffer[literal+repeat]=0x00;
       }
     }
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
         break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
-  LiberateMemory((void **) &scanline);
-  LiberateMemory((void **) &buffer);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   /*
     Write the Image record header.
   */
@@ -877,7 +879,7 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   /*
     Write the Image data.
   */
-  (void) WriteBlob(image,32,pdb_image.name);
+  (void) WriteBlob(image,32,(unsigned char *) pdb_image.name);
   (void) WriteBlobByte(image,pdb_image.version);
   (void) WriteBlobByte(image,pdb_image.type);
   (void) WriteBlobMSBLong(image,pdb_image.reserved_1);
@@ -890,9 +892,9 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,pdb_image.width);
   (void) WriteBlobMSBShort(image,pdb_image.height);
   (void) WriteBlob(image,q-p,p);
-  LiberateMemory((void **) &p);
+  p=(unsigned char *) RelinquishMagickMemory(p);
   if (pdb_info.number_records > 1)
     WriteBlobString(image,comment->value);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
