@@ -10,39 +10,26 @@
 %                            P      IIIII  X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read Alias/Wavefront RLE Image Format.                   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,11 +42,13 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,7 +61,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPIXImage reads a Alias/Wavefront RLE image file and returns it.
+%  ReadPIXImage() reads a Alias/Wavefront RLE image file and returns it.
 %  It allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -103,6 +92,9 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   Quantum
     blue,
     green,
@@ -117,13 +109,12 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
+  size_t
+    length;
 
   unsigned long
     bits_per_pixel,
     height,
-    length,
     width;
 
   /*
@@ -131,12 +122,17 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read PIX image.
   */
@@ -147,18 +143,18 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   bits_per_pixel=ReadBlobMSBShort(image);
   if ((width == (unsigned long) ~0) || (height == (unsigned long) ~0) ||
       ((bits_per_pixel != 8) && (bits_per_pixel != 24)))
-    ThrowReaderException(CorruptImageError,"NotAPIXImageFile",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   do
   {
     /*
       Initialize image structure.
     */
-    image->columns= width;
-    image->rows= height;
+    image->columns=width;
+    image->rows=height;
     if (bits_per_pixel == 8)
-      if (!AllocateImageColormap(image,256))
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    if (image_info->ping && (image_info->number_scenes != 0))
+      if (AllocateImageColormap(image,256) == MagickFalse)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -179,7 +175,7 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         if (length == 0)
           {
-            length=ReadBlobByte(image);
+            length=(size_t) ReadBlobByte(image);
             if (bits_per_pixel == 8)
               index=ScaleCharToQuantum(ReadBlobByte(image));
             else
@@ -197,17 +193,17 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         length--;
         q++;
       }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
           break;
     }
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
-    if (EOFBlob(image))
+      (void) SyncImage(image);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -222,9 +218,10 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ReadBlobMSBShort(image);
     (void) ReadBlobMSBShort(image);
     bits_per_pixel=ReadBlobMSBShort(image);
-    status=(width != (unsigned long) ~0) && (height == (unsigned long) ~0) &&
-      ((bits_per_pixel == 8) || (bits_per_pixel == 24));
-    if (status == True)
+    status=(MagickBooleanType) ((width != (unsigned long) ~0) &&
+      (height == (unsigned long) ~0) && ((bits_per_pixel == 8) ||
+      (bits_per_pixel == 24)));
+    if (status == MagickTrue)
       {
         /*
           Allocate next image structure.
@@ -238,14 +235,12 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
-  } while (status == True);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  } while (status == MagickTrue);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -259,7 +254,7 @@ static Image *ReadPIXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPIXImage adds attributes for the PIX image format to
+%  RegisterPIXImage() adds attributes for the PIX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -277,7 +272,7 @@ ModuleExport void RegisterPIXImage(void)
     *entry;
 
   entry=SetMagickInfo("PIX");
-  entry->decoder=(DecoderHandler) ReadPIXImage;
+  entry->decoder=(DecoderHandler *) ReadPIXImage;
   entry->description=AcquireString("Alias/Wavefront RLE image format");
   entry->module=AcquireString("PIX");
   (void) RegisterMagickInfo(entry);
@@ -294,7 +289,7 @@ ModuleExport void RegisterPIXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPIXImage removes format registrations made by the
+%  UnregisterPIXImage() removes format registrations made by the
 %  PIX module from the list of supported formats.
 %
 %  The format of the UnregisterPIXImage method is:
