@@ -10,39 +10,26 @@
 %                            P       W W   P                                  %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read Seattle Film Works Image Format.                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,13 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/resource.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,16 +63,16 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsPWP returns True if the image format type, identified by the
+%  IsPWP() returns MagickTrue if the image format type, identified by the
 %  magick string, is PWP.
 %
 %  The format of the IsPWP method is:
 %
-%      unsigned int IsPWP(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPWP(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPWP returns True if the image format type is PWP.
+%    o status:  Method IsPWP() returns MagickTrue if the image format type is PWP.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -91,13 +81,13 @@
 %
 %
 */
-static unsigned int IsPWP(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPWP(const unsigned char *magick,const size_t length)
 {
   if (length < 5)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"SFW95",5) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -111,7 +101,7 @@ static unsigned int IsPWP(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPWPImage reads a Seattle Film Works multi-image file and returns
+%  ReadPWPImage() reads a Seattle Film Works multi-image file and returns
 %  it.  It allocates the memory necessary for the new Image structure and
 %  returns a pointer to the new image.
 %
@@ -142,12 +132,15 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *pwp_image;
 
   ImageInfo
-    *clone_info;
+    *read_info;
 
   int
     c,
     unique_file;
 
+  MagickBooleanType
+    status;
+
   MonitorHandler
     handler;
 
@@ -157,15 +150,12 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register long
     i;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
     magick[MaxTextExtent];
 
-  unsigned int
-    status;
-
   unsigned long
     filesize;
 
@@ -174,20 +164,22 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   pwp_image=AllocateImage(image_info);
+  image=pwp_image;
   status=OpenBlob(image_info,pwp_image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",pwp_image);
-  count=ReadBlob(pwp_image,5,(char *) magick);
+  if (status == MagickFalse)
+    return(MagickFalse);
+  count=ReadBlob(pwp_image,5,magick);
   if ((count == 0) || (LocaleNCompare((char *) magick,"SFW95",5) != 0))
-    ThrowReaderException(CorruptImageError,"NotAPWPImageFile",pwp_image);
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
-  unique_file=AcquireUniqueFileResource(clone_info->filename);
-  image=(Image *) NULL;
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  read_info=CloneImageInfo(image_info);
+  read_info->blob=(void *) NULL;
+  read_info->length=0;
+  unique_file=AcquireUniqueFileResource(read_info->filename);
   for ( ; ; )
   {
     for (c=ReadBlobByte(pwp_image); c != EOF; c=ReadBlobByte(pwp_image))
@@ -201,7 +193,7 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (c == EOF)
       break;
     if (LocaleNCompare((char *) (magick+12),"SFW94A",6) != 0)
-      ThrowReaderException(CorruptImageError,"NotAPWPImageFile",pwp_image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     /*
       Dump SFW image to a temporary file.
     */
@@ -209,7 +201,12 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (unique_file != -1)
       file=fdopen(unique_file,"wb");
     if ((unique_file == -1) || (file == (FILE *) NULL))
-      ThrowReaderException(FileOpenError,"UnableToWriteFile",image);
+      {
+        ThrowFileException(exception,FileOpenError,"UnableToWriteFile",
+          image->filename);
+        DestroyImageList(image);
+        return(MagickFalse);
+      }
     (void) fwrite("SFW94A",1,6,file);
     filesize=65535L*magick[2]+256L*magick[1]+magick[0];
     for (i=0; i < (long) filesize; i++)
@@ -219,11 +216,12 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     (void) fclose(file);
     handler=SetMonitorHandler((MonitorHandler) NULL);
-    next_image=ReadImage(clone_info,exception);
+    next_image=ReadImage(read_info,exception);
     (void) SetMonitorHandler(handler);
     if (next_image == (Image *) NULL)
       break;
-    FormatString(next_image->filename,"slide_%02ld.sfw",next_image->scene);
+    (void) FormatMagickString(next_image->filename,MaxTextExtent,
+      "slide_%02ld.sfw",next_image->scene);
     if (image == (Image *) NULL)
       image=next_image;
     else
@@ -241,18 +239,18 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
     status=MagickMonitor(LoadImagesTag,TellBlob(pwp_image),GetBlobSize(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
   }
-  (void) LiberateUniqueFileResource(clone_info->filename);
-  DestroyImageInfo(clone_info);
+  (void) RelinquishUniqueFileResource(read_info->filename);
+  DestroyImageInfo(read_info);
   CloseBlob(pwp_image);
   DestroyImage(pwp_image);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
-      image->filename);
+  if (EOFBlob(image) != MagickFalse)
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+      "UnexpectedEndOfFile",image->filename,strerror(errno));
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -266,7 +264,7 @@ static Image *ReadPWPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterPWPImage adds attributes for the PWP image format to
+%  RegisterPWPImage() adds attributes for the PWP image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -284,8 +282,8 @@ ModuleExport void RegisterPWPImage(void)
     *entry;
 
   entry=SetMagickInfo("PWP");
-  entry->decoder=(DecoderHandler) ReadPWPImage;
-  entry->magick=(MagickHandler) IsPWP;
+  entry->decoder=(DecoderHandler *) ReadPWPImage;
+  entry->magick=(MagickHandler *) IsPWP;
   entry->description=AcquireString("Seattle Film Works");
   entry->module=AcquireString("PWP");
   (void) RegisterMagickInfo(entry);
@@ -302,7 +300,7 @@ ModuleExport void RegisterPWPImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterPWPImage removes format registrations made by the
+%  UnregisterPWPImage() removes format registrations made by the
 %  PWP module from the list of supported formats.
 %
 %  The format of the UnregisterPWPImage method is:
