@@ -10,39 +10,26 @@
 %                            R  R    GGG   BBBB                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write Raw RGB Image Format.                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,22 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteRGBImage(const ImageInfo *,Image *);
 
 /*
@@ -78,7 +70,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadRGBImage reads an image of raw red, green, and blue samples and
+%  ReadRGBImage() reads an image of raw red, green, and blue samples and
 %  returns it.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -106,6 +98,9 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     i,
     x;
@@ -113,49 +108,54 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  size_t
+  ssize_t
     count;
 
+  size_t
+    packet_size;
+
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
-    packet_size;
+    width;
 
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open image file.
       */
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-      if (status == False)
-        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        {
+          DestroyImageList(image);
+          return((Image *) NULL);
+        }
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
   /*
     Allocate memory for a scanline.
   */
-  packet_size=image->depth > 8 ? 6 : 3;
+  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
     {
-      image->matte=True;
-      packet_size=image->depth > 8 ? 8 : 4;
+      image->matte=MagickTrue;
+      packet_size=(size_t) (image->depth > 8 ? 8 : 4);
     }
   scanline=(unsigned char *)
-    AcquireMemory(packet_size*image->extract_info.width);
+    AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -172,7 +172,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Convert raster image to pixel packets.
     */
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     switch (image_info->interlace)
@@ -188,23 +188,24 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          if (!image->matte)
+          if (image->matte == MagickFalse)
             (void) PushImagePixels(image,RGBQuantum,scanline+x);
           else
             (void) PushImagePixels(image,RGBAQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
@@ -213,13 +214,15 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Line interlacing:  RRR...GGG...BBB...RRR...GGG...BBB...
         */
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          (void) ReadBlob(image,packet_size*image->extract_info.width,
+            scanline);
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
@@ -228,21 +231,21 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           (void) PushImagePixels(image,GreenQuantum,scanline+x);
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
           (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (image->matte)
+          if (image->matte != MagickFalse)
             {
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
               (void) PushImagePixels(image,AlphaQuantum,scanline+x);
             }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
@@ -259,40 +262,47 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             AppendImageFormat("R",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
-        packet_size=image->depth > 8 ? 2 : 1;
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         i=0;
-        span=image->rows*(image->matte ? 4 : 3);
+        span=image->rows*(image->matte != MagickFalse ? 4 : 3);
         for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,RedQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
@@ -303,24 +313,27 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,GreenQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
             status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-            if (status == False)
-              ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
         for (y=0; y < image->extract_info.y; y++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
@@ -331,18 +344,18 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(i,span))
-              if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
+            if (QuantumTick(i,span) != 0)
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == MagickFalse)
                 break;
           i++;
         }
-        count=image->extract_info.height-image->rows-image->extract_info.y;
-        for (i=0; i < (long) count; i++)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
           (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
-        if (image->matte)
+        if (image->matte != MagickFalse)
           {
             /*
               Read matte channel.
@@ -352,8 +365,11 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-                if (status == False)
-                  ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+                if (status == MagickFalse)
+                  {
+                    DestroyImageList(image);
+                    return((Image *) NULL);
+                  }
               }
             for (y=0; y < image->extract_info.y; y++)
               (void) ReadBlob(image,packet_size*image->extract_info.width,
@@ -366,27 +382,31 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (q == (PixelPacket *) NULL)
                 break;
               (void) PushImagePixels(image,AlphaQuantum,scanline+x);
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(i,span))
-                  if (!MagickMonitor(LoadImageTag,i,span,&image->exception))
-                    break;
+                if (QuantumTick(i,span) != 0)
+                  {
+                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    if (status == MagickFalse)
+                      break;
+                  }
               i++;
             }
-            count=image->extract_info.height-image->rows-image->extract_info.y;
-            for (i=0; i < (long) count; i++)
+            width=image->extract_info.height-image->rows-image->extract_info.y;
+            for (i=0; i < (long) width; i++)
               (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
         break;
       }
     }
-    if (EOFBlob(image))
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -413,15 +433,13 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (count != 0);
-  LiberateMemory((void **) &scanline);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -435,7 +453,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterRGBImage adds attributes for the RGB image format to
+%  RegisterRGBImage() adds attributes for the RGB image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -453,16 +471,16 @@ ModuleExport void RegisterRGBImage(void)
     *entry;
 
   entry=SetMagickInfo("RGB");
-  entry->decoder=(DecoderHandler) ReadRGBImage;
-  entry->encoder=(EncoderHandler) WriteRGBImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadRGBImage;
+  entry->encoder=(EncoderHandler *) WriteRGBImage;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Raw red, green, and blue samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBA");
-  entry->decoder=(DecoderHandler) ReadRGBImage;
-  entry->encoder=(EncoderHandler) WriteRGBImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadRGBImage;
+  entry->encoder=(EncoderHandler *) WriteRGBImage;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Raw red, green, blue, and matte samples");
   entry->module=AcquireString("RGB");
   (void) RegisterMagickInfo(entry);
@@ -479,7 +497,7 @@ ModuleExport void RegisterRGBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterRGBImage removes format registrations made by the
+%  UnregisterRGBImage() removes format registrations made by the
 %  RGB module from the list of supported formats.
 %
 %  The format of the UnregisterRGBImage method is:
@@ -504,16 +522,16 @@ ModuleExport void UnregisterRGBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteRGBImage writes an image to a file in red, green, and blue
+%  WriteRGBImage() writes an image to a file in red, green, and blue
 %  rasterfile format.
 %
 %  The format of the WriteRGBImage method is:
 %
-%      unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteRGBImage return True if the image is written.
+%    o status: Method WriteRGBImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -523,24 +541,26 @@ ModuleExport void UnregisterRGBImage(void)
 %
 %
 */
-static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteRGBImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
+  size_t
+    packet_size;
+
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    packet_size,
-    scene;
-
   /*
     Allocate memory for pixels.
   */
@@ -548,20 +568,22 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  packet_size=image->depth > 8 ? 6 : 3;
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
-    packet_size=image->depth > 8 ? 8 : 4;
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
+    packet_size=(size_t) (image->depth > 8 ? 8 : 4);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open output image file.
       */
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      if (status == False)
-        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        return(status);
     }
   scene=0;
   do
@@ -571,7 +593,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
-      if (!image->matte)
+      if (image->matte == MagickFalse)
         SetImageOpacity(image,OpaqueOpacity);
     switch (image_info->interlace)
     {
@@ -597,9 +619,13 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
               (void) WriteBlob(image,packet_size*image->columns,pixels);
             }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         break;
       }
@@ -614,19 +640,23 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,RedQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,GreenQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,BlueQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"RGBA") == 0)
             {
               (void) PopImagePixels(image,AlphaQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
+            }
+          if (QuantumTick(y,image->rows) != 0)
+            {
+              status=MagickMonitor(SaveImageTag,y,image->rows,
+                &image->exception);
+              if (status == MagickFalse)
+                break;
             }
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-              break;
         }
         break;
       }
@@ -639,9 +669,10 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("R",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
+            if (status == MagickFalse)
+              return(status);
           }
         for (y=0; y < (long) image->rows; y++)
         {
@@ -649,7 +680,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,RedQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -657,10 +688,10 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("G",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
-        if (!MagickMonitor(SaveImageTag,100,400,&image->exception))
+        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == MagickFalse)
           break;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -668,7 +699,7 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,GreenQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
@@ -676,10 +707,10 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("B",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
-        if (!MagickMonitor(SaveImageTag,200,400,&image->exception))
+        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == MagickFalse)
           break;
         for (y=0; y < (long) image->rows; y++)
         {
@@ -687,11 +718,11 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,BlueQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            if (!MagickMonitor(SaveImageTag,300,400,&image->exception))
+            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == MagickFalse)
               break;
             if (image_info->interlace == PartitionInterlace)
               {
@@ -699,8 +730,8 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
                 AppendImageFormat("A",image->filename);
                 status=OpenBlob(image_info,image,WriteBinaryBlobMode,
                   &image->exception);
-                if (status == False)
-                  ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+                if (status == MagickFalse)
+                  return(status);
               }
             for (y=0; y < (long) image->rows; y++)
             {
@@ -709,12 +740,13 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
               if (p == (const PixelPacket *) NULL)
                 break;
               (void) PopImagePixels(image,AlphaQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-        if (!MagickMonitor(SaveImageTag,400,400,&image->exception))
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
+        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == MagickFalse)
           break;
         break;
       }
@@ -724,13 +756,10 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  LiberateMemory((void **) &pixels);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
