@@ -10,39 +10,26 @@
 %                            R  R   LLLLL  A   A                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                      Read Alias/Wavefront Image Format.                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,11 +43,13 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,7 +62,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadRLAImage reads a run-length encoded Wavefront RLA image file
+%  ReadRLAImage() reads a run-length encoded Wavefront RLA image file
 %  and returns it.  It allocates the memory necessary for the new Image
 %  structure and returns a pointer to the new image.
 %
@@ -177,8 +166,8 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     *scanlines;
 
-  register IndexPacket
-    *indexes;
+  MagickBooleanType
+    status;
 
   register long
     i,
@@ -193,20 +182,22 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     byte;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   rla_info.window.left=ReadBlobMSBShort(image);
   rla_info.window.right=ReadBlobMSBShort(image);
   rla_info.window.bottom=ReadBlobMSBShort(image);
@@ -224,46 +215,46 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   rla_info.number_channels+=rla_info.number_matte_channels;
   rla_info.number_auxiliary_channels=ReadBlobMSBShort(image);
   rla_info.revision=ReadBlobMSBShort(image);
-  (void) ReadBlob(image,16,(char *) rla_info.gamma);
-  (void) ReadBlob(image,24,(char *) rla_info.red_primary);
-  (void) ReadBlob(image,24,(char *) rla_info.green_primary);
-  (void) ReadBlob(image,24,(char *) rla_info.blue_primary);
-  (void) ReadBlob(image,24,(char *) rla_info.white_point);
+  (void) ReadBlob(image,16,(unsigned char *) rla_info.gamma);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.red_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.green_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.blue_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.white_point);
   rla_info.job_number=(long) ReadBlobMSBLong(image);
-  (void) ReadBlob(image,128,(char *) rla_info.name);
-  (void) ReadBlob(image,128,(char *) rla_info.description);
-  (void) ReadBlob(image,64,(char *) rla_info.program);
-  (void) ReadBlob(image,32,(char *) rla_info.machine);
-  (void) ReadBlob(image,32,(char *) rla_info.user);
-  (void) ReadBlob(image,20,(char *) rla_info.date);
-  (void) ReadBlob(image,24,(char *) rla_info.aspect);
-  (void) ReadBlob(image,8,(char *) rla_info.aspect_ratio);
-  (void) ReadBlob(image,32,(char *) rla_info.chan);
+  (void) ReadBlob(image,128,(unsigned char *) rla_info.name);
+  (void) ReadBlob(image,128,(unsigned char *) rla_info.description);
+  (void) ReadBlob(image,64,(unsigned char *) rla_info.program);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.machine);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.user);
+  (void) ReadBlob(image,20,(unsigned char *) rla_info.date);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.aspect);
+  (void) ReadBlob(image,8,(unsigned char *) rla_info.aspect_ratio);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.chan);
   rla_info.field=ReadBlobMSBShort(image);
-  (void) ReadBlob(image,12,(char *) rla_info.time);
-  (void) ReadBlob(image,32,(char *) rla_info.filter);
+  (void) ReadBlob(image,12,(unsigned char *) rla_info.time);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.filter);
   rla_info.bits_per_channel=ReadBlobMSBShort(image);
   rla_info.matte_type=ReadBlobMSBShort(image);
   rla_info.matte_bits=ReadBlobMSBShort(image);
   rla_info.auxiliary_type=ReadBlobMSBShort(image);
   rla_info.auxiliary_bits=ReadBlobMSBShort(image);
-  (void) ReadBlob(image,32,(char *) rla_info.auxiliary);
-  (void) ReadBlob(image,36,(char *) rla_info.space);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.auxiliary);
+  (void) ReadBlob(image,36,(unsigned char *) rla_info.space);
   rla_info.next=(long) ReadBlobMSBLong(image);
   /*
     Initialize image structure.
   */
   image->matte=rla_info.number_matte_channels != 0;
-  image->columns=rla_info.active_window.right-rla_info.active_window.left;
-  image->rows=rla_info.active_window.top-rla_info.active_window.bottom;
-  if (image_info->ping)
+  image->columns=rla_info.active_window.right-rla_info.active_window.left+1;
+  image->rows=rla_info.active_window.top-rla_info.active_window.bottom+1;
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
-  scanlines=(long *) AcquireMemory(image->rows*sizeof(long));
+  scanlines=(long *) AcquireMagickMemory(image->rows*sizeof(long));
   if (scanlines == (long *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (*rla_info.description != '\0')
     (void) SetImageAttribute(image,"comment",rla_info.description);
   /*
@@ -275,7 +266,6 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Read image data.
   */
   x=0;
-  indexes=(IndexPacket *) NULL;
   for (y=0; y < (long) image->rows; y++)
   {
     (void) SeekBlob(image,scanlines[image->rows-y-1],SEEK_SET);
@@ -293,13 +283,12 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
         if (runlength < 0)
           {
-            q=GetImagePixels(image,(long) (x % image->columns),
-              (long) (y/image->columns),1,1);
-            if (q == (PixelPacket *) NULL)
-              break;
-            indexes=GetIndexes(image);
             while (runlength < 0)
             {
+              q=GetImagePixels(image,(long) (x % image->columns),
+                (long) (y % image->rows),1,1);
+              if (q == (PixelPacket *) NULL)
+                break;
               byte=ReadBlobByte(image);
               length--;
               switch (channel)
@@ -307,7 +296,6 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 case 0:
                 {
                   q->red=ScaleCharToQuantum(byte);
-                  *indexes=0;
                   break;
                 }
                 case 1:
@@ -327,7 +315,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   break;
                 }
               }
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               x++;
               runlength++;
@@ -340,7 +328,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         do
         {
           q=GetImagePixels(image,(long) (x % image->columns),
-            (long) (y/image->columns),1,1);
+            (long) (y % image->rows),1,1);
           if (q == (PixelPacket *) NULL)
             break;
           switch (channel)
@@ -348,7 +336,6 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             case 0:
             {
               q->red=ScaleCharToQuantum(byte);
-              *indexes=0;
               break;
             }
             case 1:
@@ -368,7 +355,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               break;
             }
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           x++;
           runlength--;
@@ -376,15 +363,15 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         while (runlength > 0);
       }
     }
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -398,7 +385,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterRLAImage adds attributes for the RLA image format to
+%  RegisterRLAImage() adds attributes for the RLA image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -416,8 +403,8 @@ ModuleExport void RegisterRLAImage(void)
     *entry;
 
   entry=SetMagickInfo("RLA");
-  entry->decoder=(DecoderHandler) ReadRLAImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadRLAImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Alias/Wavefront image");
   entry->module=AcquireString("RLA");
   (void) RegisterMagickInfo(entry);
@@ -434,7 +421,7 @@ ModuleExport void RegisterRLAImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterRLAImage removes format registrations made by the
+%  UnregisterRLAImage() removes format registrations made by the
 %  RLA module from the list of supported formats.
 %
 %  The format of the UnregisterRLAImage method is:
