@@ -12,37 +12,24 @@
 %                                                                             %
 %                    Read/Write ImageMagick Image Format.                     %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
-#include "magick/resource.h"
+#include "magick/memory_.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+#include "magick/transform.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,16 +63,16 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsSFW returns True if the image format type, identified by the
+%  IsSFW() returns MagickTrue if the image format type, identified by the
 %  magick string, is SFW.
 %
 %  The format of the IsSFW method is:
 %
-%      unsigned int IsSFW(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSFW(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsSFW returns True if the image format type is SFW.
+%    o status:  Method IsSFW() returns MagickTrue if the image format type is SFW.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -90,13 +81,13 @@
 %
 %
 */
-static unsigned int IsSFW(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSFW(const unsigned char *magick,const size_t length)
 {
   if (length < 5)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"SFW94",5) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -110,7 +101,7 @@ static unsigned int IsSFW(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadSFWImage reads a Seattle Film Works image file and returns it.
+%  ReadSFWImage() reads a Seattle Film Works image file and returns it.
 %  It allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -221,45 +212,50 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image;
 
   ImageInfo
-    *clone_info;
+    *read_info;
 
   int
     unique_file;
 
+  MagickBooleanType
+    status;
+
   register unsigned char
     *header,
     *data;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
     *buffer,
     *offset;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read image into a buffer.
   */
-  buffer=(unsigned char *) AcquireMemory((size_t) GetBlobSize(image));
+  buffer=(unsigned char *) AcquireMagickMemory((size_t) GetBlobSize(image));
   if (buffer == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  count=ReadBlob(image,(size_t) GetBlobSize(image),(char *) buffer);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,(size_t) GetBlobSize(image),buffer);
   if ((count == 0) || (LocaleNCompare((char *) buffer,"SFW",3) != 0))
-    ThrowReaderException(CorruptImageError,"NotASFWImageFile",image);
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   CloseBlob(image);
   DestroyImage(image);
   /*
@@ -269,12 +265,12 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     "\377\310\377\320",4);
   if (header == (unsigned char *) NULL)
     {
-      LiberateMemory((void **) &buffer);
-      ThrowReaderException(CorruptImageError,"NotASFWImageFile",image)
+      buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
   TranslateSFWMarker(header);  /* translate soi and app tags */
   TranslateSFWMarker(header+2);
-  (void) memcpy(header+6,"JFIF\0\001\0",7);  /* JFIF magic */
+  (void) CopyMagickMemory(header+6,"JFIF\0\001\0",7);  /* JFIF magic */
   /*
     Translate remaining markers.
   */
@@ -292,53 +288,60 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (unsigned char *) "\377\311",2);
   if (data == (unsigned char *) NULL)
     {
-      LiberateMemory((void **) &buffer);
-      ThrowReaderException(CorruptImageError,"NotASFWImageFile",image)
+      buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
   TranslateSFWMarker(data++);  /* translate eoi marker */
   /*
     Write JFIF file.
   */
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
+  read_info=CloneImageInfo(image_info);
+  read_info->blob=(void *) NULL;
+  read_info->length=0;
   file=(FILE *) NULL;
-  unique_file=AcquireUniqueFileResource(clone_info->filename);
+  unique_file=AcquireUniqueFileResource(read_info->filename);
   if (unique_file != -1)
-    file=fopen(clone_info->filename,"wb");
+    file=fopen(read_info->filename,"wb");
   if ((unique_file == -1) || (file == (FILE *) NULL))
     {
-      LiberateMemory((void **) &buffer);
-      DestroyImageInfo(clone_info);
-      (void) strncpy(image->filename,clone_info->filename,MaxTextExtent-1);
-      ThrowReaderException(FileOpenError,"UnableToCreateTemporaryFile",image)
+      buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      DestroyImageInfo(read_info);
+      (void) CopyMagickString(image->filename,read_info->filename,
+        MaxTextExtent);
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image->filename);
+      DestroyImageList(image);
+      return(MagickFalse);
     }
   (void) fwrite(header,offset-header+1,1,file);
   (void) fwrite(HuffmanTable,1,sizeof(HuffmanTable)/sizeof(*HuffmanTable),file);
   (void) fwrite(offset+1,data-offset,1,file);
   status=ferror(file);
   (void) fclose(file);
-  LiberateMemory((void **) &buffer);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   if (status)
     {
-      (void) remove(clone_info->filename);
-      DestroyImageInfo(clone_info);
-      ThrowReaderException(FileOpenError,"UnableToWriteFile",image)
+      (void) remove(read_info->filename);
+      DestroyImageInfo(read_info);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        FileOpenError,"UnableToWriteFile",image->filename,strerror(errno));
+      DestroyImageList(image);
+      return(MagickFalse);
     }
   /*
     Read JPEG image.
   */
-  image=ReadImage(clone_info,exception);
-  (void) LiberateUniqueFileResource(clone_info->filename);
-  DestroyImageInfo(clone_info);
+  image=ReadImage(read_info,exception);
+  (void) RelinquishUniqueFileResource(read_info->filename);
+  DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
-    return(image);
+    return(GetFirstImageInList(image));
   /*
     Correct image orientation.
   */
   flipped_image=FlipImage(image,exception);
   if (flipped_image == (Image *) NULL)
-    return(image);
+    return(GetFirstImageInList(image));
   DestroyImage(image);
   return(flipped_image);
 }
@@ -354,7 +357,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterSFWImage adds attributes for the SFW image format to
+%  RegisterSFWImage() adds attributes for the SFW image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -372,9 +375,9 @@ ModuleExport void RegisterSFWImage(void)
     *entry;
 
   entry=SetMagickInfo("SFW");
-  entry->decoder=(DecoderHandler) ReadSFWImage;
-  entry->magick=(MagickHandler) IsSFW;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadSFWImage;
+  entry->magick=(MagickHandler *) IsSFW;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Seattle Film Works");
   entry->module=AcquireString("SFW");
   (void) RegisterMagickInfo(entry);
@@ -391,7 +394,7 @@ ModuleExport void RegisterSFWImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterSFWImage removes format registrations made by the
+%  UnregisterSFWImage() removes format registrations made by the
 %  SFW module from the list of supported formats.
 %
 %  The format of the UnregisterSFWImage method is:
