@@ -10,39 +10,26 @@
 %                            SSSSS   GGG   IIIII                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                      Read/Write Irix RGB Image Format.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,17 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Typedef declaractions.
@@ -90,7 +82,7 @@ typedef struct _SGIInfo
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteSGIImage(const ImageInfo *,Image *);
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -103,16 +95,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsSGI returns True if the image format type, identified by the
+%  IsSGI() returns MagickTrue if the image format type, identified by the
 %  magick string, is SGI.
 %
 %  The format of the IsSGI method is:
 %
-%      unsigned int IsSGI(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsSGI returns True if the image format type is SGI.
+%    o status:  Method IsSGI() returns MagickTrue if the image format type is SGI.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -121,13 +113,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsSGI(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSGI(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\001\332",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -141,7 +133,7 @@ static unsigned int IsSGI(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadSGIImage reads a SGI RGB image file and returns it.  It
+%  ReadSGIImage() reads a SGI RGB image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -162,16 +154,19 @@ static unsigned int IsSGI(const unsigned char *magick,const size_t length)
 %
 */
 
-static void SGIDecode(const unsigned long bytes_per_pixel,
+static void SGIDecode(const unsigned long width,const size_t bytes_per_pixel,
   unsigned char *max_packets,unsigned char *pixels)
 {
-  long
-    count;
-
   register unsigned char
     *p,
     *q;
 
+  register long
+    i;
+
+  ssize_t
+    count;
+
   unsigned long
     pixel;
 
@@ -179,14 +174,15 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
   q=pixels;
   if (bytes_per_pixel == 2)
     {
-      for ( ; ; )
+      for (i=0; i < (long) width; )
       {
-        pixel=(*p++) << 8;
+        pixel=(unsigned long) (*p++) << 8;
         pixel|=(*p++);
-        count=(long) (pixel & 0x7f);
+        count=(ssize_t) (pixel & 0x7f);
+        i+=count;
         if (count == 0)
           break;
-        if (pixel & 0x80)
+        if ((pixel & 0x80) != 0)
           for ( ; count != 0; count--)
           {
             *q=(*p++);
@@ -195,7 +191,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
           }
         else
           {
-            pixel=(*p++) << 8;
+            pixel=(unsigned long) (*p++) << 8;
             pixel|=(*p++);
             for ( ; count != 0; count--)
             {
@@ -207,13 +203,14 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
       }
       return;
     }
-  for ( ; ; )
+  for (i=0; i < (long) width; )
   {
-    pixel=(*p++);
-    count=(long) (pixel & 0x7f);
+    pixel=(unsigned long) (*p++);
+    count=(ssize_t) (pixel & 0x7f);
     if (count == 0)
       break;
-    if (pixel & 0x80)
+    i+=count;
+    if ((pixel & 0x80) != 0)
       for ( ; count != 0; count--)
       {
         *q=(*p++);
@@ -221,7 +218,7 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
       }
     else
       {
-        pixel=(*p++);
+        pixel=(unsigned long) (*p++);
         for ( ; count != 0; count--)
         {
           *q=(unsigned char) pixel;
@@ -233,9 +230,6 @@ static void SGIDecode(const unsigned long bytes_per_pixel,
 
 static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  ExtendedSignedIntegralType
-    number_pixels;
-
   Image
     *image;
 
@@ -243,6 +237,12 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
     y,
     z;
 
+  MagickBooleanType
+    status;
+
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -259,26 +259,28 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   SGIInfo
     iris_info;
 
+  size_t
+    bytes_per_pixel;
+
   unsigned char
     *iris_pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    bytes_per_pixel;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read SGI raster header.
   */
@@ -289,41 +291,42 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify SGI identifier.
     */
     if (iris_info.magic != 0x01DA)
-      ThrowReaderException(CorruptImageError,"NotASGIImageFile",image);
-    iris_info.storage=ReadBlobByte(image);
-    iris_info.bytes_per_pixel=ReadBlobByte(image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    iris_info.storage=(unsigned char) ReadBlobByte(image);
+    if ((int) iris_info.storage == 0x01)
+      image->compression=RLECompression;
+    iris_info.bytes_per_pixel=(unsigned char) ReadBlobByte(image);
     iris_info.dimension=ReadBlobMSBShort(image);
     iris_info.columns=ReadBlobMSBShort(image);
     iris_info.rows=ReadBlobMSBShort(image);
     iris_info.depth=ReadBlobMSBShort(image);
     image->columns=iris_info.columns;
     image->rows=iris_info.rows;
-    image->depth=iris_info.depth <= 8 ? 8 : QuantumDepth;
+    image->depth=(unsigned long) (iris_info.depth <= 8 ? 8 : QuantumDepth);
     if (iris_info.depth < 3)
       {
         image->storage_class=PseudoClass;
         image->colors=256;
       }
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse)  && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     iris_info.minimum_value=ReadBlobMSBLong(image);
     iris_info.maximum_value=ReadBlobMSBLong(image);
-    (void) ReadBlob(image,(unsigned int) sizeof(iris_info.filler),
-      (char *) iris_info.filler);
+    (void) ReadBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
     */
-    bytes_per_pixel=iris_info.bytes_per_pixel;
-    number_pixels=(ExtendedSignedIntegralType) iris_info.columns*iris_info.rows;
-    if ((4*bytes_per_pixel*number_pixels) != (size_t)
-        (4*bytes_per_pixel*number_pixels))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    bytes_per_pixel=(size_t) iris_info.bytes_per_pixel;
+    number_pixels=(MagickSizeType) iris_info.columns*iris_info.rows;
+    if ((4*bytes_per_pixel*number_pixels) != ((MagickSizeType) (size_t)
+        (4*bytes_per_pixel*number_pixels)))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     iris_pixels=(unsigned char *)
-      AcquireMemory(4*bytes_per_pixel*iris_info.columns*iris_info.rows);
+      AcquireMagickMemory(4*bytes_per_pixel*iris_info.columns*iris_info.rows);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    if (iris_info.storage != 0x01)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    if ((int) iris_info.storage != 0x01)
       {
         unsigned char
           *scanline;
@@ -332,17 +335,16 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Read standard image format.
         */
         scanline=(unsigned char *)
-          AcquireMemory(bytes_per_pixel*iris_info.columns);
+          AcquireMagickMemory(bytes_per_pixel*iris_info.columns);
         if (scanline == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (z=0; z < (int) iris_info.depth; z++)
         {
           p=iris_pixels+bytes_per_pixel*z;
           for (y=0; y < (long) iris_info.rows; y++)
           {
             (void) ReadBlob(image,bytes_per_pixel*iris_info.columns,
-              (char *) scanline);
+              scanline);
             if (bytes_per_pixel == 2)
               for (x=0; x < (long) iris_info.columns; x++)
               {
@@ -358,10 +360,14 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
           }
         }
-        LiberateMemory((void **) &scanline);
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
       }
     else
       {
+        ssize_t
+          offset,
+          *offsets;
+
         unsigned char
           *max_packets;
 
@@ -369,40 +375,39 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
           data_order;
 
         unsigned long
-          offset,
-          *offsets,
           *runlength;
 
         /*
           Read runlength-encoded image format.
         */
-        offsets=(unsigned long *) AcquireMemory(iris_info.rows*
+        offsets=(ssize_t *) AcquireMagickMemory((size_t) iris_info.rows*
           iris_info.depth*sizeof(unsigned long));
-        max_packets=(unsigned char *) AcquireMemory(4*iris_info.columns+10);
-        runlength=(unsigned long *) AcquireMemory(iris_info.rows*
+        max_packets=(unsigned char *)
+          AcquireMagickMemory(4*(size_t) iris_info.columns+10);
+        runlength=(unsigned long *) AcquireMagickMemory(iris_info.rows*
           iris_info.depth*sizeof(unsigned long));
-        if ((offsets == (unsigned long *) NULL) ||
+        if ((offsets == (ssize_t *) NULL) ||
             (max_packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
-          offsets[i]=ReadBlobMSBLong(image);
+          offsets[i]=(ssize_t) ReadBlobMSBLong(image);
         for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
           runlength[i]=ReadBlobMSBLong(image);
         /*
           Check data order.
         */
         offset=0;
-        data_order=0;
-        for (y=0; ((y < (long) iris_info.rows) && !data_order); y++)
-          for (z=0; ((z < (int) iris_info.depth) && !data_order); z++)
+        data_order=MagickFalse;
+        for (y=0; ((y < (long) iris_info.rows) && (data_order == MagickFalse)); y++)
+          for (z=0; ((z < (int) iris_info.depth) && (data_order == MagickFalse)); z++)
           {
             if (offsets[y+z*iris_info.rows] < offset)
-              data_order=1;
+              data_order=MagickTrue;
             offset=offsets[y+z*iris_info.rows];
           }
-        offset=512+4*bytes_per_pixel*2*(iris_info.rows*iris_info.depth);
+        offset=(ssize_t) (512+4*bytes_per_pixel*2*(iris_info.rows*
+          iris_info.depth));
         if (data_order == 1)
           {
             for (z=0; z < (int) iris_info.depth; z++)
@@ -415,16 +420,21 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     offset=offsets[y+z*iris_info.rows];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,runlength[y+z*iris_info.rows],
-                  (char *) max_packets);
+                (void) ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
+                  max_packets);
                 offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z);
+                SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
+                  p+bytes_per_pixel*z);
                 p+=(iris_info.columns*4*bytes_per_pixel);
               }
             }
           }
         else
           {
+            MagickOffsetType
+              position;
+           
+            position=TellBlob(image);
             p=iris_pixels;
             for (y=0; y < (long) iris_info.rows; y++)
             {
@@ -435,22 +445,24 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     offset=offsets[y+z*iris_info.rows];
                     (void) SeekBlob(image,(long) offset,SEEK_SET);
                   }
-                (void) ReadBlob(image,runlength[y+z*iris_info.rows],
-                  (char *) max_packets);
+                (void) ReadBlob(image,(size_t) runlength[y+z*iris_info.rows],
+                  max_packets);
                 offset+=runlength[y+z*iris_info.rows];
-                SGIDecode(bytes_per_pixel,max_packets,p+bytes_per_pixel*z);
+                SGIDecode(iris_info.columns,bytes_per_pixel,max_packets,
+                  p+bytes_per_pixel*z);
               }
               p+=(iris_info.columns*4*bytes_per_pixel);
             }
+            (void) SeekBlob(image,position,SEEK_SET);
           }
-        LiberateMemory((void **) &runlength);
-        LiberateMemory((void **) &max_packets);
-        LiberateMemory((void **) &offsets);
+        runlength=(unsigned long *) RelinquishMagickMemory(runlength);
+        max_packets=(unsigned char *) RelinquishMagickMemory(max_packets);
+        offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
     /*
       Initialize image structure.
     */
-    image->matte=iris_info.depth == 4;
+    image->matte=(MagickBooleanType) (iris_info.depth == 4);
     image->columns=iris_info.columns;
     image->rows=iris_info.rows;
     /*
@@ -479,12 +491,15 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 p+=8;
                 q++;
               }
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                    break;
+                if (QuantumTick(y,image->rows) != 0)
+                  {
+                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    if (status == MagickFalse)
+                      break;
+                  }
             }
           }
         else
@@ -503,12 +518,15 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               p+=4;
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                  break;
+              if (QuantumTick(y,image->rows) != 0)
+                {
+                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
       }
     else
@@ -516,9 +534,8 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Create grayscale map.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         /*
           Convert SGI image to PseudoClass pixel packets.
         */
@@ -533,17 +550,20 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
               indexes=GetIndexes(image);
               for (x=0; x < (long) image->columns; x++)
               {
-                indexes[x]=(*p << 8);
+                indexes[x]=(IndexPacket) (*p << 8);
                 indexes[x]|=(*(p+1));
                 p+=8;
                 q++;
               }
-              if (!SyncImagePixels(image))
+              if (SyncImagePixels(image) == MagickFalse)
                 break;
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
-                  if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                    break;
+                if (QuantumTick(y,image->rows) != 0)
+                  {
+                    status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                    if (status == MagickFalse)
+                      break;
+                  }
             }
           }
         else
@@ -556,23 +576,26 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
             indexes=GetIndexes(image);
             for (x=0; x < (long) image->columns; x++)
             {
-              indexes[x]=(*p);
+              indexes[x]=(IndexPacket) (*p);
               p+=4;
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                  break;
+              if (QuantumTick(y,image->rows) != 0)
+                {
+                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
-        SyncImage(image);
+        (void) SyncImage(image);
       }
-    LiberateMemory((void **) &iris_pixels);
-    if (EOFBlob(image))
+    iris_pixels=(unsigned char *) RelinquishMagickMemory(iris_pixels);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -583,6 +606,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     iris_info.magic=ReadBlobMSBShort(image);
+printf("%d %x\n",(int) TellBlob(image),iris_info.magic);
     if (iris_info.magic == 0x01DA)
       {
         /*
@@ -597,14 +621,12 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (iris_info.magic == 0x01DA);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -618,7 +640,7 @@ static Image *ReadSGIImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterSGIImage adds attributes for the SGI image format to
+%  RegisterSGIImage() adds attributes for the SGI image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -636,12 +658,12 @@ ModuleExport void RegisterSGIImage(void)
     *entry;
 
   entry=SetMagickInfo("SGI");
-  entry->decoder=(DecoderHandler) ReadSGIImage;
-  entry->encoder=(EncoderHandler) WriteSGIImage;
-  entry->magick=(MagickHandler) IsSGI;
+  entry->decoder=(DecoderHandler *) ReadSGIImage;
+  entry->encoder=(EncoderHandler *) WriteSGIImage;
+  entry->magick=(MagickHandler *) IsSGI;
   entry->description=AcquireString("Irix RGB image");
   entry->module=AcquireString("SGI");
-  entry->seekable_stream=True;
+  entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
 }
 
@@ -656,7 +678,7 @@ ModuleExport void RegisterSGIImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterSGIImage removes format registrations made by the
+%  UnregisterSGIImage() removes format registrations made by the
 %  SGI module from the list of supported formats.
 %
 %  The format of the UnregisterSGIImage method is:
@@ -680,15 +702,15 @@ ModuleExport void UnregisterSGIImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteSGIImage writes an image in SGI RGB encoded image format.
+%  WriteSGIImage() writes an image in SGI RGB encoded image format.
 %
 %  The format of the WriteSGIImage method is:
 %
-%      unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteSGIImage return True if the image is written.
+%    o status: Method WriteSGIImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -699,7 +721,7 @@ ModuleExport void UnregisterSGIImage(void)
 %
 */
 
-static size_t SGIEncode(unsigned char *pixels,size_t count,
+static size_t SGIEncode(unsigned char *pixels,size_t length,
   unsigned char *packets)
 {
   short
@@ -714,7 +736,7 @@ static size_t SGIEncode(unsigned char *pixels,size_t count,
     *mark;
 
   p=pixels;
-  limit=p+count*4;
+  limit=p+length*4;
   q=packets;
   while (p < limit)
   {
@@ -723,12 +745,12 @@ static size_t SGIEncode(unsigned char *pixels,size_t count,
     while ((p < limit) && ((*(p-8) != *(p-4)) || (*(p-4) != *p)))
       p+=4;
     p-=8;
-    count=((p-mark) >> 2);
-    while (count)
+    length=(size_t) (p-mark) >> 2;
+    while (length != 0)
     {
-      runlength=(short) (count > 126 ? 126 : count);
-      count-=runlength;
-      *q++=0x80 | runlength;
+      runlength=(short) (length > 126 ? 126 : length);
+      length-=runlength;
+      *q++=(unsigned char) (0x80 | runlength);
       for ( ; runlength > 0; runlength--)
       {
         *q++=(*mark);
@@ -739,28 +761,34 @@ static size_t SGIEncode(unsigned char *pixels,size_t count,
     p+=4;
     while ((p < limit) && (*p == *mark))
       p+=4;
-    count=((p-mark) >> 2);
-    while (count)
+    length=(size_t) (p-mark) >> 2;
+    while (length != 0)
     {
-      runlength=(short) (count > 126 ? 126 : count);
-      count-=runlength;
+      runlength=(short) (length > 126 ? 126 : length);
+      length-=runlength;
       *q++=(unsigned char) runlength;
       *q++=(*mark);
     }
   }
-  *q++=0;
-  return(q-packets);
+  *q++='\0';
+  return((size_t) (q-packets));
 }
 
-static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteSGIImage(const ImageInfo *image_info,Image *image)
 {
-  ExtendedSignedIntegralType
-    number_pixels;
-
   long
     y,
     z;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
+  MagickSizeType
+    number_pixels;
+
   SGIInfo
     iris_info;
 
@@ -778,12 +806,6 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     *iris_pixels,
     *packets;
 
-  unsigned int
-    status;
-
-  unsigned long
-    scene;
-
   /*
     Open output image file.
   */
@@ -791,11 +813,13 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit",image);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  if ((image->columns > 65535UL) || (image->rows > 65535UL))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   scene=0;
   do
   {
@@ -804,25 +828,31 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     */
     (void) SetImageColorspace(image,RGBColorspace);
     iris_info.magic=0x01DA;
-    if (image_info->compression == NoCompression)
-      iris_info.storage=0x00;
+    if (image->compression == NoCompression)
+      iris_info.storage=(unsigned char) 0x00;
     else
-      iris_info.storage=0x01;
-    iris_info.bytes_per_pixel=1;  /* one byte per pixel */
+      iris_info.storage=(unsigned char) 0x01;
+    iris_info.bytes_per_pixel=(unsigned char) 1;  /* one byte per pixel */
     iris_info.dimension=3;
     iris_info.columns=(unsigned short) image->columns;
     iris_info.rows=(unsigned short) image->rows;
-    iris_info.depth=image->matte ? 4 : 3;
-    if ((image_info->type != TrueColorType) &&
-        IsGrayImage(image,&image->exception))
+    if (image->matte != MagickFalse)
+      iris_info.depth=4;
+    else
       {
-        iris_info.dimension=2;
-        iris_info.depth=1;
+        if ((image_info->type != TrueColorType) &&
+            (IsGrayImage(image,&image->exception) != MagickFalse))
+          {
+            iris_info.dimension=2;
+            iris_info.depth=1;
+          }
+        else
+          iris_info.depth=3;
       }
     iris_info.minimum_value=0;
-    iris_info.maximum_value=ScaleQuantumToChar(MaxRGB);
+    iris_info.maximum_value=(unsigned long) ScaleQuantumToChar(MaxRGB);
     for (i=0; i < (int) sizeof(iris_info.filler); i++)
-      iris_info.filler[i]=0;
+      iris_info.filler[i]=(unsigned char) 0;
     /*
       Write SGI header.
     */
@@ -835,17 +865,17 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobMSBShort(image,iris_info.depth);
     (void) WriteBlobMSBLong(image,iris_info.minimum_value);
     (void) WriteBlobMSBLong(image,iris_info.maximum_value);
-    (void) WriteBlob(image,sizeof(iris_info.filler),
-      (char *) iris_info.filler);
+    (void) WriteBlob(image,sizeof(iris_info.filler),iris_info.filler);
     /*
       Allocate SGI pixels.
     */
-    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
-    if ((4*number_pixels) != (size_t) (4*number_pixels))
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
-    iris_pixels=(unsigned char *) AcquireMemory(4*image->columns*image->rows);
+    number_pixels=(MagickSizeType) image->columns*image->rows;
+    if ((4*number_pixels) != ((MagickSizeType) (size_t) (4*number_pixels)))
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    iris_pixels=(unsigned char *)
+      AcquireMagickMemory(4*(size_t) image->columns*image->rows);
     if (iris_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert image pixels to uncompressed SGI pixels.
     */
@@ -864,11 +894,14 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         p++;
       }
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
     }
-    if (image_info->compression == NoCompression)
+    if (image->compression == NoCompression)
       {
         unsigned char
           *scanline;
@@ -876,10 +909,10 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Write uncompressed SGI pixels.
         */
-        scanline=(unsigned char *) AcquireMemory(iris_info.columns);
+        scanline=(unsigned char *)
+          AcquireMagickMemory((size_t) iris_info.columns);
         if (scanline == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         for (z=0; z < (int) iris_info.depth; z++)
         {
           q=iris_pixels+z;
@@ -890,46 +923,49 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
               scanline[x]=(*q);
               q+=4;
             }
-            (void) WriteBlob(image,iris_info.columns,(char *) scanline);
+            (void) WriteBlob(image,(size_t) iris_info.columns,scanline);
           }
         }
-        LiberateMemory((void **) &scanline);
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
       }
     else
       {
-        unsigned long
-          length,
-          number_packets,
+        ssize_t
           offset,
-          *offsets,
+          *offsets;
+
+        size_t
+          length,
+          number_packets;
+
+        unsigned long
           *runlength;
 
         /*
           Convert SGI uncompressed pixels.
         */
-        offsets=(unsigned long *) AcquireMemory(iris_info.rows*
+        offsets=(ssize_t *) AcquireMagickMemory(iris_info.rows*
           iris_info.depth*sizeof(unsigned long));
         packets=(unsigned char *)
-          AcquireMemory(4*(2*iris_info.columns+10)*image->rows);
-        runlength=(unsigned long *) AcquireMemory(iris_info.rows*
+          AcquireMagickMemory(4*(2*(size_t) iris_info.columns+10)*image->rows);
+        runlength=(unsigned long *) AcquireMagickMemory(iris_info.rows*
           iris_info.depth*sizeof(unsigned long));
-        if ((offsets == (unsigned long *) NULL) ||
+        if ((offsets == (ssize_t *) NULL) ||
             (packets == (unsigned char *) NULL) ||
             (runlength == (unsigned long *) NULL))
-          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        offset=512+4*2*(iris_info.rows*iris_info.depth);
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+        offset=512+4*2*((ssize_t) iris_info.rows*iris_info.depth);
         number_packets=0;
         q=iris_pixels;
         for (y=0; y < (long) iris_info.rows; y++)
         {
-          for (z=0; z < (int) iris_info.depth; z++)
+          for (z=0; z < (long) iris_info.depth; z++)
           {
-            length=
-              SGIEncode(q+z,(int) iris_info.columns,packets+number_packets);
+            length=SGIEncode(q+z,(size_t) iris_info.columns,packets+
+              number_packets);
             number_packets+=length;
             offsets[y+z*iris_info.rows]=offset;
-            runlength[y+z*iris_info.rows]=length;
+            runlength[y+z*iris_info.rows]=(unsigned long) length;
             offset+=length;
           }
           q+=(iris_info.columns*4);
@@ -937,30 +973,30 @@ static unsigned int WriteSGIImage(const ImageInfo *image_info,Image *image)
         /*
           Write out line start and length tables and runlength-encoded pixels.
         */
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
-          (void) WriteBlobMSBLong(image,offsets[i]);
-        for (i=0; i < (int) (iris_info.rows*iris_info.depth); i++)
+        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
+          (void) WriteBlobMSBLong(image,(unsigned long) offsets[i]);
+        for (i=0; i < (long) (iris_info.rows*iris_info.depth); i++)
           (void) WriteBlobMSBLong(image,runlength[i]);
-        (void) WriteBlob(image,number_packets,(char *) packets);
+        (void) WriteBlob(image,number_packets,packets);
         /*
-          Free memory.
+          Free resources.
         */
-        LiberateMemory((void **) &runlength);
-        LiberateMemory((void **) &packets);
-        LiberateMemory((void **) &offsets);
+        runlength=(unsigned long *) RelinquishMagickMemory(runlength);
+        packets=(unsigned char *) RelinquishMagickMemory(packets);
+        offsets=(ssize_t *) RelinquishMagickMemory(offsets);
       }
-    LiberateMemory((void **) &iris_pixels);
+    iris_pixels=(unsigned char *) RelinquishMagickMemory(iris_pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
+  } while (image_info->adjoin != MagickFalse);
+  if (image_info->adjoin != MagickFalse)
     while (image->previous != (Image *) NULL)
       image=image->previous;
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
