@@ -10,39 +10,26 @@
 %              SSSSS    T    EEEEE   GGG   A   A  N   N   OOO                 %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                       Write A Steganographic Image.                         %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,7 +62,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadSTEGANOImage reads a steganographic image hidden within another
+%  ReadSTEGANOImage() reads a steganographic image hidden within another
 %  image type.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -96,16 +86,16 @@
 static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define GetBit(a,i) (((a) >> (i)) & 0x01)
-#define SetBit(a,i,set) \
-  a=(Quantum) ((set) ? (a) | (1UL << (i)) : (a) & ~(1UL << (i)))
+#define GetBit(alpha,i) (((alpha) >> (unsigned long) (i)) & 0x01)
+#define SetBit(alpha,i,set) (alpha)=(Quantum) ((set) != 0 ? (alpha) | \
+  (1UL << (unsigned long) (i)) : (alpha) & ~(1UL << (unsigned long) (i)))
 
   Image
     *image,
     *watermark;
 
   ImageInfo
-    *clone_info;
+    *read_info;
 
   long
     c,
@@ -131,26 +121,28 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
-  *clone_info->magick='\0';
-  watermark=ReadImage(clone_info,exception);
-  DestroyImageInfo(clone_info);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  read_info=CloneImageInfo(image_info);
+  read_info->blob=(void *) NULL;
+  read_info->length=0;
+  *read_info->magick='\0';
+  watermark=ReadImage(read_info,exception);
+  DestroyImageInfo(read_info);
   if (watermark == (Image *) NULL)
     return((Image *) NULL);
   watermark->depth=QuantumDepth;
-  if (!AllocateImageColormap(image,MaxColormapSize))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  if (image_info->ping)
+  if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Get hidden watermark from low-order bits of image.
@@ -200,12 +192,12 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
           j++;
       }
     }
-    if (!MagickMonitor(LoadImagesTag,i,QuantumDepth,&image->exception))
+    if (MagickMonitor(LoadImagesTag,i,QuantumDepth,&image->exception) == MagickFalse)
       break;
   }
   DestroyImage(watermark);
   SyncImage(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -219,7 +211,7 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterSTEGANOImage adds attributes for the STEGANO image format to
+%  RegisterSTEGANOImage() adds attributes for the STEGANO image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -237,7 +229,7 @@ ModuleExport void RegisterSTEGANOImage(void)
     *entry;
 
   entry=SetMagickInfo("STEGANO");
-  entry->decoder=(DecoderHandler) ReadSTEGANOImage;
+  entry->decoder=(DecoderHandler *) ReadSTEGANOImage;
   entry->description=AcquireString("Steganographic image");
   entry->module=AcquireString("STEGANO");
   (void) RegisterMagickInfo(entry);
@@ -254,7 +246,7 @@ ModuleExport void RegisterSTEGANOImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterSTEGANOImage removes format registrations made by the
+%  UnregisterSTEGANOImage() removes format registrations made by the
 %  STEGANO module from the list of supported formats.
 %
 %  The format of the UnregisterSTEGANOImage method is:
