@@ -10,39 +10,26 @@
 %                          T    IIIII  LLLLL  EEEEE                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Return A Tiled Image Using Texture.                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,12 +41,15 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,7 +62,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTILEImage tiles a texture on an image.  It allocates the
+%  ReadTILEImage tiles a texture on an image.  It allocates the
 %  memory necessary for the new Image structure and returns a pointer to the
 %  new image.
 %
@@ -100,47 +90,34 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
     *tile_image;
 
   ImageInfo
-    *clone_info;
-
-  long
-    y;
-
-  register long
-    x;
+    *read_info;
 
   /*
     Initialize Image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
-  *clone_info->magick='\0';
-  tile_image=ReadImage(clone_info,exception);
-  DestroyImageInfo(clone_info);
+  read_info=CloneImageInfo(image_info);
+  read_info->blob=(void *) NULL;
+  read_info->length=0;
+  *read_info->magick='\0';
+  tile_image=ReadImage(read_info,exception);
+  DestroyImageInfo(read_info);
   if (tile_image == (Image *) NULL)
     return((Image *) NULL);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (*image_info->filename == '\0')
-    ThrowReaderException(OptionError,"MustSpecifyAnImageName",image);
-  /*
-    Tile texture onto image.
-  */
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-  for (y=0; y < (long) image->rows; y+=tile_image->rows)
-  {
-    for (x=0; x < (long) image->columns; x+=tile_image->columns)
-      (void) CompositeImage(image,CopyCompositeOp,tile_image,x,y);
-    if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-      break;
-  }
+    ThrowReaderException(OptionError,"MustSpecifyAnImageName");
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
+  (void) TextureImage(image,tile_image);
   DestroyImage(tile_image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -154,7 +131,7 @@ static Image *ReadTILEImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTILEImage adds attributes for the TILE image format to
+%  RegisterTILEImage() adds attributes for the TILE image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -172,8 +149,8 @@ ModuleExport void RegisterTILEImage(void)
     *entry;
 
   entry=SetMagickInfo("TILE");
-  entry->decoder=(DecoderHandler) ReadTILEImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadTILEImage;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Tile image with a texture");
   entry->module=AcquireString("TILE");
   (void) RegisterMagickInfo(entry);
@@ -190,7 +167,7 @@ ModuleExport void RegisterTILEImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTILEImage removes format registrations made by the
+%  UnregisterTILEImage() removes format registrations made by the
 %  TILE module from the list of supported formats.
 %
 %  The format of the UnregisterTILEImage method is:
