@@ -10,39 +10,26 @@
 %                              T    IIIII  M   M                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                           Read PSX TIM Image Format.                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,11 +42,13 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -72,7 +61,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTIMImage reads a PSX TIM image file and returns it.  It
+%  ReadTIMImage() reads a PSX TIM image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -116,6 +105,9 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -138,9 +130,6 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned short
     word;
 
-  unsigned int
-    status;
-
   unsigned long
     bytes_per_line,
     height,
@@ -153,12 +142,17 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Determine if this is a TIM file.
   */
@@ -169,7 +163,7 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Verify TIM identifier.
     */
     if (tim_info.id != 0x00000010)
-      ThrowReaderException(CorruptImageError,"NotATIMImageFile",image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     tim_info.flag=ReadBlobLSBLong(image);
     has_clut=!!(tim_info.flag & (1 << 3));
     pixel_mode=tim_info.flag & 0x07;
@@ -194,26 +188,27 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void)ReadBlobLSBShort(image);
         width=ReadBlobLSBShort(image);
         height=ReadBlobLSBShort(image);
-        if (!AllocateImageColormap(image,pixel_mode == 1 ? 256 : 16))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        tim_colormap=(unsigned char *) AcquireMemory(image->colors*2);
+        if (AllocateImageColormap(image,pixel_mode == 1 ? 256 : 16) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        tim_colormap=(unsigned char *) AcquireMagickMemory(image->colors*2);
         if (tim_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
-        (void) ReadBlob(image,2*image->colors,(char *) tim_colormap);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ReadBlob(image,2*image->colors,tim_colormap);
         p=tim_colormap;
         for (i=0; i < (long) image->colors; i++)
         {
           word=(*p++);
           word|=(unsigned short) (*p++ << 8);
-          image->colormap[i].blue=ScaleCharToQuantum(ScaleColor5to8((word >> 10) & 0x1f));
-          image->colormap[i].green=ScaleCharToQuantum(ScaleColor5to8((word >> 5) & 0x1f));
-          image->colormap[i].red=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
+          image->colormap[i].blue=
+            ScaleCharToQuantum(ScaleColor5to8((word >> 10) & 0x1f));
+          image->colormap[i].green=
+            ScaleCharToQuantum(ScaleColor5to8((word >> 5) & 0x1f));
+          image->colormap[i].red=
+            ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
         }
-        LiberateMemory((void **) &tim_colormap);
+        tim_colormap=(unsigned char *) RelinquishMagickMemory(tim_colormap);
       }
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -227,10 +222,10 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image_size=2*width*height;
     bytes_per_line=width*2;
     width=(width*16)/bits_per_pixel;
-    tim_data=(unsigned char *) AcquireMemory(image_size);
+    tim_data=(unsigned char *) AcquireMagickMemory(image_size);
     if (tim_data == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    (void) ReadBlob(image,image_size,(char *) tim_data);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    (void) ReadBlob(image,image_size,tim_data);
     tim_pixels=tim_data;
     /*
       Initialize image structure.
@@ -265,13 +260,13 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               indexes[x]=(*p >> 4) & 0xf;
               p++;
             }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
+          if (QuantumTick(y,image->rows) != 0)
             {
               status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
                 exception);
-              if (status == False)
+              if (status == MagickFalse)
                 break;
             }
         }
@@ -291,13 +286,13 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           p=tim_pixels+y*bytes_per_line;
           for (x=0; x < (long) image->columns; x++)
             indexes[x]=(*p++);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
+          if (QuantumTick(y,image->rows) != 0)
             {
               status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
                 exception);
-              if (status == False)
+              if (status == MagickFalse)
                 break;
             }
         }
@@ -323,13 +318,13 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->red=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
+          if (QuantumTick(y,image->rows) != 0)
             {
               status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
                 exception);
-              if (status == False)
+              if (status == MagickFalse)
                 break;
             }
         }
@@ -353,27 +348,27 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=ScaleCharToQuantum(*p++);
             q++;
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
+          if (QuantumTick(y,image->rows) != 0)
             {
               status=MagickMonitor(LoadImageTag,image->rows-y-1,image->rows,
                 exception);
-              if (status == False)
+              if (status == MagickFalse)
                 break;
             }
         }
         break;
       }
       default:
-        ThrowReaderException(CorruptImageError,"NotATIMImageFile",image)
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     }
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
-    LiberateMemory((void **) &tim_pixels);
-    if (EOFBlob(image))
+      (void) SyncImage(image);
+    tim_pixels=(unsigned char *) RelinquishMagickMemory(tim_pixels);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -395,14 +390,12 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (tim_info.id == 0x00000010);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -416,7 +409,7 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTIMImage adds attributes for the TIM image format to
+%  RegisterTIMImage() adds attributes for the TIM image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -434,7 +427,7 @@ ModuleExport void RegisterTIMImage(void)
     *entry;
 
   entry=SetMagickInfo("TIM");
-  entry->decoder=(DecoderHandler) ReadTIMImage;
+  entry->decoder=(DecoderHandler *) ReadTIMImage;
   entry->description=AcquireString("PSX TIM");
   entry->module=AcquireString("TIM");
   (void) RegisterMagickInfo(entry);
@@ -451,7 +444,7 @@ ModuleExport void RegisterTIMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTIMImage removes format registrations made by the
+%  UnregisterTIMImage() removes format registrations made by the
 %  TIM module from the list of supported formats.
 %
 %  The format of the UnregisterTIMImage method is:
