@@ -10,39 +10,26 @@
 %                              T      T    F                                  %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%             Return A Preview For A TrueType or Postscript Font              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,13 +41,16 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/draw.h"
 #include "magick/error.h"
-#include "magick/drawing_wand.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
-#include "magick/render.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+#include "magick/type.h"
+#include "wand/magick_wand.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,16 +63,16 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPFA() returns True if the image format type, identified by the magick
+%  IsPFA()() returns MagickTrue if the image format type, identified by the magick
 %  string, is PFA.
 %
 %  The format of the IsPFA method is:
 %
-%      unsigned int IsPFA(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPFA returns True if the image format type can be
+%    o status:  Method IsPFA() returns MagickTrue if the image format type can be
 %               handled by this module.
 %
 %    o magick: This string is generally the first few bytes of an image file
@@ -92,13 +82,13 @@
 %
 %
 */
-static unsigned int IsPFA(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPFA(const unsigned char *magick,const size_t length)
 {
   if (length < 14)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"%!PS-AdobeFont-1.0",14) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -112,16 +102,16 @@ static unsigned int IsPFA(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsTTF() returns True if the image format type, identified by the magick
+%  IsTTF()() returns MagickTrue if the image format type, identified by the magick
 %  string, is TTF.
 %
 %  The format of the IsTTF method is:
 %
-%      unsigned int IsTTF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsTTF returns True if the image format type can be
+%    o status:  Method IsTTF() returns MagickTrue if the image format type can be
 %               handled by this module.
 %
 %    o magick: This string is generally the first few bytes of an image file
@@ -131,16 +121,18 @@ static unsigned int IsPFA(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int IsTTF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsTTF(const unsigned char *magick,const size_t length)
 {
   if (length < 5)
-    return(False);
-  if ((magick[0] == 0x00) && (magick[1] == 0x01) && (magick[2] == 0x00) &&
-      (magick[3] == 0x00) && (magick[4] == 0x0))
-    return(True);
-  return(False);
+    return(MagickFalse);
+  if (((int) magick[0] == 0x00) && ((int) magick[1] == 0x01) &&
+      ((int) magick[2] == 0x00) && ((int) magick[3] == 0x00) &&
+      ((int) magick[4] == 0x0))
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
+#if defined(HasTTF)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -152,7 +144,7 @@ static unsigned int IsTTF(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTTFImage reads a TrueType font file and returns it.  It
+%  ReadTTFImage() reads a TrueType font file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -172,31 +164,33 @@ static unsigned int IsTTF(const unsigned char *magick,const size_t length)
 %
 %
 */
-#if defined(HasTTF)
 static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
     buffer[MaxTextExtent];
 
-  const unsigned char
-    *Text = (unsigned char *)
+  const char
+    *Text = (char *)
       "The quick brown fox jumps over the lazy dog 0123456789\n"
-      "ABCDEFGHIJKLMNOPQRSTUVWXYZ &#~\"\'(-`_^@)=+\260\n"
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZ &#~\"\\'(-`_^@)=+\260\n"
       "$\243^\250*\265\371%!\247:/;.,?<> "
       "\342\352\356\373\364\344\353\357\366\374\377\340\371\351\350\347 ";
 
+  const TypeInfo
+    *type_info;
+
   DrawInfo
     *draw_info;
 
-  DrawingWand
-    *wand;
-
   Image
     *image;
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PixelPacket
     background_color;
 
@@ -207,22 +201,28 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   image->columns=800;
   image->rows=480;
+  type_info=GetTypeInfo(image_info->filename,exception);
+  if ((type_info != (const TypeInfo *) NULL) &&
+      (type_info->glyphs != (char *) NULL))
+    (void) CopyMagickString(image->filename,type_info->glyphs,MaxTextExtent);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Color canvas with background color
   */
@@ -234,46 +234,51 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     for (x=0; x < (long) image->columns; x++)
       *q++=background_color;
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
   }
-  (void) strncpy(image->magick,image_info->magick,MaxTextExtent-1);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+  (void) CopyMagickString(image->magick,image_info->magick,MaxTextExtent);
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
   /*
     Prepare drawing commands
   */
   y=20;
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->font=AllocateString(image->filename);
+  draw_info->font=AcquireString(image->filename);
   draw_info->fill=image_info->pen;
-  wand=DrawAllocateWand(draw_info,image);
-  (void) DrawPushGraphicContext(wand);
-  (void) DrawSetViewbox(wand,0,0,image->columns,image->rows);
-  (void) DrawSetFont(wand,image_info->filename);
-  (void) DrawSetFontSize(wand,18);
-  (void) DrawAnnotation(wand,10,y,Text);
+  ConcatenateString(&draw_info->primitive,"push graphic-context\n");
+  (void) FormatMagickString(buffer,MaxTextExtent," viewbox 0 0 %lu %lu\n",
+    image->columns,image->rows);
+  ConcatenateString(&draw_info->primitive,buffer);
+  ConcatenateString(&draw_info->primitive," font-size 18\n");
+  (void) FormatMagickString(buffer,MaxTextExtent," text 10,%ld '",y);
+  ConcatenateString(&draw_info->primitive,buffer);
+  ConcatenateString(&draw_info->primitive,Text);
+  (void) FormatMagickString(buffer,MaxTextExtent,"'\n");
+  ConcatenateString(&draw_info->primitive,buffer);
   y+=20*MultilineCensus((char *) Text)+20;
   for (i=12; i <= 72; i+=6)
   {
     y+=i+12;
-    (void) DrawSetFontSize(wand,18);
-    (void) FormatString(buffer,"%ld",i);
-    (void) DrawAnnotation(wand,10,y,(const unsigned char *) buffer);
-    (void) DrawSetFontSize(wand,i);
-    (void) DrawAnnotation(wand,50,y,(const unsigned char *)
-      "That which does not destroy me, only makes me stronger.");
+    ConcatenateString(&draw_info->primitive," font-size 18\n");
+    (void) FormatMagickString(buffer,MaxTextExtent," text 10,%ld '%ld'\n",y,i);
+    ConcatenateString(&draw_info->primitive,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent," font-size %ld\n",i);
+    ConcatenateString(&draw_info->primitive,buffer);
+    (void) FormatMagickString(buffer,MaxTextExtent," text 50,%ld "
+      "'That which does not destroy me, only makes me stronger.'\n",y);
+    ConcatenateString(&draw_info->primitive,buffer);
     if (i >= 24)
       i+=6;
   }
-  (void) DrawPopGraphicContext(wand);
-  (void) DrawRender(wand);
+  ConcatenateString(&draw_info->primitive,"pop graphic-context");
+  (void) DrawImage(image,draw_info);
   /*
     Free resources.
   */
   DestroyDrawInfo(draw_info);
-  DestroyDrawingWand(wand);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 #endif /* HasTTF */
 
@@ -288,7 +293,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTTFImage adds attributes for the TTF image format to
+%  RegisterTTFImage() adds attributes for the TTF image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -310,25 +315,15 @@ ModuleExport void RegisterTTFImage(void)
 
   *version='\0';
 #if defined(FREETYPE_MAJOR) && defined(FREETYPE_MINOR)
-  FormatString(version,"%d.%d",FREETYPE_MAJOR,FREETYPE_MINOR);
+  (void) FormatMagickString(version,MaxTextExtent,"%d.%d",
+    FREETYPE_MAJOR,FREETYPE_MINOR);
 #endif
-  entry=SetMagickInfo("TTF");
-#if defined(HasTTF)
-  entry->decoder=(DecoderHandler) ReadTTFImage;
-#endif
-  entry->magick=(MagickHandler) IsTTF;
-  entry->adjoin=False;
-  entry->description=AcquireString("TrueType font");
-  if (*version != '\0')
-    entry->version=AcquireString(version);
-  entry->module=AcquireString("TTF");
-  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PFA");
 #if defined(HasTTF)
-  entry->decoder=(DecoderHandler) ReadTTFImage;
+  entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
-  entry->magick=(MagickHandler) IsPFA;
-  entry->adjoin=False;
+  entry->magick=(MagickHandler *) IsPFA;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Postscript Type 1 font (ASCII)");
   if (*version != '\0')
     entry->version=AcquireString(version);
@@ -336,15 +331,37 @@ ModuleExport void RegisterTTFImage(void)
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PFB");
 #if defined(HasTTF)
-  entry->decoder=(DecoderHandler) ReadTTFImage;
+  entry->decoder=(DecoderHandler *) ReadTTFImage;
 #endif
-  entry->magick=(MagickHandler) IsPFA;
-  entry->adjoin=False;
+  entry->magick=(MagickHandler *) IsPFA;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Postscript Type 1 font (binary)");
   if (*version != '\0')
     entry->version=AcquireString(version);
   entry->module=AcquireString("TTF");
   (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("TTC");
+#if defined(HasTTF)
+  entry->decoder=(DecoderHandler *) ReadTTFImage;
+#endif
+  entry->magick=(MagickHandler *) IsTTF;
+  entry->adjoin=MagickFalse;
+  entry->description=AcquireString("TrueType font collection");
+  if (*version != '\0')
+    entry->version=AcquireString(version);
+  entry->module=AcquireString("TTF");
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("TTF");
+#if defined(HasTTF)
+  entry->decoder=(DecoderHandler *) ReadTTFImage;
+#endif
+  entry->magick=(MagickHandler *) IsTTF;
+  entry->adjoin=MagickFalse;
+  entry->description=AcquireString("TrueType font");
+  if (*version != '\0')
+    entry->version=AcquireString(version);
+  entry->module=AcquireString("TTF");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -358,7 +375,7 @@ ModuleExport void RegisterTTFImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTTFImage removes format registrations made by the
+%  UnregisterTTFImage() removes format registrations made by the
 %  TTF module from the list of supported formats.
 %
 %  The format of the UnregisterTTFImage method is:
@@ -369,6 +386,8 @@ ModuleExport void RegisterTTFImage(void)
 ModuleExport void UnregisterTTFImage(void)
 {
   (void) UnregisterMagickInfo("TTF");
+  (void) UnregisterMagickInfo("TTC");
   (void) UnregisterMagickInfo("PFA");
   (void) UnregisterMagickInfo("PFB");
+  (void) UnregisterMagickInfo("PFA");
 }
