@@ -10,39 +10,26 @@
 %                              T    X   X    T                                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                      Render Text Onto A Canvas Image.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,19 +40,27 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/annotate.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/draw.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/render.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteTXTImage(const ImageInfo *,Image *);
 
 /*
@@ -73,23 +68,78 @@ static unsigned int
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   R e a d T X T I m a g e                                                   %
+%   I s T X T                                                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTXTImage reads a text file and returns it as an image.  It
-%  allocates the memory necessary for the new Image structure and returns a
-%  pointer to the new image.
+%  IsTXT() returns MagickTrue if the image format type, identified by the magick
+%  string, is TXT.
 %
-%  The format of the ReadTXTImage method is:
+%  The format of the IsTXT method is:
 %
-%      Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%      MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadTXTImage returns a pointer to the image after
+%    o status:  Method IsTXT() returns MagickTrue if the image format type is TXT.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+static MagickBooleanType IsTXT(const unsigned char *magick,const size_t length)
+{
+#define MagickID  "# ImageMagick pixel enumeration:"
+
+  char
+    format[MaxTextExtent];
+
+  ssize_t
+    count;
+
+  unsigned long
+    columns,
+    max_value,
+    rows;
+
+  if (length < 32)
+    return(MagickFalse);
+  count=(ssize_t) sscanf((char *) magick+32,"%lu,%lu,%lu,%s",&columns,&rows,
+    &max_value,format);
+  if ((LocaleNCompare((char *) magick,MagickID,strlen(MagickID)) != 0) ||
+      (count != 4))
+    return(MagickFalse);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e a d T E X T I m a g e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ReadTEXTImage() reads a text file and returns it as an image.  It
+%  allocates the memory necessary for the new Image structure and returns a
+%  pointer to the new image.
+%
+%  The format of the ReadTEXTImage method is:
+%
+%      Image *ReadTEXTImage(const ImageInfo *image_info,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image:  Method ReadTEXTImage returns a pointer to the image after
 %      reading. A null image is returned if there is a memory shortage or if
 %      the image cannot be read.
 %
@@ -99,7 +149,8 @@ static unsigned int
 %
 %
 */
-static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
+static Image *ReadTEXTImage(const ImageInfo *image_info,
+  ExceptionInfo *exception)
 {
   char
     filename[MaxTextExtent],
@@ -121,26 +172,31 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     offset;
 
+  MagickBooleanType
+    status;
+
   RectangleInfo
     page;
 
   TypeMetric
     metrics;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page geometry.
   */
@@ -151,13 +207,13 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       GeometryInfo
         geometry_info;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(PSDensityGeometry,&geometry_info);
       image->x_resolution=geometry_info.rho;
       image->y_resolution=geometry_info.sigma;
-      if (!(flags & SigmaValue))
+      if ((flags & SigmaValue) == 0)
         image->y_resolution=image->x_resolution;
     }
   page.width=612;
@@ -170,21 +226,22 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Initialize Image structure.
   */
   image->columns=(unsigned long)
-    ceil(((page.width*image->x_resolution)/dx_resolution)-0.5);
+    (((page.width*image->x_resolution)/dx_resolution)+0.5);
   image->rows=(unsigned long)
-    ceil(((page.height*image->y_resolution)/dy_resolution)-0.5);
+    (((page.height*image->y_resolution)/dy_resolution)+0.5);
   texture=(Image *) NULL;
   if (image_info->texture != (char *) NULL)
     {
       ImageInfo
-        *clone_info;
+        *read_info;
 
-      clone_info=CloneImageInfo(image_info);
-      clone_info->blob=(void *) NULL;
-      clone_info->length=0;
-      (void) strncpy(clone_info->filename,image_info->texture,MaxTextExtent-1);
-      texture=ReadImage(clone_info,exception);
-      DestroyImageInfo(clone_info);
+      read_info=CloneImageInfo(image_info);
+      read_info->blob=(void *) NULL;
+      read_info->length=0;
+      (void) CopyMagickString(read_info->filename,image_info->texture,
+        MaxTextExtent);
+      texture=ReadImage(read_info,exception);
+      DestroyImageInfo(read_info);
     }
   /*
     Annotate the text image.
@@ -193,13 +250,13 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   draw_info->fill=image_info->pen;
   (void) CloneString(&draw_info->text,image_info->filename);
-  FormatString(geometry,"0x0%+ld%+ld",page.x,page.y);
+  (void) FormatMagickString(geometry,MaxTextExtent,"0x0%+ld%+ld",page.x,page.y);
   (void) CloneString(&draw_info->geometry,geometry);
   status=GetTypeMetrics(image,draw_info,&metrics);
-  if (status == False)
-    ThrowReaderException(TypeError,"UnableToGetTypeMetrics",image);
-  (void) strncpy(filename,image_info->filename,MaxTextExtent-1);
-  if (draw_info->text != '\0')
+  if (status == MagickFalse)
+    ThrowReaderException(TypeError,"UnableToGetTypeMetrics");
+  (void) CopyMagickString(filename,image_info->filename,MaxTextExtent);
+  if (*draw_info->text != '\0')
     *draw_info->text='\0';
   p=ReadBlobString(image,text);
   for (offset=2*page.y; p != (char *) NULL; )
@@ -212,8 +269,12 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     offset+=(long) (metrics.ascent-metrics.descent);
     if (image->previous == (Image *) NULL)
       if (QuantumTick(offset,image->rows))
-        if (!MagickMonitor(LoadImageTag,offset,image->rows,&image->exception))
-          break;
+        {
+          status=MagickMonitor(LoadImageTag,offset,image->rows,
+            &image->exception);
+          if (status == MagickFalse)
+            break;
+        }
     p=ReadBlobString(image,text);
     if ((offset < (long) image->rows) && (p != (char *) NULL))
       continue;
@@ -243,11 +304,11 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->next->columns=image->columns;
     image->next->rows=image->rows;
     image=SyncNextImageInList(image);
-    (void) strncpy(image->filename,filename,MaxTextExtent-1);
+    (void) CopyMagickString(image->filename,filename,MaxTextExtent);
     SetImage(image,OpaqueOpacity);
     status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
       exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
   }
   if (texture != (Image *) NULL)
@@ -256,17 +317,156 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
         handler;
 
       handler=SetMonitorHandler((MonitorHandler) NULL);
-      TextureImage(image,texture);
+      (void) TextureImage(image,texture);
       (void) SetMonitorHandler(handler);
     }
   (void) AnnotateImage(image,draw_info);
   if (texture != (Image *) NULL)
     DestroyImage(texture);
   DestroyDrawInfo(draw_info);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e a d T X T I m a g e                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ReadTXTImage() reads a text file and returns it as an image.  It
+%  allocates the memory necessary for the new Image structure and returns a
+%  pointer to the new image.
+%
+%  The format of the ReadTXTImage method is:
+%
+%      Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image:  Method ReadTXTImage returns a pointer to the image after
+%      reading. A null image is returned if there is a memory shortage or if
+%      the image cannot be read.
+%
+%    o image_info: The image info.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+%
+*/
+static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
+{
+  char
+    format[MaxTextExtent],
+    text[MaxTextExtent];
+
+  Image
+    *image;
+
+  long
+    x,
+    y;
+
+  LongPixelPacket
+    pixel;
+
+  MagickBooleanType
+    status;
+
+  Quantum
+    *scale;
+
+  register long
+    i;
+
+  register PixelPacket
+    *q;
+
+  ssize_t
+    count;
+
+  unsigned long
+    max_value;
+
+  /*
+    Open image file.
+  */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  image=AllocateImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  (void) ReadBlobString(image,text);
+  count=(ssize_t) sscanf(text+32,"%lu,%lu,%lu,%s",&image->columns,&image->rows,
+    &max_value,format);
+  if ((LocaleNCompare(text,"# ImageMagick pixel enumeration:",32) != 0) ||
+      (count != 4))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if (LocaleNCompare(format,"CMYK",4) == 0)
+    image->colorspace=CMYKColorspace;
+  image->matte=(MagickBooleanType) (strchr(format,'A') != (char *) NULL);
+  scale=(Quantum *) NULL;
+  if (max_value != MaxRGB)
+    {
+      /*
+        Compute pixel scaling table.
+      */
+      scale=(Quantum *)
+        AcquireMagickMemory((size_t) (max_value+1)*sizeof(Quantum));
+      if (scale == (Quantum *) NULL)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      for (i=0; i <= (long) max_value; i++)
+        scale[i]=ScaleXToQuantum(i,max_value);
+    }
+  pixel.opacity=OpaqueOpacity;
+  while (ReadBlobString(image,text) != (char *) NULL)
+  {
+    if (image->matte == MagickFalse)
+      {
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu)",&x,&y,&pixel.red,
+          &pixel.green,&pixel.blue);
+        if (count != 5)
+          continue;
+      }
+    else
+      {
+        count=(ssize_t) sscanf(text,"%ld,%ld: (%lu,%lu,%lu,%lu)",&x,&y,
+          &pixel.red,&pixel.green,&pixel.blue,&pixel.opacity);
+        if (count != 6)
+          continue;
+      }
+    if (scale != (Quantum *) NULL)
+      {
+        pixel.red=scale[pixel.red];
+        pixel.green=scale[pixel.green];
+        pixel.blue=scale[pixel.blue];
+        pixel.opacity=scale[pixel.opacity];
+      }
+    q=SetImagePixels(image,x,y,1,1);
+    if (q == (PixelPacket *) NULL)
+      continue;
+    q->red=(Quantum) pixel.red;
+    q->green=(Quantum) pixel.green;
+    q->blue=(Quantum) pixel.blue;
+    q->opacity=(Quantum) pixel.opacity;
+    (void) SyncImagePixels(image);
+  }
+  if (scale != (Quantum *) NULL)
+    scale=(Quantum *) RelinquishMagickMemory(scale);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -280,7 +480,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTXTImage adds attributes for the TXT image format to
+%  RegisterTXTImage() adds attributes for the TXT image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -298,16 +498,17 @@ ModuleExport void RegisterTXTImage(void)
     *entry;
 
   entry=SetMagickInfo("TEXT");
-  entry->decoder=(DecoderHandler) ReadTXTImage;
-  entry->encoder=(EncoderHandler) WriteTXTImage;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadTEXTImage;
+  entry->encoder=(EncoderHandler *) WriteTXTImage;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("Text");
   entry->module=AcquireString("TXT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TXT");
-  entry->decoder=(DecoderHandler) ReadTXTImage;
-  entry->encoder=(EncoderHandler) WriteTXTImage;
+  entry->decoder=(DecoderHandler *) ReadTXTImage;
+  entry->encoder=(EncoderHandler *) WriteTXTImage;
   entry->description=AcquireString("Text");
+  entry->magick=(MagickHandler *) IsTXT;
   entry->module=AcquireString("TXT");
   (void) RegisterMagickInfo(entry);
 }
@@ -323,7 +524,7 @@ ModuleExport void RegisterTXTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTXTImage removes format registrations made by the
+%  UnregisterTXTImage() removes format registrations made by the
 %  TXT module from the list of supported formats.
 %
 %  The format of the UnregisterTXTImage method is:
@@ -348,15 +549,15 @@ ModuleExport void UnregisterTXTImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteTXTImage writes the pixel values as text numbers.
+%  WriteTXTImage writes the pixel values as text numbers.
 %
 %  The format of the WriteTXTImage method is:
 %
-%      unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteTXTImage return True if the image is written.
+%    o status: Method WriteTXTImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -366,26 +567,28 @@ ModuleExport void UnregisterTXTImage(void)
 %
 %
 */
-static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
 {
   char
-    buffer[MaxTextExtent];
+    buffer[MaxTextExtent],
+    format[MaxTextExtent],
+    tuple[MaxTextExtent];
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p;
 
   register long
     x;
 
-  unsigned int
-    status;
-
-  unsigned long
-    scene;
-
   /*
     Open output image file.
   */
@@ -393,9 +596,22 @@ static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
+  if (image->colorspace == CMYKColorspace)
+    (void) strcpy(format,"CMYK");
+  else
+    (void) strcpy(format,"RGB");
+  if (image->matte != MagickFalse)
+    (void) ConcatenateMagickString(format,"A",MaxTextExtent);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "# ImageMagick pixel enumeration: %lu,%lu,%lu,%s\n",image->columns,
+    image->rows,image->depth == QuantumDepth ? (unsigned long) MaxRGB :
+    (1L << image->depth)-1,format);
+  (void) WriteBlobString(image,buffer);
   scene=0;
   do
   {
@@ -410,40 +626,32 @@ static unsigned int WriteTXTImage(const ImageInfo *image_info,Image *image)
         break;
       for (x=0; x < (long) image->columns; x++)
       {
-        if (image->matte)
-          {
-            FormatString(buffer,"%ld,%ld: %u,%u,%u,%u ",x,y,
-              p->red,p->green,p->blue,p->opacity);
-            (void) WriteBlobString(image,buffer);
-          }
-        else
-          {
-            FormatString(buffer,"%ld,%ld: %u,%u,%u ",x,y,
-              p->red,p->green,p->blue);
-            (void) WriteBlobString(image,buffer);
-            (void) QueryColorname(image,p,SVGCompliance,buffer,
-              &image->exception);
-            (void) WriteBlobString(image,buffer);
-          }
-        (void) WriteBlobByte(image,'\n');
-        if (image->previous == (Image *) NULL)
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-              break;
+        (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
+        (void) WriteBlobString(image,buffer);
+        GetColorTuple(p,image->depth,image->matte,MagickFalse,tuple);
+        (void) ConcatenateMagickString(tuple," ",MaxTextExtent);
+        (void) WriteBlobString(image,tuple);
+        (void) QueryColorname(image,p,SVGCompliance,tuple,&image->exception);
+        (void) WriteBlobString(image,tuple);
+        (void) WriteBlobString(image,"\n");
         p++;
       }
+      if (image->previous == (Image *) NULL)
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == MagickFalse)
+              break;
+          }
     }
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
