@@ -10,39 +10,26 @@
 %                         UUU     Y      V      Y                             %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read/Write 16bit/pixel Interleaved YUV Image Format.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,21 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteUYVYImage(const ImageInfo *,Image *);
 
 /*
@@ -78,7 +69,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadUYVYImage reads an image in the UYVY format and returns it.  It
+%  ReadUYVYImage() reads an image in the UYVY format and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -108,6 +99,9 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     x;
 
@@ -123,30 +117,29 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
     y1,
     y2;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image)
+  if (status == MagickFalse)
+    return(MagickFalse);
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   image->depth=8;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Accumulate UYVY, then unpack into two pixels.
@@ -158,10 +151,10 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) (image->columns >> 1); x++)
     {
-      u=ReadBlobByte(image);
-      y1=ReadBlobByte(image);
-      v=ReadBlobByte(image);
-      y2=ReadBlobByte(image);
+      u=(unsigned char) ReadBlobByte(image);
+      y1=(unsigned char) ReadBlobByte(image);
+      v=(unsigned char) ReadBlobByte(image);
+      y2=(unsigned char) ReadBlobByte(image);
       q->red=ScaleCharToQuantum(y1);
       q->green=ScaleCharToQuantum(u);
       q->blue=ScaleCharToQuantum(v);
@@ -171,19 +164,19 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
       q->blue=ScaleCharToQuantum(v);
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
   image->colorspace=YCbCrColorspace;
   (void) SetImageColorspace(image,RGBColorspace);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -197,7 +190,7 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterUYVYImage adds attributes for the UYVY image format to
+%  RegisterUYVYImage() adds attributes for the UYVY image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -215,18 +208,18 @@ ModuleExport void RegisterUYVYImage(void)
     *entry;
 
   entry=SetMagickInfo("PAL");
-  entry->decoder=(DecoderHandler) ReadUYVYImage;
-  entry->encoder=(EncoderHandler) WriteUYVYImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadUYVYImage;
+  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("UYVY");
-  entry->decoder=(DecoderHandler) ReadUYVYImage;
-  entry->encoder=(EncoderHandler) WriteUYVYImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadUYVYImage;
+  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
@@ -243,7 +236,7 @@ ModuleExport void RegisterUYVYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterUYVYImage removes format registrations made by the
+%  UnregisterUYVYImage() removes format registrations made by the
 %  UYVY module from the list of supported formats.
 %
 %  The format of the UnregisterUYVYImage method is:
@@ -268,17 +261,17 @@ ModuleExport void UnregisterUYVYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteUYVYImage writes an image to a file in the digital UYVY
+%  WriteUYVYImage() writes an image to a file in the digital UYVY
 %  format.  This format, used by AccomWSD, is not dramatically higher quality
 %  than the 12bit/pixel YUV format, but has better locality.
 %
 %  The format of the WriteUYVYImage method is:
 %
-%      unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteUYVYImage return True if the image is written.
+%    o status: Method WriteUYVYImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -288,9 +281,9 @@ ModuleExport void UnregisterUYVYImage(void)
 %      Implicit assumption: number of columns is even.
 %
 */
-static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,Image *image)
 {
-  DoublePixelPacket
+  MagickPixelPacket
     pixel;
 
   Image
@@ -299,16 +292,16 @@ static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    full,
+    status;
+
   register const PixelPacket
     *p;
 
   register long
     x;
 
-  unsigned int
-    full,
-    status;
-
   /*
     Open output image file.
   */
@@ -316,18 +309,20 @@ static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   /*
     Accumulate two pixels, then output.
   */
-  uyvy_image=CloneImage(image,0,0,True,&image->exception);
+  uyvy_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (uyvy_image == (Image *) NULL)
-    ThrowWriterException(ResourceLimitError,image->exception.reason,image);
+    ThrowWriterException(ResourceLimitError,image->exception.reason);
   (void) SetImageColorspace(uyvy_image,YCbCrColorspace);
-  full=False;
-  memset(&pixel,0,sizeof(DoublePixelPacket));
+  full=MagickFalse;
+  (void) ResetMagickMemory(&pixel,0,sizeof(MagickPixelPacket));
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(uyvy_image,0,y,image->columns,1,&image->exception);
@@ -335,7 +330,7 @@ static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      if (full)
+      if (full != MagickFalse)
         {
           pixel.green=(pixel.green+p->green)/2;
           pixel.blue=(pixel.blue+p->blue)/2;
@@ -344,17 +339,17 @@ static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.blue));
           (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
         }
-      pixel.red=p->red;
-      pixel.green=p->green;
-      pixel.blue=p->blue;
-      full=!full;
+      pixel.red=(double) p->red;
+      pixel.green=(double) p->green;
+      pixel.blue=(double) p->blue;
+      full=(MagickBooleanType) (full == MagickFalse);
       p++;
     }
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
         break;
   }
   DestroyImage(uyvy_image);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
