@@ -10,39 +10,26 @@
 %                              V    IIIII  DDDD                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%             Return a Visual Image Directory for matching images.            %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,18 +42,25 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/montage.h"
+#include "magick/resize.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteVIDImage(const ImageInfo *,Image *);
 
 /*
@@ -80,7 +74,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadVIDImage reads one of more images and creates a Visual Image
+%  ReadVIDImage reads one of more images and creates a Visual Image
 %  Directory file.  It allocates the memory necessary for the new Image
 %  structure and returns a pointer to the new image.
 %
@@ -105,21 +99,24 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define ClientName  "montage"
 
   char
-    **filelist,
-    **list;
+    **filelist;
 
   Image
     *image,
+    *images,
     *montage_image,
     *next_image,
     *thumbnail_image;
 
   ImageInfo
-    *clone_info;
+    *read_info;
 
   int
     number_files;
 
+  MagickBooleanType
+    status;
+
   MonitorHandler
     handler;
 
@@ -132,59 +129,54 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register long
     i;
 
-  unsigned int
-    status;
-
   /*
     Expand the filename.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"enter");
   image=AllocateImage(image_info);
-  list=(char **) AcquireMemory(sizeof(char *));
-  if (list == (char **) NULL)
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
-    }
-  list[0]=AllocateString(image_info->filename);
+  filelist=(char **) AcquireMagickMemory(sizeof(char *));
+  if (filelist == (char **) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  filelist[0]=(char *) image_info->filename;
   number_files=1;
-  filelist=list;
   status=ExpandFilenames(&number_files,&filelist);
-  if ((status == False) || (number_files == 0))
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
-    }
+  if ((status == MagickFalse) || (number_files == 0))
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   DestroyImage(image);
   /*
     Read each image and convert them to a tile.
   */
-  image=(Image *) NULL;
-  clone_info=CloneImageInfo(image_info);
-  clone_info->blob=(void *) NULL;
-  clone_info->length=0;
-  if (clone_info->size == (char *) NULL)
-    CloneString(&clone_info->size,DefaultTileGeometry);
-  for (i=0; i < number_files; i++)
+  images=NewImageList();
+  read_info=CloneImageInfo(image_info);
+  read_info->blob=(void *) NULL;
+  read_info->length=0;
+  if (read_info->size == (char *) NULL)
+    (void) CloneString(&read_info->size,DefaultTileGeometry);
+  for (i=0; i < (long) number_files; i++)
   {
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"name: %.1024s",
-      filelist[i]);
+    if (image_info->debug != MagickFalse)
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"name: %s",
+        filelist[i]);
     handler=SetMonitorHandler((MonitorHandler) NULL);
-    (void) strncpy(clone_info->filename,filelist[i],MaxTextExtent-1);
-    *clone_info->magick='\0';
-    next_image=ReadImage(clone_info,exception);
-    LiberateMemory((void **) &filelist[i]);
+    (void) CopyMagickString(read_info->filename,filelist[i],MaxTextExtent);
+    filelist[i]=(char *) RelinquishMagickMemory(filelist[i]);
+    *read_info->magick='\0';
+    next_image=ReadImage(read_info,exception);
+    if (exception->severity != UndefinedException)
+      CatchException(exception);
     if (next_image != (Image *) NULL)
       {
         (void) SetImageAttribute(next_image,"label",DefaultTileLabel);
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"geometry: %ldx%ld",
-          next_image->columns,next_image->rows);
+        if (image_info->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "geometry: %ldx%ld",next_image->columns,next_image->rows);
         SetGeometry(next_image,&geometry);
-        (void) ParseMetaGeometry(clone_info->size,&geometry.x,&geometry.y,
+        (void) ParseMetaGeometry(read_info->size,&geometry.x,&geometry.y,
           &geometry.width,&geometry.height);
         thumbnail_image=ZoomImage(next_image,geometry.width,geometry.height,
           exception);
@@ -193,46 +185,33 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             DestroyImage(next_image);
             next_image=thumbnail_image;
           }
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "thumbnail geometry: %ldx%ld",next_image->columns,next_image->rows);
-        if (image == (Image *) NULL)
-          image=next_image;
-        else
-          {
-            image->next=next_image;
-            image->next->previous=image;
-            image=SyncNextImageInList(image);
-          }
+        if (image_info->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "thumbnail geometry: %ldx%ld",next_image->columns,next_image->rows);
+        AppendImageToList(&images,next_image);
       }
     (void) SetMonitorHandler(handler);
-    if (!MagickMonitor(LoadImageTag,i,number_files,&image->exception))
+    status=MagickMonitor(LoadImageTag,(MagickOffsetType) i,(MagickSizeType)
+      number_files,exception);
+    if (status == MagickFalse)
       break;
   }
-  DestroyImageInfo(clone_info);
-  LiberateMemory((void **) &filelist);
-  if (image == (Image *) NULL)
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
-      ThrowReaderException(CorruptImageError,"UnableToReadVIDImage",image)
-    }
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  DestroyImageInfo(read_info);
+  filelist=(char **) RelinquishMagickMemory(filelist);
+  if (images == (Image *) NULL)
+    ThrowReaderException(CorruptImageError,"UnableToReadVIDImage");
   /*
     Create the visual image directory.
   */
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"creating montage");
-  montage_image=MontageImages(image,montage_info,exception);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"creating montage");
+  montage_image=MontageImages(GetFirstImageInList(images),montage_info,
+    exception);
   DestroyMontageInfo(montage_info);
   if (montage_image == (Image *) NULL)
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
-      ThrowReaderException(CorruptImageError,"UnableToReadVIDImage",image)
-    }
-  DestroyImageList(image);
-  LiberateMemory((void **) &list[0]);
-  LiberateMemory((void **) &list);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"return");
+    ThrowReaderException(CorruptImageError,"UnableToReadVIDImage");
+  DestroyImageList(images);
   return(montage_image);
 }
 
@@ -247,7 +226,7 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterVIDImage adds attributes for the VID image format to
+%  RegisterVIDImage() adds attributes for the VID image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -265,8 +244,8 @@ ModuleExport void RegisterVIDImage(void)
     *entry;
 
   entry=SetMagickInfo("VID");
-  entry->decoder=(DecoderHandler) ReadVIDImage;
-  entry->encoder=(EncoderHandler) WriteVIDImage;
+  entry->decoder=(DecoderHandler *) ReadVIDImage;
+  entry->encoder=(EncoderHandler *) WriteVIDImage;
   entry->description=AcquireString("Visual Image Directory");
   entry->module=AcquireString("VID");
   (void) RegisterMagickInfo(entry);
@@ -283,7 +262,7 @@ ModuleExport void RegisterVIDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterVIDImage removes format registrations made by the
+%  UnregisterVIDImage() removes format registrations made by the
 %  VID module from the list of supported formats.
 %
 %  The format of the UnregisterVIDImage method is:
@@ -307,15 +286,15 @@ ModuleExport void UnregisterVIDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteVIDImage writes an image to a file in VID X image format.
+%  WriteVIDImage() writes an image to a file in VID X image format.
 %
 %  The format of the WriteVIDImage method is:
 %
-%      unsigned int WriteVIDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteVIDImage return True if the image is written.
+%    o status: Method WriteVIDImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -325,20 +304,20 @@ ModuleExport void UnregisterVIDImage(void)
 %
 %
 */
-static unsigned int WriteVIDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteVIDImage(const ImageInfo *image_info,Image *image)
 {
   Image
     *montage_image;
 
+  MagickBooleanType
+    status;
+
   MontageInfo
     *montage_info;
 
   register Image
     *p;
 
-  unsigned int
-    status;
-
   /*
     Create the visual image directory.
   */
@@ -347,8 +326,9 @@ static unsigned int WriteVIDImage(const ImageInfo *image_info,Image *image)
   montage_info=CloneMontageInfo(image_info,(MontageInfo *) NULL);
   montage_image=MontageImages(image,montage_info,&image->exception);
   if (montage_image == (Image *) NULL)
-    ThrowWriterException(CorruptImageError,image->exception.reason,image);
-  FormatString(montage_image->filename,"miff:%.1024s",image->filename);
+    ThrowWriterException(CorruptImageError,image->exception.reason);
+  (void) FormatMagickString(montage_image->filename,MaxTextExtent,
+    "miff:%s",image->filename);
   status=WriteImage(image_info,montage_image);
   DestroyImageList(montage_image);
   return(status);
