@@ -10,39 +10,26 @@
 %                          V    IIIII  F      F                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                Read/Write Khoros Visualization Image Format.                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,17 +42,22 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteVIFFImage(const ImageInfo *,Image *);
 
 /*
@@ -79,16 +71,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsVIFF returns True if the image format type, identified by the
+%  IsVIFF() returns MagickTrue if the image format type, identified by the
 %  magick string, is VIFF.
 %
 %  The format of the IsVIFF method is:
 %
-%      unsigned int IsVIFF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsVIFF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsVIFF returns True if the image format type is VIFF.
+%    o status:  Method IsVIFF() returns MagickTrue if the image format type is VIFF.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -97,13 +89,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsVIFF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsVIFF(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\253\001",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -117,7 +109,7 @@ static unsigned int IsVIFF(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadVIFFImage reads a Khoros Visualization image file and returns
+%  ReadVIFFImage() reads a Khoros Visualization image file and returns
 %  it.  It allocates the memory necessary for the new Image structure and
 %  returns a pointer to the new image.
 %
@@ -212,9 +204,6 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     scale_factor,
     value;
 
-  ExtendedSignedIntegralType
-    number_pixels;
-
   Image
     *image;
 
@@ -224,6 +213,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickSizeType
+    number_pixels;
+
   register IndexPacket
     *indexes;
 
@@ -239,16 +234,13 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
     buffer[7],
     *viff_pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bytes_per_pixel,
     lsb_first,
@@ -262,32 +254,37 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read VIFF header (1024 bytes).
   */
-  count=ReadBlob(image,1,(char *) &viff_info.identifier);
+  count=ReadBlob(image,1,&viff_info.identifier);
   do
   {
     /*
       Verify VIFF identifier.
     */
     if ((count == 0) || ((unsigned char) viff_info.identifier != 0xab))
-      ThrowReaderException(CorruptImageError,"NotAVIFFImage",image);
+      ThrowReaderException(CorruptImageError,"NotAVIFFImage");
     /*
       Initialize VIFF image.
     */
-    (void) ReadBlob(image,7,(char *) buffer);
+    (void) ReadBlob(image,7,buffer);
     viff_info.file_type=buffer[0];
     viff_info.release=buffer[1];
     viff_info.version=buffer[2];
     viff_info.machine_dependency=buffer[3];
-    (void) ReadBlob(image,512,(char *) viff_info.comment);
+    (void) ReadBlob(image,512,(unsigned char *) viff_info.comment);
     viff_info.comment[511]='\0';
     if (strlen(viff_info.comment) > 4)
       (void) SetImageAttribute(image,"comment",viff_info.comment);
@@ -348,36 +345,35 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     /*
       Verify that we can read this VIFF image.
     */
-    number_pixels=(ExtendedSignedIntegralType) viff_info.columns*viff_info.rows;
+    number_pixels=(MagickSizeType) viff_info.columns*viff_info.rows;
     if (number_pixels != (size_t) number_pixels)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if (number_pixels == 0)
-      ThrowReaderException(CoderError,"ImageColumnOrRowSizeIsNotSupported",
-        image);
+      ThrowReaderException(CoderError,"ImageColumnOrRowSizeIsNotSupported");
     if ((viff_info.data_storage_type != VFF_TYP_BIT) &&
         (viff_info.data_storage_type != VFF_TYP_1_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_2_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_4_BYTE) &&
         (viff_info.data_storage_type != VFF_TYP_FLOAT) &&
         (viff_info.data_storage_type != VFF_TYP_DOUBLE))
-      ThrowReaderException(CoderError,"DataStorageTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,"DataStorageTypeIsNotSupported");
     if (viff_info.data_encode_scheme != VFF_DES_RAW)
-      ThrowReaderException(CoderError,"DataEncodingSchemeIsNotSupported",image);
+      ThrowReaderException(CoderError,"DataEncodingSchemeIsNotSupported");
     if ((viff_info.map_storage_type != VFF_MAPTYP_NONE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_1_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_2_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_4_BYTE) &&
         (viff_info.map_storage_type != VFF_MAPTYP_FLOAT) &&
         (viff_info.map_storage_type != VFF_MAPTYP_DOUBLE))
-      ThrowReaderException(CoderError,"MapStorageTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,"MapStorageTypeIsNotSupported");
     if ((viff_info.color_space_model != VFF_CM_NONE) &&
         (viff_info.color_space_model != VFF_CM_ntscRGB) &&
         (viff_info.color_space_model != VFF_CM_genericRGB))
-      ThrowReaderException(CoderError,"ColorspaceModelIsNotSupported",image);
+      ThrowReaderException(CoderError,"ColorspaceModelIsNotSupported");
     if (viff_info.location_type != VFF_LOC_IMPLICIT)
-      ThrowReaderException(CoderError,"LocationTypeIsNotSupported",image);
+      ThrowReaderException(CoderError,"LocationTypeIsNotSupported");
     if (viff_info.number_of_images != 1)
-      ThrowReaderException(CoderError,"NumberOfImagesIsNotSupported",image);
+      ThrowReaderException(CoderError,"NumberOfImagesIsNotSupported");
     if (viff_info.map_rows == 0)
       viff_info.map_scheme=VFF_MS_NONE;
     switch ((int) viff_info.map_scheme)
@@ -392,9 +388,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             image->colors=image->depth <= 8 ? 256 : 65536L;
             if (viff_info.data_storage_type == VFF_TYP_BIT)
               image->colors=2;
-            if (!AllocateImageColormap(image,image->colors))
-              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-                image);
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         break;
       }
@@ -417,19 +412,17 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           default: bytes_per_pixel=1; break;
         }
         image->colors=viff_info.map_columns;
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         viff_colormap=(unsigned char *)
-          AcquireMemory(bytes_per_pixel*image->colors*viff_info.map_rows);
+          AcquireMagickMemory(bytes_per_pixel*image->colors*viff_info.map_rows);
         if (viff_colormap == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         /*
           Read VIFF raster colormap.
         */
-        (void) ReadBlob(image,bytes_per_pixel*
-          image->colors*viff_info.map_rows,(char *) viff_colormap);
+        (void) ReadBlob(image,bytes_per_pixel*image->colors*viff_info.map_rows,
+          viff_colormap);
         lsb_first=1;
         if (*(char *) &lsb_first &&
             ((viff_info.machine_dependency != VFF_DEP_DECORDER) &&
@@ -464,7 +457,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           if (i < (long) image->colors)
             {
               image->colormap[i].red=ScaleCharToQuantum((unsigned long) value);
-              image->colormap[i].green=ScaleCharToQuantum((unsigned long) value);
+              image->colormap[i].green=
+                ScaleCharToQuantum((unsigned long) value);
               image->colormap[i].blue=ScaleCharToQuantum((unsigned long) value);
             }
           else
@@ -476,11 +470,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
                 image->colormap[i % image->colors].blue=
                   ScaleCharToQuantum((unsigned long) value);
         }
-        LiberateMemory((void **) &viff_colormap);
+        viff_colormap=(unsigned char *) RelinquishMagickMemory(viff_colormap);
         break;
       }
       default:
-        ThrowReaderException(CoderError,"ColormapTypeNotSupported",image)
+        ThrowReaderException(CoderError,"ColormapTypeNotSupported");
     }
     /*
       Initialize image structure.
@@ -490,7 +484,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       (viff_info.number_data_bands < 3 ? PseudoClass : DirectClass);
     image->columns=viff_info.rows;
     image->rows=viff_info.columns;
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -509,10 +503,10 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     else
       max_packets=number_pixels*viff_info.number_data_bands;
     viff_pixels=(unsigned char *)
-      AcquireMemory(bytes_per_pixel*max_packets*sizeof(Quantum));
+      AcquireMagickMemory(bytes_per_pixel*max_packets*sizeof(Quantum));
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-    (void) ReadBlob(image,bytes_per_pixel*max_packets,(char *) viff_pixels);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    (void) ReadBlob(image,bytes_per_pixel*max_packets,viff_pixels);
     lsb_first=1;
     if (*(char *) &lsb_first &&
         ((viff_info.machine_dependency != VFF_DEP_DECORDER) &&
@@ -612,7 +606,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     p=(unsigned char *) viff_pixels;
     if (viff_info.data_storage_type == VFF_TYP_BIT)
       {
-        unsigned int
+        MagickBooleanType
           polarity;
 
         /*
@@ -643,11 +637,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
                   ((*p) & (0x01 << bit) ? !polarity : polarity);
               p++;
             }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
       }
@@ -661,11 +655,11 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           indexes=GetIndexes(image);
           for (x=0; x < (long) image->columns; x++)
             indexes[x]=(*p++);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
         }
       else
@@ -673,7 +667,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           /*
             Convert DirectColor scanline.
           */
-          number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+          number_pixels=(MagickSizeType) image->columns*image->rows;
           for (y=0; y < (long) image->rows; y++)
           {
             q=SetImagePixels(image,0,y,image->columns,1);
@@ -695,20 +689,23 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
               p++;
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
-                  break;
+              if (QuantumTick(y,image->rows) != 0)
+                {
+                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         }
-    LiberateMemory((void **) &viff_pixels);
+    viff_pixels=(unsigned char *) RelinquishMagickMemory(viff_pixels);
     if (image->storage_class == PseudoClass)
-      SyncImage(image);
-    if (EOFBlob(image))
+      (void) SyncImage(image);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -718,7 +715,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    count=ReadBlob(image,1,(char *) &viff_info.identifier);
+    count=ReadBlob(image,1,&viff_info.identifier);
     if ((count != 0) && (viff_info.identifier == 0xab))
       {
         /*
@@ -733,14 +730,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while ((count != 0) && (viff_info.identifier == 0xab));
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -754,7 +749,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterVIFFImage adds attributes for the VIFF image format to
+%  RegisterVIFFImage() adds attributes for the VIFF image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -772,16 +767,17 @@ ModuleExport void RegisterVIFFImage(void)
     *entry;
 
   entry=SetMagickInfo("VIFF");
-  entry->decoder=(DecoderHandler) ReadVIFFImage;
-  entry->encoder=(EncoderHandler) WriteVIFFImage;
-  entry->magick=(MagickHandler) IsVIFF;
+  entry->decoder=(DecoderHandler *) ReadVIFFImage;
+  entry->encoder=(EncoderHandler *) WriteVIFFImage;
+  entry->magick=(MagickHandler *) IsVIFF;
   entry->description=AcquireString("Khoros Visualization image");
   entry->module=AcquireString("VIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XV");
-  entry->decoder=(DecoderHandler) ReadVIFFImage;
-  entry->encoder=(EncoderHandler) WriteVIFFImage;
+  entry->decoder=(DecoderHandler *) ReadVIFFImage;
+  entry->encoder=(EncoderHandler *) WriteVIFFImage;
   entry->description=AcquireString("Khoros Visualization image");
+  entry->module=AcquireString("VIFF");
   (void) RegisterMagickInfo(entry);
 }
 
@@ -796,7 +792,7 @@ ModuleExport void RegisterVIFFImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterVIFFImage removes format registrations made by the
+%  UnregisterVIFFImage() removes format registrations made by the
 %  VIFF module from the list of supported formats.
 %
 %  The format of the UnregisterVIFFImage method is:
@@ -821,15 +817,15 @@ ModuleExport void UnregisterVIFFImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteVIFFImage writes an image to a file in the VIFF image format.
+%  WriteVIFFImage() writes an image to a file in the VIFF image format.
 %
 %  The format of the WriteVIFFImage method is:
 %
-%      unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteVIFFImage return True if the image is written.
+%    o status: Method WriteVIFFImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -839,7 +835,7 @@ ModuleExport void UnregisterVIFFImage(void)
 %
 %
 */
-static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,Image *image)
 {
 #define VFF_CM_genericRGB  15
 #define VFF_CM_NONE  0
@@ -897,12 +893,19 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   const ImageAttribute
     *attribute;
 
-  ExtendedSignedIntegralType
-    number_pixels;
-
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
+  MagickSizeType
+    number_pixels,
+    packets;
+
   register const PixelPacket
     *p;
 
@@ -922,13 +925,6 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     buffer[8],
     *viff_pixels;
 
-  unsigned int
-    status;
-
-  unsigned long
-    packets,
-    scene;
-
   ViffInfo
     viff_info;
 
@@ -939,10 +935,12 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
-  memset(&viff_info,0,sizeof(ViffInfo));
+  if (status == MagickFalse)
+    return(status);
+  (void) ResetMagickMemory(&viff_info,0,sizeof(ViffInfo));
   scene=0;
   do
   {
@@ -981,7 +979,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     viff_info.map_subrows=0;
     viff_info.map_enable=1;  /* no colormap */
     viff_info.maps_per_cycle=0;
-    number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+    number_pixels=(MagickSizeType) image->columns*image->rows;
     if (image->storage_class == DirectClass)
       {
         /*
@@ -998,7 +996,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
         viff_info.color_space_model=VFF_CM_NONE;
         viff_info.data_storage_type=VFF_TYP_1_BYTE;
         packets=number_pixels;
-        if (!IsGrayImage(image,&image->exception))
+        if (IsGrayImage(image,&image->exception) == MagickFalse)
           {
             /*
               Colormapped VIFF raster.
@@ -1029,8 +1027,8 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     buffer[5]=viff_info.reserve[0];
     buffer[6]=viff_info.reserve[1];
     buffer[7]=viff_info.reserve[2];
-    (void) WriteBlob(image,8,(char *) buffer);
-    (void) WriteBlob(image,512,(char *) viff_info.comment);
+    (void) WriteBlob(image,8,buffer);
+    (void) WriteBlob(image,512,(unsigned char *) viff_info.comment);
     (void) WriteBlobMSBLong(image,viff_info.rows);
     (void) WriteBlobMSBLong(image,viff_info.columns);
     (void) WriteBlobMSBLong(image,viff_info.subrows);
@@ -1059,16 +1057,16 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to VIFF raster pixels.
     */
-    viff_pixels=(unsigned char *) AcquireMemory(packets);
+    viff_pixels=(unsigned char *) AcquireMagickMemory(packets);
     if (viff_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     q=viff_pixels;
     if (image->storage_class == DirectClass)
       {
         /*
           Convert DirectClass packet to VIFF RGB pixel.
         */
-        number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
+        number_pixels=(MagickSizeType) image->columns*image->rows;
         for (y=0; y < (long) image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -1079,19 +1077,23 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             *q=ScaleQuantumToChar(p->red);
             *(q+number_pixels)=ScaleQuantumToChar(p->green);
             *(q+number_pixels*2)=ScaleQuantumToChar(p->blue);
-            if (image->matte)
+            if (image->matte != MagickFalse)
               *(q+number_pixels*3)=ScaleQuantumToChar(MaxRGB-p->opacity);
             p++;
             q++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       }
     else
-      if (!IsGrayImage(image,&image->exception))
+      if (IsGrayImage(image,&image->exception) == MagickFalse)
         {
           unsigned char
             *viff_colormap;
@@ -1099,10 +1101,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
           /*
             Dump colormap to file.
           */
-          viff_colormap=(unsigned char *) AcquireMemory(3*image->colors);
+          viff_colormap=(unsigned char *) AcquireMagickMemory(3*image->colors);
           if (viff_colormap == (unsigned char *) NULL)
-            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",
-              image);
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
           q=viff_colormap;
           for (i=0; i < (long) image->colors; i++)
             *q++=ScaleQuantumToChar(image->colormap[i].red);
@@ -1110,8 +1111,8 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             *q++=ScaleQuantumToChar(image->colormap[i].green);
           for (i=0; i < (long) image->colors; i++)
             *q++=ScaleQuantumToChar(image->colormap[i].blue);
-          (void) WriteBlob(image,3*image->colors,(char *) viff_colormap);
-          LiberateMemory((void **) &viff_colormap);
+          (void) WriteBlob(image,3*image->colors,viff_colormap);
+          viff_colormap=(unsigned char *) RelinquishMagickMemory(viff_colormap);
           /*
             Convert PseudoClass packet to VIFF colormapped pixels.
           */
@@ -1125,9 +1126,13 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) image->columns; x++)
               *q++=indexes[x];
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
-                if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                  break;
+              if (QuantumTick(y,image->rows) != 0)
+                {
+                  status=MagickMonitor(SaveImageTag,y,image->rows,
+                    &image->exception);
+                  if (status == MagickFalse)
+                    break;
+                }
           }
         }
       else
@@ -1175,11 +1180,11 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
               if (bit != 0)
                 *q++=byte >> (8-bit);
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if (QuantumTick(y,image->rows) != 0)
                   {
                     status=MagickMonitor(SaveImageTag,y,image->rows,
                       &image->exception);
-                    if (status == False)
+                    if (status == MagickFalse)
                       break;
                   }
             }
@@ -1201,28 +1206,25 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if (QuantumTick(y,image->rows) != 0)
                   {
                     status=MagickMonitor(SaveImageTag,y,image->rows,
                       &image->exception);
-                    if (status == False)
+                    if (status == MagickFalse)
                       break;
                   }
             }
           }
-    (void) WriteBlob(image,packets,(char *) viff_pixels);
-    LiberateMemory((void **) &viff_pixels);
+    (void) WriteBlob(image,packets,viff_pixels);
+    viff_pixels=(unsigned char *) RelinquishMagickMemory(viff_pixels);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
