@@ -10,39 +10,26 @@
 %                                   X   X                                     %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write Image from/to X11 Server.                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,25 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/client.h"
+#include "magick/display.h"
 #include "magick/error.h"
+#include "magick/image.h"
+#include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 #include "magick/xwindow.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+#if defined(HasX11)
+static MagickBooleanType
   WriteXImage(const ImageInfo *,Image *);
+#endif
 
 #if defined(HasX11)
 /*
@@ -78,7 +73,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Procedure ReadXImage reads an image from an X window.
+%  Procedure ReadXImage() reads an image from an X window.
 %
 %  The format of the ReadXImage method is:
 %
@@ -100,13 +95,6 @@ static Image *ReadXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   XGetImportInfo(&ximage_info);
   return(XImportImage(image_info,&ximage_info));
 }
-#else
-static Image *ReadXImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  (void) ThrowException(exception,MissingDelegateError,"XWindowLibraryIsNotAvailable",
-    image_info->filename);
-  return((Image *) NULL);
-}
 #endif
 
 /*
@@ -120,7 +108,7 @@ static Image *ReadXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterXImage adds attributes for the X image format to
+%  RegisterXImage() adds attributes for the X image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -138,9 +126,10 @@ ModuleExport void RegisterXImage(void)
     *entry;
 
   entry=SetMagickInfo("X");
-  entry->decoder=(DecoderHandler) ReadXImage;
-  entry->encoder=(EncoderHandler) WriteXImage;
-  entry->adjoin=False;
+#if defined(HasX11)
+  entry->decoder=(DecoderHandler *) ReadXImage;
+  entry->encoder=(EncoderHandler *) WriteXImage;
+#endif
   entry->description=AcquireString("X Image");
   entry->module=AcquireString("X");
   (void) RegisterMagickInfo(entry);
@@ -157,7 +146,7 @@ ModuleExport void RegisterXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterXImage removes format registrations made by the
+%  UnregisterXImage() removes format registrations made by the
 %  X module from the list of supported formats.
 %
 %  The format of the UnregisterXImage method is:
@@ -182,15 +171,15 @@ ModuleExport void UnregisterXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteXImage writes an image to an X server.
+%  WriteXImage() writes an image to an X server.
 %
 %  The format of the WriteXImage method is:
 %
-%      unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXImage return True if the image is displayed on
+%    o status: Method WriteXImage returns True if the image is displayed on
 %      the X server.  False is returned is there is a memory shortage or if
 %      the image file fails to write.
 %
@@ -200,52 +189,8 @@ ModuleExport void UnregisterXImage(void)
 %
 %
 */
-static unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXImage(const ImageInfo *image_info,Image *image)
 {
-  char
-    *client_name;
-
-  Display
-    *display;
-
-  unsigned long
-    state;
-
-  XResourceInfo
-    resource_info;
-
-  XrmDatabase
-    resource_database;
-
-  /*
-    Open X server connection.
-  */
-  display=XOpenDisplay(image_info->server_name);
-  if (display == (Display *) NULL)
-    ThrowWriterException(XServerError,"UnableToOpenXServer",image);
-  /*
-    Set our forgiving error handler.
-  */
-  (void) XSetErrorHandler(XError);
-  /*
-    Get user defaults from X resource database.
-  */
-  client_name=SetClientName((char *) NULL);
-  resource_database=XGetResourceDatabase(display,client_name);
-  XGetResourceInfo(resource_database,client_name,&resource_info);
-  resource_info.immutable=True;
-  /*
-    Display image.
-  */
-  state=DefaultState;
-  (void) XDisplayImage(display,&resource_info,&client_name,1,&image,&state);
-  (void) XCloseDisplay(display);
-  return(True);
-}
-#else
-static unsigned int WriteXImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
-    image->filename);
+  return(DisplayImages(image_info,image));
 }
 #endif
