@@ -10,39 +10,26 @@
 %                            X   X  BBBB   M   M                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                  Read/Write X Windows System Bitmap Format.                 %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,21 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteXBMImage(const ImageInfo *,Image *);
 
 /*
@@ -78,16 +69,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsXBM returns True if the image format type, identified by the
+%  IsXBM() returns MagickTrue if the image format type, identified by the
 %  magick string, is XBM.
 %
 %  The format of the IsXBM method is:
 %
-%      unsigned int IsXBM(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXBM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXBM returns True if the image format type is XBM.
+%    o status:  Method IsXBM() returns MagickTrue if the image format type is XBM.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -96,13 +87,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsXBM(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXBM(const unsigned char *magick,const size_t length)
 {
   if (length < 7)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"#define",7) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -116,7 +107,7 @@ static unsigned int IsXBM(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadXBMImage reads an X11 bitmap image file and returns it.  It
+%  ReadXBMImage() reads an X11 bitmap image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -155,13 +146,13 @@ static int XBMInteger(Image *image,short int *hex_digits)
         break;
       }
     c&=0xff;
-    if (isxdigit(c))
+    if (isxdigit(c) != MagickFalse)
       {
-        value=(value << 4)+hex_digits[c];
+        value=(int) ((unsigned long) value << 4)+hex_digits[c];
         flag++;
         continue;
       }
-    if ((hex_digits[c]) < 0 && flag)
+    if ((hex_digits[c]) < 0 && (flag != 0))
       break;
   }
   return(value);
@@ -176,12 +167,12 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  int
-    bit;
-
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -201,10 +192,8 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned char
     *data;
 
-  unsigned int
-    status;
-
   unsigned long
+    bit,
     byte,
     bytes_per_line,
     padding,
@@ -216,12 +205,17 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read X bitmap header.
   */
@@ -262,20 +256,23 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (LocaleCompare("bits[]",(char *) p) == 0)
       break;
   }
-  if ((image->columns == 0) || (image->rows == 0) || EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"NotAXBMImageFile",image);
+  if ((image->columns == 0) || (image->rows == 0) ||
+      (EOFBlob(image) != MagickFalse))
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   /*
     Initialize image structure.
   */
-  if (!AllocateImageColormap(image,image->colors))
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+  if (AllocateImageColormap(image,image->colors) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   padding=0;
-  if ((image->columns % 16) && ((image->columns % 16) < 9)  && (version == 10))
+  if (((image->columns % 16) != 0) &&
+      ((image->columns % 16) < 9) && (version == 10))
     padding=1;
   bytes_per_line=(image->columns+7)/8+padding;
-  data=(unsigned char *) AcquireMemory(bytes_per_line*image->rows);
+  data=(unsigned char *)
+    AcquireMagickMemory((size_t) bytes_per_line*image->rows);
   if (data == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize colormap.
   */
@@ -285,42 +282,42 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->colormap[1].red=0;
   image->colormap[1].green=0;
   image->colormap[1].blue=0;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Initialize hex values.
   */
-  hex_digits['0']=0;
-  hex_digits['1']=1;
-  hex_digits['2']=2;
-  hex_digits['3']=3;
-  hex_digits['4']=4;
-  hex_digits['5']=5;
-  hex_digits['6']=6;
-  hex_digits['7']=7;
-  hex_digits['8']=8;
-  hex_digits['9']=9;
-  hex_digits['A']=10;
-  hex_digits['B']=11;
-  hex_digits['C']=12;
-  hex_digits['D']=13;
-  hex_digits['E']=14;
-  hex_digits['F']=15;
-  hex_digits['a']=10;
-  hex_digits['b']=11;
-  hex_digits['c']=12;
-  hex_digits['d']=13;
-  hex_digits['e']=14;
-  hex_digits['f']=15;
-  hex_digits['x']=0;
-  hex_digits[' ']=(-1);
-  hex_digits[',']=(-1);
-  hex_digits['}']=(-1);
-  hex_digits['\n']=(-1);
-  hex_digits['\t']=(-1);
+  hex_digits[(int) '0']=0;
+  hex_digits[(int) '1']=1;
+  hex_digits[(int) '2']=2;
+  hex_digits[(int) '3']=3;
+  hex_digits[(int) '4']=4;
+  hex_digits[(int) '5']=5;
+  hex_digits[(int) '6']=6;
+  hex_digits[(int) '7']=7;
+  hex_digits[(int) '8']=8;
+  hex_digits[(int) '9']=9;
+  hex_digits[(int) 'A']=10;
+  hex_digits[(int) 'B']=11;
+  hex_digits[(int) 'C']=12;
+  hex_digits[(int) 'D']=13;
+  hex_digits[(int) 'E']=14;
+  hex_digits[(int) 'F']=15;
+  hex_digits[(int) 'a']=10;
+  hex_digits[(int) 'b']=11;
+  hex_digits[(int) 'c']=12;
+  hex_digits[(int) 'd']=13;
+  hex_digits[(int) 'e']=14;
+  hex_digits[(int) 'f']=15;
+  hex_digits[(int) 'x']=0;
+  hex_digits[(int) ' ']=(-1);
+  hex_digits[(int) ',']=(-1);
+  hex_digits[(int) '}']=(-1);
+  hex_digits[(int) '\n']=(-1);
+  hex_digits[(int) '\t']=(-1);
   /*
     Read hex image data.
   */
@@ -328,15 +325,15 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (version == 10)
     for (i=0; i < (long) (bytes_per_line*image->rows); (i+=2))
     {
-      value=XBMInteger(image,hex_digits);
+      value=(unsigned long) XBMInteger(image,hex_digits);
       *p++=(unsigned char) value;
-      if (!padding || ((i+2) % bytes_per_line))
+      if ((padding == 0) || (((i+2) % bytes_per_line) != 0))
         *p++=(unsigned char) (value >> 8);
     }
   else
     for (i=0; i < (long) (bytes_per_line*image->rows); i++)
     {
-      value=XBMInteger(image,hex_digits);
+      value=(unsigned long) XBMInteger(image,hex_digits);
       *p++=(unsigned char) value;
     }
   /*
@@ -354,23 +351,23 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     for (x=0; x < (long) image->columns; x++)
     {
       if (bit == 0)
-        byte=(*p++);
-      indexes[x]=byte & 0x01 ? 0x01 : 0x00;
+        byte=(unsigned long) (*p++);
+      indexes[x]=(IndexPacket) ((byte & 0x01) != 0 ? 0x01 : 0x00);
       bit++;
       byte>>=1;
       if (bit == 8)
         bit=0;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
-  LiberateMemory((void **) &data);
+  data=(unsigned char *) RelinquishMagickMemory(data);
   SyncImage(image);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -384,7 +381,7 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterXBMImage adds attributes for the XBM image format to
+%  RegisterXBMImage() adds attributes for the XBM image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -402,10 +399,10 @@ ModuleExport void RegisterXBMImage(void)
     *entry;
 
   entry=SetMagickInfo("XBM");
-  entry->decoder=(DecoderHandler) ReadXBMImage;
-  entry->encoder=(EncoderHandler) WriteXBMImage;
-  entry->magick=(MagickHandler) IsXBM;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadXBMImage;
+  entry->encoder=(EncoderHandler *) WriteXBMImage;
+  entry->magick=(MagickHandler *) IsXBM;
+  entry->adjoin=MagickFalse;
   entry->description=
     AcquireString("X Windows system bitmap (black and white)");
   entry->module=AcquireString("XBM");
@@ -423,7 +420,7 @@ ModuleExport void RegisterXBMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterXBMImage removes format registrations made by the
+%  UnregisterXBMImage() removes format registrations made by the
 %  XBM module from the list of supported formats.
 %
 %  The format of the UnregisterXBMImage method is:
@@ -447,15 +444,15 @@ ModuleExport void UnregisterXBMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Procedure WriteXBMImage writes an image to a file in the X bitmap format.
+%  Procedure WriteXBMImage() writes an image to a file in the X bitmap format.
 %
 %  The format of the WriteXBMImage method is:
 %
-%      unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXBMImage return True if the image is written.
+%    o status: Method WriteXBMImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -465,15 +462,21 @@ ModuleExport void UnregisterXBMImage(void)
 %
 %
 */
-static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
 {
   char
     basename[MaxTextExtent],
     buffer[MaxTextExtent];
 
-  int
+  IndexPacket
+    polarity;
+
+  long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -483,16 +486,12 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
   register IndexPacket
     *indexes;
 
-  unsigned char
-    bit,
-    byte,
-    polarity;
-
-  unsigned int
-    status;
+  ssize_t
+    count;
 
   unsigned long
-    count;
+    bit,
+    byte;
 
   /*
     Open output image file.
@@ -501,37 +500,43 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Write X bitmap header.
   */
   GetPathComponent(image->filename,BasePath,basename);
-  FormatString(buffer,"#define %.1024s_width %lu\n",basename,image->columns);
-  (void) WriteBlob(image,strlen(buffer),buffer);
-  FormatString(buffer,"#define %.1024s_height %lu\n",basename,image->rows);
-  (void) WriteBlob(image,strlen(buffer),buffer);
-  FormatString(buffer,"static char %.1024s_bits[] = {\n",basename);
-  (void) WriteBlob(image,strlen(buffer),buffer);
+  (void) FormatMagickString(buffer,MaxTextExtent,"#define %s_width %lu\n",
+    basename,image->columns);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
+  (void) FormatMagickString(buffer,MaxTextExtent,"#define %s_height %lu\n",
+    basename,image->rows);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "static char %s_bits[] = {\n",basename);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
   (void) strcpy(buffer," ");
-  (void) WriteBlob(image,strlen(buffer),buffer);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
   /*
     Convert MIFF to X bitmap pixels.
   */
-  SetImageType(image,BilevelType);
-  polarity=PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2);
+  (void) SetImageType(image,BilevelType);
+  polarity=(IndexPacket)
+    (PixelIntensityToQuantum(&image->colormap[0]) < (MaxRGB/2));
   if (image->colors == 2)
-    polarity=PixelIntensityToQuantum(&image->colormap[0]) <
-      PixelIntensityToQuantum(&image->colormap[1]);
+    polarity=(IndexPacket) (PixelIntensityToQuantum(&image->colormap[0]) <
+      PixelIntensityToQuantum(&image->colormap[1]));
   bit=0;
   byte=0;
   count=0;
   x=0;
   y=0;
   (void) strcpy(buffer," ");
-  (void) WriteBlob(image,strlen(buffer),buffer);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
   for (y=0; y < (long) image->rows; y++)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -549,13 +554,14 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
           /*
             Write a bitmap byte to the image file.
           */
-          FormatString(buffer,"0x%02X, ",(unsigned int) (byte & 0xff));
-          (void) WriteBlob(image,strlen(buffer),buffer);
+          (void) FormatMagickString(buffer,MaxTextExtent,"0x%02X, ",
+            (unsigned int) (byte & 0xff));
+          (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
           count++;
           if (count == 12)
             {
               (void) strcpy(buffer,"\n  ");
-              (void) WriteBlob(image,strlen(buffer),buffer);
+              (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
               count=0;
             };
           bit=0;
@@ -569,24 +575,25 @@ static unsigned int WriteXBMImage(const ImageInfo *image_info,Image *image)
           Write a bitmap byte to the image file.
         */
         byte>>=(8-bit);
-        FormatString(buffer,"0x%02X, ",(unsigned int) (byte & 0xff));
-        (void) WriteBlob(image,strlen(buffer),buffer);
+        (void) FormatMagickString(buffer,MaxTextExtent,"0x%02X, ",
+          (unsigned int) (byte & 0xff));
+        (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
         count++;
         if (count == 12)
           {
             (void) strcpy(buffer,"\n  ");
-            (void) WriteBlob(image,strlen(buffer),buffer);
+            (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
             count=0;
           };
         bit=0;
         byte=0;
       };
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
         break;
   }
   (void) strcpy(buffer,"};\n");
-  (void) WriteBlob(image,strlen(buffer),buffer);
+  (void) WriteBlob(image,strlen(buffer),(unsigned char *) buffer);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
