@@ -10,39 +10,26 @@
 %                            X   X  P      M   M                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                  Read/Write X Windows system Pixmap Format.                 %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,19 +41,28 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/quantize.h"
+#include "magick/resize.h"
+#include "magick/resource_.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePICONImage(const ImageInfo *,Image *),
   WriteXPMImage(const ImageInfo *,Image *);
 
@@ -81,16 +77,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsXPM returns True if the image format type, identified by the
+%  IsXPM() returns MagickTrue if the image format type, identified by the
 %  magick string, is XPM.
 %
 %  The format of the IsXPM method is:
 %
-%      unsigned int IsXPM(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXPM returns True if the image format type is XPM.
+%    o status:  Method IsXPM() returns MagickTrue if the image format type is XPM.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -99,13 +95,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsXPM(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXPM(const unsigned char *magick,const size_t length)
 {
   if (length < 9)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick+1,"* XPM *",7) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -119,7 +115,7 @@ static unsigned int IsXPM(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadXPMImage reads an X11 pixmap image file and returns it.  It
+%  ReadXPMImage() reads an X11 pixmap image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -164,7 +160,7 @@ static char *ParseColor(char *data)
     {
       if (*r != *q)
         continue;
-      if (!isspace((int) (*(r-1))))
+      if (isspace((int) ((unsigned char) (*(r-1)))) == 0)
         continue;
       p=r;
       for ( ; ; )
@@ -192,7 +188,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  int
+  ssize_t
     count;
 
   long
@@ -200,6 +196,9 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     none,
     y;
 
+  MagickBooleanType
+    status;
+
   register char
     *p,
     *q;
@@ -216,11 +215,10 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register long
     i;
 
-  unsigned int
-    status;
+  size_t
+    length;
 
   unsigned long
-    length,
     width;
 
   /*
@@ -228,17 +226,22 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read XPM file.
   */
   length=MaxTextExtent;
-  xpm_buffer=(char *) AcquireMemory(length);
+  xpm_buffer=(char *) AcquireMagickMemory((size_t) length);
   p=xpm_buffer;
   if (xpm_buffer != (char *) NULL)
     while (ReadBlobString(image,p) != (char *) NULL)
@@ -249,16 +252,16 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ((*p == '}') && (*(p+1) == ';'))
         break;
       p+=strlen(p);
-      if ((p-xpm_buffer+MaxTextExtent+1) < (long) length)
+      if (((size_t) (p-xpm_buffer)+MaxTextExtent+1) < length)
         continue;
       length<<=1;
-      ReacquireMemory((void **) &xpm_buffer,length);
+      xpm_buffer=(char *) ResizeMagickMemory(xpm_buffer,(size_t) length);
       if (xpm_buffer == (char *) NULL)
         break;
       p=xpm_buffer+strlen(xpm_buffer);
     }
   if (xpm_buffer == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Remove comments.
   */
@@ -267,14 +270,14 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     if (*p != '"')
       continue;
-    count=sscanf(p+1,"%lu %lu %lu %lu",&image->columns,&image->rows,
+    count=(ssize_t) sscanf(p+1,"%lu %lu %lu %lu",&image->columns,&image->rows,
       &image->colors,&width);
     if (count == 4)
       break;
   }
   if ((count != 4) || (width > 2) || (image->columns == 0) ||
       (image->rows == 0) || (image->colors == 0))
-    ThrowReaderException(CorruptImageError,"NotAXPMImageFile",image)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   image->depth=16;
   /*
     Remove unquoted characters.
@@ -287,26 +290,27 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     for (q=p+1; *q != '\0'; q++)
       if (*q == '"')
         break;
-    (void) strncpy(xpm_buffer+i,p+1,q-p-1);
+    (void) strncpy(xpm_buffer+i,p+1,(size_t) (q-p)-1);
     i+=q-p-1;
     xpm_buffer[i++]='\n';
     p=q+1;
   }
   xpm_buffer[i]='\0';
   textlist=StringToList(xpm_buffer);
-  LiberateMemory((void **) &xpm_buffer);
+  xpm_buffer=(char *) RelinquishMagickMemory(xpm_buffer);
   if (textlist == (char **) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize image structure.
   */
-  keys=(char **) AcquireMemory(image->colors*sizeof(char *));
-  if (!AllocateImageColormap(image,image->colors) || (keys == (char **) NULL))
+  keys=(char **) AcquireMagickMemory((size_t) image->colors*sizeof(char *));
+  if ((AllocateImageColormap(image,image->colors) == MagickFalse) ||
+      (keys == (char **) NULL))
     {
       for (i=0; textlist[i] != (char *) NULL; i++)
-        LiberateMemory((void **) &textlist[i]);
-      LiberateMemory((void **) &textlist);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
+        textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+      textlist=(char **) RelinquishMagickMemory(textlist);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Read image colormap.
@@ -318,17 +322,17 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     p=textlist[i++];
     if (p == (char *) NULL)
       break;
-    keys[j]=(char *) AcquireMemory(width+1);
+    keys[j]=(char *) AcquireMagickMemory((size_t) width+1);
     if (keys[j] == (char *) NULL)
       {
         for (i=0; textlist[i] != (char *) NULL; i++)
-          LiberateMemory((void **) &textlist[i]);
-        LiberateMemory((void **) &textlist);
-        LiberateMemory((void **) &keys);
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image)
+          textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+        textlist=(char **) RelinquishMagickMemory(textlist);
+        keys=(char **) RelinquishMagickMemory(keys);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       }
     keys[j][width]='\0';
-    (void) strncpy(keys[j],p,width);
+    (void) strncpy(keys[j],p,(size_t) width);
     /*
       Parse color.
     */
@@ -336,34 +340,34 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     q=ParseColor(p+width);
     if (q != (char *) NULL)
       {
-        while (!isspace((int) (*q)) && (*q != '\0'))
+        while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
           q++;
-        (void) strncpy(target,q,MaxTextExtent-1);
+        (void) CopyMagickString(target,q,MaxTextExtent);
         q=ParseColor(target);
         if (q != (char *) NULL)
           *q='\0';
       }
-    Strip(target);
+    StripString(target);
     if (LocaleCompare(target,"none") == 0)
       {
         image->storage_class=DirectClass;
-        image->matte=True;
+        image->matte=MagickTrue;
         none=j;
         (void) strcpy(target,"black");
       }
-    if (!QueryColorDatabase(target,&image->colormap[j],exception))
+    if (QueryColorDatabase(target,&image->colormap[j],exception) == MagickFalse)
       break;
   }
   if (j < (long) image->colors)
     {
       for (i=0; textlist[i] != (char *) NULL; i++)
-        LiberateMemory((void **) &textlist[i]);
-      LiberateMemory((void **) &textlist);
-      ThrowReaderException(CorruptImageError,"CorruptXPMImage",image)
+        textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+      textlist=(char **) RelinquishMagickMemory(textlist);
+      ThrowReaderException(CorruptImageError,"CorruptImage");
     }
   j=0;
   key[width]='\0';
-  if (!image_info->ping)
+  if (image_info->ping == MagickFalse)
     {
       /*
         Read image pixels.
@@ -379,7 +383,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          (void) strncpy(key,p,width);
+          (void) strncpy(key,p,(size_t) width);
           if (strcmp(key,keys[j]) != 0)
             for (j=0; j < (long) Max(image->colors-1,1); j++)
               if (strcmp(key,keys[j]) == 0)
@@ -392,23 +396,23 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           r++;
           p+=width;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
       if (y < (long) image->rows)
-        ThrowReaderException(CorruptImageError,"NotEnoughPixelData",image);
+        ThrowReaderException(CorruptImageError,"NotEnoughPixelData");
     }
   /*
     Free resources.
   */
   for (i=0; i < (long) image->colors; i++)
-    LiberateMemory((void **) &keys[i]);
-  LiberateMemory((void **) &keys);
+    keys[i]=(char *) RelinquishMagickMemory(keys[i]);
+  keys=(char **) RelinquishMagickMemory(keys);
   for (i=0; textlist[i] != (char *) NULL; i++)
-    LiberateMemory((void **) &textlist[i]);
-  LiberateMemory((void **) &textlist);
+    textlist[i]=(char *) RelinquishMagickMemory(textlist[i]);
+  textlist=(char **) RelinquishMagickMemory(textlist);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -422,7 +426,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterXPMImage adds attributes for the XPM image format to
+%  RegisterXPMImage() adds attributes for the XPM image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -440,24 +444,25 @@ ModuleExport void RegisterXPMImage(void)
     *entry;
 
   entry=SetMagickInfo("PICON");
-  entry->decoder=(DecoderHandler) ReadXPMImage;
-  entry->encoder=(EncoderHandler) WritePICONImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadXPMImage;
+  entry->encoder=(EncoderHandler *) WritePICONImage;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Personal Icon");
-  entry->module=AcquireString("PICON");
+  entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PM");
-  entry->decoder=(DecoderHandler) ReadXPMImage;
-  entry->encoder=(EncoderHandler) WriteXPMImage;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadXPMImage;
+  entry->encoder=(EncoderHandler *) WriteXPMImage;
+  entry->adjoin=MagickFalse;
+  entry->stealth=MagickTrue;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XPM");
-  entry->decoder=(DecoderHandler) ReadXPMImage;
-  entry->encoder=(EncoderHandler) WriteXPMImage;
-  entry->magick=(MagickHandler) IsXPM;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadXPMImage;
+  entry->encoder=(EncoderHandler *) WriteXPMImage;
+  entry->magick=(MagickHandler *) IsXPM;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("X Windows system pixmap (color)");
   entry->module=AcquireString("XPM");
   (void) RegisterMagickInfo(entry);
@@ -474,7 +479,7 @@ ModuleExport void RegisterXPMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterXPMImage removes format registrations made by the
+%  UnregisterXPMImage() removes format registrations made by the
 %  XPM module from the list of supported formats.
 %
 %  The format of the UnregisterXPMImage method is:
@@ -500,16 +505,16 @@ ModuleExport void UnregisterXPMImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Procedure WritePICONImage writes an image to a file in the Personal Icon
+%  Procedure WritePICONImage() writes an image to a file in the Personal Icon
 %  format.
 %
 %  The format of the WritePICONImage method is:
 %
-%      unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePICONImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePICONImage return True if the image is written.
+%    o status: Method WritePICONImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -519,7 +524,7 @@ ModuleExport void UnregisterXPMImage(void)
 %
 %
 */
-static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WritePICONImage(const ImageInfo *image_info,Image *image)
 {
 #define ColormapExtent  155
 #define GraymapExtent  95
@@ -570,13 +575,18 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     *picon,
     *map;
 
-  int
-    j;
+  ImageInfo
+    *blob_info;
 
   long
+    j,
     k,
     y;
 
+  MagickBooleanType
+    status,
+    transparent;
+
   RectangleInfo
     geometry;
 
@@ -593,10 +603,6 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   register PixelPacket
     *q;
 
-  unsigned int
-    status,
-    transparent;
-
   unsigned long
     characters_per_pixel,
     colors;
@@ -608,37 +614,43 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   SetGeometry(image,&geometry);
   (void) ParseMetaGeometry(PiconGeometry,&geometry.x,&geometry.y,
     &geometry.width,&geometry.height);
   picon=ResizeImage(image,geometry.width,geometry.height,TriangleFilter,1.0,
     &image->exception);
+  blob_info=CloneImageInfo(image_info);
+  (void) AcquireUniqueFilename(blob_info->filename);
   if ((image_info->type != TrueColorType) &&
-      IsGrayImage(image,&image->exception))
-    map=BlobToImage(image_info,Graymap,GraymapExtent,&image->exception);
+      (IsGrayImage(image,&image->exception) != MagickFalse))
+    map=BlobToImage(blob_info,Graymap,GraymapExtent,&image->exception);
   else
-    map=BlobToImage(image_info,Colormap,ColormapExtent,&image->exception);
+    map=BlobToImage(blob_info,Colormap,ColormapExtent,&image->exception);
+  DestroyImageInfo(blob_info);
   if ((picon == (Image *) NULL) || (map == (Image *) NULL))
-    return(False);
+    return(MagickFalse);
   status=MapImage(picon,map,image_info->dither);
+  (void) RelinquishUniqueFileResource(blob_info->filename);
   DestroyImage(map);
-  transparent=False;
+  transparent=MagickFalse;
   if (picon->storage_class == PseudoClass)
     {
       CompressImageColormap(picon);
-      if (picon->matte)
-        transparent=True;
+      if (picon->matte != MagickFalse)
+        transparent=MagickTrue;
     }
   else
     {
       /*
         Convert DirectClass to PseudoClass picon.
       */
-      if (picon->matte)
+      if (picon->matte != MagickFalse)
         {
           /*
             Map all the transparent pixels.
@@ -651,22 +663,25 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) picon->columns; x++)
             {
               if (q->opacity == TransparentOpacity)
-                transparent=True;
+                transparent=MagickTrue;
               else
                 q->opacity=OpaqueOpacity;
               q++;
             }
-            if (!SyncImagePixels(picon))
+            if (SyncImagePixels(picon) == MagickFalse)
               break;
           }
         }
-      SetImageType(picon,PaletteType);
+      (void) SetImageType(picon,PaletteType);
     }
   colors=picon->colors;
-  if (transparent)
+  if (transparent != MagickFalse)
     {
       colors++;
-      ReacquireMemory((void **) &picon->colormap,colors*sizeof(PixelPacket));
+      picon->colormap=(PixelPacket *) ResizeMagickMemory((void **)
+        &picon->colormap,(size_t) colors*sizeof(PixelPacket));
+      if (picon->colormap == (PixelPacket *) NULL)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
       for (y=0; y < (long) picon->rows; y++)
       {
         q=GetImagePixels(picon,0,y,picon->columns,1);
@@ -679,7 +694,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
             indexes[x]=(IndexPacket) picon->colors;
           q++;
         }
-        if (!SyncImagePixels(picon))
+        if (SyncImagePixels(picon) == MagickFalse)
           break;
       }
     }
@@ -694,11 +709,12 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
   */
   (void) WriteBlobString(image,"/* XPM */\n");
   GetPathComponent(picon->filename,BasePath,basename);
-  (void) FormatString(buffer,"static char *%.1024s[] = {\n",basename);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "static char *%s[] = {\n",basename);
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"/* columns rows colors chars-per-pixel */\n");
-  FormatString(buffer,"\"%lu %lu %lu %ld\",\n",picon->columns,picon->rows,
-    colors,characters_per_pixel);
+  (void) FormatMagickString(buffer,MaxTextExtent,"\"%lu %lu %lu %ld\",\n",
+    picon->columns,picon->rows,colors,characters_per_pixel);
   (void) WriteBlobString(image,buffer);
   for (i=0; i < (long) colors; i++)
   {
@@ -708,7 +724,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     picon->colormap[i].opacity=OpaqueOpacity;
     (void) QueryColorname(picon,picon->colormap+i,XPMCompliance,name,
       &picon->exception);
-    if (transparent)
+    if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
           (void) strcpy(name,"grey75");
@@ -724,7 +740,8 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
       symbol[j]=Cixel[k];
     }
     symbol[j]='\0';
-    FormatString(buffer,"\"%.1024s c %.1024s\",\n",symbol,name);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s c %s\",\n",
+       symbol,name);
     (void) WriteBlobString(image,buffer);
   }
   /*
@@ -740,7 +757,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"\"");
     for (x=0; x < (long) picon->columns; x++)
     {
-      k=indexes[x] % MaxCixels;
+      k=(long) (indexes[x] % MaxCixels);
       symbol[0]=Cixel[k];
       for (j=1; j < (long) characters_per_pixel; j++)
       {
@@ -748,20 +765,20 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
-      (void) strncpy(buffer,symbol,MaxTextExtent-1);
+      (void) CopyMagickString(buffer,symbol,MaxTextExtent);
       (void) WriteBlobString(image,buffer);
     }
-    FormatString(buffer,"\"%.1024s\n",
+    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
       (y == (long) (picon->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
-    if (QuantumTick(y,picon->rows))
-      if (!MagickMonitor(SaveImageTag,y,picon->rows,&image->exception))
+    if (QuantumTick(y,picon->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,picon->rows,&image->exception) == MagickFalse)
         break;
   }
   DestroyImage(picon);
   (void) WriteBlobString(image,"};\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -775,15 +792,15 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Procedure WriteXPMImage writes an image to a file in the X pixmap format.
+%  Procedure WriteXPMImage() writes an image to a file in the X pixmap format.
 %
 %  The format of the WriteXPMImage method is:
 %
-%      unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXPMImage return True if the image is written.
+%    o status: Method WriteXPMImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -793,7 +810,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
 %
 %
 */
-static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
 {
 #define MaxCixels  92
 
@@ -807,13 +824,15 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     name[MaxTextExtent],
     symbol[MaxTextExtent];
 
-  int
-    j;
-
   long
+    j,
     k,
     y;
 
+  MagickBooleanType
+    status,
+    transparent;
+
   register const PixelPacket
     *p;
 
@@ -827,10 +846,6 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   register PixelPacket
     *q;
 
-  unsigned int
-    status,
-    transparent;
-
   unsigned long
     characters_per_pixel,
     colors;
@@ -842,23 +857,25 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
-  transparent=False;
+  transparent=MagickFalse;
   if (image->storage_class == PseudoClass)
     {
       CompressImageColormap(image);
-      if (image->matte)
-        transparent=True;
+      if (image->matte != MagickFalse)
+        transparent=MagickTrue;
     }
   else
     {
       /*
         Convert DirectClass to PseudoClass image.
       */
-      if (image->matte)
+      if (image->matte != MagickFalse)
         {
           /*
             Map all the transparent pixels.
@@ -871,22 +888,25 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) image->columns; x++)
             {
               if (q->opacity == TransparentOpacity)
-                transparent=True;
+                transparent=MagickTrue;
               else
                 q->opacity=OpaqueOpacity;
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
           }
         }
-      SetImageType(image,PaletteType);
+      (void) SetImageType(image,PaletteType);
     }
   colors=image->colors;
-  if (transparent)
+  if (transparent != MagickFalse)
     {
       colors++;
-      ReacquireMemory((void **) &image->colormap,colors*sizeof(PixelPacket));
+      image->colormap=(PixelPacket *)
+        ResizeMagickMemory(image->colormap,(size_t) colors*sizeof(PixelPacket));
+      if (image->colormap == (PixelPacket *) NULL)
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationError");
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
@@ -899,7 +919,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
             indexes[x]=(IndexPacket) image->colors;
           q++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
       }
     }
@@ -914,11 +934,12 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
   */
   (void) WriteBlobString(image,"/* XPM */\n");
   GetPathComponent(image->filename,BasePath,basename);
-  (void) FormatString(buffer,"static char *%.1024s[] = {\n",basename);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "static char *%s[] = {\n",basename);
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"/* columns rows colors chars-per-pixel */\n");
-  FormatString(buffer,"\"%lu %lu %lu %ld\",\n",image->columns,
-    image->rows,colors,characters_per_pixel);
+  (void) FormatMagickString(buffer,MaxTextExtent,"\"%lu %lu %lu %ld\",\n",
+    image->columns,image->rows,colors,characters_per_pixel);
   (void) WriteBlobString(image,buffer);
   for (i=0; i < (long) colors; i++)
   {
@@ -928,7 +949,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     image->colormap[i].opacity=OpaqueOpacity;
     (void) QueryColorname(image,image->colormap+i,XPMCompliance,name,
       &image->exception);
-    if (transparent)
+    if (transparent != MagickFalse)
       {
         if (i == (long) (colors-1))
           (void) strcpy(name,"None");
@@ -944,7 +965,8 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
       symbol[j]=Cixel[k];
     }
     symbol[j]='\0';
-    FormatString(buffer,"\"%.1024s c %.1024s\",\n",symbol,name);
+    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s c %s\",\n",
+      symbol,name);
     (void) WriteBlobString(image,buffer);
   }
   /*
@@ -960,7 +982,7 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,"\"");
     for (x=0; x < (long) image->columns; x++)
     {
-      k=indexes[x] % MaxCixels;
+      k=(long) (indexes[x] % MaxCixels);
       symbol[0]=Cixel[k];
       for (j=1; j < (long) characters_per_pixel; j++)
       {
@@ -968,17 +990,17 @@ static unsigned int WriteXPMImage(const ImageInfo *image_info,Image *image)
         symbol[j]=Cixel[k];
       }
       symbol[j]='\0';
-      (void) strncpy(buffer,symbol,MaxTextExtent-1);
+      (void) CopyMagickString(buffer,symbol,MaxTextExtent);
       (void) WriteBlobString(image,buffer);
     }
-    FormatString(buffer,"\"%.1024s\n",
+    (void) FormatMagickString(buffer,MaxTextExtent,"\"%s\n",
       (y == (long) (image->rows-1) ? "" : ","));
     (void) WriteBlobString(image,buffer);
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == MagickFalse)
         break;
   }
   (void) WriteBlobString(image,"};\n");
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
