@@ -10,39 +10,26 @@
 %                            X   X   W W   DDDD                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                Read/Write X Windows System Window Dump Format.              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,18 +42,32 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
 #include "magick/error.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+#if defined(HasX11)
+#include "magick/xwindow.h"
+#if !defined(vms)
+#include <X11/XWDFile.h>
+#else
+#include "XWDFile.h"
+#endif
+#endif
 
 /*
   Forward declarations.
 */
-static unsigned int
+#if defined(HasX11)
+static MagickBooleanType
   WriteXWDImage(const ImageInfo *,Image *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -79,16 +80,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsXWD returns True if the image format type, identified by the
+%  IsXWD() returns MagickTrue if the image format type, identified by the
 %  magick string, is XWD.
 %
 %  The format of the IsXWD method is:
 %
-%      unsigned int IsXWD(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsXWD returns True if the image format type is XWD.
+%    o status:  Method IsXWD() returns MagickTrue if the image format type is XWD.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -97,22 +98,21 @@ static unsigned int
 %
 %
 */
-static unsigned int IsXWD(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsXWD(const unsigned char *magick,const size_t length)
 {
   if (length < 8)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick+1,"\000\000",2) == 0)
     {
       if (memcmp(magick+4,"\007\000\000",3) == 0)
-        return(True);
+        return(MagickTrue);
       if (memcmp(magick+5,"\000\000\007",3) == 0)
-        return(True);
+        return(MagickTrue);
     }
-  return(False);
+  return(MagickFalse);
 }
 
 #if defined(HasX11)
-#include "magick/xwindow.h"
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -124,7 +124,7 @@ static unsigned int IsXWD(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadXWDImage reads an X Window System window dump image file and
+%  ReadXWDImage() reads an X Window System window dump image file and
 %  returns it.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -152,12 +152,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  int
-    status;
+  IndexPacket
+    index;
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register IndexPacket
     *indexes;
 
@@ -174,15 +177,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     pixel;
 
   size_t
-    count,
     length;
 
+  ssize_t
+    count;
+
   unsigned long
     lsb_first;
 
-  unsigned short
-    index;
-
   XColor
     *colors;
 
@@ -197,18 +199,23 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
      Read in header information.
   */
-  count=ReadBlob(image,sz_XWDheader,(char *) &header);
+  count=ReadBlob(image,sz_XWDheader,(unsigned char *) &header);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader",image);
+    ThrowReaderException(CorruptImageError,"UnableToReadImageHeader");
   image->columns=header.pixmap_width;
   image->rows=header.pixmap_height;
   image->depth=8;
@@ -216,32 +223,32 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Ensure the header byte-order is most-significant byte first.
   */
   lsb_first=1;
-  if (*(char *) &lsb_first)
+  if ((int) (*(char *) &lsb_first) != 0)
     MSBOrderLong((unsigned char *) &header,sz_XWDheader);
   /*
     Check to see if the dump file is in the proper format.
   */
   if (header.file_version != XWD_FILE_VERSION)
-    ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch",image);
+    ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch");
   if (header.header_size < sz_XWDheader)
-    ThrowReaderException(CorruptImageError,"CorruptXWDImage",image);
-  length=header.header_size-sz_XWDheader;
-  comment=(char *) AcquireMemory(length+1);
+    ThrowReaderException(CorruptImageError,"CorruptImage");
+  length=(size_t) header.header_size-sz_XWDheader;
+  comment=(char *) AcquireMagickMemory(length+1);
   if (comment == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  count=ReadBlob(image,length,comment);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,length,(unsigned char *) comment);
   comment[length]='\0';
   (void) SetImageAttribute(image,"comment",comment);
-  LiberateMemory((void **) &comment);
+  comment=(char *) RelinquishMagickMemory(comment);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"UnableToReadWindowNameFromDumpFile",
-      image);
+    ThrowReaderException(CorruptImageError,
+      "UnableToReadWindowNameFromDumpFile");
   /*
     Initialize the X image.
   */
-  ximage=(XImage *) AcquireMemory(sizeof(XImage));
+  ximage=(XImage *) AcquireMagickMemory(sizeof(XImage));
   if (ximage == (XImage *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   ximage->depth=(int) header.pixmap_depth;
   ximage->format=(int) header.pixmap_format;
   ximage->xoffset=(int) header.xoffset;
@@ -257,9 +264,9 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ximage->red_mask=header.red_mask;
   ximage->green_mask=header.green_mask;
   ximage->blue_mask=header.blue_mask;
-  status=XInitImage(ximage);
-  if (status == False)
-    ThrowReaderException(CorruptImageError,"UnrecognizedXWDHeader",image);
+  status=(MagickBooleanType) XInitImage(ximage);
+  if (status == MagickFalse)
+    ThrowReaderException(CorruptImageError,"UnrecognizedXWDHeader");
   /*
     Read colormap.
   */
@@ -269,26 +276,27 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       XWDColor
         color;
 
-      colors=(XColor *) AcquireMemory(header.ncolors*sizeof(XColor));
+      colors=(XColor *)
+        AcquireMagickMemory((size_t) header.ncolors*sizeof(XColor));
       if (colors == (XColor *) NULL)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) header.ncolors; i++)
       {
-        count=ReadBlob(image,sz_XWDColor,(char *) &color);
+        count=ReadBlob(image,sz_XWDColor,(unsigned char *) &color);
         if (count == 0)
           ThrowReaderException(CorruptImageError,
-            "UnableToReadColormapFromDumpFile",image);
+            "UnableToReadColormapFromDumpFile");
         colors[i].pixel=color.pixel;
         colors[i].red=color.red;
         colors[i].green=color.green;
         colors[i].blue=color.blue;
-        colors[i].flags=color.flags;
+        colors[i].flags=(char) color.flags;
       }
       /*
         Ensure the header byte-order is most-significant byte first.
       */
       lsb_first=1;
-      if (*(char *) &lsb_first)
+      if ((int) (*(char *) &lsb_first) != 0)
         for (i=0; i < (long) header.ncolors; i++)
         {
           MSBOrderLong((unsigned char *) &colors[i].pixel,
@@ -301,28 +309,27 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Allocate the pixel buffer.
   */
   if (ximage->format == ZPixmap)
-    length=ximage->bytes_per_line*ximage->height;
+    length=(size_t) ximage->bytes_per_line*ximage->height;
   else
-    length=ximage->bytes_per_line*ximage->height*ximage->depth;
-  ximage->data=(char *) AcquireMemory(length);
+    length=(size_t) ximage->bytes_per_line*ximage->height*ximage->depth;
+  ximage->data=(char *) AcquireMagickMemory(length);
   if (ximage->data == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
-  count=ReadBlob(image,length,ximage->data);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  count=ReadBlob(image,length,(unsigned char *) ximage->data);
   if (count == 0)
-    ThrowReaderException(CorruptImageError,"UnableToReadPixmapFromDumpFile",
-      image);
+    ThrowReaderException(CorruptImageError,"UnableToReadPixmapFromDumpFile");
   /*
     Convert image to MIFF format.
   */
-  image->columns=ximage->width;
-  image->rows=ximage->height;
+  image->columns=(unsigned long) ximage->width;
+  image->rows=(unsigned long) ximage->height;
   if ((colors == (XColor *) NULL) || (ximage->red_mask != 0) ||
       (ximage->green_mask != 0) || (ximage->blue_mask != 0))
     image->storage_class=DirectClass;
   else
     image->storage_class=PseudoClass;
   image->colors=header.ncolors;
-  if (!image_info->ping)
+  if (image_info->ping == MagickFalse)
     switch (image->storage_class)
     {
       case DirectClass:
@@ -375,18 +382,18 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (x=0; x < (long) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
-              index=(unsigned short) ((pixel >> red_shift) & red_mask);
+              index=(IndexPacket) ((pixel >> red_shift) & red_mask);
               q->red=ScaleShortToQuantum(colors[index].red);
-              index=(unsigned short) ((pixel >> green_shift) & green_mask);
+              index=(IndexPacket) ((pixel >> green_shift) & green_mask);
               q->green=ScaleShortToQuantum(colors[index].green);
-              index=(unsigned short) ((pixel >> blue_shift) & blue_mask);
+              index=(IndexPacket) ((pixel >> blue_shift) & blue_mask);
               q->blue=ScaleShortToQuantum(colors[index].blue);
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
           }
         else
@@ -406,10 +413,10 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               q->blue=ScaleShortToQuantum((color*65535L)/blue_mask);
               q++;
             }
-            if (!SyncImagePixels(image))
+            if (SyncImagePixels(image) == MagickFalse)
               break;
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+            if (QuantumTick(y,image->rows) != 0)
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
                 break;
           }
         break;
@@ -419,9 +426,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Convert X image to PseudoClass packets.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",
-            image);
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
           image->colormap[i].red=ScaleShortToQuantum(colors[i].red);
@@ -436,15 +442,15 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           indexes=GetIndexes(image);
           for (x=0; x < (long) image->columns; x++)
           {
-            index=
-              ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,(int) y));
+            index=ConstrainColormapIndex(image,XGetPixel(ximage,(int) x,
+              (int) y));
             indexes[x]=index;
             *q++=image->colormap[index];
           }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == MagickFalse)
             break;
-          if (QuantumTick(y,image->rows))
-            if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+          if (QuantumTick(y,image->rows) != 0)
+            if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
               break;
         }
         break;
@@ -454,21 +460,14 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Free image and colormap.
   */
   if (header.ncolors != 0)
-    LiberateMemory((void **) &colors);
-  LiberateMemory((void **) &ximage->data);
-  LiberateMemory((void **) &ximage);
-  if (EOFBlob(image))
-    (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+    colors=(XColor *) RelinquishMagickMemory(colors);
+  ximage->data=(char *) RelinquishMagickMemory(ximage->data);
+  ximage=(XImage *) RelinquishMagickMemory(ximage);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
   CloseBlob(image);
-  return(image);
-}
-#else
-static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  (void) ThrowException(exception,MissingDelegateError,"XWindowLibraryIsNotAvailable",
-    image_info->filename);
-  return((Image *) NULL);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -483,7 +482,7 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterXWDImage adds attributes for the XWD image format to
+%  RegisterXWDImage() adds attributes for the XWD image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -501,10 +500,12 @@ ModuleExport void RegisterXWDImage(void)
     *entry;
 
   entry=SetMagickInfo("XWD");
-  entry->decoder=(DecoderHandler) ReadXWDImage;
-  entry->encoder=(EncoderHandler) WriteXWDImage;
-  entry->magick=(MagickHandler) IsXWD;
-  entry->adjoin=False;
+#if defined(HasX11)
+  entry->decoder=(DecoderHandler *) ReadXWDImage;
+  entry->encoder=(EncoderHandler *) WriteXWDImage;
+#endif
+  entry->magick=(MagickHandler *) IsXWD;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("X Windows system window dump (color)");
   entry->module=AcquireString("XWD");
   (void) RegisterMagickInfo(entry);
@@ -521,7 +522,7 @@ ModuleExport void RegisterXWDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterXWDImage removes format registrations made by the
+%  UnregisterXWDImage() removes format registrations made by the
 %  XWD module from the list of supported formats.
 %
 %  The format of the UnregisterXWDImage method is:
@@ -531,9 +532,7 @@ ModuleExport void RegisterXWDImage(void)
 */
 ModuleExport void UnregisterXWDImage(void)
 {
-#if defined(HasX11)
   (void) UnregisterMagickInfo("XWD");
-#endif
 }
 
 #if defined(HasX11)
@@ -548,16 +547,16 @@ ModuleExport void UnregisterXWDImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteXWDImage writes an image to a file in X window dump
+%  WriteXWDImage() writes an image to a file in X window dump
 %  rasterfile format.
 %
 %  The format of the WriteXWDImage method is:
 %
-%      unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteXWDImage return True if the image is written.
+%    o status: Method WriteXWDImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -567,11 +566,17 @@ ModuleExport void UnregisterXWDImage(void)
 %
 %
 */
-static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteXWDImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  const ImageAttribute
+	  *attribute;
+
+  long
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *p;
 
@@ -587,12 +592,12 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   register unsigned char
     *q;
 
+  size_t
+    length;
+
   unsigned char
     *pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bits_per_pixel,
     bytes_per_line,
@@ -609,14 +614,20 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+  if (status == MagickFalse)
+    return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize XWD file header.
   */
-  xwd_info.header_size=(CARD32) (sz_XWDheader+strlen(image->filename)+1);
+  xwd_info.header_size=(CARD32) sz_XWDheader;
+  attribute=GetImageAttribute(image,"comment");
+  if (attribute != (const ImageAttribute *) NULL)
+    xwd_info.header_size+=strlen(attribute->value);
+  xwd_info.header_size++;
   xwd_info.file_version=(CARD32) XWD_FILE_VERSION;
   xwd_info.pixmap_format=(CARD32) ZPixmap;
   xwd_info.pixmap_depth=(CARD32) (image->storage_class == DirectClass ? 24 : 8);
@@ -627,7 +638,7 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
   xwd_info.bitmap_unit=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
   xwd_info.bitmap_bit_order=(CARD32) MSBFirst;
   xwd_info.bitmap_pad=(CARD32) (image->storage_class == DirectClass ? 32 : 8);
-  bits_per_pixel=(image->storage_class == DirectClass ? 24 : 8);
+  bits_per_pixel=(unsigned long) (image->storage_class == DirectClass ? 24 : 8);
   xwd_info.bits_per_pixel=(CARD32) bits_per_pixel;
   bytes_per_line=(CARD32) ((((xwd_info.bits_per_pixel*
     xwd_info.pixmap_width)+((xwd_info.bitmap_pad)-1))/
@@ -654,10 +665,13 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     Write XWD header.
   */
   lsb_first=1;
-  if (*(char *) &lsb_first)
+  if ((int) (*(char *) &lsb_first) != 0)
     MSBOrderLong((unsigned char *) &xwd_info,sizeof(xwd_info));
-  (void) WriteBlob(image,sz_XWDheader,(char *) &xwd_info);
-  (void) WriteBlob(image,strlen(image->filename)+1,(char *) image->filename);
+  (void) WriteBlob(image,sz_XWDheader,(unsigned char *) &xwd_info);
+  if (attribute != (const ImageAttribute *) NULL)
+    (void) WriteBlob(image,strlen(attribute->value),(unsigned char *)
+      attribute->value);
+  (void) WriteBlob(image,1,(const unsigned char *) "\0");
   if (image->storage_class == PseudoClass)
     {
       XColor
@@ -669,18 +683,19 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
       /*
         Dump colormap to file.
       */
-      colors=(XColor *) AcquireMemory(image->colors*sizeof(XColor));
+      colors=(XColor *)
+        AcquireMagickMemory((size_t) image->colors*sizeof(XColor));
       if (colors == (XColor *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (long) image->colors; i++)
       {
-        colors[i].pixel=i;
+        colors[i].pixel=(unsigned long) i;
         colors[i].red=ScaleQuantumToShort(image->colormap[i].red);
         colors[i].green=ScaleQuantumToShort(image->colormap[i].green);
         colors[i].blue=ScaleQuantumToShort(image->colormap[i].blue);
-        colors[i].flags=DoRed | DoGreen | DoBlue;
-        colors[i].pad=0;
-        if (*(char *) &lsb_first)
+        colors[i].flags=(char) (DoRed | DoGreen | DoBlue);
+        colors[i].pad='\0';
+        if ((int) (*(char *) &lsb_first) != 0)
           {
             MSBOrderLong((unsigned char *) &colors[i].pixel,sizeof(long));
             MSBOrderShort((unsigned char *) &colors[i].red,3*sizeof(short));
@@ -692,18 +707,20 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
         color.red=colors[i].red;
         color.green=colors[i].green;
         color.blue=colors[i].blue;
-        color.flags=colors[i].flags;
-        (void) WriteBlob(image,sz_XWDColor,(char *) &color);
+        color.flags=(CARD8) colors[i].flags;
+        (void) WriteBlob(image,sz_XWDColor,(unsigned char *) &color);
       }
-      LiberateMemory((void **) &colors);
+      colors=(XColor *) RelinquishMagickMemory(colors);
     }
   /*
     Allocate memory for pixels.
   */
-  pixels=(unsigned char *)
-    AcquireMemory(image->columns*sizeof(PixelPacket));
+  length=3*bytes_per_line;
+  if (image->storage_class == PseudoClass)
+    length=bytes_per_line;
+  pixels=(unsigned char *) AcquireMagickMemory(length);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Convert MIFF to XWD raster pixels.
   */
@@ -713,36 +730,34 @@ static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
     q=pixels;
-    for (x=0; x < (long) image->columns; x++)
-    {
-      if (image->storage_class == PseudoClass)
-        *q++=indexes[x];
-      else
-        {
-          *q++=ScaleQuantumToChar(p->red);
-          *q++=ScaleQuantumToChar(p->green);
-          *q++=ScaleQuantumToChar(p->blue);
-        }
-      p++;
-    }
+    if (image->storage_class == PseudoClass)
+      {
+        indexes=GetIndexes(image);
+        for (x=0; x < (long) image->columns; x++)
+          *q++=(unsigned char) indexes[x];
+      }
+    else
+      for (x=(long) image->columns-1; x >= 0; x--)
+      {
+        *q++=ScaleQuantumToChar(p->red);
+        *q++=ScaleQuantumToChar(p->green);
+        *q++=ScaleQuantumToChar(p->blue);
+        p++;
+      }
     for (x=0; x < (long) scanline_pad; x++)
-      *q++=0;
-    (void) WriteBlob(image,q-pixels,(char *) pixels);
+      *q++='\0';
+    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
     if (image->previous == (Image *) NULL)
-      if (QuantumTick(y,image->rows))
-        if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-          break;
+      if (QuantumTick(y,image->rows) != 0)
+        {
+          status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+          if (status == MagickFalse)
+            break;
+        }
   }
-  LiberateMemory((void **) &pixels);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
-  return(True);
-}
-#else
-static unsigned int WriteXWDImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
-    image->filename);
+  return(MagickTrue);
 }
 #endif
