@@ -10,39 +10,26 @@
 %                              Y     UUU     V                                %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,17 +41,24 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
 #include "magick/list.h"
 #include "magick/magick.h"
+#include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/resize.h"
 #include "magick/static.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteYUVImage(const ImageInfo *,Image *);
 
 /*
@@ -78,7 +72,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadYUVImage reads an image with digital YUV (CCIR 601 4:1:1, plane
+%  ReadYUVImage() reads an image with digital YUV (CCIR 601 4:1:1, plane
 %  or partition interlaced, or 4:2:2 plane, partition interlaced or
 %  noninterlaced) bytes and returns it.  It allocates the memory necessary
 %  for the new Image structure and returns a pointer to the new image.
@@ -106,11 +100,17 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *image,
     *resize_image;
 
+  InterlaceType
+    interlace;
+
   long
     horizontal_factor,
     vertical_factor,
     y;
 
+  MagickBooleanType
+    status;
+
   register const PixelPacket
     *r;
 
@@ -127,28 +127,24 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
-  size_t
+  ssize_t
     count;
 
   unsigned char
     *scanline;
 
-  unsigned int
-    status;
-
-  InterlaceType
-    interlace;
-
   /*
     Allocate image structure.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"MustSpecifyImageSize",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   image->depth=8;
   interlace=image_info->interlace;
   horizontal_factor=2;
@@ -158,18 +154,17 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       GeometryInfo
         geometry_info;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
       horizontal_factor=(long) geometry_info.rho;
       vertical_factor=(long) geometry_info.sigma;
-      if (!(flags & SigmaValue))
+      if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
           (vertical_factor != 1) && (vertical_factor != 2))
-        ThrowReaderException(CorruptImageError,"UnexpectedSamplingFactor",
-          image);
+        ThrowReaderException(CorruptImageError,"UnexpectedSamplingFactor");
     }
   if ((interlace == UndefinedInterlace) ||
       ((interlace == NoInterlace) && (vertical_factor == 2)))
@@ -184,8 +179,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Open image file.
       */
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-      if (status == False)
-        ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        {
+          DestroyImageList(image);
+          return((Image *) NULL);
+        }
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
@@ -193,36 +191,40 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
-    scanline=(unsigned char *) AcquireMemory(2*image->columns+2);
+    scanline=(unsigned char *)
+      AcquireMagickMemory((size_t) (2*image->columns+2));
   else
-    scanline=(unsigned char *) AcquireMemory(image->columns);
+    scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   do
   {
     chroma_image=CloneImage(image,image->columns/horizontal_factor,
-      image->rows/vertical_factor,True,exception);
+      image->rows/vertical_factor,MagickTrue,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert raster image to pixel packets.
     */
-    if (image_info->ping && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     if (interlace == PartitionInterlace)
       {
         AppendImageFormat("Y",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-        if (status == False)
-          ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+        if (status == MagickFalse)
+          {
+            DestroyImageList(image);
+            return((Image *) NULL);
+          }
       }
     for (y=0; y < (long) image->rows; y++)
     {
       if (interlace == NoInterlace)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,2*image->columns,scanline);
+            (void) ReadBlob(image,(size_t) (2*image->columns),scanline);
           p=scanline;
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
@@ -249,7 +251,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       else
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,image->columns,scanline);
+            (void) ReadBlob(image,(size_t) image->columns,scanline);
           p=scanline;
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
@@ -262,13 +264,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q++;
           }
         }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (interlace == NoInterlace)
-        if (!SyncImagePixels(chroma_image))
+        if (SyncImagePixels(chroma_image) == MagickFalse)
           break;
       if (image->previous == (Image *) NULL)
-        if (!MagickMonitor(LoadImageTag,y,image->rows,exception))
+        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
           break;
     }
     if (interlace == PartitionInterlace)
@@ -276,14 +278,17 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         CloseBlob(image);
         AppendImageFormat("U",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-        if (status == False)
-          ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+        if (status == MagickFalse)
+          {
+            DestroyImageList(image);
+            return((Image *) NULL);
+          }
       }
     if (interlace != NoInterlace)
       {
         for (y=0; y < (long) chroma_image->rows; y++)
         {
-          (void) ReadBlob(image,chroma_image->columns,scanline);
+          (void) ReadBlob(image,(size_t) chroma_image->columns,scanline);
           p=scanline;
           q=SetImagePixels(chroma_image,0,y,chroma_image->columns,1);
           if (q == (PixelPacket *) NULL)
@@ -295,7 +300,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->blue=0;
             q++;
           }
-          if (!SyncImagePixels(chroma_image))
+          if (SyncImagePixels(chroma_image) == MagickFalse)
             break;
         }
       if (interlace == PartitionInterlace)
@@ -303,12 +308,15 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           CloseBlob(image);
           AppendImageFormat("V",image->filename);
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-          if (status == False)
-            ThrowReaderException(FileOpenError,"UnableToOpenFile",image);
+          if (status == MagickFalse)
+            {
+              DestroyImageList(image);
+              return((Image *) NULL);
+            }
         }
       for (y=0; y < (long) chroma_image->rows; y++)
       {
-        (void) ReadBlob(image,chroma_image->columns,scanline);
+        (void) ReadBlob(image,(size_t) chroma_image->columns,scanline);
         p=scanline;
         q=GetImagePixels(chroma_image,0,y,chroma_image->columns,1);
         if (q == (PixelPacket *) NULL)
@@ -318,7 +326,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q->blue=ScaleCharToQuantum(*p++);
           q++;
         }
-        if (!SyncImagePixels(chroma_image))
+        if (SyncImagePixels(chroma_image) == MagickFalse)
           break;
       }
     }
@@ -329,7 +337,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       TriangleFilter,1.0,exception);
     DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed",image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=0; y < (long) image->rows; y++)
     {
       q=GetImagePixels(image,0,y,image->columns,1);
@@ -344,17 +352,18 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         r++;
         q++;
       }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == MagickFalse)
         break;
     }
     DestroyImage(resize_image);
     image->colorspace=YCbCrColorspace;
     (void) SetImageColorspace(image,RGBColorspace);
     if (interlace == PartitionInterlace)
-      (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-    if (EOFBlob(image))
+      (void) CopyMagickString(image->filename,image_info->filename,
+        MaxTextExtent);
+    if (EOFBlob(image) != MagickFalse)
       {
-        (void) ThrowException(exception,CorruptImageError,"UnexpectedEndOfFile",
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
         break;
       }
@@ -365,9 +374,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     if (interlace == NoInterlace)
-      count=ReadBlob(image,2*image->columns,(char *) scanline);
+      count=ReadBlob(image,(size_t) (2*image->columns),scanline);
     else
-      count=ReadBlob(image,image->columns,(char *) scanline);
+      count=ReadBlob(image,(size_t) image->columns,scanline);
     if (count != 0)
       {
         /*
@@ -382,15 +391,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image=SyncNextImageInList(image);
         status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
           exception);
-        if (status == False)
+        if (status == MagickFalse)
           break;
       }
   } while (count != 0);
-  LiberateMemory((void **) &scanline);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -404,7 +411,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterYUVImage adds attributes for the YUV image format to
+%  RegisterYUVImage() adds attributes for the YUV image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -422,10 +429,10 @@ ModuleExport void RegisterYUVImage(void)
     *entry;
 
   entry=SetMagickInfo("YUV");
-  entry->decoder=(DecoderHandler) ReadYUVImage;
-  entry->encoder=(EncoderHandler) WriteYUVImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadYUVImage;
+  entry->encoder=(EncoderHandler *) WriteYUVImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
   entry->description=AcquireString("CCIR 601 4:1:1 or 4:2:2");
   entry->module=AcquireString("YUV");
   (void) RegisterMagickInfo(entry);
@@ -442,7 +449,7 @@ ModuleExport void RegisterYUVImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterYUVImage removes format registrations made by the
+%  UnregisterYUVImage() removes format registrations made by the
 %  YUV module from the list of supported formats.
 %
 %  The format of the UnregisterYUVImage method is:
@@ -466,17 +473,17 @@ ModuleExport void UnregisterYUVImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteYUVImage writes an image to a file in the digital YUV
+%  WriteYUVImage() writes an image to a file in the digital YUV
 %  (CCIR 601 4:1:1, plane or partition interlaced, or 4:2:2 plane, partition
 %  interlaced or noninterlaced) bytes and returns it.
 %
 %  The format of the WriteYUVImage method is:
 %
-%      unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteYUVImage return True if the image is written.
+%    o status: Method WriteYUVImage returns True if the image is written.
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
@@ -486,7 +493,7 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %
 */
-static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
 {
   Image
     *chroma_image,
@@ -500,6 +507,12 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     vertical_factor,
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   register const PixelPacket
     *p,
     *s;
@@ -507,11 +520,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
   register long
     x;
 
-  unsigned int
-    status;
-
   unsigned long
-    scene,
     height,
     width;
 
@@ -519,6 +528,8 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   interlace=image_info->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -527,18 +538,17 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
       GeometryInfo
         geometry_info;
 
-      int
+      MagickStatusType
         flags;
 
       flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
       horizontal_factor=(long) geometry_info.rho;
       vertical_factor=(long) geometry_info.sigma;
-      if (!(flags & SigmaValue))
+      if ((flags & SigmaValue) == 0)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
           (vertical_factor != 1) && (vertical_factor != 2))
-        ThrowWriterException(CorruptImageError,"UnexpectedSamplingFactor",
-          image);
+        ThrowWriterException(CorruptImageError,"UnexpectedSamplingFactor");
     }
   if ((interlace == UndefinedInterlace) ||
       ((interlace == NoInterlace) && (vertical_factor == 2)))
@@ -553,15 +563,15 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
         Open output image file.
       */
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      if (status == False)
-        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        return(status);
     }
   else
     {
       AppendImageFormat("Y",image->filename);
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-      if (status == False)
-        ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+      if (status == MagickFalse)
+        return(status);
     }
   scene=0;
   do
@@ -576,7 +586,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
       &image->exception);
     if (yuv_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason,image);
+      ThrowWriterException(ResourceLimitError,image->exception.reason);
     (void) SetImageColorspace(yuv_image,YCbCrColorspace);
     /*
       Downsample image.
@@ -584,7 +594,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     chroma_image=ResizeImage(image,width/horizontal_factor,
       height/vertical_factor,TriangleFilter,1.0,&image->exception);
     if (chroma_image == (Image *) NULL)
-      ThrowWriterException(ResourceLimitError,image->exception.reason,image);
+      ThrowWriterException(ResourceLimitError,image->exception.reason);
     (void) SetImageColorspace(chroma_image,YCbCrColorspace);
     if (interlace == NoInterlace)
       {
@@ -613,9 +623,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             x++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         DestroyImage(yuv_image);
       }
@@ -636,9 +650,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageTag,y,image->rows,&image->exception))
-                break;
+            if (QuantumTick(y,image->rows) != 0)
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == MagickFalse)
+                  break;
+              }
         }
         DestroyImage(yuv_image);
         /*
@@ -650,8 +668,8 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("U",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
@@ -674,8 +692,8 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             AppendImageFormat("V",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
-            if (status == False)
-              ThrowWriterException(FileOpenError,"UnableToOpenFile",image);
+            if (status == MagickFalse)
+              return(status);
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
@@ -692,18 +710,16 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
       }
     DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
-      (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
+      (void) CopyMagickString(image->filename,image_info->filename,
+        MaxTextExtent);
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
     status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
       &image->exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
