@@ -1,5 +1,5 @@
 /*
-  Image Compression/Decompression Methods.
+  ImageMagick Binary Large OBjects Methods.
 */
 #ifndef _MAGICK_BLOB_H
 #define _MAGICK_BLOB_H
@@ -8,21 +8,9 @@
 extern "C" {
 #endif
 
-#include "image.h"
+#include "magick/image.h"
+#include "magick/stream.h"
 
-/*
-  Define declarations.
-*/
-#ifdef S_IRUSR
-#define S_MODE      (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
-#else
-#define S_MODE      0644 
-#endif 
-#define MinBlobExtent  65536L
-
-/*
-  Enum declarations.
-*/
 typedef enum
 {
   UndefinedBlobMode,
@@ -40,69 +28,12 @@ typedef enum
   IOMode
 } MapMode;
 
-typedef enum
-{
-  UndefinedStream,
-  FileStream,
-  StandardStream,
-  PipeStream,
-  ZipStream,
-  BZipStream,
-  FifoStream,
-  BlobStream
-} StreamType;
-
-/*
-  Typedef declarations.
-*/
 typedef int
   *(*BlobFifo)(const Image *,const void *,const size_t);
 
 typedef struct _BlobInfo
-{
-  size_t
-    length,
-    extent,
-    quantum;
+  BlobInfo;
 
-  unsigned int
-    mapped,
-    eof;
-
-  ExtendedSignedIntegralType
-    offset,
-    size;
-
-  unsigned int
-    exempt,
-    status,
-    temporary;
-
-  StreamType
-    type;
-
-  FILE
-    *file;
-
-  StreamHandler
-    stream;
-
-  unsigned char
-    *data;
-
-  SemaphoreInfo
-    *semaphore;
-
-  long
-    reference_count;
-
-  unsigned long
-    signature;
-} BlobInfo;
-
-/*
-  Blob methods.
-*/
 extern MagickExport BlobInfo
   *CloneBlobInfo(const BlobInfo *),
   *ReferenceBlob(BlobInfo *);
@@ -110,10 +41,8 @@ extern MagickExport BlobInfo
 extern MagickExport char
   *ReadBlobString(Image *,char *);
 
-extern MagickExport ExtendedSignedIntegralType
-  GetBlobSize(const Image *image),
-  SeekBlob(Image *,const ExtendedSignedIntegralType,const int),
-  TellBlob(const Image *image);
+extern MagickExport FILE
+  *GetBlobFileHandle(const Image *);
 
 extern MagickExport Image
   *BlobToImage(const ImageInfo *,const void *,const size_t,ExceptionInfo *),
@@ -124,28 +53,42 @@ extern MagickExport int
   ReadBlobByte(Image *),
   SyncBlob(Image *);
 
-extern MagickExport size_t
-  ReadBlob(Image *,const size_t,void *),
-  WriteBlob(Image *,const size_t,const void *),
-  WriteBlobByte(Image *,const unsigned long),
-  WriteBlobLSBLong(Image *,const unsigned long),
-  WriteBlobLSBShort(Image *,const unsigned long),
-  WriteBlobMSBLong(Image *,const unsigned long),
-  WriteBlobMSBShort(Image *,const unsigned long),
-  WriteBlobString(Image *,const char *);
-
-extern MagickExport StreamType
-  GetBlobStreamType(const Image *);
-
-extern MagickExport unsigned char
-  *GetBlobStreamData(const Image *);
-
-extern MagickExport unsigned int
-  BlobToFile(const char *,const void *,const size_t,ExceptionInfo *),
-  ImageToFile(Image *,const char *,ExceptionInfo *),
+extern MagickExport  MagickBooleanType
+  BlobToFile(char *,const void *,const size_t,ExceptionInfo *),
+  GetBlobError(const Image *),
+  ImageToFile(Image *,char *,ExceptionInfo *),
+  IsBlobExempt(const Image *),
+  IsBlobSeekable(const Image *),
+  IsBlobTemporary(const Image *),
   OpenBlob(const ImageInfo *,Image *,const BlobMode,ExceptionInfo *),
   UnmapBlob(void *,const size_t);
 
+extern MagickExport MagickOffsetType
+  SeekBlob(Image *,const MagickOffsetType,const int),
+  TellBlob(const Image *image);
+
+extern MagickExport MagickSizeType
+  GetBlobSize(const Image *image);
+
+extern MagickExport ssize_t
+  ReadBlob(Image *,const size_t,unsigned char *),
+  WriteBlob(Image *,const size_t,const unsigned char *),
+  WriteBlobByte(Image *,const unsigned char),
+  WriteBlobLSBLong(Image *,const unsigned long),
+  WriteBlobLSBShort(Image *,const unsigned short),
+  WriteBlobMSBLong(Image *,const unsigned long),
+  WriteBlobMSBShort(Image *,const unsigned short),
+  WriteBlobString(Image *,const char *);
+
+extern MagickExport StreamHandler
+  GetBlobStreamHandler(const Image *);
+
+extern MagickExport unsigned char
+  *FileToBlob(const char *,size_t *,ExceptionInfo *),
+  *GetBlobStreamData(const Image *),
+  *ImageToBlob(const ImageInfo *,Image *,size_t *,ExceptionInfo *),
+  *MapBlob(int,const MapMode,const MagickOffsetType,const size_t);
+
 extern MagickExport unsigned long
   ReadBlobLSBLong(Image *),
   ReadBlobMSBLong(Image *);
@@ -157,17 +100,12 @@ extern MagickExport unsigned short
 extern MagickExport void
   AttachBlob(BlobInfo *,const void *,const size_t),
   CloseBlob(Image *),
-  DestroyBlobInfo(BlobInfo *),
+  DestroyBlob(Image *),
   DetachBlob(BlobInfo *),
-  *FileToBlob(const char *,size_t *,ExceptionInfo *),
   GetBlobInfo(BlobInfo *),
-  *GetConfigureBlob(const char *,char *,size_t *,ExceptionInfo *),
-  *GetTypeBlob(const char *,char *,size_t *,ExceptionInfo *),
-  *GetModuleBlob(const char *,char *,size_t *,ExceptionInfo *),
-  *ImageToBlob(const ImageInfo *,Image *,size_t *,ExceptionInfo *),
-  *MapBlob(int,const MapMode,off_t,size_t),
   MSBOrderLong(unsigned char *,const size_t),
-  MSBOrderShort(unsigned char *,const size_t);
+  MSBOrderShort(unsigned char *,const size_t),
+  SetBlobExempt(Image *,const MagickBooleanType);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
