@@ -15,6 +15,7 @@
 %                         V V     I    E      WW WW                           %
 %                          V    IIIII  EEEEE  W   W                           %
 %                                                                             %
+%                                                                             %
 %                       ImageMagick Cache View Methods                        %
 %                                                                             %
 %                              Software Design                                %
@@ -22,31 +23,19 @@
 %                               February 2000                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -60,8 +49,9 @@
 #include "magick/studio.h"
 #include "magick/cache.h"
 #include "magick/cache_view.h"
+#include "magick/memory_.h"
 #include "magick/error.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -74,9 +64,9 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method AcquireCacheView gets pixels from the in-memory or disk pixel cache
-%  as defined by the geometry parameters.   A pointer to the pixels is returned
-%  if the pixels are transferred, otherwise a NULL is returned.
+%  AcquireCacheView() gets pixels from the in-memory or disk pixel cache as
+%  defined by the geometry parameters.   A pointer to the pixels is returned if
+%  the pixels are transferred, otherwise a NULL is returned.
 %
 %  The format of the AcquireCacheView method is:
 %
@@ -104,6 +94,9 @@ MagickExport const PixelPacket *AcquireCacheView(const ViewInfo *view,
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(AcquireCacheNexus(view->image,x,y,columns,rows,view->id,exception));
 }
 
@@ -134,8 +127,12 @@ MagickExport void CloseCacheView(ViewInfo *view)
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
-  DestroyCacheNexus(view->image->cache,view->id);
-  LiberateMemory((void **) &view);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
+  if (view->id != 0)
+    DestroyCacheNexus(view->image->cache,view->id);
+  view=(ViewInfo *) RelinquishMagickMemory(view);
 }
 
 /*
@@ -175,6 +172,9 @@ MagickExport PixelPacket *GetCacheView(ViewInfo *view,const long x,const long y,
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(GetCacheNexus(view->image,x,y,columns,rows,view->id));
 }
 
@@ -189,7 +189,7 @@ MagickExport PixelPacket *GetCacheView(ViewInfo *view,const long x,const long y,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetCacheViewIndexes returns the indexes associated with the specified
+%  GetCacheViewIndexes() returns the indexes associated with the specified
 %  view.
 %
 %  The format of the GetCacheViewIndexes method is:
@@ -209,6 +209,9 @@ MagickExport IndexPacket *GetCacheViewIndexes(const ViewInfo *view)
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(GetNexusIndexes(view->image->cache,view->id));
 }
 
@@ -240,6 +243,9 @@ MagickExport PixelPacket *GetCacheViewPixels(const ViewInfo *view)
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(GetNexusPixels(view->image->cache,view->id));
 }
 
@@ -273,19 +279,16 @@ MagickExport ViewInfo *OpenCacheView(Image *image)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  view=(ViewInfo *) AcquireMemory(sizeof(ViewInfo));
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  view=(ViewInfo *) AcquireMagickMemory(sizeof(ViewInfo));
   if (view == (ViewInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateCacheView");
-  (void) memset(view,0,sizeof(ViewInfo));
+    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+      image->filename);
+  (void) ResetMagickMemory(view,0,sizeof(ViewInfo));
   view->id=GetNexus(image->cache);
   view->image=image;
   view->signature=MagickSignature;
-  if (view->id == 0)
-    {
-      CloseCacheView(view);
-      return((ViewInfo *) NULL);
-    }
   return(view);
 }
 
@@ -323,6 +326,9 @@ MagickExport PixelPacket *SetCacheView(ViewInfo *view,const long x,const long y,
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(SetCacheNexus(view->image,x,y,columns,rows,view->id));
 }
 
@@ -338,11 +344,11 @@ MagickExport PixelPacket *SetCacheView(ViewInfo *view,const long x,const long y,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SyncCacheView() saves the view pixels to the in-memory or disk cache.
-%  The method returns True if the pixel region is synced, otherwise False.
+%  The method returns MagickTrue if the pixel region is synced, otherwise MagickFalse.
 %
 %  The format of the SyncCacheView method is:
 %
-%      unsigned int SyncCacheView(ViewInfo *view)
+%      MagickBooleanType SyncCacheView(ViewInfo *view)
 %
 %  A description of each parameter follows:
 %
@@ -350,9 +356,12 @@ MagickExport PixelPacket *SetCacheView(ViewInfo *view,const long x,const long y,
 %
 %
 */
-MagickExport unsigned int SyncCacheView(ViewInfo *view)
+MagickExport MagickBooleanType SyncCacheView(ViewInfo *view)
 {
   assert(view != (ViewInfo *) NULL);
   assert(view->signature == MagickSignature);
+  assert(view->image != (Image *) NULL);
+  if (view->image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),view->image->filename);
   return(SyncCacheNexus(view->image,view->id));
 }
