@@ -8,30 +8,17 @@
 extern "C" {
 #endif
 
-/*
-  Typedef declaration.
-*/
-typedef struct _Ascii85Info
-{
-  long
-    offset,
-    line_break;
-
-  unsigned char
-    buffer[10];
-} Ascii85Info;
-
-/*
-  Compress methods.
-*/
-extern MagickExport unsigned int
+extern MagickExport MagickBooleanType
   HuffmanDecodeImage(Image *),
   HuffmanEncodeImage(const ImageInfo *,Image *),
+  Huffman2DEncodeImage(const ImageInfo *,Image *),
+  JPEGEncodeImage(const ImageInfo *,Image *),
   LZWEncodeImage(Image *,const size_t,unsigned char *),
-  PackbitsEncodeImage(Image *,const size_t,unsigned char *);
+  PackbitsEncodeImage(Image *,const size_t,unsigned char *),
+  ZLIBEncodeImage(Image *,const size_t,const unsigned long,unsigned char *);
 
 extern MagickExport void
-  Ascii85Encode(Image *,const unsigned long),
+  Ascii85Encode(Image *,const unsigned char),
   Ascii85Flush(Image *),
   Ascii85Initialize(Image *);
 
