@@ -12,37 +12,24 @@
 %                                                                             %
 %                    Graphic Gems - Graphic Support Methods                   %
 %                                                                             %
-%                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2001 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,6 +42,10 @@
 */
 #include "magick/studio.h"
 #include "magick/gem.h"
+#include "magick/image.h"
+#include "magick/log.h"
+#include "magick/memory_.h"
+#include "magick/random.h"
 #include "magick/signature.h"
 
 /*
@@ -68,8 +59,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method Contrast enhances the intensity differences between the lighter
-%  and darker elements of the image.
+%  Contrast() enhances the intensity differences between the lighter and darker
+%  elements of the image.
 %
 %  The format of the Contrast method is:
 %
@@ -98,7 +89,10 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   assert(red != (Quantum *) NULL);
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
-  TransformHSL(*red,*green,*blue,&hue,&saturation,&brightness);
+  hue=0.0;
+  saturation=0.0;
+  brightness=0.0;
+  TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
   alpha=0.5+MagickEpsilon;
   brightness+=
     alpha*sign*(alpha*(sin(MagickPI*(brightness-alpha))+1.0)-brightness);
@@ -107,7 +101,7 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
   else
     if (brightness < 0.0)
       brightness=0.0;
-  HSLTransform(hue,saturation,brightness,red,green,blue);
+  HSBTransform(hue,saturation,brightness,red,green,blue);
 }
 
 /*
@@ -121,10 +115,10 @@ MagickExport void Contrast(const int sign,Quantum *red,Quantum *green,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ExpandAffine computes the affine's expansion factor, i.e. the
-%  square root of the factor by which the affine transform affects area. In an
-%  affine transform composed of scaling, rotation, shearing, and translation,
-%  returns the amount of scaling.
+%  ExpandAffine() computes the affine's expansion factor, i.e. the square root
+%  of the factor by which the affine transform affects area. In an affine
+%  transform composed of scaling, rotation, shearing, and translation, returns
+%  the amount of scaling.
 %
 %  The format of the ExpandAffine method is:
 %
@@ -149,133 +143,6 @@ MagickExport double ExpandAffine(const AffineMatrix *affine)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e n e r a t e N o i s e                                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Method GenerateNoise adds noise to a pixel.
-%
-%  The format of the GenerateNoise method is:
-%
-%      Quantum GenerateNoise(const Quantum pixel,const NoiseType noise_type)
-%
-%  A description of each parameter follows:
-%
-%    o pixel: A structure of type Quantum.
-%
-%    o noise_type:  The type of noise: Gaussian, multiplicative Gaussian,
-%      impulse, laplacian, or Poisson.
-%
-%
-*/
-MagickExport Quantum GenerateNoise(const Quantum pixel,
-  const NoiseType noise_type)
-{
-#define NoiseEpsilon  1.0e-5
-#define SigmaUniform  4.0
-#define SigmaGaussian  4.0
-#define SigmaImpulse  0.10
-#define SigmaLaplacian 10.0
-#define SigmaMultiplicativeGaussian  0.5
-#define SigmaPoisson  0.05
-#define TauGaussian  20.0
-
-  double
-    alpha,
-    beta,
-    sigma,
-    value;
-
-  alpha=(double) rand()/RAND_MAX;
-  if (alpha == 0.0)
-    alpha=1.0;
-  switch (noise_type)
-  {
-    case UniformNoise:
-    default:
-    {
-      value=(double) pixel+SigmaUniform*(alpha-0.5);
-      break;
-    }
-    case GaussianNoise:
-    {
-      double
-        tau;
-
-      beta=(double) rand()/RAND_MAX;
-      sigma=sqrt(-2.0*log(alpha))*cos(2.0*MagickPI*beta);
-      tau=sqrt(-2.0*log(alpha))*sin(2.0*MagickPI*beta);
-      value=(double) pixel+sqrt((double) pixel)*SigmaGaussian*sigma+
-        TauGaussian*tau;
-      break;
-    }
-    case MultiplicativeGaussianNoise:
-    {
-      if (alpha <= NoiseEpsilon)
-        sigma=MaxRGB;
-      else
-        sigma=sqrt(-2.0*log(alpha));
-      beta=(double) rand()/RAND_MAX;
-      value=(double) pixel+pixel*SigmaMultiplicativeGaussian*sigma*
-        cos(2.0*MagickPI*beta);
-      break;
-    }
-    case ImpulseNoise:
-    {
-      if (alpha < (SigmaImpulse/2.0))
-        value=0;
-       else
-         if (alpha >= (1.0-(SigmaImpulse/2.0)))
-           value=MaxRGB;
-         else
-           value=pixel;
-      break;
-    }
-    case LaplacianNoise:
-    {
-      if (alpha <= 0.5)
-        {
-          if (alpha <= NoiseEpsilon)
-            value=(double) pixel-(double) MaxRGB;
-          else
-            value=(double) pixel+SigmaLaplacian*log(2.0*alpha);
-          break;
-        }
-      beta=1.0-alpha;
-      if (beta <= (0.5*NoiseEpsilon))
-        value=(double) pixel+MaxRGB;
-      else
-        value=(double) pixel-SigmaLaplacian*log(2.0*beta);
-      break;
-    }
-    case PoissonNoise:
-    {
-      register long
-        i;
-
-      for (i=0; alpha > exp(-SigmaPoisson*pixel); i++)
-      {
-        beta=(double) rand()/RAND_MAX;
-        alpha=alpha*beta;
-      }
-      value=i/SigmaPoisson;
-      break;
-    }
-  }
-  if (value < 0.0)
-    return(0);
-  if (value > MaxRGB)
-    return(MaxRGB);
-  return((Quantum) (value+0.5));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t O p t i m a l K e r n e l W i d t h                                 %
 %                                                                             %
 %                                                                             %
@@ -288,7 +155,8 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
 %
 %  The format of the GetOptimalKernelWidth method is:
 %
-%      int GetOptimalKernelWidth(const double radius,const double sigma)
+%      unsigned long GetOptimalKernelWidth(const double radius,
+%        const double sigma)
 %
 %  A description of each parameter follows:
 %
@@ -303,21 +171,23 @@ MagickExport Quantum GenerateNoise(const Quantum pixel,
 %
 */
 
-MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
+MagickExport unsigned long GetOptimalKernelWidth1D(const double radius,
+  const double sigma)
 {
-  double
-    normalize,
-    value;
-
   long
     width;
 
+  MagickRealType
+    normalize,
+    value;
+
   register long
     u;
 
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
   assert(sigma != 0.0);
   if (radius > 0.0)
-    return((int) (2.0*ceil(radius)+1.0));
+    return(2*((unsigned long) radius)+1);
   for (width=5; ;)
   {
     normalize=0.0;
@@ -329,26 +199,28 @@ MagickExport int GetOptimalKernelWidth1D(const double radius,const double sigma)
       break;
     width+=2;
   }
-  return(width-2);
+  return((unsigned long) (width-2));
 }
 
-MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
+MagickExport unsigned long GetOptimalKernelWidth2D(const double radius,
+  const double sigma)
 {
-  double
-    alpha,
-    normalize,
-    value;
 
   long
     width;
 
+  MagickRealType
+    alpha,
+    normalize,
+    value;
   register long
     u,
     v;
 
   assert(sigma != 0.0);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
   if (radius > 0.0)
-    return((int) (2.0*ceil(radius)+1.0));
+    return(2*((unsigned long) radius)+1);
   for (width=5; ;)
   {
     normalize=0.0;
@@ -366,10 +238,11 @@ MagickExport int GetOptimalKernelWidth2D(const double radius,const double sigma)
       break;
     width+=2;
   }
-  return(width-2);
+  return((unsigned long) (width-2));
 }
 
-MagickExport int GetOptimalKernelWidth(const double radius,const double sigma)
+MagickExport unsigned long  GetOptimalKernelWidth(const double radius,
+  const double sigma)
 {
   return(GetOptimalKernelWidth1D(radius,sigma));
 }
@@ -379,14 +252,120 @@ MagickExport int GetOptimalKernelWidth(const double radius,const double sigma)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   H S B T r a n s f o r m                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  HSBTransform() converts a (hue, saturation, brightness) to a (red, green,
+%  blue) triple.
+%
+%  The format of the HSBTransformImage method is:
+%
+%      void HSBTransform(const double hue,const double saturation,
+%        const double brightness,Quantum *red,Quantum *green,Quantum *blue)
+%
+%  A description of each parameter follows:
+%
+%    o hue, saturation, brightness: A double value representing a
+%      component of the HSB color space.
+%
+%    o red, green, blue: A pointer to a pixel component of type Quantum.
+%
+%
+*/
+MagickExport void HSBTransform(const double hue,const double saturation,
+  const double brightness,Quantum *red,Quantum *green,Quantum *blue)
+{
+  MagickRealType
+    f,
+    h,
+    p,
+    q,
+    t;
+
+  /*
+    Convert HSB to RGB colorspace.
+  */
+  assert(red != (Quantum *) NULL);
+  assert(green != (Quantum *) NULL);
+  assert(blue != (Quantum *) NULL);
+  if (saturation == 0.0)
+    {
+      *red=RoundToQuantum(MaxRGB*brightness);
+      *green=(*red);
+      *blue=(*red);
+      return;
+    }
+  h=6.0*hue;
+  if (h == 6.0)
+    h=0.0;
+  f=h-(int) h;
+  p=brightness*(1.0-saturation);
+  q=brightness*(1.0-saturation*f);
+  t=brightness*(1.0-saturation*(1.0-f));
+  switch ((int) h)
+  {
+    case 0:
+    default:
+    {
+      *red=RoundToQuantum(MaxRGB*brightness);
+      *green=RoundToQuantum(MaxRGB*t);
+      *blue= RoundToQuantum(MaxRGB*p);
+      break;
+    }
+    case 1:
+    {
+      *red=RoundToQuantum(MaxRGB*q);
+      *green=RoundToQuantum(MaxRGB*brightness);
+      *blue= RoundToQuantum(MaxRGB*p);
+      break;
+    }
+    case 2:
+    {
+      *red=RoundToQuantum(MaxRGB*p);
+      *green=RoundToQuantum(MaxRGB*brightness);
+      *blue=RoundToQuantum(MaxRGB*t);
+      break;
+    }
+    case 3:
+    {
+      *red=RoundToQuantum(MaxRGB*p);
+      *green=RoundToQuantum(MaxRGB*q);
+      *blue=RoundToQuantum(MaxRGB*brightness);
+      break;
+    }
+    case 4:
+    {
+      *red=RoundToQuantum(MaxRGB*t);
+      *green=RoundToQuantum(MaxRGB*p);
+      *blue=RoundToQuantum(MaxRGB*brightness);
+      break;
+    }
+    case 5:
+    {
+      *red=RoundToQuantum(MaxRGB*brightness);
+      *green=RoundToQuantum(MaxRGB*p);
+      *blue=RoundToQuantum(MaxRGB*q);
+      break;
+    }
+  }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   H S L T r a n s f o r m                                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method HSLTransform converts a (hue, saturation, luminosity) to a
-%  (red, green, blue) triple.
+%  HSLTransform() converts a (hue, saturation, luminosity) to a (red, green,
+%  blue) triple.
 %
 %  The format of the HSLTransformImage method is:
 %
@@ -405,7 +384,7 @@ MagickExport int GetOptimalKernelWidth(const double radius,const double sigma)
 MagickExport void HSLTransform(const double hue,const double saturation,
   const double luminosity,Quantum *red,Quantum *green,Quantum *blue)
 {
-  double
+  MagickRealType
     b,
     g,
     r,
@@ -442,9 +421,9 @@ MagickExport void HSLTransform(const double hue,const double saturation,
     case 5: r=v; g=y; b=z; break;
     default: r=v; g=x; b=y; break;
   }
-  *red=(Quantum) (MaxRGB*r+0.5);
-  *green=(Quantum) (MaxRGB*g+0.5);
-  *blue=(Quantum) (MaxRGB*b+0.5);
+  *red=RoundToQuantum(MaxRGB*r);
+  *green=RoundToQuantum(MaxRGB*g);
+  *blue=RoundToQuantum(MaxRGB*b);
 }
 
 /*
@@ -458,8 +437,8 @@ MagickExport void HSLTransform(const double hue,const double saturation,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method HWBTransform converts a (hue, whiteness, blackness) to a
-%  (red, green, blue) triple.
+%  HWBTransform() converts a (hue, whiteness, blackness) to a (red, green,
+%  blue) triple.
 %
 %  The format of the HWBTransformImage method is:
 %
@@ -478,7 +457,7 @@ MagickExport void HSLTransform(const double hue,const double saturation,
 MagickExport void HWBTransform(const double hue,const double whiteness,
   const double blackness,Quantum *red,Quantum *green,Quantum *blue)
 {
-  double
+  MagickRealType
     b,
     f,
     g,
@@ -505,7 +484,7 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
     }
   i=(long) floor(hue);
   f=hue-i;
-  if (i & 0x01)
+  if ((i & 0x01) != 0)
     f=1.0-f;
   n=whiteness+f*(v-whiteness);  /* linear interpolation */
   switch (i)
@@ -535,11 +514,10 @@ MagickExport void HWBTransform(const double hue,const double whiteness,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method Hull implements the eight hull algorithm described in Applied
-%  Optics, Vol. 24, No. 10, 15 May 1985, "Geometric filter for Speckle
-%  Reduction", by Thomas R Crimmins.  Each pixel in the image is replaced by
-%  one of its eight of its surrounding pixels using a polarity and negative
-%  hull function.
+%  Hull() implements the eight hull algorithm described in Applied Optics, Vol
+%  24, No. 10, 15 May 1985, "Geometric filter for Speckle Reduction", by Thomas
+%  R Crimmins.  Each pixel in the image is replaced by one of its eight of its
+%  surrounding pixels using a polarity and negative hull function.
 %
 %  The format of the Hull method is:
 %
@@ -564,10 +542,12 @@ MagickExport void Hull(const long x_offset,const long y_offset,
   const unsigned long columns,const unsigned long rows,Quantum *f,Quantum *g,
   const int polarity)
 {
-  double
-    v,
+  long
     y;
 
+  MagickRealType
+    v;
+
   register long
     x;
 
@@ -590,8 +570,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
     if (polarity > 0)
       for (x=(long) columns; x > 0; x--)
       {
-        v=(*p);
-        if (*r >= (v+ScaleCharToQuantum(2)))
+        v=(MagickRealType) (*p);
+        if ((MagickRealType) *r >= (v+(MagickRealType) ScaleCharToQuantum(2)))
           v+=ScaleCharToQuantum(1);
         *q=(Quantum) v;
         p++;
@@ -601,8 +581,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
     else
       for (x=(long) columns; x > 0; x--)
       {
-        v=(*p);
-        if (*r <= (v-(long) ScaleCharToQuantum(2)))
+        v=(MagickRealType) (*p);
+        if ((MagickRealType) *r <= (v-(MagickRealType) ScaleCharToQuantum(2)))
           v-=(long) ScaleCharToQuantum(1);
         *q=(Quantum) v;
         p++;
@@ -626,8 +606,10 @@ MagickExport void Hull(const long x_offset,const long y_offset,
     if (polarity > 0)
       for (x=(long) columns; x > 0; x--)
       {
-        v=(*q);
-        if ((*s >= (v+ScaleCharToQuantum(2))) && (*r > v))
+        v=(MagickRealType) (*q);
+        if (((MagickRealType) *s >=
+             (v+(MagickRealType) ScaleCharToQuantum(2))) &&
+            ((MagickRealType) *r > v))
           v+=ScaleCharToQuantum(1);
         *p=(Quantum) v;
         p++;
@@ -638,9 +620,11 @@ MagickExport void Hull(const long x_offset,const long y_offset,
     else
       for (x=(long) columns; x > 0; x--)
       {
-        v=(*q);
-        if ((*s <= (v-(long) ScaleCharToQuantum(2))) && (*r < v))
-          v-=(long) ScaleCharToQuantum(1);
+        v=(MagickRealType) (*q);
+        if (((MagickRealType) *s <=
+             (v-(MagickRealType) ScaleCharToQuantum(2))) &&
+            ((MagickRealType) *r < v))
+          v-=(MagickRealType) ScaleCharToQuantum(1);
         *p=(Quantum) v;
         p++;
         q++;
@@ -665,8 +649,8 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method InterpolateColor applies bi-linear interpolation between a pixel and
-%  it's neighbors.
+%  InterpolateColor() applies bi-linear interpolation between a pixel and its
+%  neighbors.
 %
 %  The format of the InterpolateColor method is:
 %
@@ -685,7 +669,7 @@ MagickExport void Hull(const long x_offset,const long y_offset,
 MagickExport PixelPacket InterpolateColor(const Image *image,
   const double x_offset,const double y_offset,ExceptionInfo *exception)
 {
-  double
+  MagickRealType
     alpha,
     beta;
 
@@ -697,6 +681,8 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   p=AcquireImagePixels(image,(long) x_offset,(long) y_offset,2,2,exception);
   if (p == (const PixelPacket *) NULL)
     return(AcquireOnePixel(image,(long) x_offset,(long) y_offset,exception));
@@ -708,8 +694,11 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
     alpha*p[1].green)+beta*((1.0-alpha)*p[2].green+alpha*p[3].green)+0.5);
   color.blue=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].blue+
     alpha*p[1].blue)+beta*((1.0-alpha)*p[2].blue+alpha*p[3].blue)+0.5);
-  color.opacity=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].opacity+
-    alpha*p[1].opacity)+beta*((1.0-alpha)*p[2].opacity+alpha*p[3].opacity)+0.5);
+  if (image->matte == MagickFalse)
+    color.opacity=p[0].opacity;
+  else
+    color.opacity=(Quantum) ((1.0-beta)*((1.0-alpha)*p[0].opacity+alpha*
+      p[1].opacity)+beta*((1.0-alpha)*p[2].opacity+alpha*p[3].opacity)+0.5);
   return(color);
 }
 
@@ -724,8 +713,7 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method Modulate modulates the hue, saturation, and brightness of an
-%  image.
+%  Modulate() modulates the hue, saturation, and brightness of an image.
 %
 %  The format of the Modulate method is:
 %
@@ -736,7 +724,7 @@ MagickExport PixelPacket InterpolateColor(const Image *image,
 %  A description of each parameter follows:
 %
 %    o percent_hue, percent_saturation, percent_brightness: A double value
-%      representing the percent change in a component of the HSL color space.
+%      representing the percent change in a component of the HSB color space.
 %
 %    o red, green, blue: A pointer to a pixel component of type Quantum.
 %
@@ -757,26 +745,88 @@ MagickExport void Modulate(const double percent_hue,
   assert(red != (Quantum *) NULL);
   assert(green != (Quantum *) NULL);
   assert(blue != (Quantum *) NULL);
-  TransformHSL(*red,*green,*blue,&hue,&saturation,&brightness);
-  brightness*=(0.01+MagickEpsilon)*percent_brightness;
-  if (brightness < 0.0)
-    brightness=0.0;
-  else
-    if (brightness > 1.0)
-      brightness=1.0;
-  saturation*=(0.01+MagickEpsilon)*percent_saturation;
-  if (saturation < 0.0)
-    saturation=0.0;
-  else
-    if (saturation > 1.0)
-      saturation=1.0;
-  hue*=(0.01+MagickEpsilon)*percent_hue;
-  if (hue < 0.0)
+  TransformHSB(*red,*green,*blue,&hue,&saturation,&brightness);
+  brightness*=0.01*percent_brightness;
+  saturation*=0.01*percent_saturation;
+  hue*=0.01*percent_hue;
+  while (hue < 0.0)
     hue+=1.0;
+  while (hue > 1.0)
+    hue-=1.0;
+  HSBTransform(hue,saturation,brightness,red,green,blue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   T r a n s f o r m H S B                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransformHSB() converts a (red, green, blue) to a (hue, saturation,
+%  brightness) triple.
+%
+%  The format of the TransformHSB method is:
+%
+%      void TransformHSB(const Quantum red,const Quantum green,
+%        const Quantum blue,double *hue,double *saturation,double *brightness)
+%
+%  A description of each parameter follows:
+%
+%    o red, green, blue: A Quantum value representing the red, green, and
+%      blue component of a pixel..
+%
+%    o hue, saturation, brightness: A pointer to a double value representing a
+%      component of the HSB color space.
+%
+%
+*/
+MagickExport void TransformHSB(const Quantum red,const Quantum green,
+  const Quantum blue,double *hue,double *saturation,double *brightness)
+{
+  MagickRealType
+    delta,
+    max,
+    min;
+
+  /*
+    Convert RGB to HSB colorspace.
+  */
+  assert(hue != (double *) NULL);
+  assert(saturation != (double *) NULL);
+  assert(brightness != (double *) NULL);
+  max=(MagickRealType) (red > green ? red : green);
+  if ((MagickRealType) blue > max)
+    max=(MagickRealType) blue;
+  min=(MagickRealType) (red < green ? red : green);
+  if ((MagickRealType) blue < min)
+    min=(MagickRealType) blue;
+  *hue=0.0;
+  *saturation=0.0;
+  if (max != 0.0)
+    *saturation=(max-min)/max;
+  *brightness=QuantumScale*max;
+  if (*saturation == 0.0)
+    return;
+  delta=max-min;
+  if ((MagickRealType) red == max)
+    *hue=(green-(MagickRealType) blue)/delta;
   else
-    if (hue > 1.0)
-      hue-=1.0;
-  HSLTransform(hue,saturation,brightness,red,green,blue);
+    if ((MagickRealType) green == max)
+      *hue=2.0+(blue-(MagickRealType) red)/delta;
+    else
+      if ((MagickRealType) blue == max)
+        *hue=4.0+(red-(MagickRealType) green)/delta;
+  *hue/=6.0;
+  if (*hue < 0.0)
+    *hue+=1.0;
+  else
+    if (*hue > 1.0)
+      *hue-=1.0;
 }
 
 /*
@@ -790,7 +840,7 @@ MagickExport void Modulate(const double percent_hue,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method TransformHSL converts a (red, green, blue) to a (hue, saturation,
+%  TransformHSL() converts a (red, green, blue) to a (hue, saturation,
 %  luminosity) triple.
 %
 %  The format of the TransformHSL method is:
@@ -811,7 +861,7 @@ MagickExport void Modulate(const double percent_hue,
 MagickExport void TransformHSL(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *saturation,double *luminosity)
 {
-  double
+  MagickRealType
     b,
     delta,
     g,
@@ -825,9 +875,9 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   assert(hue != (double *) NULL);
   assert(saturation != (double *) NULL);
   assert(luminosity != (double *) NULL);
-  r=(double) red/MaxRGB;
-  g=(double) green/MaxRGB;
-  b=(double) blue/MaxRGB;
+  r=(MagickRealType) red/MaxRGB;
+  g=(MagickRealType) green/MaxRGB;
+  b=(MagickRealType) blue/MaxRGB;
   max=Max(r,Max(g,b));
   min=Min(r,Min(g,b));
   *hue=0.0;
@@ -836,7 +886,7 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
   delta=max-min;
   if (delta == 0.0)
     return;
-  *saturation=delta/((*luminosity <= 0.5) ? (min+max) : (2.0-max-min));
+  *saturation=delta/((*luminosity < 0.5) ? (min+max) : (2.0-max-min));
   if (r == max)
     *hue=(g == min ? 5.0+(max-b)/delta : 1.0-(max-g)/delta);
   else
@@ -858,8 +908,8 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method TransformHWB converts a (red, green, blue) to a
-%  (hue, whiteness, blackness) triple.
+%  TransformHWB() converts a (red, green, blue) to a (hue, whiteness,
+%  blackness) triple.
 %
 %  The format of the TransformHWB method is:
 %
@@ -879,7 +929,7 @@ MagickExport void TransformHSL(const Quantum red,const Quantum green,
 MagickExport void TransformHWB(const Quantum red,const Quantum green,
   const Quantum blue,double *hue,double *whiteness,double *blackness)
 {
-  double
+  MagickRealType
     f;
 
   register long
@@ -904,11 +954,11 @@ MagickExport void TransformHWB(const Quantum red,const Quantum green,
       *whiteness=1.0-(*blackness);
       return;
     }
-  f=(red == w) ? (double) green-blue : ((green == w) ? (double) blue-red :
-    (double) red-green);
+  f=(red == w) ? (MagickRealType) green-blue : ((green == w) ?
+    (MagickRealType) blue-red : (MagickRealType) red-green);
   i=(red == w) ? 3 : ((green == w) ? 5 : 1);
-  *hue=i-f/(v-w);
-  *whiteness=(double) w/MaxRGB;
+  *hue=(MagickRealType) (i-f/(v-w));
+  *whiteness=(MagickRealType) w/MaxRGB;
 }
 
 /*
@@ -968,27 +1018,30 @@ MagickExport void Upsample(const unsigned long width,const unsigned long height,
       p--;
       q-=2;
       *q=(*p);
-      *(q+1)=(unsigned char) ((((long) *p)+((long) *(p+1))+1) >> 1);
+      *(q+1)=(unsigned char) ((((unsigned long) *p)+
+        ((unsigned long) *(p+1))+1) >> 1);
     }
   }
   for (y=0; y < (long) (height-1); y++)
   {
-    p=pixels+(y << 1)*scaled_width;
+    p=pixels+((unsigned long) y << 1)*scaled_width;
     q=p+scaled_width;
     r=q+scaled_width;
     for (x=0; x < (long) (width-1); x++)
     {
-      *q=(unsigned char) ((((long) *p)+((long) *r)+1) >> 1);
-      *(q+1)=(unsigned char)
-        ((((long) *p)+((long) *(p+2))+((long) *r)+((long) *(r+2))+2) >> 2);
+      *q=(unsigned char) ((((unsigned long) *p)+((unsigned long) *r)+1) >> 1);
+      *(q+1)=(unsigned char) ((((unsigned long) *p)+((unsigned long) *(p+2))+
+        ((unsigned long) *r)+((unsigned long) *(r+2))+2) >> 2);
       q+=2;
       p+=2;
       r+=2;
     }
-    *q++=(unsigned char) ((((long) *p++)+((long) *r++)+1) >> 1);
-    *q++=(unsigned char) ((((long) *p++)+((long) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((unsigned long) *p++)+
+      ((unsigned long) *r++)+1) >> 1);
+    *q++=(unsigned char) ((((unsigned long) *p++)+
+      ((unsigned long) *r++)+1) >> 1);
   }
   p=pixels+(2*height-2)*scaled_width;
   q=pixels+(2*height-1)*scaled_width;
-  (void) memcpy(q,p,2*width);
+  (void) CopyMagickMemory(q,p,(size_t) (2*width));
 }
