@@ -12,37 +12,24 @@
 %                                                                             %
 %                       ImageMagick Geometry Methods                          %
 %                                                                             %
-%                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,8 +40,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
-#include "magick/utility.h"
+#include "magick/geometry.h"
+#include "magick/memory_.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -75,7 +65,7 @@
 %
 %  The format of the GetGeometry method is:
 %
-%      unsigned int GetGeometry(const char *geometry,long *x,long *y,
+%      MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
 %        unsigned long *width,unsigned long *height)
 %
 %  A description of each parameter follows:
@@ -89,7 +79,7 @@
 %
 %
 */
-MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
+MagickExport MagickStatusType GetGeometry(const char *geometry,long *x,long *y,
   unsigned long *width,unsigned long *height)
 {
   char
@@ -97,21 +87,23 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
     pedantic_geometry[MaxTextExtent],
     *q;
 
-  unsigned int
+  MagickStatusType
     flags;
 
   /*
     Remove whitespace and meta characters from geometry specification.
   */
-  if ((geometry == (char *) NULL) || (*geometry == '\0'))
-    return(NoValue);
   flags=NoValue;
-  (void) strncpy(pedantic_geometry,geometry,MaxTextExtent-1);
+  if ((geometry == (char *) NULL) || (*geometry == '\0'))
+    return(flags);
+  if (strlen(geometry) >= MaxTextExtent)
+    return(flags);
+  (void) CopyMagickString(pedantic_geometry,geometry,MaxTextExtent);
   for (p=pedantic_geometry; *p != '\0'; )
   {
-    if (isspace((int) (*p)))
+    if (isspace((int) ((unsigned char) *p)) != 0)
       {
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         continue;
       }
     switch (*p)
@@ -119,35 +111,60 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
       case '%':
       {
         flags|=PercentValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '!':
       {
         flags|=AspectValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '<':
       {
         flags|=LessValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '>':
       {
         flags|=GreaterValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '@':
       {
         flags|=AreaValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
+      case '(':
+      case ')':
+      {
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
+      case '-':
+      case '.':
+      case '+':
+      case '0':
+      case '1':
+      case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6':
+      case '7':
+      case '8':
+      case '9':
+      case 'x':
+      case 'X':
+      {
+        p++;
         break;
       }
       default:
-        p++;
+        return(flags);
     }
   }
   /*
@@ -157,7 +174,9 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
   if (*p == '\0')
     return(flags);
   q=p;
-  (void) strtol(p,&q,10);
+  (void) strtod(p,&q);
+  if (LocaleNCompare(p,"0x",2) == 0)
+    (void) strtol(p,&q,10);
   if ((*q == 'x') || (*q == 'X') || (*q == '\0'))
     {
       /*
@@ -167,7 +186,7 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
       if (LocaleNCompare(p,"0x",2) == 0)
         *width=(unsigned long) strtol(p,&p,10);
       else
-        *width=(unsigned long) floor(strtod(p,&p)+0.5);
+        *width=(unsigned long) (strtod(p,&p)+0.5);
       if (p != q)
         flags|=WidthValue;
     }
@@ -180,7 +199,7 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
             Parse height.
           */
           q=p;
-          *height=(unsigned long) floor(strtod(p,&p)+0.5);
+          *height=(unsigned long) (strtod(p,&p)+0.5);
           if (p != q)
             flags|=HeightValue;
         }
@@ -193,7 +212,7 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
       if (*p == '-')
         flags|=XNegative;
       q=p;
-      *x=(long) ceil(strtod(p,&p)-0.5);
+      *x=(long) (strtod(p,&p)+0.5);
       if (p != q)
         flags|=XValue;
       if ((*p == '+') || (*p == '-'))
@@ -204,39 +223,11 @@ MagickExport unsigned int GetGeometry(const char *geometry,long *x,long *y,
           if (*p == '-')
             flags|=YNegative;
           q=p;
-          *y=(long) ceil(strtod(p,&p)-0.5);
+          *y=(long) (strtod(p,&p)+0.5);
           if (p != q)
             flags|=YValue;
         }
     }
-  if (flags == NoValue)
-    {
-      ExceptionInfo
-        exception;
-
-      Image
-        *image;
-
-      ImageInfo
-        *image_info;
-
-      /*
-        See if we can grab geometry from an image.
-      */
-      GetExceptionInfo(&exception);
-      image_info=CloneImageInfo((ImageInfo *) NULL);
-      (void) strncpy(image_info->filename,geometry,MaxTextExtent-1);
-      image=PingImage(image_info,&exception);
-      if (image != (Image *) NULL)
-        {
-          *width=image->columns;
-          *height=image->rows;
-          flags|=WidthValue | HeightValue;
-          DestroyImage(image);
-        }
-      DestroyImageInfo(image_info);
-      DestroyExceptionInfo(&exception);
-    }
   return(flags);
 }
 
@@ -349,26 +340,27 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
     i;
 
   assert(page_geometry != (char *) NULL);
-  page=AllocateString(page_geometry);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),page_geometry);
+  page=AcquireString(page_geometry);
   for (i=0; *PageSizes[i] != (char *) NULL; i++)
     if (LocaleNCompare(PageSizes[i][0],page,strlen(PageSizes[i][0])) == 0)
       {
         RectangleInfo
           geometry;
 
-        unsigned int
+        MagickStatusType
           flags;
 
         /*
           Replace mneumonic with the equivalent size in dots-per-inch.
         */
-        (void) strncpy(page,PageSizes[i][1],MaxTextExtent-1);
-        (void) strncat(page,page_geometry+strlen(PageSizes[i][0]),
-          MaxTextExtent-strlen(page)-2);
+        (void) CopyMagickString(page,PageSizes[i][1],MaxTextExtent);
+        (void) ConcatenateMagickString(page,page_geometry+
+          strlen(PageSizes[i][0]),MaxTextExtent);
         flags=GetGeometry(page,&geometry.x,&geometry.y,&geometry.width,
           &geometry.height);
-        if (!(flags & GreaterValue))
-          (void) strcat(page,">");
+        if ((flags & GreaterValue) == 0)
+          (void) ConcatenateMagickString(page,">",MaxTextExtent);
         break;
       }
   return(page);
@@ -384,12 +376,12 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsGeometry() returns True if the geometry specification is valid.  Examples
+%  IsGeometry() returns MagickTrue if the geometry specification is valid.  Examples
 %  are 100, 100x200, x200, 100x200+10+20, +10+20, 200%, 200x200!, etc.
 %
 %  The format of the IsGeometry method is:
 %
-%      unsigned int IsGeometry(const char *geometry)
+%      MagickBooleanType IsGeometry(const char *geometry)
 %
 %  A description of each parameter follows:
 %
@@ -397,18 +389,18 @@ MagickExport char *GetPageGeometry(const char *page_geometry)
 %
 %
 */
-MagickExport unsigned int IsGeometry(const char *geometry)
+MagickExport MagickBooleanType IsGeometry(const char *geometry)
 {
   GeometryInfo
     geometry_info;
 
-  unsigned int
+  MagickStatusType
     flags;
 
   if (geometry == (const char *) NULL)
-    return(False);
+    return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
-  return(flags != NoValue);
+  return((MagickBooleanType) (flags != NoValue));
 }
 
 /*
@@ -421,13 +413,13 @@ MagickExport unsigned int IsGeometry(const char *geometry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsSceneGeometry() returns True if the geometry is a valid scene
+%  IsSceneGeometry() returns MagickTrue if the geometry is a valid scene
 %  specification (e.g. [1], [1-9], [1,7,4]).
 %
 %  The format of the IsSceneGeometry method is:
 %
-%      unsigned int IsSceneGeometry(const char *geometry,
-%        const unsigned int pedantic)
+%      MagickBooleanType IsSceneGeometry(const char *geometry,
+%        const MagickBooleanType pedantic)
 %
 %  A description of each parameter follows:
 %
@@ -438,17 +430,24 @@ MagickExport unsigned int IsGeometry(const char *geometry)
 %
 %
 */
-MagickExport unsigned int IsSceneGeometry(const char *geometry,
-  const unsigned int pedantic)
+MagickExport MagickBooleanType IsSceneGeometry(const char *geometry,
+  const MagickBooleanType pedantic)
 {
+  char
+    *p;
+
   if (geometry == (const char *) NULL)
-    return(False);
+    return(MagickFalse);
+  p=(char *) geometry;
+  (void) strtod(geometry,&p);
+  if (p == geometry)
+    return(MagickFalse);
   if ((strchr(geometry,'x') != (char *) NULL) ||
       (strchr(geometry,'X') != (char *) NULL))
-    return(False);
-  if (pedantic && (strchr(geometry,',') != (char *) NULL))
-    return(False);
-  return(True);
+    return(MagickFalse);
+  if ((pedantic != MagickFalse) && (strchr(geometry,',') != (char *) NULL))
+    return(MagickFalse);
+  return(MagickTrue);
 }
 
 /*
@@ -466,7 +465,7 @@ MagickExport unsigned int IsSceneGeometry(const char *geometry,
 %
 %  The format of the ParseAbsoluteGeometry method is:
 %
-%      unsigned int ParseAbsoluteGeometry(const char *geometry,
+%      MagickStatusType ParseAbsoluteGeometry(const char *geometry,
 %        RectangeInfo *region_info)
 %
 %  A description of each parameter follows:
@@ -477,10 +476,10 @@ MagickExport unsigned int IsSceneGeometry(const char *geometry,
 %
 %
 */
-MagickExport unsigned int ParseAbsoluteGeometry(const char *geometry,
+MagickExport MagickStatusType ParseAbsoluteGeometry(const char *geometry,
   RectangleInfo *region_info)
 {
-  unsigned int
+  MagickStatusType
     flags;
 
   flags=GetGeometry(geometry,&region_info->x,&region_info->y,
@@ -507,7 +506,7 @@ MagickExport unsigned int ParseAbsoluteGeometry(const char *geometry,
 %
 %  The format of the ParseGeometry method is:
 %
-%      unsigned int ParseGeometry(const char *geometry,
+%      MagickStatusType ParseGeometry(const char *geometry,
 %        GeometryInfo *geometry_info)
 %
 %  A description of each parameter follows:
@@ -518,7 +517,7 @@ MagickExport unsigned int ParseAbsoluteGeometry(const char *geometry,
 %
 %
 */
-MagickExport unsigned int ParseGeometry(const char *geometry,
+MagickExport MagickStatusType ParseGeometry(const char *geometry,
   GeometryInfo *geometry_info)
 {
   char
@@ -526,22 +525,24 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
     pedantic_geometry[MaxTextExtent],
     *q;
 
-  unsigned int
+  MagickStatusType
     flags;
 
   /*
     Remove whitespaces meta characters from geometry specification.
   */
   assert(geometry_info != (GeometryInfo *) NULL);
-  if ((geometry == (char *) NULL) || (*geometry == '\0'))
-    return(NoValue);
   flags=NoValue;
-  (void) strncpy(pedantic_geometry,geometry,MaxTextExtent-1);
+  if ((geometry == (char *) NULL) || (*geometry == '\0'))
+    return(flags);
+  if (strlen(geometry) >= MaxTextExtent)
+    return(flags);
+  (void) CopyMagickString(pedantic_geometry,geometry,MaxTextExtent);
   for (p=pedantic_geometry; *p != '\0'; )
   {
-    if (isspace((int) (*p)))
+    if (isspace((int) ((unsigned char) *p)) != 0)
       {
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         continue;
       }
     switch (*p)
@@ -549,35 +550,62 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
       case '%':
       {
         flags|=PercentValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '!':
       {
         flags|=AspectValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '<':
       {
         flags|=LessValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '>':
       {
         flags|=GreaterValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
         break;
       }
       case '@':
       {
         flags|=AreaValue;
-        (void) strcpy(p,p+1);
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
+      case '(':
+      case ')':
+      {
+        (void) CopyMagickString(p,p+1,MaxTextExtent);
+        break;
+      }
+      case '-':
+      case '.':
+      case '+':
+      case '0':
+      case '1':
+      case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6':
+      case '7':
+      case '8':
+      case '9':
+      case 'x':
+      case 'X':
+      case ',':
+      case '/':
+      {
+        p++;
         break;
       }
       default:
-        p++;
+        return(flags);
     }
   }
   /*
@@ -588,6 +616,8 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
     return(flags);
   q=p;
   (void) strtod(p,&q);
+  if (LocaleNCompare(p,"0x",2) == 0)
+    (void) strtol(p,&q,10);
   if ((*q == 'x') || (*q == 'X') || (*q == '/') || (*q == ',') || (*q =='\0'))
     {
       /*
@@ -595,7 +625,7 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
       */
       q=p;
       if (LocaleNCompare(p,"0x",2) == 0)
-        geometry_info->rho=strtol(p,&p,10);
+        geometry_info->rho=(double) strtol(p,&p,10);
       else
         geometry_info->rho=strtod(p,&p);
       if (p != q)
@@ -643,34 +673,22 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
                 flags|=PsiNegative;
             }
         }
-    }
-  if (flags == NoValue)
-    {
-      ExceptionInfo
-        exception;
-
-      Image
-        *image;
-
-      ImageInfo
-        *image_info;
-
-      /*
-        See if we can grab geometry from an image.
-      */
-      GetExceptionInfo(&exception);
-      image_info=CloneImageInfo((ImageInfo *) NULL);
-      (void) strncpy(image_info->filename,geometry,MaxTextExtent-1);
-      image=PingImage(image_info,&exception);
-      if (image != (Image *) NULL)
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
         {
-          geometry_info->rho=image->columns;
-          geometry_info->sigma=image->rows;
-          flags|=RhoValue | SigmaValue;
-          DestroyImage(image);
+          /*
+            Parse chi value.
+          */
+          if ((*p == ',') || (*p == '/'))
+            p++;
+          q=p;
+          geometry_info->chi=strtod(p,&p);
+          if (p != q)
+            {
+              flags|=ChiValue;
+              if (*q == '-')
+                flags|=ChiNegative;
+            }
         }
-      DestroyImageInfo(image_info);
-      DestroyExceptionInfo(&exception);
     }
   return(flags);
 }
@@ -691,8 +709,8 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
 %
 %  The format of the ParseGravityGeometry method is:
 %
-%      unsigned int ParseGravityGeometry(const Image *image,
-%        const char *geometry,RectangeInfo *region_info)
+%      MagickStatusType ParseGravityGeometry(Image *image,const char *geometry,
+%        RectangeInfo *region_info)
 %
 %  A description of each parameter follows:
 %
@@ -703,19 +721,24 @@ MagickExport unsigned int ParseGeometry(const char *geometry,
 %
 %
 */
-MagickExport unsigned int ParseGravityGeometry(const Image *image,
+MagickExport MagickStatusType ParseGravityGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
 {
   char
     region_geometry[MaxTextExtent];
 
-  unsigned int
+  MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
-  FormatString(region_geometry,"%.1024s!",geometry);
-  flags=ParseMetaGeometry(region_geometry,&region_info->x,&region_info->y,
-    &region_info->width,&region_info->height);
+  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
+  flags=ParseAbsoluteGeometry(region_geometry,region_info);
+  if (flags == NoValue)
+    {
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        OptionError,"InvalidGeometry",geometry);
+      return(flags);
+    }
   switch (image->gravity)
   {
     case ForgetGravity:
@@ -768,6 +791,31 @@ MagickExport unsigned int ParseGravityGeometry(const Image *image,
       break;
     }
   }
+  if ((flags & PercentValue) != 0)
+    {
+      GeometryInfo
+        geometry_info;
+
+      MagickRealType
+        x_scale,
+        y_scale;
+
+      MagickStatusType
+        flags;
+
+      /*
+        Geometry is a percentage of the image size.
+      */
+      flags=ParseGeometry(geometry,&geometry_info);
+      x_scale=geometry_info.rho;
+      if ((flags & RhoValue) == 0)
+        x_scale=100.0;
+      y_scale=geometry_info.sigma;
+      if ((flags & SigmaValue) == 0)
+        y_scale=x_scale;
+      region_info->width=(unsigned long) ((x_scale*image->columns/100.0)+0.5);
+      region_info->height=(unsigned long) ((y_scale*image->rows/100.0)+0.5);
+    }
   return(flags);
 }
 
@@ -788,7 +836,7 @@ MagickExport unsigned int ParseGravityGeometry(const Image *image,
 %
 %  The format of the ParseMetaGeometry method is:
 %
-%      unsigned int ParseMetaGeometry(const char *geometry,long *x,long *y,
+%      MagickStatusType ParseMetaGeometry(const char *geometry,long *x,long *y,
 %        unsigned long *width,unsigned long *height)
 %
 %  A description of each parameter follows:
@@ -802,13 +850,13 @@ MagickExport unsigned int ParseGravityGeometry(const Image *image,
 %
 %
 */
-MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
+MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,long *x,
   long *y,unsigned long *width,unsigned long *height)
 {
   GeometryInfo
     geometry_info;
 
-  unsigned int
+  MagickStatusType
     flags;
 
   unsigned long
@@ -824,6 +872,7 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
   assert(height != (unsigned long *) NULL);
   if ((geometry == (char *) NULL) || (*geometry == '\0'))
     return(NoValue);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),geometry);
   /*
     Parse geometry using GetGeometry.
   */
@@ -831,13 +880,13 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
   former_width=(*width);
   former_height=(*height);
   flags=GetGeometry(geometry,x,y,width,height);
-  if (flags & PercentValue)
+  if ((flags & PercentValue) != 0)
     {
-      double
+      MagickRealType
         x_scale,
         y_scale;
 
-      unsigned int
+      MagickStatusType
         flags;
 
       /*
@@ -845,35 +894,20 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
       */
       flags=ParseGeometry(geometry,&geometry_info);
       x_scale=geometry_info.rho;
+      if ((flags & RhoValue) == 0)
+        x_scale=100.0;
       y_scale=geometry_info.sigma;
-      if (!(flags & SigmaValue))
+      if ((flags & SigmaValue) == 0)
         y_scale=x_scale;
-      *width=(unsigned long) floor((x_scale*former_width/100.0)+0.5);
-      *height=(unsigned long) floor((y_scale*former_height/100.0)+0.5);
+      *width=(unsigned long) ((x_scale*former_width/100.0)+0.5);
+      *height=(unsigned long) ((y_scale*former_height/100.0)+0.5);
       former_width=(*width);
       former_height=(*height);
     }
-  if (flags & AreaValue)
-    {
-      double
-        area,
-        distance;
-
-      /*
-        Geometry is a maximum area in pixels.
-      */
-      (void) ParseGeometry(geometry,&geometry_info);
-      area=geometry_info.rho;
-      distance=sqrt((double) former_width*former_height);
-      *width=(unsigned long) (former_width/(distance/sqrt(area)));
-      *height=(unsigned long) (former_height/(distance/sqrt(area)));
-      former_width=(*width);
-      former_height=(*height);
-    }
-  if (!(flags & AspectValue) &&
+  if (((flags & AspectValue) == 0) &&
       ((*width != former_width) || (*height != former_height)))
     {
-      double
+      MagickRealType
         scale_factor;
 
       /*
@@ -884,32 +918,56 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
       else
         if (((flags & RhoValue) != 0) && (flags & SigmaValue) != 0)
           {
-            scale_factor=(double) *width/former_width;
-            if (scale_factor > ((double) *height/former_height))
-              scale_factor=(double) *height/former_height;
+            scale_factor=(MagickRealType) *width/(MagickRealType) former_width;
+            if (scale_factor > ((MagickRealType) *height/
+                (MagickRealType) former_height))
+              scale_factor=(MagickRealType)
+                *height/(MagickRealType) former_height;
           }
         else
           if ((flags & RhoValue) != 0)
-            scale_factor=(double) *width/former_width;
+            scale_factor=(MagickRealType) *width/(MagickRealType) former_width;
           else
-            scale_factor=(double) *height/former_height;
-    *width=(unsigned long) floor(scale_factor*former_width+0.5);
-    *height=(unsigned long) floor(scale_factor*former_height+0.5);
+            scale_factor=(MagickRealType)
+              *height/(MagickRealType) former_height;
+    *width=(unsigned long) (scale_factor*former_width+0.5);
+    *height=(unsigned long) (scale_factor*former_height+0.5);
   }
-  if (flags & GreaterValue)
+  if ((flags & GreaterValue) != 0)
     {
       if (former_width < *width)
         *width=former_width;
       if (former_height < *height)
         *height=former_height;
     }
-  if (flags & LessValue)
+  if ((flags & LessValue) != 0)
     {
       if (former_width > *width)
         *width=former_width;
       if (former_height > *height)
         *height=former_height;
     }
+  if ((flags & AreaValue) != 0)
+    {
+      MagickRealType
+        area,
+        distance;
+
+      /*
+        Geometry is a maximum area in pixels.
+      */
+      (void) ParseGeometry(geometry,&geometry_info);
+      area=geometry_info.rho;
+      distance=sqrt((double) former_width*former_height);
+      if (((former_width/(distance/sqrt(area))) < *width) ||
+          ((former_height/(distance/sqrt(area))) < *height))
+        {
+          *width=(unsigned long) (former_width/(distance/sqrt(area)));
+          *height=(unsigned long) (former_height/(distance/sqrt(area)));
+        }
+      former_width=(*width);
+      former_height=(*height);
+    }
   return(flags);
 }
 
@@ -929,7 +987,7 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
 %
 %  The format of the ParsePageGeometry method is:
 %
-%      unsigned int ParsePageGeometry(const Image *image,const char *geometry,
+%      MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
 %        RectangeInfo *region_info)
 %
 %  A description of each parameter follows:
@@ -941,19 +999,22 @@ MagickExport unsigned int ParseMetaGeometry(const char *geometry,long *x,
 %
 %
 */
-MagickExport unsigned int ParsePageGeometry(const Image *image,
-  const char *geometry,RectangleInfo *region_info)
+MagickExport MagickStatusType ParsePageGeometry(Image *image,const char *geometry,
+  RectangleInfo *region_info)
 {
   char
     region_geometry[MaxTextExtent];
 
-  unsigned int
+  MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
-  FormatString(region_geometry,"%.1024s!",geometry);
+  (void) FormatMagickString(region_geometry,MaxTextExtent,"%s!",geometry);
   flags=ParseMetaGeometry(region_geometry,&region_info->x,&region_info->y,
     &region_info->width,&region_info->height);
+  if (flags == NoValue)
+    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
+      "InvalidGeometry",geometry);
   return(flags);
 }
 
@@ -973,7 +1034,7 @@ MagickExport unsigned int ParsePageGeometry(const Image *image,
 %
 %  The format of the ParseSizeGeometry method is:
 %
-%      unsigned int ParseSizeGeometry(const Image *image,const char *geometry,
+%      MagickStatusType ParseSizeGeometry(Image *image,const char *geometry,
 %        RectangeInfo *region_info)
 %
 %  A description of each parameter follows:
@@ -984,15 +1045,18 @@ MagickExport unsigned int ParsePageGeometry(const Image *image,
 %
 %
 */
-MagickExport unsigned int ParseSizeGeometry(const Image *image,
+MagickExport MagickStatusType ParseSizeGeometry(Image *image,
   const char *geometry,RectangleInfo *region_info)
 {
-  unsigned int
+  MagickStatusType
     flags;
 
   SetGeometry(image,region_info);
   flags=ParseMetaGeometry(geometry,&region_info->x,&region_info->y,
     &region_info->width,&region_info->height);
+  if (flags == NoValue)
+    (void) ThrowMagickException(&image->exception,GetMagickModule(),OptionError,
+      "InvalidGeometry",geometry);
   return(flags);
 }
 
@@ -1024,8 +1088,11 @@ MagickExport unsigned int ParseSizeGeometry(const Image *image,
 MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 {
   assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(geometry != (RectangleInfo *) NULL);
-  (void) memset(geometry,0,sizeof(RectangleInfo));
+  (void) ResetMagickMemory(geometry,0,sizeof(RectangleInfo));
   geometry->width=image->columns;
   geometry->height=image->rows;
 }
@@ -1056,5 +1123,6 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
 MagickExport void SetGeometryInfo(GeometryInfo *geometry_info)
 {
   assert(geometry_info != (GeometryInfo *) NULL);
-  (void) memset(geometry_info,0,sizeof(GeometryInfo));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
+  (void) ResetMagickMemory(geometry_info,0,sizeof(GeometryInfo));
 }
