@@ -13,56 +13,49 @@
 %                                                                             %
 %                  Windows NT Feature Methods for ImageMagick                 %
 %                                                                             %
-%                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %
 */
 
-#if defined(WIN32) || defined(__CYGWIN__)
 /*
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/cache.h"
-#include "magick/error.h"
-#include "magick/utility.h"
-#include "magick/monitor.h"
-#include "magick/nt_feature.h"
+#if defined(__WINDOWS__) || defined(__CYGWIN__)
 #define WIN32_LEAN_AND_MEAN
 #define VC_EXTRALEAN
 #include <windows.h>
+#include "magick/cache.h"
+#include "magick/colorspace.h"
+#include "magick/draw.h"
+#include "magick/error.h"
+#include "magick/hashmap.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/string_.h"
+#include "magick/token.h"
+#include "magick/utility.h"
+#include "magick/nt_feature.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -75,10 +68,10 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method CropImageToHBITMAP extracts a specified region of the image and
-%  returns it as a Windows HBITMAP. While the same functionality can be
-%  accomplished by invoking CropImage() followed by ImageToHBITMAP(), this
-%  method is more efficient since it copies pixels directly to the HBITMAP.
+%  CropImageToHBITMAP() extracts a specified region of the image and returns
+%  it as a Windows HBITMAP. While the same functionality can be accomplished by
+%  invoking CropImage() followed by ImageToHBITMAP(), this method is more
+%  efficient since it copies pixels directly to the HBITMAP.
 %
 %  The format of the CropImageToHBITMAP method is:
 %
@@ -130,6 +123,8 @@ MagickExport void *CropImageToHBITMAP(Image *image,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -137,8 +132,7 @@ MagickExport void *CropImageToHBITMAP(Image *image,
       ((geometry->y+(long) geometry->height) < 0) ||
       (geometry->x >= (long) image->columns) ||
       (geometry->y >= (long) image->rows))
-    ThrowImageException(OptionError,"GeometryDoesNotContainImage",
-      "UnableToCropImage");
+    ThrowImageException(OptionError,"GeometryDoesNotContainImage");
   page=(*geometry);
   if ((page.x+(long) page.width) > (long) image->columns)
     page.width=image->columns-page.x;
@@ -156,8 +150,7 @@ MagickExport void *CropImageToHBITMAP(Image *image,
     }
 
   if ((page.width == 0) || (page.height == 0))
-    ThrowImageException(OptionError,"GeometryDimensionsAreZero",
-      "UnableToCropImage");
+    ThrowImageException(OptionError,"GeometryDimensionsAreZero");
   /*
     Initialize crop image attributes.
   */
@@ -172,21 +165,16 @@ MagickExport void *CropImageToHBITMAP(Image *image,
   bitmap_bitsH = (HANDLE) GlobalAlloc (GMEM_MOVEABLE | GMEM_DDESHARE,
                page.width*page.height*bitmap.bmBitsPixel);
   if (bitmap_bitsH == NULL)
-    return( NULL ); 
-
-  bitmap_bits = (RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
-
+    return(NULL);
+  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
   if ( bitmap.bmBits == NULL )
     bitmap.bmBits = bitmap_bits;
-
   if (image->colorspace != RGBColorspace)
     SetImageColorspace(image,RGBColorspace);
-
   /*
     Extract crop image.
   */
-  q = bitmap_bits;
-
+  q=bitmap_bits;
   for (y=0; y < (long) page.height; y++)
   {
     p=AcquireImagePixels(image,page.x,page.y+y,page.width,1,exception);
@@ -195,7 +183,7 @@ MagickExport void *CropImageToHBITMAP(Image *image,
 
 #if QuantumDepth == 8
       /* Form of PixelPacket is identical to RGBQUAD when QuantumDepth==8 */
-      memcpy((void*)q,(const void*)p,page.width*sizeof(PixelPacket));
+      CopyMagickMemory((void*)q,(const void*)p,page.width*sizeof(PixelPacket));
       q += page.width;
 
 #else  /* 16 or 32 bit Quantum */
@@ -216,7 +204,7 @@ MagickExport void *CropImageToHBITMAP(Image *image,
       }
 #endif
     if (QuantumTick(y,page.height))
-      if (!MagickMonitor(CropImageText,y,page.height-1,exception))
+      if (MagickMonitor(CropImageText,y,page.height-1,exception) == MagickFalse)
         break;
   }
   if (y < (long) page.height)
@@ -245,28 +233,25 @@ MagickExport void *CropImageToHBITMAP(Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsMagickConflict returns true if the image format conflicts with
-%  a logical drive (.e.g. X:).
+%  IsMagickConflict() returns true if the image format conflicts with a logical
+%  drive (.e.g. X:).
 %
 %  The format of the IsMagickConflict method is:
 %
-%      unsigned int IsMagickConflict(const char *magick)
+%      MagickBooleanType IsMagickConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method IsMagickConflict returns true if the image format
-%      conflicts with a logical drive.
-%
 %    o magick: Specifies the image format.
 %
 %
 */
-MagickExport unsigned int NTIsMagickConflict(const char *magick)
+MagickExport MagickBooleanType NTIsMagickConflict(const char *magick)
 {
   assert(magick != (char *) NULL);
   if (strlen(magick) > 1)
-    return(False);
-  return((GetLogicalDrives()) & (1 << ((toupper((int) (*magick)))-'A')));
+    return(MagickFalse);
+  return((GetLogicalDrives() & (1 << ((toupper((int) (*magick)))-'A'))) != 0);
 }
 
 /*
@@ -280,35 +265,20 @@ MagickExport unsigned int NTIsMagickConflict(const char *magick)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method NTGetTypeList returns a TypeInfo list corresponding to installed
-%  Windows TrueType fonts. The user is responsible for destroying the
-%  returned list once it is no longer needed. NULL is returned if a fatal
-%  error is encountered.
+%  NTLoadTypeLists() loads a Windows TrueType fonts.
 %
-%  The format of the NTGetTypeList method is:
+%  The format of the NTLoadTypeLists method is:
 %
-%      TypeInfo* NTGetTypeList( void )
+%      MagickBooleanType NTLoadTypeLists(LinkedListInfo *type_list)
 %
 %  A description of each parameter follows:
 %
-%    o returns: A linked list of TypeInfo structures is returned.
+%    o type_list: A linked list of fonts.
 %
 */
-static int TypeInfoCompare(const void *x,const void *y)
+MagickExport MagickBooleanType NTLoadTypeLists(LinkedListInfo *type_list,
+  ExceptionInfo *exception)
 {
-  TypeInfo
-    **info_1,
-    **info_2;
-
-  info_1=(TypeInfo **) x;
-  info_2=(TypeInfo **) y;
-  return( strcmp((*info_1)->name, (*info_2)->name) );
-}
-MagickExport TypeInfo* NTGetTypeList( void )
-{
-  TypeInfo
-    *type_list;
-
   HKEY
     reg_key = (HKEY) INVALID_HANDLE_VALUE;
 
@@ -321,35 +291,52 @@ MagickExport TypeInfo* NTGetTypeList( void )
 
   char
     buffer[MaxTextExtent],
+    system_root[MaxTextExtent],
     font_root[MaxTextExtent];
 
-  type_list = (TypeInfo*) NULL;
+  DWORD
+    type,
+    system_root_length;
 
-  res = RegOpenKeyExA (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Fonts",
-                       0, KEY_READ | KEY_ENUMERATE_SUB_KEYS, &reg_key);
+  MagickBooleanType
+    status;
 
+  /*
+    Try to find the right Windows*\CurrentVersion key, the SystemRoot and
+    then the Fonts key
+  */
+  res = RegOpenKeyExA (HKEY_LOCAL_MACHINE,
+    "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", 0, KEY_READ, &reg_key);
+  if (res == ERROR_SUCCESS) {
+    system_root_length=sizeof(system_root)-1;
+    res = RegQueryValueExA(reg_key,"SystemRoot",NULL, &type,
+      (BYTE*) system_root, &system_root_length);
+  }
+  if (res != ERROR_SUCCESS) {
+    res = RegOpenKeyExA (HKEY_LOCAL_MACHINE,
+      "SOFTWARE\\Microsoft\\Windows\\CurrentVersion", 0, KEY_READ, &reg_key);
+    if (res == ERROR_SUCCESS) {
+      system_root_length=sizeof(system_root)-1;
+      res = RegQueryValueExA(reg_key,"SystemRoot",NULL, &type,
+        (BYTE*)system_root, &system_root_length);
+    }
+  }
+  if (res == ERROR_SUCCESS)
+    res = RegOpenKeyExA (reg_key, "Fonts",0, KEY_READ, &reg_key);
   if (res != ERROR_SUCCESS)
-    res = RegOpenKeyExA (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Fonts",
-                         0, KEY_READ | KEY_ENUMERATE_SUB_KEYS, &reg_key);
-
-  if (res != ERROR_SUCCESS)
-    return (TypeInfo *) type_list;
-
+    return(MagickFalse);
   *font_root='\0';
-  if ( getenv("SystemRoot") != (char *) NULL)
+  (void) CopyMagickString(buffer,system_root,MaxTextExtent);
+  (void) ConcatenateMagickString(buffer,"\\fonts\\arial.ttf",MaxTextExtent);
+  if (IsAccessible(buffer) != MagickFalse)
     {
-      strncpy(buffer,getenv("SystemRoot"),sizeof(buffer)-1);
-      strcat(buffer,"\\fonts\\arial.ttf");
-      if (IsAccessible(buffer))
-        {
-          strncpy(font_root,getenv("SystemRoot"),sizeof(buffer)-1);
-          strcat(font_root,"\\fonts\\");
-        }
-      else
-        {
-          strncpy(font_root,getenv("SystemRoot"),sizeof(buffer)-1);
-          strcat(font_root,"\\");
-        }
+      (void) CopyMagickString(font_root,system_root,MaxTextExtent);
+      (void) ConcatenateMagickString(font_root,"\\fonts\\",MaxTextExtent);
+    }
+  else
+    {
+      (void) CopyMagickString(font_root,system_root,MaxTextExtent);
+      (void) ConcatenateMagickString(font_root,"\\",MaxTextExtent);
     }
 
   {
@@ -368,7 +355,7 @@ MagickExport TypeInfo* NTGetTypeList( void )
 
     res = ERROR_SUCCESS;
 
-    while ( (res != ERROR_NO_MORE_ITEMS) && (res == ERROR_SUCCESS) )
+    while (res != ERROR_NO_MORE_ITEMS)
       {
         char
           *family_extent,
@@ -378,24 +365,26 @@ MagickExport TypeInfo* NTGetTypeList( void )
 
         value_name_length = sizeof(value_name) - 1;
         value_data_size = sizeof(value_data) - 1;
-        res = RegEnumValueA ( reg_key, registry_index, value_name, &value_name_length,
-                              0, &type, (BYTE*)value_data, &value_data_size);
+        res = RegEnumValueA ( reg_key, registry_index, value_name,
+          &value_name_length, 0, &type, (BYTE*)value_data, &value_data_size);
         registry_index++;
+        if (res != ERROR_SUCCESS)
+          continue;
         if ( (pos = strstr(value_name, " (TrueType)")) == (char*) NULL )
           continue;
         *pos='\0'; /* Remove (TrueType) from string */
 
-        type_info=(TypeInfo *) AcquireMemory(sizeof(TypeInfo));
+        type_info=(TypeInfo *) AcquireMagickMemory(sizeof(TypeInfo));
         if (type_info == (TypeInfo *) NULL)
-          MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-            "UnableToAllocateTypeInfo");
-        (void) memset(type_info,0,sizeof(TypeInfo));
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "UnableToAllocateTypeInfo",value_name);
+        (void) ResetMagickMemory(type_info,0,sizeof(TypeInfo));
 
         type_info->path=AcquireString("Windows Fonts");
         type_info->signature=MagickSignature;
 
         /* Name */
-        strncpy(buffer,value_name,MaxTextExtent-1);
+        (void) CopyMagickString(buffer,value_name,MaxTextExtent);
         for(pos = buffer; *pos != 0 ; pos++)
           if (*pos == ' ')
             *pos = '-';
@@ -408,13 +397,13 @@ MagickExport TypeInfo* NTGetTypeList( void )
         type_info->format=AcquireString("truetype");
 
         /* Glyphs */
-        if (strchr(value_data,'\\') == (char *) NULL)
-          {
-            strncpy(buffer,font_root,MaxTextExtent-1);
-            strcat(buffer,value_data);
-          }
+        if (strchr(value_data,'\\') != (char *) NULL)
+          (void) CopyMagickString(buffer,value_data,MaxTextExtent);
         else
-          strncpy(buffer,value_data,MaxTextExtent-1);
+          {
+            (void) CopyMagickString(buffer,font_root,MaxTextExtent);
+            (void) ConcatenateMagickString(buffer,value_data,MaxTextExtent);
+          }
 
         LocaleLower(buffer);
         type_info->glyphs=AcquireString(buffer);
@@ -544,59 +533,19 @@ MagickExport TypeInfo* NTGetTypeList( void )
               }
           }
 
-        strncpy(buffer,value_name,family_extent-value_name);
+        (void) strncpy(buffer,value_name,family_extent-value_name);
         buffer[family_extent-value_name]='\0';
         type_info->family=AcquireString(buffer);
 
         list_entries++;
-        if (type_list == (TypeInfo *) NULL)
-          {
-            type_list=type_info;
-            continue;
-          }
-        type_list->next=type_info;
-        type_info->previous=type_list;
-        type_list=type_list->next;
-        continue;
+        status=AppendElementToLinkedList(type_list,type_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed",type_info->name);
       }
   }
-
   RegCloseKey ( reg_key );
-
-  /* Sort entries */
-  {
-    TypeInfo
-      **type_array;
-
-    int
-      array_index = 0;
-
-    type_array = (TypeInfo**) AcquireMemory(sizeof(TypeInfo*)*list_entries);
-
-    while (type_list->previous != (TypeInfo *) NULL)
-      type_list=type_list->previous;
-
-    for (array_index=0; array_index< list_entries; array_index++)
-      {
-        type_array[array_index] = type_list;
-        type_list=type_list->next;
-      }
-
-    qsort((void *) type_array, list_entries, sizeof(TypeInfo *),TypeInfoCompare);
-
-    type_list=type_array[0];
-    type_list->previous=(TypeInfo *) NULL;
-    for(array_index=1; array_index < list_entries; array_index++)
-      {
-        type_array[array_index-1]->next = type_array[array_index];
-        type_array[array_index]->previous = type_array[array_index-1];
-        type_array[array_index]->next=(TypeInfo *) NULL;
-      }
-
-    LiberateMemory((void**) &type_array);
-  }
-
-  return (TypeInfo *) type_list;
+  return(MagickTrue);
 }
 
 /*
@@ -610,7 +559,7 @@ MagickExport TypeInfo* NTGetTypeList( void )
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ImageToHBITMAP creates a Windows HBITMAP from an IM Image
+%  ImageToHBITMAP() creates a Windows HBITMAP from an IM Image
 %
 %  The format of the ImageToHBITMAP method is:
 %
@@ -624,8 +573,7 @@ MagickExport TypeInfo* NTGetTypeList( void )
 */
 MagickExport void *ImageToHBITMAP(Image* image)
 {
-  unsigned long 
-    nPixels,
+  unsigned long
     row;
 
   long
@@ -647,8 +595,6 @@ MagickExport void *ImageToHBITMAP(Image* image)
     *pDestPixel,
     *bitmap_bits;
 
-  nPixels = image->columns * image->rows;
-
   bitmap.bmType         = 0;
   bitmap.bmWidth        = image->columns;
   bitmap.bmHeight       = image->rows;
@@ -657,26 +603,23 @@ MagickExport void *ImageToHBITMAP(Image* image)
   bitmap.bmBitsPixel    = 32;
   bitmap.bmBits         = NULL;
 
-  memSize = nPixels * bitmap.bmBitsPixel;
+  memSize = bitmap.bmWidthBytes * bitmap.bmHeight;
   bitmap_bitsH = (HANDLE) GlobalAlloc (GMEM_MOVEABLE | GMEM_DDESHARE, memSize);
 
   if (bitmap_bitsH == NULL)
-    return( NULL ); 
-  
-  bitmap_bits = (RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
-  pDestPixel = bitmap_bits;
-
+    return(NULL);
+  bitmap_bits=(RGBQUAD *) GlobalLock((HGLOBAL) bitmap_bitsH);
+  pDestPixel=bitmap_bits;
   if ( bitmap.bmBits == NULL )
-    bitmap.bmBits = bitmap_bits;
-
+    bitmap.bmBits=bitmap_bits;
   SetImageColorspace(image,RGBColorspace);
-  for( row = 0 ; row < image->rows ; row++ )
-    {
+  for (row = 0 ; row < image->rows ; row++ )
+  {
       pPixels = AcquireImagePixels(image,0,row,image->columns,1,
                                    &image->exception);
 #if QuantumDepth == 8
       /* Form of PixelPacket is identical to RGBQUAD when QuantumDepth==8 */
-      memcpy((void*)pDestPixel,(const void*)pPixels,
+      CopyMagickMemory((void*)pDestPixel,(const void*)pPixels,
              sizeof(PixelPacket)*image->columns);
       pDestPixel += image->columns;
 
@@ -702,6 +645,7 @@ MagickExport void *ImageToHBITMAP(Image* image)
   bitmapH = CreateBitmapIndirect( &bitmap );
 
   GlobalUnlock((HGLOBAL) bitmap_bitsH);
+  GlobalFree((HGLOBAL) bitmap_bitsH);
 
   return (void *)bitmapH;
 }
