@@ -11,37 +11,24 @@
 %                                                                             %
 %                        Methods to Paint on an Image                         %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,27 +39,28 @@
  Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/color.h"
+#include "magick/composite.h"
+#include "magick/composite_private.h"
+#include "magick/draw.h"
+#include "magick/draw_private.h"
 #include "magick/error.h"
 #include "magick/monitor.h"
 #include "magick/paint.h"
-#include "magick/render.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Define declarations.
 */
-#define FuzzyOpacityCompare(image,color,target) \
-  (((color)->opacity == (target)->opacity) && \
-   FuzzyColorCompare(image,color,target))
 #define MaxStacksize  (1 << 15)
 #define Push(up,left,right,delta) \
   if ((s < (segment_stack+MaxStacksize)) && (((up)+(delta)) >= 0) && \
       (((up)+(delta)) < (long) image->rows)) \
     { \
-      s->y1=(up); \
-      s->x1=(left); \
-      s->x2=(right); \
-      s->y2=(delta); \
+      s->y1=(MagickRealType) (up); \
+      s->x1=(MagickRealType) (left); \
+      s->x2=(MagickRealType) (right); \
+      s->y2=(MagickRealType) (delta); \
       s++; \
     }
 
@@ -101,9 +89,9 @@
 %
 %  The format of the ColorFloodfillImage method is:
 %
-%      unsigned int ColorFloodfillImage(Image *image,const DrawInfo *draw_info,
-%        const PixelPacket target,const long x_offset,const long y_offset,
-%        const PaintMethod method)
+%      MagickBooleanType ColorFloodfillImage(Image *image,
+%        const DrawInfo *draw_info,const PixelPacket target,
+%        const long x_offset,const long y_offset,const PaintMethod method)
 %
 %  A description of each parameter follows:
 %
@@ -119,33 +107,10 @@
 %
 %
 */
-
-static inline void AlphaComposite(const PixelPacket *p,const double alpha,
-  const PixelPacket *q,const double beta,PixelPacket *composite)
-{
-  composite->red=(Quantum)
-    (((MaxRGB-alpha)*p->red+alpha*(MaxRGB-beta)*q->red/MaxRGB)/MaxRGB+0.5);
-  composite->green=(Quantum)
-    (((MaxRGB-alpha)*p->green+alpha*(MaxRGB-beta)*q->green/MaxRGB)/MaxRGB+0.5);
-  composite->blue=(Quantum)
-    (((MaxRGB-alpha)*p->blue+alpha*(MaxRGB-beta)*q->blue/MaxRGB)/MaxRGB+0.5);
-  composite->opacity=(Quantum)
-    (MaxRGB-((MaxRGB-alpha)+alpha*(MaxRGB-beta)/MaxRGB)+0.5);
-}
-
-MagickExport unsigned int ColorFloodfillImage(Image *image,
+MagickExport MagickBooleanType ColorFloodfillImage(Image *image,
   const DrawInfo *draw_info,const PixelPacket target,const long x_offset,
   const long y_offset,const PaintMethod method)
 {
-  ExtendedSignedIntegralType
-    number_pixels;
-
-  Image
-    *pattern;
-
-  int
-    skip;
-
   long
     offset,
     start,
@@ -153,8 +118,17 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
     x2,
     y;
 
+  MagickBooleanType
+    skip;
+
+  MagickSizeType
+    number_pixels;
+
   PixelPacket
-    color;
+    fill_color;
+
+  register const PixelPacket
+    *p;
 
   register long
     x;
@@ -176,32 +150,36 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
   if ((x_offset < 0) || (x_offset >= (long) image->columns))
-    return(False);
+    return(MagickFalse);
   if ((y_offset < 0) || (y_offset >= (long) image->rows))
-    return(False);
+    return(MagickFalse);
   /*
     Set floodfill color.
   */
-  if (FuzzyColorCompare(image,&draw_info->fill,&target))
-    return(False);
-  number_pixels=(ExtendedSignedIntegralType) image->columns*image->rows;
-  if (number_pixels != (size_t) number_pixels)
+  number_pixels=(MagickSizeType) image->columns*image->rows;
+  if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToFloodfillImage");
-  floodplane=(unsigned char *) AcquireMemory(image->columns*image->rows);
-  segment_stack=(SegmentInfo *) AcquireMemory(MaxStacksize*sizeof(SegmentInfo));
+      image->filename);
+  floodplane=(unsigned char *) AcquireMagickMemory((size_t) number_pixels);
+  segment_stack=(SegmentInfo *)
+    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
   if ((floodplane== (unsigned char *) NULL) ||
       (segment_stack == (SegmentInfo *) NULL))
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToFloodfillImage");
-  (void) memset(floodplane,False,image->columns*image->rows);
+      image->filename);
+  (void) ResetMagickMemory(floodplane,MagickFalse,(size_t) number_pixels);
+  floodplane[y_offset*image->columns+x_offset]=(unsigned char) MagickTrue;
   /*
     Push initial segment on stack.
   */
   image->storage_class=DirectClass;
+  if (image->matte == MagickFalse)
+    SetImageOpacity(image,OpaqueOpacity);
   x=x_offset;
   y=y_offset;
   start=0;
@@ -221,29 +199,27 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
     /*
       Recolor neighboring pixels.
     */
-    q=GetImagePixels(image,0,y,x1+1,1);
-    if (q == (PixelPacket *) NULL)
+    p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    q+=x1;
+    p+=x1;
     for (x=x1; x >= 0; x--)
     {
       if (method == FloodfillMethod)
         {
-          if (!FuzzyColorCompare(image,q,&target))
+          if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
+              ((int) floodplane[y*image->columns+x] != MagickFalse))
             break;
         }
       else
-        if (FuzzyColorCompare(image,q,&target) ||
-            FuzzyColorCompare(image,q,&draw_info->fill))
+        if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
+            ((int) floodplane[y*image->columns+x] != MagickFalse))
           break;
-      floodplane[y*image->columns+x]=True;
-      *q=draw_info->fill;
-      q--;
+      floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+      p--;
     }
-    if (!SyncImagePixels(image))
-      break;
-    skip=x >= x1;
-    if (!skip)
+    skip=(MagickBooleanType) (x >= x1);
+    if (skip == MagickFalse)
       {
         start=x+1;
         if (start < x1)
@@ -252,110 +228,86 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
       }
     do
     {
-      if (!skip)
+      if (skip == MagickFalse)
         {
           if (x < (long) image->columns)
             {
-              q=GetImagePixels(image,x,y,image->columns-x,1);
-              if (q == (PixelPacket *) NULL)
+              p=AcquireImagePixels(image,x,y,image->columns-x,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
                 break;
               for ( ; x < (long) image->columns; x++)
               {
                 if (method == FloodfillMethod)
                   {
-                    if (!FuzzyColorCompare(image,q,&target))
+                    if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
+                        ((int) floodplane[y*image->columns+x] != MagickFalse))
                       break;
                   }
                 else
-                  if (FuzzyColorCompare(image,q,&target) ||
-                      FuzzyColorCompare(image,q,&draw_info->fill))
+                  if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
+                      ((int) floodplane[y*image->columns+x] != MagickFalse))
                     break;
-                floodplane[y*image->columns+x]=True;
-                *q=draw_info->fill;
-                q++;
+                floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+                p++;
               }
-              if (!SyncImagePixels(image))
-                break;
             }
           Push(y,start,x-1,offset);
           if (x > (x2+1))
             Push(y,x2+1,x-1,-offset);
         }
-      skip=False;
+      skip=MagickFalse;
       x++;
       if (x <= x2)
         {
-          q=GetImagePixels(image,x,y,x2-x+1,1);
-          if (q == (PixelPacket *) NULL)
+          p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
+            &image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           for ( ; x <= x2; x++)
           {
             if (method == FloodfillMethod)
               {
-                if (FuzzyColorCompare(image,q,&target))
+                if ((FuzzyColorCompare(image,p,&target) != MagickFalse) &&
+                    ((int) floodplane[y*image->columns+x] == MagickFalse))
                   break;
               }
             else
-              if (!FuzzyColorCompare(image,q,&target) &&
-                  !FuzzyColorCompare(image,q,&draw_info->fill))
-                break;
-            q++;
+              {
+                if ((FuzzyColorCompare(image,p,&target) == MagickFalse) &&
+                    ((int) floodplane[y*image->columns+x] == MagickFalse))
+                  break;
+              }
+            p++;
           }
         }
       start=x;
     } while (x <= x2);
   }
-  pattern=draw_info->fill_pattern;
-  if (pattern == (Image *) NULL)
-    for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (long) image->rows; y++)
+  {
+    /*
+      Tile fill color onto floodplane.
+    */
+    q=GetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    for (x=0; x < (long) image->columns; x++)
     {
-      /*
-        Tile fill color onto floodplane.
-      */
-      q=GetImagePixels(image,0,y,image->columns,1);
-      if (q == (PixelPacket *) NULL)
-        break;
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if (floodplane[y*image->columns+x])
-          *q=draw_info->fill;
-        q++;
-      }
-      if (!SyncImagePixels(image))
-        break;
-    }
-  else
-    {
-      /*
-        Tile image onto floodplane.
-      */
-      for (y=0; y < (long) image->rows; y++)
-      {
-        q=GetImagePixels(image,0,y,image->columns,1);
-        if (q == (PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
+      if ((int) floodplane[y*image->columns+x] != MagickFalse)
         {
-          if (floodplane[y*image->columns+x])
-            {
-              color=AcquireOnePixel(pattern,(long) ((unsigned long)
-                (x-pattern->extract_info.x) % pattern->columns),(long)
-                ((unsigned long) (y-pattern->extract_info.y) % pattern->rows),
-                &image->exception);
-              if (!pattern->matte)
-                color.opacity=OpaqueOpacity;
-              if (color.opacity != TransparentOpacity)
-                AlphaComposite(&color,color.opacity,q,q->opacity,q);
-            }
-          q++;
+          fill_color=GetFillColor(draw_info,x,y);
+          *q=MagickCompositeOver(&fill_color,(MagickRealType)
+            fill_color.opacity,q,(MagickRealType) q->opacity);
         }
-        if (!SyncImagePixels(image))
-          break;
-      }
+      q++;
     }
-  LiberateMemory((void **) &segment_stack);
-  LiberateMemory((void **) &floodplane);
-  return(True);
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+  }
+  segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
+  floodplane=(unsigned char *) RelinquishMagickMemory(floodplane);
+  return((MagickBooleanType) (y == (long) image->rows));
 }
 
 /*
@@ -383,9 +335,9 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
 %
 %  The format of the MatteFloodfillImage method is:
 %
-%      unsigned int MatteFloodfillImage(Image *image,const PixelPacket target,
-%        const unsigned int opacity,const long x_offset,const long y_offset,
-%        const PaintMethod method)
+%      MagickBooleanType MatteFloodfillImage(Image *image,
+%        const PixelPacket target,const Quantum opacity,const long x_offset,
+%        const long y_offset,const PaintMethod method)
 %
 %  A description of each parameter follows:
 %
@@ -402,13 +354,10 @@ MagickExport unsigned int ColorFloodfillImage(Image *image,
 %
 %
 */
-MagickExport unsigned int MatteFloodfillImage(Image *image,
-  const PixelPacket target,const unsigned int opacity,const long x_offset,
+MagickExport MagickBooleanType MatteFloodfillImage(Image *image,
+  const PixelPacket target,const Quantum opacity,const long x_offset,
   const long y_offset,const PaintMethod method)
 {
-  int
-    skip;
-
   long
     offset,
     start,
@@ -416,6 +365,15 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
     x2,
     y;
 
+  MagickBooleanType
+    skip;
+
+  MagickSizeType
+    number_pixels;
+
+  register const PixelPacket
+    *p;
+
   register long
     x;
 
@@ -428,34 +386,42 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
   SegmentInfo
     *segment_stack;
 
+  unsigned char
+    *floodplane;
+
   /*
     Check boundary conditions.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if ((x_offset < 0) || (x_offset >= (long) image->columns))
-    return(False);
+    return(MagickFalse);
   if ((y_offset < 0) || (y_offset >= (long) image->rows))
-    return(False);
-  if (target.opacity == opacity)
-    return(False);
-  q=GetImagePixels(image,x_offset,y_offset,1,1);
-  if (q == (PixelPacket *) NULL)
-    return(False);
-  if (q->opacity == opacity)
-    return(False);
+    return(MagickFalse);
   /*
-    Allocate segment stack.
+    Set floodfill color.
   */
-  segment_stack=(SegmentInfo *) AcquireMemory(MaxStacksize*sizeof(SegmentInfo));
-  if (segment_stack == (SegmentInfo *) NULL)
+  number_pixels=(MagickSizeType) image->columns*image->rows;
+  if (number_pixels != (MagickSizeType) ((size_t) number_pixels))
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToFloodfillImage");
+      image->filename);
+  floodplane=(unsigned char *) AcquireMagickMemory((size_t) number_pixels);
+  segment_stack=(SegmentInfo *)
+    AcquireMagickMemory(MaxStacksize*sizeof(*segment_stack));
+  if ((floodplane== (unsigned char *) NULL) ||
+      (segment_stack == (SegmentInfo *) NULL))
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  (void) ResetMagickMemory(floodplane,MagickFalse,(size_t) number_pixels);
+  floodplane[y_offset*image->columns+x_offset]=(unsigned char) MagickTrue;
   /*
     Push initial segment on stack.
   */
   image->storage_class=DirectClass;
-  image->matte=True;
+  if (image->matte == MagickFalse)
+    SetImageOpacity(image,OpaqueOpacity);
   x=x_offset;
   y=y_offset;
   start=0;
@@ -473,29 +439,29 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
     offset=(long) s->y2;
     y=(long) s->y1+offset;
     /*
-      Recolor neighboring points.
+      Recolor neighboring pixels.
     */
-    q=GetImagePixels(image,0,y,image->columns,1);
-    if (q == (PixelPacket *) NULL)
+    p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    q+=x1;
+    p+=x1;
     for (x=x1; x >= 0; x--)
     {
       if (method == FloodfillMethod)
         {
-          if (!FuzzyOpacityCompare(image,q,&target))
+          if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
+              ((int) floodplane[y*image->columns+x] != MagickFalse))
             break;
         }
       else
-        if (FuzzyOpacityCompare(image,q,&target) || (q->opacity == opacity))
+        if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
+            ((int) floodplane[y*image->columns+x] != MagickFalse))
           break;
-      q->opacity=opacity;
-      q--;
+      floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+      p--;
     }
-    if (!SyncImagePixels(image))
-      break;
-    skip=x >= x1;
-    if (!skip)
+    skip=(MagickBooleanType) (x >= x1);
+    if (skip == MagickFalse)
       {
         start=x+1;
         if (start < x1)
@@ -504,53 +470,82 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
       }
     do
     {
-      if (!skip)
+      if (skip == MagickFalse)
         {
-          q=GetImagePixels(image,0,y,image->columns,1);
-          if (q == (PixelPacket *) NULL)
-            break;
-          q+=x;
-          for ( ; x < (long) image->columns; x++)
-          {
-            if (method == FloodfillMethod)
-              {
-                if (!FuzzyOpacityCompare(image,q,&target))
-                  break;
-              }
-            else
-              if (FuzzyOpacityCompare(image,q,&target) || (q->opacity == opacity))
+          if (x < (long) image->columns)
+            {
+              p=AcquireImagePixels(image,x,y,image->columns-x,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
                 break;
-            q->opacity=opacity;
-            q++;
-          }
-          if (!SyncImagePixels(image))
-            break;
+              for ( ; x < (long) image->columns; x++)
+              {
+                if (method == FloodfillMethod)
+                  {
+                    if ((FuzzyColorCompare(image,p,&target) == MagickFalse) ||
+                        ((int) floodplane[y*image->columns+x] != MagickFalse))
+                      break;
+                  }
+                else
+                  if ((FuzzyColorCompare(image,p,&target) != MagickFalse) ||
+                      ((int) floodplane[y*image->columns+x] != MagickFalse))
+                    break;
+                floodplane[y*image->columns+x]=(unsigned char) MagickTrue;
+                p++;
+              }
+            }
           Push(y,start,x-1,offset);
           if (x > (x2+1))
             Push(y,x2+1,x-1,-offset);
         }
-      skip=False;
-      q=GetImagePixels(image,0,y,image->columns,1);
-      if (q == (PixelPacket *) NULL)
-        break;
-      q+=x;
-      for (x++; x <= x2; x++)
-      {
-        q++;
-        if (method == FloodfillMethod)
-          {
-            if (FuzzyOpacityCompare(image,q,&target))
-              break;
-          }
-        else
-          if (!FuzzyOpacityCompare(image,q,&target) && (q->opacity != opacity))
+      skip=MagickFalse;
+      x++;
+      if (x <= x2)
+        {
+          p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
+            &image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
-      }
+          for ( ; x <= x2; x++)
+          {
+            if (method == FloodfillMethod)
+              {
+                if ((FuzzyColorCompare(image,p,&target) != MagickFalse) &&
+                    ((int) floodplane[y*image->columns+x] == MagickFalse))
+                  break;
+              }
+            else
+              {
+                if ((FuzzyColorCompare(image,p,&target) == MagickFalse) &&
+                    ((int) floodplane[y*image->columns+x] == MagickFalse))
+                  break;
+              }
+            p++;
+          }
+        }
       start=x;
     } while (x <= x2);
   }
-  LiberateMemory((void **) &segment_stack);
-  return(True);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    /*
+      Tile fill color onto floodplane.
+    */
+    q=GetImagePixels(image,0,y,image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if ((int) floodplane[y*image->columns+x] != MagickFalse)
+        q->opacity=opacity;
+      q++;
+    }
+    if (SyncImagePixels(image) == MagickFalse)
+      break;
+  }
+  segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
+  floodplane=(unsigned char *) RelinquishMagickMemory(floodplane);
+  return((MagickBooleanType) (y == (long) image->rows));
 }
 
 /*
@@ -560,7 +555,6 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
 %     O p a q u e I m a g e                                                   %
 %                                                                             %
 %                                                                             %
-%                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  OpaqueImage() changes any pixel that matches color with the color
@@ -574,7 +568,7 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
 %
 %  The format of the OpaqueImage method is:
 %
-%      unsigned int OpaqueImage(Image *image,const PixelPacket target,
+%      MagickBooleanType OpaqueImage(Image *image,const PixelPacket target,
 %        const PixelPacket fill)
 %
 %  A description of each parameter follows:
@@ -587,14 +581,17 @@ MagickExport unsigned int MatteFloodfillImage(Image *image,
 %
 %
 */
-MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
-  const PixelPacket fill)
+MagickExport MagickBooleanType OpaqueImage(Image *image,
+  const PixelPacket target,const PixelPacket fill)
 {
 #define OpaqueImageTag  "Opaque/Image"
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     x;
 
@@ -609,6 +606,8 @@ MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   switch (image->storage_class)
   {
     case DirectClass:
@@ -624,15 +623,19 @@ MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
           break;
         for (x=0; x < (long) image->columns; x++)
         {
-          if (FuzzyColorCompare(image,q,&target))
+          if (FuzzyColorCompare(image,q,&target) != MagickFalse)
             *q=fill;
           q++;
         }
-        if (!SyncImagePixels(image))
+        if (SyncImagePixels(image) == MagickFalse)
           break;
-        if (QuantumTick(y,image->rows))
-          if (!MagickMonitor(OpaqueImageTag,y,image->rows,&image->exception))
-            break;
+        if (QuantumTick(y,image->rows) != 0)
+          {
+            status=MagickMonitor(OpaqueImageTag,y,image->rows,
+              &image->exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -643,17 +646,40 @@ MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
       */
       for (i=0; i < (long) image->colors; i++)
       {
-        if (FuzzyColorCompare(image,&image->colormap[i],&target))
+        if (FuzzyColorCompare(image,&image->colormap[i],&target) != MagickFalse)
           image->colormap[i]=fill;
         if (QuantumTick(i,image->colors))
-          if (!MagickMonitor(OpaqueImageTag,i,image->colors,&image->exception))
-            break;
+          {
+            status=MagickMonitor(OpaqueImageTag,i,image->colors,
+              &image->exception);
+            if (status == MagickFalse)
+              break;
+          }
       }
-      SyncImage(image);
+      if (fill.opacity != OpaqueOpacity)
+        {
+          for (y=0; y < (long) image->rows; y++)
+          {
+            q=GetImagePixels(image,0,y,image->columns,1);
+            if (q == (PixelPacket *) NULL)
+              break;
+            for (x=0; x < (long) image->columns; x++)
+            {
+              if (FuzzyColorCompare(image,q,&target) != MagickFalse)
+                q->opacity=fill.opacity;
+              q++;
+            }
+            if (SyncImagePixels(image) == MagickFalse)
+              break;
+          }
+        }
+      (void) SyncImage(image);
       break;
     }
   }
-  return(True);
+  if (fill.opacity != OpaqueOpacity)
+    image->matte=MagickTrue;
+  return(MagickTrue);
 }
 
 /*
@@ -677,8 +703,8 @@ MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
 %
 %  The format of the TransparentImage method is:
 %
-%      unsigned int TransparentImage(Image *image,const PixelPacket target,
-%        const unsigned int opacity)
+%      MagickBooleanType TransparentImage(Image *image,const PixelPacket target,
+%        const Quantum opacity)
 %
 %  A description of each parameter follows:
 %
@@ -690,14 +716,17 @@ MagickExport unsigned int OpaqueImage(Image *image,const PixelPacket target,
 %
 %
 */
-MagickExport unsigned int TransparentImage(Image *image,
-  const PixelPacket target,const unsigned int opacity)
+MagickExport MagickBooleanType TransparentImage(Image *image,
+  const PixelPacket target,const Quantum opacity)
 {
 #define TransparentImageTag  "Transparent/Image"
 
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     x;
 
@@ -709,7 +738,9 @@ MagickExport unsigned int TransparentImage(Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (!image->matte)
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  if (image->matte == MagickFalse)
     SetImageOpacity(image,OpaqueOpacity);
   for (y=0; y < (long) image->rows; y++)
   {
@@ -718,15 +749,19 @@ MagickExport unsigned int TransparentImage(Image *image,
       break;
     for (x=(long) image->columns-1; x >= 0; x--)
     {
-      if (FuzzyColorCompare(image,q,&target))
+      if (FuzzyColorCompare(image,q,&target) != MagickFalse)
         q->opacity=opacity;
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(TransparentImageTag,y,image->rows,&image->exception))
-        break;
+    if (QuantumTick(y,image->rows) != 0)
+      {
+        status=MagickMonitor(TransparentImageTag,y,image->rows,
+          &image->exception);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  return(True);
+  return(MagickTrue);
 }
