@@ -11,37 +11,24 @@
 %                                                                             %
 %                   Methods to Generate Random Numbers                        %
 %                                                                             %
-%                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,11 +39,17 @@
 /*
   Include declarations.
 */
+#if defined(__VMS)
+#include <time.h>
+#endif
 #include "magick/studio.h"
 #include "magick/error.h"
+#include "magick/memory_.h"
 #include "magick/semaphore.h"
-#include "magick/resource.h"
+#include "magick/random.h"
+#include "magick/resource_.h"
 #include "magick/signature.h"
+#include "magick/string_.h"
 #include "magick/utility.h"
 
 /*
@@ -102,14 +95,15 @@ MagickExport void DestroyRandomReservoir(void)
   AcquireSemaphoreInfo(&random_semaphore);
   if (reservoir != (SignatureInfo *) NULL)
     {
-      (void) memset(reservoir,0,sizeof(SignatureInfo));
-      LiberateMemory((void **) &reservoir);
+      (void) ResetMagickMemory(reservoir,0,sizeof(SignatureInfo));
+      reservoir=(SignatureInfo *) RelinquishMagickMemory(reservoir);
     }
   if (roulette != (unsigned long *) NULL)
     {
-      (void) memset(roulette,0,sizeof(unsigned long));
-      LiberateMemory((void **) &roulette);
+      (void) ResetMagickMemory(roulette,0,sizeof(unsigned long));
+      roulette=(unsigned long *) RelinquishMagickMemory(roulette);
     }
+  RelinquishSemaphoreInfo(&random_semaphore);
   DestroySemaphoreInfo(&random_semaphore);
 }
 
@@ -150,22 +144,23 @@ MagickExport void DistillRandomEvent(const unsigned char *event,
   /*
     Distill a random event.
   */
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
   assert(event != (const unsigned char *) NULL);
   if ((reservoir == (SignatureInfo *) NULL) ||
       (roulette == (unsigned long *) NULL))
     {
       AcquireSemaphoreInfo(&random_semaphore);
       if (reservoir == (SignatureInfo *) NULL)
-        reservoir=(SignatureInfo *) AcquireMemory(sizeof(SignatureInfo));
+        reservoir=(SignatureInfo *) AcquireMagickMemory(sizeof(SignatureInfo));
       if (roulette == (unsigned long *) NULL)
-        roulette=(unsigned long *) AcquireMemory(sizeof(unsigned long));
-      LiberateSemaphoreInfo(&random_semaphore);
+        roulette=(unsigned long *) AcquireMagickMemory(sizeof(unsigned long));
+      RelinquishSemaphoreInfo(&random_semaphore);
       if ((reservoir == (SignatureInfo *) NULL) ||
           (roulette == (unsigned long *) NULL))
-        MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-          "UnableToDistillRandomEvent");
-      (void) memset(reservoir,0,sizeof(SignatureInfo));
-      (void) memset(roulette,0,sizeof(unsigned long));
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "UnableToDistillRandomEvent",strerror(errno));
+      (void) ResetMagickMemory(reservoir,0,sizeof(SignatureInfo));
+      (void) ResetMagickMemory(roulette,0,sizeof(unsigned long));
     }
   AcquireSemaphoreInfo(&random_semaphore);
   GetSignatureInfo(&digest_info);
@@ -173,8 +168,9 @@ MagickExport void DistillRandomEvent(const unsigned char *event,
     sizeof(reservoir->digest));
   UpdateSignature(&digest_info,event,length);
   FinalizeSignature(&digest_info);
-  memcpy(reservoir->digest,digest_info.digest,sizeof(reservoir->digest));
-  LiberateSemaphoreInfo(&random_semaphore);
+  (void) CopyMagickMemory(reservoir->digest,digest_info.digest,
+    sizeof(reservoir->digest));
+  RelinquishSemaphoreInfo(&random_semaphore);
 }
 
 /*
@@ -215,7 +211,7 @@ MagickExport void GetRandomKey(unsigned char *key,const size_t length)
       (reservoir == (SignatureInfo *) NULL))
     InitializeRandomReservoir();
   AcquireSemaphoreInfo(&random_semaphore);
-  for (n=length; n > 0; n-=sizeof(reservoir->digest))
+  for (n=(long) length; n > 0; n-=sizeof(reservoir->digest))
   {
     GetSignatureInfo(&digest_info);
     UpdateSignature(&digest_info,(const unsigned char *) reservoir->digest,
@@ -224,11 +220,11 @@ MagickExport void GetRandomKey(unsigned char *key,const size_t length)
       sizeof(roulette));
     FinalizeSignature(&digest_info);
     (*roulette)++;
-    memcpy(key,digest_info.digest,
-      n < (long) sizeof(reservoir->digest) ? n : sizeof(reservoir->digest));
+    (void) CopyMagickMemory(key,digest_info.digest,(size_t) (n < (long)
+      sizeof(reservoir->digest) ? n : (long) sizeof(reservoir->digest)));
     key+=sizeof(reservoir->digest);
   }
-  LiberateSemaphoreInfo(&random_semaphore);
+  RelinquishSemaphoreInfo(&random_semaphore);
 }
 
 /*
@@ -243,7 +239,8 @@ MagickExport void GetRandomKey(unsigned char *key,const size_t length)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetRandomValue() return a non-negative double-precision floating-point
-%  value uniformly distributed over the interval [0.0, 1.0).
+%  value uniformly distributed over the interval [0.0, 1.0) with a 2 to the
+%  128th-1 period.
 %
 %  The format of the GetRandomValue method is:
 %
@@ -253,38 +250,37 @@ MagickExport void GetRandomKey(unsigned char *key,const size_t length)
 */
 MagickExport double GetRandomValue(void)
 {
-  static long
-    i = 8;
-
-  static SignatureInfo
-    digest_info;
+  static unsigned long
+    w = (~0UL),
+    x = (~0UL),
+    y = (~0UL),
+    z = (~0UL);
 
   unsigned long
-    key,
-    range;
+    range,
+    t;
 
   if ((roulette == (unsigned long *) NULL) ||
       (reservoir == (SignatureInfo *) NULL))
     InitializeRandomReservoir();
-  AcquireSemaphoreInfo(&random_semaphore);
+  while ((w == ~0UL) || (x == (~0UL)) || (y == (~0UL)) || (z == (~0UL)))
+  {
+    GetRandomKey((unsigned char *) &w,sizeof(unsigned long));
+    GetRandomKey((unsigned char *) &x,sizeof(unsigned long));
+    GetRandomKey((unsigned char *) &y,sizeof(unsigned long));
+    GetRandomKey((unsigned char *) &z,sizeof(unsigned long));
+  }
   range=(unsigned long) (~0);
   do
   {
-    if (i == 8)
-      {
-        GetSignatureInfo(&digest_info);
-        UpdateSignature(&digest_info,(const unsigned char *) reservoir->digest,
-          sizeof(reservoir->digest));
-        UpdateSignature(&digest_info,(const unsigned char *) roulette,
-          sizeof(roulette));
-        FinalizeSignature(&digest_info);
-        (*roulette)++;
-        i=0;
-      }
-    key=digest_info.digest[i++];
-  } while (key == range);
-  LiberateSemaphoreInfo(&random_semaphore);
-  return((double) key/range);
+    t=(x ^ (x << 11));
+    x=y;
+    y=z;
+    z=w;
+    w=(w ^ (w >> 19)) ^ (t ^ (t >> 8));
+  }
+  while (w == range);
+  return((double) w/range);
 }
 
 /*
@@ -365,24 +361,27 @@ static void InitializeRandomReservoir(void)
     nanoseconds=timer.tms_utime+timer.tms_stime;
   }
 #else
-#if defined(WIN32)
+#if defined(__WINDOWS__)
   nanoseconds=NTElapsedTime()+NTUserTime();
 #else
   nanoseconds=clock();
 #endif
 #endif
   DistillRandomEvent((const unsigned char *) &nanoseconds,sizeof(time_t));
-  pid=getpid();
+  pid=(long) getpid();
   DistillRandomEvent((const unsigned char *) &pid,sizeof(long));
   DistillRandomEvent((const unsigned char *) &roulette,sizeof(unsigned long *));
   (void) AcquireUniqueFilename(filename);
-  DistillRandomEvent((const unsigned char *) filename,MaxTextExtent);
-  (void) LiberateUniqueFileResource(filename);
-  file=open("/dev/random",O_RDONLY | O_BINARY);
+  DistillRandomEvent((const unsigned char *) filename,strlen(filename));
+  (void) RelinquishUniqueFileResource(filename);
+  file=open("/dev/urandom",O_RDONLY | O_BINARY);
   if (file != -1)
     {
-      (void) read(file,random,MaxTextExtent);
+      ssize_t
+        count;
+
+      count=read(file,random,(size_t) MaxTextExtent);
       (void) close(file);
-      DistillRandomEvent(random,MaxTextExtent);
+      DistillRandomEvent(random,(size_t) count);
     }
 }
