@@ -12,37 +12,24 @@
 %                                                                             %
 %                            ImageMagick Registry.                            %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                October 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,22 +41,51 @@
 */
 #include "magick/studio.h"
 #include "magick/error.h"
+#include "magick/hashmap.h"
+#include "magick/image.h"
 #include "magick/list.h"
+#include "magick/memory_.h"
 #include "magick/registry.h"
 #include "magick/semaphore.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
+/*
+  Typedef declaractions.
+*/
+typedef struct _RegistryInfo
+{
+  long
+    id;
+
+  RegistryType
+    type;
+
+  void
+    *blob;
+
+  size_t
+    length;
+
+  unsigned long
+    signature;
+} RegistryInfo;
 
 /*
   Global declarations.
 */
-static SemaphoreInfo
-  *registry_semaphore = (SemaphoreInfo *) NULL;
+static LinkedListInfo
+  *registry_list = (LinkedListInfo *) NULL;
 
 static long
   id = 0;
 
-static RegistryInfo
-  *registry_list = (RegistryInfo *) NULL;
+static SemaphoreInfo
+  *registry_semaphore = (SemaphoreInfo *) NULL;
+
+/*
+  Forward declarations.
+*/
+static void
+  *DestroyRegistryElement(void *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -83,12 +99,12 @@ static RegistryInfo
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  DeleteMagickRegistry() deletes an entry in the registry as defined by the id.
-%  It returns True if the entry is deleted otherwise False if no entry is found
+%  It returns MagickTrue if the entry is deleted otherwise MagickFalse if no entry is found
 %  in the registry that matches the id.
 %
 %  The format of the DeleteMagickRegistry method is:
 %
-%      unsigned int DeleteMagickRegistry(const long id)
+%      MagickBooleanType DeleteMagickRegistry(const long id)
 %
 %  A description of each parameter follows:
 %
@@ -96,50 +112,35 @@ static RegistryInfo
 %
 %
 */
-MagickExport unsigned int DeleteMagickRegistry(const long id)
+MagickExport MagickBooleanType DeleteMagickRegistry(const long id)
 {
-  register RegistryInfo
+  register const RegistryInfo
     *p;
 
-  RegistryInfo
-    *registry_info;
-
+  if (registry_list == (LinkedListInfo *) NULL)
+    return(MagickFalse);
+  if (IsLinkedListEmpty(registry_list) == MagickTrue)
+    return(MagickFalse);
   AcquireSemaphoreInfo(&registry_semaphore);
-  for (p=registry_list; p != (RegistryInfo *) NULL; p=p->next)
+  ResetLinkedListIterator(registry_list);
+  p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
+  while (p != (const RegistryInfo *) NULL)
   {
-    if (id != p->id)
-      continue;
-    registry_info=p;
-    switch (registry_info->type)
-    {
-      case ImageRegistryType:
-      {
-        DestroyImage((Image *) registry_info->blob);
-        break;
-      }
-      case ImageInfoRegistryType:
-      {
-        DestroyImageInfo((ImageInfo *) registry_info->blob);
-        break;
-      }
-      default:
-      {
-        LiberateMemory((void **) &registry_info->blob);
-        break;
-      }
-    }
-    if (registry_info == registry_list)
-      registry_list=registry_info->next;
-    if (registry_info->previous != (RegistryInfo *) NULL)
-      registry_info->previous->next=registry_info->next;
-    if (registry_info->next != (RegistryInfo *) NULL)
-      registry_info->next->previous=registry_info->previous;
-    LiberateMemory((void **) &registry_info);
-    registry_info=(RegistryInfo *) NULL;
-    break;
+    if (p->id == id)
+      break;
+    p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   }
-  LiberateSemaphoreInfo(&registry_semaphore);
-  return(p != (RegistryInfo *) NULL);
+  if (p != (const RegistryInfo *) NULL)
+    {
+      RegistryInfo
+        *registry_info;
+
+      registry_info=(RegistryInfo *)
+        RemoveElementByValueFromLinkedList(registry_list,p);
+      (void) DestroyRegistryElement(registry_info);
+    }
+  RelinquishSemaphoreInfo(&registry_semaphore);
+  return((MagickBooleanType) (p != (RegistryInfo *) NULL));
 }
 
 /*
@@ -161,40 +162,42 @@ MagickExport unsigned int DeleteMagickRegistry(const long id)
 %
 %
 */
-MagickExport void DestroyMagickRegistry(void)
+
+static void *DestroyRegistryElement(void *registry_info)
 {
   register RegistryInfo
     *p;
-
-  RegistryInfo
-    *registry_info;
-
-  AcquireSemaphoreInfo(&registry_semaphore);
-  for (p=registry_list; p != (RegistryInfo *) NULL; )
+                                                                                
+  p=(RegistryInfo *) registry_info;
+  switch (p->type)
   {
-    registry_info=p;
-    p=p->next;
-    switch (registry_info->type)
+    case ImageRegistryType:
     {
-      case ImageRegistryType:
-      {
-        DestroyImage((Image *) registry_info->blob);
-        break;
-      }
-      case ImageInfoRegistryType:
-      {
-        DestroyImageInfo((ImageInfo *) registry_info->blob);
-        break;
-      }
-      default:
-      {
-        LiberateMemory((void **) &registry_info->blob);
-        break;
-      }
+      DestroyImage((Image *) p->blob);
+      break;
+    }
+    case ImageInfoRegistryType:
+    {
+      DestroyImageInfo((ImageInfo *) p->blob);
+      break;
+    }
+    default:
+    {
+      p->blob=(char *) RelinquishMagickMemory(p->blob);
+      break;
     }
-    LiberateMemory((void **) &registry_info);
   }
-  registry_list=(RegistryInfo *) NULL;
+  p=(RegistryInfo *) RelinquishMagickMemory(p);
+  return((void *) NULL);
+}
+
+MagickExport void DestroyMagickRegistry(void)
+{
+  AcquireSemaphoreInfo(&registry_semaphore);
+  if (registry_list != (LinkedListInfo *) NULL)
+    DestroyLinkedList(registry_list,DestroyRegistryElement);
+  registry_list=(LinkedListInfo *) NULL;
+  RelinquishSemaphoreInfo(&registry_semaphore);
   DestroySemaphoreInfo(&registry_semaphore);
 }
 
@@ -233,26 +236,33 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
   Image
     *image;
 
-  register RegistryInfo
+  register const RegistryInfo
     *p;
 
+  if (registry_list == (LinkedListInfo *) NULL)
+    return((Image *) NULL);
+  if (IsLinkedListEmpty(registry_list) == MagickTrue)
+    return((Image *) NULL);
   *id=(-1);
   image=(Image *) NULL;
   AcquireSemaphoreInfo(&registry_semaphore);
-  for (p=registry_list; p != (RegistryInfo *) NULL; p=p->next)
+  ResetLinkedListIterator(registry_list);
+  p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
+  while (p != (const RegistryInfo *) NULL)
   {
-    if (p->type != ImageRegistryType)
-      continue;
-    if (LocaleCompare(((Image *) p->blob)->filename,name) == 0)
+    if ((p->type == ImageRegistryType) &&
+        (LocaleCompare(((Image *) p->blob)->filename,name) == 0))
       {
         *id=p->id;
         image=CloneImageList((Image *) p->blob,exception);
         break;
       }
+    p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   }
-  LiberateSemaphoreInfo(&registry_semaphore);
+  RelinquishSemaphoreInfo(&registry_semaphore);
   if (image == (Image *) NULL)
-    (void) ThrowException(exception,RegistryError,"UnableToLocateImage",name);
+    (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+      "UnableToLocateImage",name);
   return(image);
 }
 
@@ -290,70 +300,75 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
 MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
   size_t *length,ExceptionInfo *exception)
 {
-  register RegistryInfo
+  register const RegistryInfo
     *p;
 
   void
     *blob;
 
-  RegistryInfo
-    *registry_info;
-
   blob=(void *) NULL;
   *type=UndefinedRegistryType;
   *length=0;
+  if (registry_list == (LinkedListInfo *) NULL)
+    return((Image *) NULL);
+  if (IsLinkedListEmpty(registry_list) == MagickTrue)
+    return((Image *) NULL);
   AcquireSemaphoreInfo(&registry_semaphore);
-  for (p=registry_list; p != (RegistryInfo *) NULL; p=p->next)
+  ResetLinkedListIterator(registry_list);
+  p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
+  while (p != (const RegistryInfo *) NULL)
   {
-    if (id != p->id)
-      continue;
-    registry_info=p;
-    switch (registry_info->type)
-    {
-      case ImageRegistryType:
-      {
-        Image
-          *image;
-
-        image=(Image *) registry_info->blob;
-        blob=(void *) CloneImageList(image,exception);
-        break;
-      }
-      case ImageInfoRegistryType:
-      {
-        ImageInfo
-          *image_info;
-
-        image_info=(ImageInfo *) registry_info->blob;
-        blob=(void *) CloneImageInfo(image_info);
-        break;
-      }
-      default:
-      {
-        blob=(void *) AcquireMemory(registry_info->length);
-        if (blob == (void *) NULL)
-          {
-            (void) ThrowException(exception,ResourceLimitError,
-              "MemoryAllocationFailed","UnableToGetFromRegistry");
-            break;
-          }
-        (void) memcpy(blob,registry_info->blob,registry_info->length);
-        break;
-      }
-    }
-    *type=registry_info->type;
-    *length=registry_info->length;
-    break;
+    if (id == p->id)
+      break;
+    p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   }
-  LiberateSemaphoreInfo(&registry_semaphore);
+  if (p != (const RegistryInfo *) NULL)
+    {
+      switch (p->type)
+      {
+        case ImageRegistryType:
+        {
+          Image
+            *image;
+
+          image=(Image *) p->blob;
+          blob=(void *) CloneImageList(image,exception);
+          break;
+        }
+        case ImageInfoRegistryType:
+        {
+          ImageInfo
+            *image_info;
+
+          image_info=(ImageInfo *) p->blob;
+          blob=(void *) CloneImageInfo(image_info);
+          break;
+        }
+        default:
+        {
+          blob=(void *) AcquireMagickMemory(p->length);
+          if (blob == (void *) NULL)
+            {
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                ResourceLimitError,"MemoryAllocationFailed",strerror(errno));
+              break;
+            }
+          (void) CopyMagickMemory(blob,p->blob,p->length);
+          break;
+        }
+      }
+      *type=p->type;
+      *length=p->length;
+    }
+  RelinquishSemaphoreInfo(&registry_semaphore);
   if (blob == (void *) NULL)
     {
       char
-        description[MaxTextExtent];
+        reason[MaxTextExtent];
 
-      FormatString(description,"id=%ld",id);
-      (void) ThrowException(exception,RegistryError,"UnableToLocateRegistryID",
-        description);
+      (void) FormatMagickString(reason,MaxTextExtent,"id=%ld",id);
+      (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+        "UnableToGetRegistryID",reason);
     }
   return(blob);
 }
@@ -393,6 +408,9 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
 MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
   const size_t length,ExceptionInfo *exception)
 {
+  MagickBooleanType
+    status;
+
   RegistryInfo
     *registry_info;
 
@@ -409,14 +427,14 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image=(Image *) blob;
       if (length != sizeof(Image))
         {
-          (void) ThrowException(exception,RegistryError,"UnableToSetRegistry",
-            "StructureSizeMismatch");
+          (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+            "UnableToSetRegistry",strerror(errno));
           return(-1);
         }
       if (image->signature != MagickSignature)
         {
-          (void) ThrowException(exception,RegistryError,"UnableToSetRegistry",
-            "ImageExpected");
+          (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+            "UnableToSetRegistry",strerror(errno));
           return(-1);
         }
       clone_blob=(void *) CloneImageList(image,exception);
@@ -432,14 +450,14 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       image_info=(ImageInfo *) blob;
       if (length != sizeof(ImageInfo))
         {
-          (void) ThrowException(exception,RegistryError,"UnableToSetRegistry",
-            "StructureSizeMismatch");
+          (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+            "UnableToSetRegistry",strerror(errno));
           return(-1);
         }
       if (image_info->signature != MagickSignature)
         {
-          (void) ThrowException(exception,RegistryError,"UnableToSetRegistry",
-            "ImageInfoExpected");
+          (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+            "UnableToSetRegistry",strerror(errno));
           return(-1);
         }
       clone_blob=(void *) CloneImageInfo(image_info);
@@ -449,34 +467,34 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
     }
     default:
     {
-      clone_blob=(void *) AcquireMemory(length);
+      clone_blob=(void *) AcquireMagickMemory(length);
       if (clone_blob == (void *) NULL)
         return(-1);
-      (void) memcpy(clone_blob,blob,length);
+      (void) CopyMagickMemory(clone_blob,blob,length);
     }
   }
-  registry_info=(RegistryInfo *) AcquireMemory(sizeof(RegistryInfo));
+  registry_info=(RegistryInfo *) AcquireMagickMemory(sizeof(RegistryInfo));
   if (registry_info == (RegistryInfo *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateRegistryInfo");
-  (void) memset(registry_info,0,sizeof(RegistryInfo));
+    ThrowMagickFatalException(ResourceLimitFatalError,
+      "UnableToAllocateRegistryInfo",strerror(errno));
+  (void) ResetMagickMemory(registry_info,0,sizeof(RegistryInfo));
   registry_info->type=type;
   registry_info->blob=clone_blob;
   registry_info->length=length;
   registry_info->signature=MagickSignature;
   AcquireSemaphoreInfo(&registry_semaphore);
   registry_info->id=id++;
-  if (registry_list == (RegistryInfo *) NULL)
-    registry_list=registry_info;
-  else
+  if (registry_list == (LinkedListInfo *) NULL)
     {
-      register RegistryInfo
-        *p;
-
-      for (p=registry_list; p->next != (RegistryInfo *) NULL; p=p->next);
-      registry_info->previous=p;
-      p->next=registry_info;
+      registry_list=NewLinkedList(0);
+      if (registry_list == (LinkedListInfo *) NULL)
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "UnableToAllocateRegistryInfo",strerror(errno));
     }
-  LiberateSemaphoreInfo(&registry_semaphore);
+  status=AppendElementToLinkedList(registry_list,registry_info);
+  if (status == MagickFalse)
+    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
+      "MemoryAllocationFailed",strerror(errno));
+  RelinquishSemaphoreInfo(&registry_semaphore);
   return(registry_info->id);
 }
