@@ -12,37 +12,24 @@
 %                                                                             %
 %                        Get/Set ImageMagick Resources.                       %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -54,49 +41,46 @@
 */
 #include "magick/studio.h"
 #include "magick/error.h"
+#include "magick/hashmap.h"
 #include "magick/log.h"
-#include "magick/resource.h"
+#include "magick/image.h"
+#include "magick/memory_.h"
+#include "magick/random.h"
+#include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/signature.h"
+#include "magick/string_.h"
+#include "magick/token.h"
 #include "magick/utility.h"
 
 /*
   Define  declarations.
 */
 #define ResourceInfinity  (~0UL)
-#define ResourceToMegabytes(value) ((double) (value)*1024.0*1024.0)
-#define MegabytesToResource(value) ((unsigned long) ((value)/1024.0/1024.0))
-#define GigabytesToResource(value) \
-  ((unsigned long) ((value)/1024.0/1024.0/1024.0))
+#define BytesToMegabytes(value) ((unsigned long) ((value)/1024/1024))
+#define BytesToGigabytes(value) ((unsigned long) ((value)/1024/1024/1024))
+#define GigabytesToBytes(value) ((MagickSizeType) (value)*1024*1024*1024)
+#define MegabytesToBytes(value) ((MagickSizeType) (value)*1024*1024)
 
 /*
   Typedef declarations.
 */
 typedef struct _ResourceInfo
 {
-  double
-    file,
+  MagickOffsetType
+    area,
     memory,
     map,
-    disk;
+    disk,
+    file;
 
   unsigned long
-    file_limit,
+    area_limit,
     memory_limit,
     map_limit,
-    disk_limit;
+    disk_limit,
+    file_limit;
 } ResourceInfo;
-
-typedef struct _UniqueFileResourceInfo
-{
-  const char
-    *name;
-
-  struct _UniqueFileResourceInfo
-    *next,
-    *previous;
-} UniqueFileResourceInfo;
-
 
 /*
   Global declarations.
@@ -104,17 +88,12 @@ typedef struct _UniqueFileResourceInfo
 static SemaphoreInfo
   *resource_semaphore = (SemaphoreInfo *) NULL;
 
-static SemaphoreInfo
-  *unique_semaphore = (SemaphoreInfo *) NULL;
-
 static ResourceInfo
   resource_info =
-  {
-    0, 0, 0, 0, 1024, 1024, 4096, ResourceInfinity
-  };
+    { 0, 0, 0, 0, 0, 128, 512, 1024, ResourceInfinity, 768 };
 
-static UniqueFileResourceInfo
-  *temporary_resources = (UniqueFileResourceInfo *) NULL;
+static LinkedListInfo
+  *temporary_resources = (LinkedListInfo *) NULL;
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -127,13 +106,14 @@ static UniqueFileResourceInfo
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AcquireMagickResource() acquires resources of the specified type.  True is
-%  returned if the specified resource is available otherwise False.
+%  AcquireMagickResource() acquires resources of the specified type.
+%  MagickTrue is returned if the specified resource is available otherwise
+%  MagickFalse.
 %
 %  The format of the AcquireMagickResource() method is:
 %
-%      unsigned int AcquireMagickResource(const ResourceType type,
-%        const ExtendedSignedIntegralType size)
+%      MagickBooleanType AcquireMagickResource(const ResourceType type,
+%        const MagickSizeType size)
 %
 %  A description of each parameter follows:
 %
@@ -143,71 +123,86 @@ static UniqueFileResourceInfo
 %
 %
 */
-MagickExport unsigned int AcquireMagickResource(const ResourceType type,
-  const ExtendedSignedIntegralType size)
+MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
+  const MagickSizeType size)
 {
   char
-    message[MaxTextExtent];
+    resource_current[MaxTextExtent],
+    resource_limit[MaxTextExtent],
+    resource_request[MaxTextExtent],
+    resource_type[MaxTextExtent];
 
-  unsigned int
+  MagickBooleanType
     status;
 
-  status=True;
+  MagickSizeType
+    limit;
+
+  status=MagickFalse;
+  FormatSize(size,resource_request);
   AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
-    case FileResource:
+    case AreaResource:
     {
-      resource_info.file+=size;
-      if (resource_info.file_limit == ResourceInfinity)
-        break;
-      status=resource_info.file <= resource_info.file_limit;
-      FormatString(message,"file +%lu/%lu/%lu",(unsigned long) size,
-        (unsigned long) resource_info.file,resource_info.file_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
+      resource_info.area+=size;
+      limit=MegabytesToBytes(resource_info.area_limit);
+      status=(resource_info.area_limit == ResourceInfinity) || (size < limit);
+      (void) FormatMagickString(resource_type,MaxTextExtent,"area");
+      FormatSize((MagickSizeType) resource_info.area,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
       break;
     }
     case MemoryResource:
     {
       resource_info.memory+=size;
-      if (resource_info.memory_limit == ResourceInfinity)
-        break;
-      status=resource_info.memory <=
-        ResourceToMegabytes(resource_info.memory_limit);
-      FormatString(message,"memory +%lumb/%lumb/%lumb",
-        MegabytesToResource(size),MegabytesToResource(resource_info.memory),
-        resource_info.memory_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
+      limit=MegabytesToBytes(resource_info.memory_limit);
+      status=(resource_info.memory_limit == ResourceInfinity) ||
+        ((MagickSizeType) (resource_info.memory+size) < limit);
+      (void) FormatMagickString(resource_type,MaxTextExtent,"memory");
+      FormatSize((MagickSizeType) resource_info.memory,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
       break;
     }
     case MapResource:
     {
       resource_info.map+=size;
-      if (resource_info.map_limit == ResourceInfinity)
-        break;
-      status=resource_info.disk <=
-        ResourceToMegabytes(resource_info.map_limit);
-      FormatString(message,"map +%lumb/%lumb/%lumb",MegabytesToResource(size),
-        MegabytesToResource(resource_info.map),resource_info.map_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
+      limit=MegabytesToBytes(resource_info.map_limit);
+      status=(resource_info.map_limit == ResourceInfinity) ||
+        ((MagickSizeType) (resource_info.map+size) < limit);
+      (void) FormatMagickString(resource_type,MaxTextExtent,"map");
+      FormatSize((MagickSizeType) resource_info.map,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
       break;
     }
     case DiskResource:
     {
       resource_info.disk+=size;
-      if (resource_info.disk_limit == ResourceInfinity)
-        break;
-      status=resource_info.disk <=
-        ResourceToMegabytes(resource_info.disk_limit);
-      FormatString(message,"disk +%lumb/%lugb/%lugb",MegabytesToResource(size),
-        GigabytesToResource(resource_info.disk),resource_info.disk_limit/1024);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
+      limit=GigabytesToBytes(resource_info.disk_limit);
+      status=(resource_info.disk_limit == ResourceInfinity) ||
+        ((MagickSizeType) (resource_info.disk+size) < limit);
+      (void) FormatMagickString(resource_type,MaxTextExtent,"disk");
+      FormatSize((MagickSizeType) resource_info.disk,resource_current);
+      FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
+      break;
+    }
+    case FileResource:
+    {
+      resource_info.file+=size;
+      limit=resource_info.file_limit;
+      status=(resource_info.file_limit == ResourceInfinity) ||
+        ((MagickSizeType) (resource_info.file+size) < limit);
+      (void) FormatMagickString(resource_type,MaxTextExtent,"file");
+      FormatSize((MagickSizeType) resource_info.file,resource_current);
+      FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
       break;
     }
     default:
       break;
   }
-  LiberateSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(&resource_semaphore);
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+    resource_type,resource_request,resource_current,resource_limit);
   return(status);
 }
 
@@ -216,6 +211,47 @@ MagickExport unsigned int AcquireMagickResource(const ResourceType type,
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   A s y n c h r o n o u s D e s t r o y M a g i c k R e s o u r c e s       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AsynchronousDestroyMagickResources() destroys the resource environment.
+%  It differs from DestroyMagickResources() in that it can be called from a
+%  asynchronous signal handler.
+%
+%  The format of the DestroyMagickResources() method is:
+%
+%      DestroyMagickResources(void)
+%
+%
+*/
+MagickExport void AsynchronousDestroyMagickResources(void)
+{
+  if (temporary_resources != (LinkedListInfo *) NULL)
+    {
+      const char
+        *path;
+
+      /*
+        Remove any lingering temporary files.
+      */
+      ResetLinkedListIterator(temporary_resources);
+      path=(const char *) GetNextElementInLinkedList(temporary_resources);
+      while (path != (const char *) NULL)
+      {
+        (void) remove(path);
+        path=(const char *) GetNextElementInLinkedList(temporary_resources);
+      }
+    }
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   A c q u i r e U n i q u e F i l e R e s o u r c e                         %
 %                                                                             %
 %                                                                             %
@@ -237,7 +273,7 @@ MagickExport unsigned int AcquireMagickResource(const ResourceType type,
 %
 */
 
-static unsigned int GetPathTemplate(char *path)
+static MagickBooleanType GetPathTemplate(char *path)
 {
   char
     *directory;
@@ -248,30 +284,34 @@ static unsigned int GetPathTemplate(char *path)
   struct stat
     file_info;
 
-  (void) strcpy(path,"magicXXXXXX");
-  directory=getenv("TMPDIR");
-#if defined(WIN32)
+  (void) strcpy(path,"magick-XXXXXXXX");
+   directory=getenv("TMPDIR");
+#if defined(__WINDOWS__)
   if (directory == (char *) NULL)
     directory=getenv("TMP");
   if (directory == (char *) NULL)
     directory=getenv("TEMP");
 #endif
+#if defined(__VMS)
+   directory=getenv("MTMPDIR");
+#endif
 #if defined(P_tmpdir)
   if (directory == (char *) NULL)
     directory=P_tmpdir;
 #endif
   if (directory == (char *) NULL)
-    return(True);
+    return(MagickTrue);
   if (strlen(directory) > (MaxTextExtent-15))
-    return(True);
+    return(MagickTrue);
   status=stat(directory,&file_info);
   if ((status != 0) || !S_ISDIR(file_info.st_mode))
-    return(True);
+    return(MagickTrue);
   if (directory[strlen(directory)-1] == *DirectorySeparator)
-    FormatString(path,"%smagicXXXXXX",directory);
+    (void) FormatMagickString(path,MaxTextExtent,"%smagick-XXXXXXXX",directory);
   else
-    FormatString(path,"%s%smagicXXXXXX",directory,DirectorySeparator);
-  return(True);
+    (void) FormatMagickString(path,MaxTextExtent,"%s%smagick-XXXXXXXX",
+      directory,DirectorySeparator);
+  return(MagickTrue);
 }
 
 MagickExport int AcquireUniqueFileResource(char *path)
@@ -294,9 +334,10 @@ MagickExport int AcquireUniqueFileResource(char *path)
     i;
 
   unsigned char
-    key[6];
+    key[8];
 
   assert(path != (char *) NULL);
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
   file=(-1);
   for (i=0; i < TMP_MAX; i++)
   {
@@ -309,48 +350,29 @@ MagickExport int AcquireUniqueFileResource(char *path)
     if (file != -1)
       break;
 #endif
-    GetRandomKey(key,6);
-    p=path+strlen(path)-6;
-    for (i=0; i < 6; i++)
+    GetRandomKey(key,8);
+    p=path+strlen(path)-8;
+    for (i=0; i < 8; i++)
     {
-      c=key[i] & 0x1f;
-      *p++=c > 9 ? c+'a'-10 : c+'0';
+      c=(int) (key[i] & 0x1f);
+      *p++=(char) (c > 9 ? c+(int) 'a'-10 : c+(int) '0');
     }
     file=open(path,O_RDWR | O_CREAT | O_EXCL | O_BINARY | O_NOFOLLOW,S_MODE);
     if ((file > 0) || (errno != EEXIST))
       break;
   }
-  if (file != -1)
-    {
-      UniqueFileResourceInfo
-        *resource_info;
-
-      if (!AcquireMagickResource(FileResource,1))
-        {
-          (void) close(file);
-          (void) remove(path);
-          return(-1);
-        }
-      AcquireSemaphoreInfo(&unique_semaphore);
-      resource_info=(UniqueFileResourceInfo *)
-        AcquireMemory(sizeof(UniqueFileResourceInfo));
-      if (resource_info == (UniqueFileResourceInfo *) NULL)
-        MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-          "UnableToAcquireString");
-      resource_info->name=AcquireString(path);
-      resource_info->next=(UniqueFileResourceInfo *) NULL;
-      resource_info->previous=(UniqueFileResourceInfo *) NULL;
-      if (temporary_resources == (UniqueFileResourceInfo *) NULL)
-        temporary_resources=resource_info;
-      else
-        {
-          resource_info->next=temporary_resources;
-          temporary_resources->previous=resource_info;
-          temporary_resources=temporary_resources->previous;
-        }
-      LiberateSemaphoreInfo(&unique_semaphore);
-    }
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),path);
+  if (file == -1)
+    return(file);
+  if (AcquireMagickResource(FileResource,1) == MagickFalse)
+    {
+      (void) close(file);
+      (void) remove(path);
+      return(-1);
+    }
+  if (temporary_resources == (LinkedListInfo *) NULL)
+    temporary_resources=NewLinkedList(0);
+  (void) AppendElementToLinkedList(temporary_resources,AcquireString(path));
   return(file);
 }
 
@@ -375,29 +397,26 @@ MagickExport int AcquireUniqueFileResource(char *path)
 */
 MagickExport void DestroyMagickResources(void)
 {
-  UniqueFileResourceInfo
-    *resource_info;
-
-  register UniqueFileResourceInfo
-    *p;
-
-  AcquireSemaphoreInfo(&unique_semaphore);
-  for (p=temporary_resources; p != (UniqueFileResourceInfo *) NULL; )
-  {
-    resource_info=p;
-    p=p->next;
-    if (resource_info->name != (char *) NULL)
-      {
-        (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-          resource_info->name);
-        (void) remove(resource_info->name);
-        LiberateMemory((void **) &resource_info->name);
-      }
-    LiberateMemory((void **) &resource_info);
-  }
-  temporary_resources=(UniqueFileResourceInfo *) NULL;
-  DestroySemaphoreInfo(&unique_semaphore);
   AcquireSemaphoreInfo(&resource_semaphore);
+  if (temporary_resources != (LinkedListInfo *) NULL)
+    {
+      const char
+        *path;
+
+      /*
+        Remove any lingering temporary files.
+      */
+      ResetLinkedListIterator(temporary_resources);
+      path=(const char *) GetNextElementInLinkedList(temporary_resources);
+      while (path != (const char *) NULL)
+      {
+        (void) remove(path);
+        path=(const char *) GetNextElementInLinkedList(temporary_resources);
+      }
+      DestroyLinkedList(temporary_resources,RelinquishMagickMemory);
+      temporary_resources=(LinkedListInfo *) NULL;
+    }
+  RelinquishSemaphoreInfo(&resource_semaphore);
   DestroySemaphoreInfo(&resource_semaphore);
 }
 
@@ -433,30 +452,35 @@ MagickExport unsigned long GetMagickResource(const ResourceType type)
   AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
+    case AreaResource:
+    {
+      resource=BytesToMegabytes(resource_info.area);
+      break;
+    }
+    case MemoryResource:
+    {
+      resource=BytesToMegabytes(resource_info.memory);
+      break;
+    }
+    case MapResource:
+    {
+      resource=BytesToMegabytes(resource_info.map);
+      break;
+    }
+    case DiskResource:
+    {
+      resource=BytesToGigabytes(resource_info.disk);
+      break;
+    }
     case FileResource:
     {
       resource=(unsigned long) resource_info.file;
       break;
     }
-    case MemoryResource:
-    {
-      resource=MegabytesToResource(resource_info.memory);
-      break;
-    }
-    case MapResource:
-    {
-      resource=MegabytesToResource(resource_info.map);
-      break;
-    }
-    case DiskResource:
-    {
-      resource=MegabytesToResource(resource_info.disk);
-      break;
-    }
     default:
       break;
   }
-  LiberateSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(&resource_semaphore);
   return(resource);
 }
 
@@ -486,184 +510,60 @@ MagickExport void InitializeMagickResources(void)
 
   long
     files,
-    limit,
-    pagesize,
-    pages;
+    pages,
+    pagesize;
+
+  unsigned long
+    memory;
 
   /*
     Set Magick resource limits.
   */
-  p=getenv("MAGICK_DISK_LIMIT");
-  if (p != (const char *) NULL)
-    SetMagickResourceLimit(DiskResource,atol(p));
-  files=0;
-#if defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
-  files=sysconf(_SC_OPEN_MAX);
-#endif
-  pagesize=0;
-#if defined(HAVE_SYSCONF) && defined(_SC_PAGE_SIZE)
-  pagesize=sysconf(_SC_PAGE_SIZE);
-#endif
-#if defined(HAVE_GETPAGESIZE)
+  pagesize=(-1);
+#if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+  pagesize=sysconf(_SC_PAGESIZE);
+#elif defined(HAVE_GETPAGESIZE) && defined(POSIX)
   pagesize=getpagesize();
 #endif
-  pages=0;
+  if (pagesize <= 0)
+    pagesize=4096;
+  pages=(-1);
 #if defined(HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
   pages=sysconf(_SC_PHYS_PAGES);
 #endif
-  limit=((pages+512)/1024)*((pagesize+512)/1024);
+  if (pages <= 0)
+    pages=62500;
+  memory=(unsigned long) ((pages+512)/1024)*((pagesize+512)/1024);
 #if defined(PixelCacheThreshold)
-  limit=PixelCacheThreshold;
+  memory=PixelCacheThreshold;
 #endif
-  SetMagickResourceLimit(FileResource,files > 0 ? 3*files/4 : 1024);
-  p=getenv("MAGICK_FILES_LIMIT");
+  (void) SetMagickResourceLimit(AreaResource,memory/sizeof(PixelPacket));
+  (void) SetMagickResourceLimit(MemoryResource,memory);
+  (void) SetMagickResourceLimit(MapResource,2*memory);
+  p=getenv("MAGICK_AREA_LIMIT");
   if (p != (const char *) NULL)
-    SetMagickResourceLimit(FileResource,atol(p));
-  SetMagickResourceLimit(MemoryResource,limit > 0 ? 2*limit : 1024);
+    (void) SetMagickResourceLimit(AreaResource,(unsigned long) atol(p));
   p=getenv("MAGICK_MEMORY_LIMIT");
   if (p != (const char *) NULL)
-    SetMagickResourceLimit(MemoryResource,atol(p));
-  SetMagickResourceLimit(MapResource,limit > 0 ? 8*limit : 4096);
+    (void) SetMagickResourceLimit(MemoryResource,(unsigned long) atol(p));
   p=getenv("MAGICK_MAP_LIMIT");
   if (p != (const char *) NULL)
-    SetMagickResourceLimit(MapResource,atol(p));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L i b e r a t e M a g i c k R e s o u r c e                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  LiberateMagickResource() liberates resources of the specified type.
-%
-%  The format of the LiberateMagickResource() method is:
-%
-%      void LiberateMagickResource(const ResourceType type,
-%        const ExtendedSignedIntegralType size)
-%
-%  A description of each parameter follows:
-%
-%    o type: The type of resource.
-%
-%    o size: The size of the resource.
-%
-%
-*/
-MagickExport void LiberateMagickResource(const ResourceType type,
-  const ExtendedSignedIntegralType size)
-{
-  char
-    message[MaxTextExtent];
-
-  AcquireSemaphoreInfo(&resource_semaphore);
-  switch (type)
-  {
-    case FileResource:
-    {
-      resource_info.file-=size;
-      FormatString(message,"file -%lu/%lu/%lu",(unsigned long) size,
-        (unsigned long) resource_info.file,resource_info.file_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
-      break;
-    }
-    case MemoryResource:
-    {
-      resource_info.memory-=size;
-      FormatString(message,"memory -%lumb/%lumb/%lumb",
-        MegabytesToResource(size),MegabytesToResource(resource_info.memory),
-        resource_info.memory_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
-      break;
-    }
-    case MapResource:
-    {
-      resource_info.map-=size;
-      FormatString(message,"map -%lumb/%lumb/%lumb",MegabytesToResource(size),
-        MegabytesToResource(resource_info.map),resource_info.map_limit);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
-      break;
-    }
-    case DiskResource:
-    {
-      resource_info.disk-=size;
-      FormatString(message,"disk -%lumb/%lugb/%lugb",MegabytesToResource(size),
-        GigabytesToResource(resource_info.disk),resource_info.disk_limit/1024);
-      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),message);
-      break;
-    }
-    default:
-      break;
-  }
-  LiberateSemaphoreInfo(&resource_semaphore);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   L i b e r a t e U n i q u e F i l e R e s o u r c e                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  (void) LiberateUniqueFileResource() deletes a unique file resource.
-%
-%  The format of the (void) LiberateUniqueFileResource() method is:
-%
-%      unsigned int (void) LiberateUniqueFileResource(char *name)
-%
-%  A description of each parameter follows:
-%
-%    o name: the name of the temporary resource.
-%
-%
-*/
-MagickExport unsigned int LiberateUniqueFileResource(const char *name)
-{
-  register UniqueFileResourceInfo
-    *p;
-
-  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),name);
-  AcquireSemaphoreInfo(&unique_semaphore);
-  for (p=temporary_resources; p != (UniqueFileResourceInfo *) NULL; p=p->next)
-    if (strcmp(p->name,name) == 0)
-      {
-        UniqueFileResourceInfo
-          *resource_info;
-
-        if ((p->previous == (UniqueFileResourceInfo *) NULL) &&
-            (p->next == (UniqueFileResourceInfo *) NULL))
-          temporary_resources=(UniqueFileResourceInfo *) NULL;
-        else
-          {
-            if (p->previous != (UniqueFileResourceInfo *) NULL)
-              {
-                p->previous->next=p->next;
-                temporary_resources=p->previous;
-              }
-            if (p->next != (UniqueFileResourceInfo *) NULL)
-              {
-                p->next->previous=p->previous;
-                temporary_resources=p->next;
-              }
-          }
-        resource_info=p;
-        LiberateMemory((void **) &resource_info->name);
-        LiberateMemory((void **) &resource_info);
-        LiberateMagickResource(FileResource,1);
-        break;
-      }
-  (void) remove(name);
-  LiberateSemaphoreInfo(&unique_semaphore);
-  return(p == (UniqueFileResourceInfo *) NULL);
+    (void) SetMagickResourceLimit(MapResource,(unsigned long) atol(p));
+  p=getenv("MAGICK_DISK_LIMIT");
+  if (p != (const char *) NULL)
+    (void) SetMagickResourceLimit(DiskResource,(unsigned long) atol(p));
+  files=(-1);
+#if defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
+  files=sysconf(_SC_OPEN_MAX);
+#elif defined(HAVE_GETDTABLESIZE) && defined(POSIX)
+  files=getdtablesize();
+#endif
+  if (files <= 0)
+    files=64;
+  (void) SetMagickResourceLimit(FileResource,(unsigned long) Max(3*files/4,8));
+  p=getenv("MAGICK_FILES_LIMIT");
+  if (p != (const char *) NULL)
+    (void) SetMagickResourceLimit(FileResource,(unsigned long) atol(p));
 }
 
 /*
@@ -677,11 +577,12 @@ MagickExport unsigned int LiberateUniqueFileResource(const char *name)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ListMagickResourceInfo lists the resource info to a file.
+%  ListMagickResourceInfo() lists the resource info to a file.
 %
 %  The format of the ListMagickResourceInfo method is:
 %
-%      unsigned int ListMagickResourceInfo(FILE *file,ExceptionInfo *exception)
+%      MagickBooleanType ListMagickResourceInfo(FILE *file,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -691,21 +592,180 @@ MagickExport unsigned int LiberateUniqueFileResource(const char *name)
 %
 %
 */
-MagickExport unsigned int ListMagickResourceInfo(FILE *file,
+MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
   ExceptionInfo *exception)
 {
+  char
+    area_limit[MaxTextExtent],
+    disk_limit[MaxTextExtent],
+    file_limit[MaxTextExtent],
+    map_limit[MaxTextExtent],
+    memory_limit[MaxTextExtent];
+
   if (file == (const FILE *) NULL)
     file=stdout;
   AcquireSemaphoreInfo(&resource_semaphore);
-  (void) fprintf(file,"File    Memory       Map       Disk\n");
-  (void) fprintf(file,"-----------------------------------\n");
-  (void) fprintf(file,"%4lu  %6lumb  %6lumb  %6lugb\n",
-    resource_info.file_limit,resource_info.memory_limit,resource_info.map_limit,
-    resource_info.disk_limit/1024);
+  FormatSize(MegabytesToBytes(resource_info.area_limit),area_limit);
+  FormatSize(GigabytesToBytes(resource_info.disk_limit),disk_limit);
+  FormatSize((MagickSizeType) resource_info.file_limit,file_limit);
+  FormatSize(MegabytesToBytes(resource_info.map_limit),map_limit);
+  FormatSize(MegabytesToBytes(resource_info.memory_limit),memory_limit);
+  (void) fprintf(file,"File       Area     Memory        Map       Disk\n");
+  (void) fprintf(file,"------------------------------------------------\n");
+  (void) fprintf(file,"%4s %10s %10s %10s %10s\n",file_limit,area_limit,
+    memory_limit,map_limit,disk_limit);
   (void) fflush(file);
-  LiberateSemaphoreInfo(&resource_semaphore);
-  return(True);
+  RelinquishSemaphoreInfo(&resource_semaphore);
+  return(MagickTrue);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e l i n q u i s h M a g i c k R e s o u r c e                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RelinquishMagickResource() relinquishes resources of the specified type.
+%
+%  The format of the RelinquishMagickResource() method is:
+%
+%      void RelinquishMagickResource(const ResourceType type,
+%        const MagickSizeType size)
+%
+%  A description of each parameter follows:
+%
+%    o type: The type of resource.
+%
+%    o size: The size of the resource.
+%
+%
+*/
+MagickExport void RelinquishMagickResource(const ResourceType type,
+  const MagickSizeType size)
+{
+  char
+    resource_current[MaxTextExtent],
+    resource_limit[MaxTextExtent],
+    resource_request[MaxTextExtent],
+    resource_type[MaxTextExtent];
+
+  FormatSize(size,resource_request);
+  AcquireSemaphoreInfo(&resource_semaphore);
+  switch (type)
+  {
+    case AreaResource:
+    {
+      resource_info.area-=size;
+      (void) FormatMagickString(resource_type,MaxTextExtent,"area");
+      FormatSize((MagickSizeType) resource_info.area,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.area_limit),resource_limit);
+      break;
+    }
+    case MemoryResource:
+    {
+      resource_info.memory-=size;
+      (void) FormatMagickString(resource_type,MaxTextExtent,"memory");
+      FormatSize((MagickSizeType) resource_info.memory,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.memory_limit),resource_limit);
+      break;
+    }
+    case MapResource:
+    {
+      resource_info.map-=size;
+      (void) FormatMagickString(resource_type,MaxTextExtent,"map");
+      FormatSize((MagickSizeType) resource_info.map,resource_current);
+      FormatSize(MegabytesToBytes(resource_info.map_limit),resource_limit);
+      break;
+    }
+    case DiskResource:
+    {
+      resource_info.disk-=size;
+      (void) FormatMagickString(resource_type,MaxTextExtent,"disk");
+      FormatSize((MagickSizeType) resource_info.disk,resource_current);
+      FormatSize(GigabytesToBytes(resource_info.disk_limit),resource_limit);
+      break;
+    }
+    case FileResource:
+    {
+      resource_info.file-=size;
+      (void) FormatMagickString(resource_type,MaxTextExtent,"file");
+      FormatSize((MagickSizeType) resource_info.file,resource_current);
+      FormatSize((MagickSizeType) resource_info.file_limit,resource_limit);
+      break;
+    }
+    default:
+      break;
+  }
+  RelinquishSemaphoreInfo(&resource_semaphore);
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
+    resource_type,resource_request,resource_current,resource_limit);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%    R e l i n q u i s h U n i q u e F i l e R e s o u r c e                  %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  RelinquishUniqueFileResource() relinquishes a unique file resource.
+%
+%  The format of the RelinquishUniqueFileResource() method is:
+%
+%      MagickBooleanType RelinquishUniqueFileResource(const char *path)
+%
+%  A description of each parameter follows:
+%
+%    o name: the name of the temporary resource.
+%
+%
+*/
+MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
+{
+  char
+    cache_path[MaxTextExtent];
+
+  assert(path != (const char *) NULL);
+  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),path);
+  if (temporary_resources != (LinkedListInfo *) NULL)
+    {
+      register const char
+        *p;
+
+      ResetLinkedListIterator(temporary_resources);
+      p=(const char *) GetNextElementInLinkedList(temporary_resources);
+      while (p != (const char *) NULL)
+      {
+        if (LocaleCompare(p,path) == 0)
+          break;
+        p=(const char *) GetNextElementInLinkedList(temporary_resources);
+      }
+      if (p != (const char *) NULL)
+        {
+          char
+            *element;
+
+          element=(char *)
+            RemoveElementByValueFromLinkedList(temporary_resources,p);
+          element=(char *) RelinquishMagickMemory(element);
+          RelinquishMagickResource(FileResource,1);
+        }
+    }
+  (void) CopyMagickString(cache_path,path,MaxTextExtent);
+  AppendImageFormat("cache",cache_path);
+  (void) remove(cache_path);
+  return((MagickBooleanType) (remove(path) == 0));
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -722,7 +782,7 @@ MagickExport unsigned int ListMagickResourceInfo(FILE *file,
 %
 %  The format of the SetMagickResourceLimit() method is:
 %
-%      void SetMagickResourceLimit(const ResourceType type,
+%      MagickBooleanType SetMagickResourceLimit(const ResourceType type,
 %        const unsigned long limit)
 %
 %  A description of each parameter follows:
@@ -733,15 +793,15 @@ MagickExport unsigned int ListMagickResourceInfo(FILE *file,
 %
 %
 */
-MagickExport void SetMagickResourceLimit(const ResourceType type,
+MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   const unsigned long limit)
 {
   AcquireSemaphoreInfo(&resource_semaphore);
   switch (type)
   {
-    case FileResource:
+    case AreaResource:
     {
-      resource_info.file_limit=limit;
+      resource_info.area_limit=limit;
       break;
     }
     case MemoryResource:
@@ -759,8 +819,14 @@ MagickExport void SetMagickResourceLimit(const ResourceType type,
       resource_info.disk_limit=limit;
       break;
     }
+    case FileResource:
+    {
+      resource_info.file_limit=limit;
+      break;
+    }
     default:
       break;
   }
-  LiberateSemaphoreInfo(&resource_semaphore);
+  RelinquishSemaphoreInfo(&resource_semaphore);
+  return(MagickTrue);
 }
