@@ -11,37 +11,24 @@
 %                                                                             %
 %             Methods to Compute a Message Digest for an Image                %
 %                                                                             %
-%                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -55,8 +42,10 @@
 #include "magick/studio.h"
 #include "magick/error.h"
 #include "magick/attribute.h"
+#include "magick/image.h"
+#include "magick/memory_.h"
 #include "magick/signature.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Define declarations.
@@ -109,14 +98,16 @@ MagickExport void FinalizeSignature(SignatureInfo *signature_info)
   low_order=signature_info->low_order;
   high_order=signature_info->high_order;
   count=(long) ((low_order >> 3) & 0x3f);
-  signature_info->message[count++]=0x80;
-  if (count <= (SignatureSize-8))
-    (void) memset(signature_info->message+count,0,SignatureSize-8-count);
+  signature_info->message[count++]=(unsigned char) 0x80;
+  if (count <= (MagickSignatureSize-8))
+    (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
+      (MagickSignatureSize-8-count));
   else
     {
-      (void) memset(signature_info->message+count,0,SignatureSize-count);
+      (void) ResetMagickMemory(signature_info->message+count,0,(size_t)
+        (MagickSignatureSize-count));
       TransformSignature(signature_info);
-      (void) memset(signature_info->message,0,SignatureSize-8);
+      (void) ResetMagickMemory(signature_info->message,0,MagickSignatureSize-8);
     }
   signature_info->message[56]=(unsigned char) (high_order >> 24);
   signature_info->message[57]=(unsigned char) (high_order >> 16);
@@ -158,7 +149,7 @@ MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
     lsb_first;
 
   assert(signature_info != (SignatureInfo *) NULL);
-  (void) memset(signature_info,0,sizeof(SignatureInfo));
+  (void) ResetMagickMemory(signature_info,0,sizeof(SignatureInfo));
   signature_info->digest[0]=0x6a09e667UL;
   signature_info->digest[1]=0xbb67ae85UL;
   signature_info->digest[2]=0x3c6ef372UL;
@@ -168,7 +159,7 @@ MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
   signature_info->digest[6]=0x1f83d9abUL;
   signature_info->digest[7]=0x5be0cd19UL;
   lsb_first=1;
-  signature_info->lsb_first=(*(char *) &lsb_first);
+  signature_info->lsb_first=(MagickBooleanType) (*(char *) &lsb_first);
   signature_info->signature=MagickSignature;
 }
 
@@ -190,7 +181,7 @@ MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
 %
 %  The format of the SignatureImage method is:
 %
-%      unsigned int SignatureImage(Image *image)
+%      MagickBooleanType SignatureImage(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -198,7 +189,7 @@ MagickExport void GetSignatureInfo(SignatureInfo *signature_info)
 %
 %
 */
-MagickExport unsigned int SignatureImage(Image *image)
+MagickExport MagickBooleanType SignatureImage(Image *image)
 {
   char
     signature[MaxTextExtent];
@@ -232,10 +223,12 @@ MagickExport unsigned int SignatureImage(Image *image)
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  message=(unsigned char *) AcquireMemory(20*image->columns);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  message=(unsigned char *) AcquireMagickMemory((size_t) (20*image->columns));
   if (message == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToComputeImageSignature");
+      image->filename);
   /*
     Compute image digital signature.
   */
@@ -264,54 +257,39 @@ MagickExport unsigned int SignatureImage(Image *image)
       *q++=(unsigned char) (quantum >> 16);
       *q++=(unsigned char) (quantum >> 8);
       *q++=(unsigned char) quantum;
-      if (!image->matte)
-        {
-          if (image->colorspace == CMYKColorspace)
-            {
-              quantum=ScaleQuantumToLong(p->opacity);
-              *q++=(unsigned char) (quantum >> 24);
-              *q++=(unsigned char) (quantum >> 16);
-              *q++=(unsigned char) (quantum >> 8);
-              *q++=(unsigned char) quantum;
-            }
-          quantum=ScaleQuantumToLong(OpaqueOpacity);
-          *q++=(unsigned char) (quantum >> 24);
-          *q++=(unsigned char) (quantum >> 16);
-          *q++=(unsigned char) (quantum >> 8);
-          *q++=(unsigned char) quantum;
-        }
-      else
+      if (image->matte != MagickFalse)
         {
           quantum=ScaleQuantumToLong(p->opacity);
           *q++=(unsigned char) (quantum >> 24);
           *q++=(unsigned char) (quantum >> 16);
           *q++=(unsigned char) (quantum >> 8);
           *q++=(unsigned char) quantum;
-          if (image->colorspace == CMYKColorspace)
-            {
-              quantum=ScaleQuantumToLong(indexes[x]);
-              *q++=(unsigned char) (quantum >> 24);
-              *q++=(unsigned char) (quantum >> 16);
-              *q++=(unsigned char) (quantum >> 8);
-              *q++=(unsigned char) quantum;
-            }
+        }
+      if (image->colorspace == CMYKColorspace)
+        {
+          quantum=ScaleQuantumToLong(indexes[x]);
+          *q++=(unsigned char) (quantum >> 24);
+          *q++=(unsigned char) (quantum >> 16);
+          *q++=(unsigned char) (quantum >> 8);
+          *q++=(unsigned char) quantum;
         }
       p++;
     }
-    UpdateSignature(&signature_info,message,q-message);
+    UpdateSignature(&signature_info,message,(size_t) (q-message));
   }
   FinalizeSignature(&signature_info);
-  LiberateMemory((void **) &message);
+  message=(unsigned char *) RelinquishMagickMemory(message);
   /*
     Convert digital signature to a 64 character hex string.
   */
-  FormatString(signature,"%08lx%08lx%08lx%08lx%08lx%08lx%08lx%08lx",
-    signature_info.digest[0],signature_info.digest[1],signature_info.digest[2],
-    signature_info.digest[3],signature_info.digest[4],signature_info.digest[5],
-    signature_info.digest[6],signature_info.digest[7]);
+  (void) FormatMagickString(signature,MaxTextExtent,
+    "%08lx%08lx%08lx%08lx%08lx%08lx%08lx%08lx",signature_info.digest[0],
+    signature_info.digest[1],signature_info.digest[2],signature_info.digest[3],
+    signature_info.digest[4],signature_info.digest[5],signature_info.digest[6],
+    signature_info.digest[7]);
   (void) SetImageAttribute(image,"signature",(char *) NULL);
   (void) SetImageAttribute(image,"signature",signature);
-  return(True);
+  return(MagickTrue);
 }
 
 /*
@@ -348,8 +326,7 @@ static void TransformSignature(SignatureInfo *signature_info)
 #define Suma1(x)  (RotateRight(x,6) ^ RotateRight(x,11) ^ RotateRight(x,25))
 
   long
-    j,
-    shift;
+    j;
 
   register long
     i;
@@ -384,6 +361,7 @@ static void TransformSignature(SignatureInfo *signature_info)
     F,
     G,
     H,
+    shift,
     T,
     T1,
     T2,
@@ -391,7 +369,7 @@ static void TransformSignature(SignatureInfo *signature_info)
 
   shift=32;
   p=signature_info->message;
-  if (!signature_info->lsb_first)
+  if (signature_info->lsb_first == MagickFalse)
     {
       if (sizeof(unsigned long) <= 4)
         for (i=0; i < 16; i++)
@@ -511,32 +489,34 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
   */
   assert(signature_info != (SignatureInfo *) NULL);
   assert(signature_info->signature == MagickSignature);
-  n=length;
+  n=(unsigned long) length;
   count=Trunc32(signature_info->low_order+(n << 3));
   if (count < signature_info->low_order)
     signature_info->high_order++;
   signature_info->low_order=count;
   signature_info->high_order+=(n >> 29);
-  if (signature_info->offset)
+  if (signature_info->offset != 0)
     {
-      i=SignatureSize-signature_info->offset;
+      i=MagickSignatureSize-signature_info->offset;
       if (i > (long) n)
         i=(long) n;
-      (void) memcpy(signature_info->message+signature_info->offset,message,i);
+      (void) CopyMagickMemory(signature_info->message+signature_info->offset,
+        message,(size_t) i);
       n-=i;
       message+=i;
       signature_info->offset+=i;
-      if (signature_info->offset != SignatureSize)
+      if (signature_info->offset != MagickSignatureSize)
         return;
       TransformSignature(signature_info);
     }
-  while (n >= SignatureSize)
+  while (n >= MagickSignatureSize)
   {
-    (void) memcpy(signature_info->message,message,SignatureSize);
-    message+=SignatureSize;
-    n-=SignatureSize;
+    (void) CopyMagickMemory(signature_info->message,message,
+      MagickSignatureSize);
+    message+=MagickSignatureSize;
+    n-=MagickSignatureSize;
     TransformSignature(signature_info);
   }
-  (void) memcpy(signature_info->message,message,n);
+  (void) CopyMagickMemory(signature_info->message,message,(size_t) n);
   signature_info->offset=(long) n;
 }
