@@ -12,37 +12,24 @@
 %                                                                             %
 %                         ImageMagick Static Methods                          %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,7 +43,7 @@
 #include "magick/studio.h"
 #include "magick/module.h"
 #include "magick/static.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,8 +60,8 @@
 %
 %  The format of the ExecuteStaticModuleProcess method is:
 %
-%      unsigned int ExecuteStaticModuleProcess(const char *tag,Image **image,
-%        const int argc,char **argv)
+%      MagickBooleanType ExecuteStaticModuleProcess(const char *tag,
+%        Image **image,const int argc,char **argv)
 %
 %  A description of each parameter follows:
 %
@@ -87,17 +74,41 @@
 %    o argv: A text array containing the command line arguments.
 %
 */
-MagickExport unsigned int ExecuteStaticModuleProcess(const char *tag,
+#if defined(SupportMagickModules)
+MagickExport MagickBooleanType ExecuteStaticModuleProcess(const char *tag,
   Image **image,const int argc,char **argv)
 {
+  MagickBooleanType
+    status;
+
+  assert(image != (Image **) NULL);
+  assert((*image)->signature == MagickSignature);
+  if ((*image)->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*image)->filename);
+  status=MagickFalse;
 #if !defined(BuildMagickModules)
-#if !defined(_VISUALC_)
-  if (LocaleCompare("analyze",tag) == 0)
-    return(AnalyzeImage(image,argc,argv));
+  {
+    MagickBooleanType
+      (*module)(Image **,const int,char **);
+
+    module=(MagickBooleanType (*)(Image **,const int,char **)) NULL;
+    if (LocaleCompare("analyze",tag) == 0)
+      module=AnalyzeImage;
+    if (module != (MagickBooleanType (*)(Image **,const int,char **)) NULL)
+      {
+        if ((*image)->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "Invoking \"%s\" static filter module",tag);
+        status=(*module)(image,argc,argv);
+        if ((*image)->debug != MagickFalse)
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"\"%s\" completes",
+            tag);
+      }
+  }
 #endif
-#endif
-  return(False);
+  return(status);
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -127,6 +138,9 @@ MagickExport void RegisterStaticModules(void)
   RegisterAVSImage();
   RegisterBMPImage();
   RegisterCAPTIONImage();
+  RegisterCINImage();
+  RegisterCIPImage();
+  RegisterCLIPImage();
   RegisterCLIPBOARDImage();
   RegisterCMYKImage();
   RegisterCUTImage();
@@ -182,9 +196,11 @@ MagickExport void RegisterStaticModules(void)
   RegisterPS3Image();
   RegisterPSDImage();
   RegisterPWPImage();
+  RegisterRAWImage();
   RegisterRGBImage();
   RegisterRLAImage();
   RegisterRLEImage();
+  RegisterSCRImage();
   RegisterSCTImage();
   RegisterSFWImage();
   RegisterSGIImage();
@@ -211,7 +227,135 @@ MagickExport void RegisterStaticModules(void)
   RegisterXCImage();
   RegisterXCFImage();
   RegisterXPMImage();
+#if defined(_VISUALC_)
+  RegisterXTRNImage();
+#endif
   RegisterXWDImage();
   RegisterYUVImage();
 #endif
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   U n r e g i s t e r S t a t i c M o d u l e s                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  UnregisterStaticModules() statically unregisters all the available module
+%  handlers.
+%
+%  The format of the UnregisterStaticModules method is:
+%
+%      UnregisterStaticModules(void)
+%
+%
+*/
+MagickExport void UnregisterStaticModules(void)
+{
+#if !defined(BuildMagickModules)
+  UnregisterARTImage();
+  UnregisterAVIImage();
+  UnregisterAVSImage();
+  UnregisterBMPImage();
+  UnregisterCAPTIONImage();
+  UnregisterCINImage();
+  UnregisterCIPImage();
+  UnregisterCLIPImage();
+  UnregisterCLIPBOARDImage();
+  UnregisterCMYKImage();
+  UnregisterCUTImage();
+  UnregisterDCMImage();
+  UnregisterDIBImage();
+  UnregisterDPSImage();
+  UnregisterDPXImage();
+  UnregisterEMFImage();
+  UnregisterEPTImage();
+  UnregisterFAXImage();
+  UnregisterFITSImage();
+  UnregisterFPXImage();
+  UnregisterGIFImage();
+  UnregisterGRAYImage();
+  UnregisterGRADIENTImage();
+  UnregisterHISTOGRAMImage();
+  UnregisterHTMLImage();
+  UnregisterICONImage();
+  UnregisterJBIGImage();
+  UnregisterJPEGImage();
+  UnregisterJP2Image();
+  UnregisterLABELImage();
+  UnregisterMAGICKImage();
+  UnregisterMAPImage();
+  UnregisterMATImage();
+  UnregisterMATTEImage();
+  UnregisterMETAImage();
+  UnregisterMIFFImage();
+  UnregisterMONOImage();
+  UnregisterMPCImage();
+  UnregisterMPEGImage();
+  UnregisterMPRImage();
+  UnregisterMSLImage();
+  UnregisterMTVImage();
+  UnregisterMVGImage();
+  UnregisterNULLImage();
+  UnregisterOTBImage();
+  UnregisterPALMImage();
+  UnregisterPATTERNImage();
+  UnregisterPCDImage();
+  UnregisterPCLImage();
+  UnregisterPCXImage();
+  UnregisterPDBImage();
+  UnregisterPDFImage();
+  UnregisterPICTImage();
+  UnregisterPIXImage();
+  UnregisterPLASMAImage();
+  UnregisterPNGImage();
+  UnregisterPNMImage();
+  UnregisterPREVIEWImage();
+  UnregisterPSImage();
+  UnregisterPS2Image();
+  UnregisterPS3Image();
+  UnregisterPSDImage();
+  UnregisterPWPImage();
+  UnregisterRAWImage();
+  UnregisterRGBImage();
+  UnregisterRLAImage();
+  UnregisterRLEImage();
+  UnregisterSCRImage();
+  UnregisterSCTImage();
+  UnregisterSFWImage();
+  UnregisterSGIImage();
+  UnregisterSTEGANOImage();
+  UnregisterSUNImage();
+  UnregisterSVGImage();
+  UnregisterTGAImage();
+  UnregisterTIFFImage();
+  UnregisterTILEImage();
+  UnregisterTIMImage();
+  UnregisterTTFImage();
+  UnregisterTXTImage();
+  UnregisterUILImage();
+  UnregisterURLImage();
+  UnregisterUYVYImage();
+  UnregisterVICARImage();
+  UnregisterVIDImage();
+  UnregisterVIFFImage();
+  UnregisterWBMPImage();
+  UnregisterWMFImage();
+  UnregisterWPGImage();
+  UnregisterXImage();
+  UnregisterXBMImage();
+  UnregisterXCImage();
+  UnregisterXCFImage();
+  UnregisterXPMImage();
+#if defined(_VISUALC_)
+  UnregisterXTRNImage();
+#endif
+  UnregisterXWDImage();
+  UnregisterYUVImage();
+#endif
+}
