@@ -12,37 +12,24 @@
 %                                                                             %
 %                      ImageMagick Pixel Stream Methods                       %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,9 +43,11 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/cache.h"
+#include "magick/constitute.h"
 #include "magick/error.h"
+#include "magick/memory_.h"
 #include "magick/semaphore.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 
 /*
   Typedef declaractions.
@@ -89,7 +78,7 @@ static PixelPacket
   *SetPixelStream(Image *,const long,const long,const unsigned long,
     const unsigned long);
 
-static unsigned int
+static MagickBooleanType
   SyncPixelStream(Image *);
 
 static void
@@ -110,8 +99,8 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method AcquireOnePixelFromStream() returns a single pixel at the specified
-%  (x,y) location.  The image background color is returned if an error occurs.
+%  AcquireOnePixelFromStream() returns a single pixel at the specified (x.y)
+%  location.  The image background color is returned if an error occurs.
 %
 %  The format of the AcquireOnePixelFromStream() method is:
 %
@@ -156,10 +145,10 @@ static PixelPacket AcquireOnePixelFromStream(const Image *image,const long x,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method AcquirePixelStream() gets pixels from the in-memory or disk pixel
-%  cache as defined by the geometry parameters.   A pointer to the pixels is
-%  returned if the pixels are transferred, otherwise a NULL is returned.  For
-%  streams this method is a no-op.
+%  AcquirePixelStream() gets pixels from the in-memory or disk pixel cache as
+%  defined by the geometry parameters.   A pointer to the pixels is returned if
+%  the pixels are transferred, otherwise a NULL is returned.  For streams this
+%  method is a no-op.
 %
 %  The format of the AcquirePixelStream() method is:
 %
@@ -185,7 +174,7 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
   const long y,const unsigned long columns,const unsigned long rows,
   ExceptionInfo *exception)
 {
-  ExtendedSignedIntegralType
+  MagickSizeType
     number_pixels;
 
   size_t
@@ -198,38 +187,42 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
     Validate pixel cache geometry.
   */
   assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if ((x < 0) || (y < 0) || ((x+(long) columns) > (long) image->columns) ||
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
-      (void) ThrowException(exception,StreamError,"UnableToAcquirePixelStream",
-        "ImageDoesNotContainTheStreamGeometry");
+      (void) ThrowMagickException(exception,GetMagickModule(),StreamError,
+        "ImageDoesNotContainTheStreamGeometry",image->filename);
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   if (stream_info->type == UndefinedCache)
     {
-      (void) ThrowException(exception,StreamError,"PixelCacheIsNotOpen",
-        image->filename);
+      (void) ThrowMagickException(exception,GetMagickModule(),StreamError,
+        "PixelCacheIsNotOpen",image->filename);
       return((PixelPacket *) NULL);
     }
   /*
     Pixels are stored in a temporary buffer until they are synced to the cache.
   */
-  number_pixels=(ExtendedSignedIntegralType) columns*rows;
-  length=number_pixels*sizeof(PixelPacket);
+  number_pixels=(MagickSizeType) columns*rows;
+  length=(size_t) number_pixels*sizeof(PixelPacket);
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
     length+=number_pixels*sizeof(IndexPacket);
   if (stream_info->pixels == (PixelPacket *) NULL)
-    stream_info->pixels=(PixelPacket *) AcquireMemory(length);
+    stream_info->pixels=(PixelPacket *) AcquireMagickMemory(length);
   else
-    if (length != stream_info->length)
-      ReacquireMemory((void **) &stream_info->pixels,length);
+    if ((MagickSizeType) length != stream_info->length)
+      stream_info->pixels=(PixelPacket *)
+        ResizeMagickMemory(stream_info->pixels,length);
   if (stream_info->pixels == (void *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateCacheInfo");
-  stream_info->length=length;
+    ThrowMagickFatalException(ResourceLimitFatalError,
+      "UnableToAllocateCacheInfo",image->filename);
+  stream_info->length=(MagickSizeType) length;
   stream_info->indexes=(IndexPacket *) NULL;
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
@@ -248,8 +241,7 @@ static const PixelPacket *AcquirePixelStream(const Image *image,const long x,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method DestroyPixelStream() deallocates memory associated with the pixel
-%  stream.
+%  DestroyPixelStream() deallocates memory associated with the pixel stream.
 %
 %  The format of the DestroyPixelStream() method is:
 %
@@ -268,21 +260,23 @@ static void DestroyPixelStream(Image *image)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
-  AcquireSemaphoreInfo(&stream_info->semaphore);
+  AcquireSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
   stream_info->reference_count--;
   if (stream_info->reference_count > 0)
     {
-      LiberateSemaphoreInfo(&stream_info->semaphore);
+      RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
       return;
     }
-  LiberateSemaphoreInfo(&stream_info->semaphore);
   if (stream_info->pixels != (PixelPacket *) NULL)
-    LiberateMemory((void **) &stream_info->pixels);
-  if (stream_info->semaphore != (SemaphoreInfo *) NULL)
-    DestroySemaphoreInfo(&stream_info->semaphore);
-  LiberateMemory((void **) &stream_info);
+    stream_info->pixels=(PixelPacket *)
+      RelinquishMagickMemory(stream_info->pixels);
+  RelinquishSemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  DestroySemaphoreInfo((SemaphoreInfo **) &stream_info->semaphore);
+  stream_info=(StreamInfo *) RelinquishMagickMemory(stream_info);
 }
 
 /*
@@ -296,8 +290,8 @@ static void DestroyPixelStream(Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetIndexesFromStream() returns the indexes associated with the last
-%  call to SetPixelStream() or GetPixelStream().
+%  GetIndexesFromStream() returns the indexes associated with the last call to
+%  SetPixelStream() or GetPixelStream().
 %
 %  The format of the GetIndexesFromStream() method is:
 %
@@ -319,6 +313,8 @@ static IndexPacket *GetIndexesFromStream(const Image *image)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   return(stream_info->indexes);
@@ -335,7 +331,7 @@ static IndexPacket *GetIndexesFromStream(const Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetOnePixelFromStream() returns a single pixel at the specified (x,y)
+%  GetOnePixelFromStream() returns a single pixel at the specified (x,y)
 %  location.  The image background color is returned if an error occurs.
 %
 %  The format of the GetOnePixelFromStream() method is:
@@ -377,7 +373,7 @@ static PixelPacket GetOnePixelFromStream(Image *image,const long x,const long y)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetPixelStream() gets pixels from the in-memory or disk pixel cache as
+%  GetPixelStream() gets pixels from the in-memory or disk pixel cache as
 %  defined by the geometry parameters.   A pointer to the pixels is returned if
 %  the pixels are transferred, otherwise a NULL is returned.  For streams
 %  this method is a no-op.
@@ -407,6 +403,8 @@ static PixelPacket *GetPixelStream(Image *image,const long x,const long y,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   pixels=SetPixelStream(image,x,y,columns,rows);
   return(pixels);
 }
@@ -422,8 +420,8 @@ static PixelPacket *GetPixelStream(Image *image,const long x,const long y,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetPixelsFromStream() returns the pixels associated with the last
-%  call to SetPixelStream() or GetPixelStream().
+%  GetPixelsFromStream() returns the pixels associated with the last call to
+%  SetPixelStream() or GetPixelStream().
 %
 %  The format of the GetPixelsFromStream() method is:
 %
@@ -445,6 +443,8 @@ static PixelPacket *GetPixelsFromStream(const Image *image)
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
   return(stream_info->pixels);
@@ -487,23 +487,25 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
     *image;
 
   ImageInfo
-    *clone_info;
+    *read_info;
 
   /*
     Stream image pixels.
   */
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  clone_info=CloneImageInfo(image_info);
-  GetCacheInfo(&clone_info->cache);
-  SetPixelCacheMethods(clone_info->cache,AcquirePixelStream,GetPixelStream,
+  read_info=CloneImageInfo(image_info);
+  GetCacheInfo(&read_info->cache);
+  SetPixelCacheMethods(read_info->cache,AcquirePixelStream,GetPixelStream,
     SetPixelStream,SyncPixelStream,GetPixelsFromStream,GetIndexesFromStream,
     AcquireOnePixelFromStream,GetOnePixelFromStream,DestroyPixelStream);
-  clone_info->stream=stream;
-  image=ReadImage(clone_info,exception);
-  DestroyImageInfo(clone_info);
+  read_info->stream=stream;
+  image=ReadImage(read_info,exception);
+  DestroyImageInfo(read_info);
   return(image);
 }
 
@@ -518,8 +520,8 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method SetPixelStream() allocates an area to store image pixels as defined
-%  by the region rectangle and returns a pointer to the area.  This area is
+%  SetPixelStream() allocates an area to store image pixels as defined by the
+%  region rectangle and returns a pointer to the area.  This area is
 %  subsequently transferred from the pixel cache with method SyncPixelStream().
 %  A pointer to the pixels is returned if the pixels are transferred,
 %  otherwise a NULL is returned.
@@ -544,12 +546,15 @@ MagickExport Image *ReadStream(const ImageInfo *image_info,StreamHandler stream,
 static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   const unsigned long columns,const unsigned long rows)
 {
-  ExtendedSignedIntegralType
+  MagickSizeType
     number_pixels;
 
   size_t
     length;
 
+  StreamHandler
+    stream_handler;
+
   StreamInfo
     *stream_info;
 
@@ -560,14 +565,15 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   if ((x < 0) || (y < 0) || ((x+(long) columns) > (long) image->columns) ||
       ((y+(long) rows) > (long) image->rows) || (columns == 0) || (rows == 0))
     {
-      (void) ThrowException(&image->exception,StreamError,"UnableToSetPixelStream",
-        "ImageDoesNotContainTheStreamGeometry");
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        StreamError,"ImageDoesNotContainTheStreamGeometry",image->filename);
       return((PixelPacket *) NULL);
     }
-  if (image->blob->stream == (StreamHandler) NULL)
+  stream_handler=GetBlobStreamHandler(image);
+  if (stream_handler == (StreamHandler) NULL)
     {
-      (void) ThrowException(&image->exception,StreamError,"UnableToSetPixelStream",
-        "NoStreamHandlerIsDefined");
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        StreamError,"NoStreamHandlerIsDefined",image->filename);
       return((PixelPacket *) NULL);
     }
   stream_info=(StreamInfo *) image->cache;
@@ -576,8 +582,8 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
       (image->colorspace != GetCacheColorspace(image->cache)))
     {
       if (GetCacheClass(image->cache) == UndefinedClass)
-        (void) image->blob->stream(image,(const void *) NULL,
-          stream_info->columns);
+        (void) stream_handler(image,(const void *) NULL,
+          (size_t) stream_info->columns);
       stream_info->storage_class=image->storage_class;
       stream_info->colorspace=image->colorspace;
       stream_info->columns=image->columns;
@@ -589,25 +595,26 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
   */
   stream_info->columns=columns;
   stream_info->rows=rows;
-  number_pixels=(ExtendedSignedIntegralType) columns*rows;
-  length=number_pixels*sizeof(PixelPacket);
+  number_pixels=(MagickSizeType) columns*rows;
+  length=(size_t) number_pixels*sizeof(PixelPacket);
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
     length+=number_pixels*sizeof(IndexPacket);
   if (stream_info->pixels == (PixelPacket *) NULL)
     {
-      stream_info->pixels=(PixelPacket *) AcquireMemory(length);
-      stream_info->length=length;
+      stream_info->pixels=(PixelPacket *) AcquireMagickMemory(length);
+      stream_info->length=(MagickSizeType) length;
     }
   else
-    if (stream_info->length < length)
+    if (stream_info->length < (MagickSizeType) length)
       {
-        ReacquireMemory((void **) &stream_info->pixels,length);
-        stream_info->length=length;
+        stream_info->pixels=(PixelPacket *)
+          ResizeMagickMemory(stream_info->pixels,length);
+        stream_info->length=(MagickSizeType) length;
       }
   if (stream_info->pixels == (void *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToAllocateImagePixels");
+    ThrowMagickFatalException(ResourceLimitFatalError,
+      "UnableToAllocateImagePixels",image->filename);
   stream_info->indexes=(IndexPacket *) NULL;
   if ((image->storage_class == PseudoClass) ||
       (image->colorspace == CMYKColorspace))
@@ -626,38 +633,49 @@ static PixelPacket *SetPixelStream(Image *image,const long x,const long y,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method SyncPixelStream() calls the user supplied callback method with the
-%  latest stream of pixels.
+%  SyncPixelStream() calls the user supplied callback method with the latest
+%  stream of pixels.
 %
 %  The format of the SyncPixelStream method is:
 %
-%      unsigned int SyncPixelStream(Image *image)
+%      MagickBooleanType SyncPixelStream(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method SyncPixelStream() returns True if the image pixels are
-%      transferred to the in-memory or disk cache otherwise False.
+%    o status: Method SyncPixelStream() returns MagickTrue if the image pixels are
+%      transferred to the in-memory or disk cache otherwise MagickFalse.
 %
 %    o image: The image.
 %
 %
 */
-static unsigned int SyncPixelStream(Image *image)
+static MagickBooleanType SyncPixelStream(Image *image)
 {
+  MagickBooleanType
+    status;
+
+  StreamHandler
+    stream_handler;
+
   StreamInfo
     *stream_info;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   stream_info=(StreamInfo *) image->cache;
   assert(stream_info->signature == MagickSignature);
-  if (image->blob->stream == (StreamHandler) NULL)
+  stream_handler=GetBlobStreamHandler(image);
+  if (stream_handler == (StreamHandler) NULL)
     {
-      (void) ThrowException(&image->exception,StreamError,"UnableToSyncPixelStream",
-        "NoStreamHandlerIsDefined");
-      return(False);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        StreamError,"NoStreamHandlerIsDefined",image->filename);
+      return(MagickFalse);
     }
-  return(image->blob->stream(image,stream_info->pixels,stream_info->columns));
+  status=stream_handler(image,stream_info->pixels,(size_t)
+    stream_info->columns);
+  return(status);
 }
 
 /*
@@ -677,7 +695,7 @@ static unsigned int SyncPixelStream(Image *image)
 %
 %  The format of the WriteStream() method is:
 %
-%      unsigned int WriteStream(const ImageInfo *image_info,Image *,
+%      MagickBooleanType WriteStream(const ImageInfo *image_info,Image *,
 %        StreamHandler stream)
 %
 %  A description of each parameter follows:
@@ -688,22 +706,24 @@ static unsigned int SyncPixelStream(Image *image)
 %
 %
 */
-MagickExport unsigned int WriteStream(const ImageInfo *image_info,Image *image,
-  StreamHandler stream)
+MagickExport MagickBooleanType WriteStream(const ImageInfo *image_info,
+  Image *image,StreamHandler stream)
 {
   ImageInfo
-    *clone_info;
+    *write_info;
 
-  unsigned int
+  MagickBooleanType
     status;
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  clone_info=CloneImageInfo(image_info);
-  clone_info->stream=stream;
-  status=WriteImage(clone_info,image);
-  DestroyImageInfo(clone_info);
+  write_info=CloneImageInfo(image_info);
+  write_info->stream=stream;
+  status=WriteImage(write_info,image);
+  DestroyImageInfo(write_info);
   return(status);
 }
