@@ -12,37 +12,24 @@
 %                                                                             %
 %                   ImageMagick Image Transform Methods                       %
 %                                                                             %
-%                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -53,10 +40,19 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/color.h"
+#include "magick/composite.h"
+#include "magick/effect.h"
 #include "magick/error.h"
-#include "magick/monitor.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/memory_.h"
 #include "magick/list.h"
-#include "magick/utility.h"
+#include "magick/monitor.h"
+#include "magick/resource_.h"
+#include "magick/resize.h"
+#include "magick/string_.h"
+#include "magick/transform.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -100,7 +96,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     y;
 
   RectangleInfo
-    clone_info;
+    extent;
 
   register const PixelPacket
     *p;
@@ -121,6 +117,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   assert(chop_info != (RectangleInfo *) NULL);
@@ -128,28 +126,27 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       ((chop_info->y+(long) chop_info->height) < 0) ||
       (chop_info->x > (long) image->columns) ||
       (chop_info->y > (long) image->rows))
-    ThrowImageException(OptionError,"GeometryDoesNotContainImage",
-      "UnableToChopImage");
-  clone_info=(*chop_info);
-  if ((clone_info.x+(long) clone_info.width) > (long) image->columns)
-    clone_info.width=(unsigned long) ((long) image->columns-clone_info.x);
-  if ((clone_info.y+(long) clone_info.height) > (long) image->rows)
-    clone_info.height=(unsigned long) ((long) image->rows-clone_info.y);
-  if (clone_info.x < 0)
+    ThrowImageException(OptionError,"GeometryDoesNotContainImage");
+  extent=(*chop_info);
+  if ((extent.x+(long) extent.width) > (long) image->columns)
+    extent.width=(unsigned long) ((long) image->columns-extent.x);
+  if ((extent.y+(long) extent.height) > (long) image->rows)
+    extent.height=(unsigned long) ((long) image->rows-extent.y);
+  if (extent.x < 0)
     {
-      clone_info.width-=(unsigned long) (-clone_info.x);
-      clone_info.x=0;
+      extent.width-=(unsigned long) (-extent.x);
+      extent.x=0;
     }
-  if (clone_info.y < 0)
+  if (extent.y < 0)
     {
-      clone_info.height-=(unsigned long) (-clone_info.y);
-      clone_info.y=0;
+      extent.height-=(unsigned long) (-extent.y);
+      extent.y=0;
     }
   /*
     Initialize chop image attributes.
   */
-  chop_image=CloneImage(image,image->columns-clone_info.width,
-    image->rows-clone_info.height,False,exception);
+  chop_image=CloneImage(image,image->columns-extent.width,
+    image->rows-extent.height,MagickFalse,exception);
   if (chop_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -157,7 +154,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   */
   i=0;
   j=0;
-  for (y=0; y < (long) clone_info.y; y++)
+  for (y=0; y < (long) extent.y; y++)
   {
     p=AcquireImagePixels(image,0,i++,image->columns,1,exception);
     q=SetImagePixels(chop_image,0,j++,chop_image->columns,1);
@@ -167,7 +164,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     chop_indexes=GetIndexes(chop_image);
     for (x=0; x < (long) image->columns; x++)
     {
-      if ((x < clone_info.x) || (x >= (long) (clone_info.x+clone_info.width)))
+      if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
         {
           if ((indexes != (IndexPacket *) NULL) &&
               (chop_indexes != (IndexPacket *) NULL))
@@ -177,17 +174,17 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
         }
       p++;
     }
-    if (!SyncImagePixels(chop_image))
+    if (SyncImagePixels(chop_image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(ChopImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0 != 0)
+      if (MagickMonitor(ChopImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
   /*
     Extract chop image.
   */
-  i+=clone_info.height;
-  for (y=0; y < (long) (image->rows-(clone_info.y+clone_info.height)); y++)
+  i+=extent.height;
+  for (y=0; y < (long) (image->rows-(extent.y+extent.height)); y++)
   {
     p=AcquireImagePixels(image,0,i++,image->columns,1,exception);
     q=SetImagePixels(chop_image,0,j++,chop_image->columns,1);
@@ -197,7 +194,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     chop_indexes=GetIndexes(chop_image);
     for (x=0; x < (long) image->columns; x++)
     {
-      if ((x < clone_info.x) || (x >= (long) (clone_info.x+clone_info.width)))
+      if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
         {
           if ((indexes != (IndexPacket *) NULL) &&
               (chop_indexes != (IndexPacket *) NULL))
@@ -207,10 +204,10 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
         }
       p++;
     }
-    if (!SyncImagePixels(chop_image))
+    if (SyncImagePixels(chop_image) == MagickFalse)
       break;
-    if (QuantumTick(i,image->rows))
-      if (!MagickMonitor(ChopImageTag,i,image->rows,exception))
+    if (QuantumTick(i,image->rows) != 0)
+      if (MagickMonitor(ChopImageTag,i,image->rows,exception) == MagickFalse)
         break;
   }
   return(chop_image);
@@ -258,22 +255,18 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException(ImageError,"ImageSequenceIsRequired",
-      "UnableToCoalesceImageSequence");
-  /*
-    Clone first image in sequence.
-  */
-  coalesce_image=CloneImage(image,0,0,True,exception);
+  coalesce_image=CloneImage(image,image->page.width,image->page.height,
+    MagickTrue,exception);
   if (coalesce_image == (Image *) NULL)
     return((Image *) NULL);
-  (void) memset(&coalesce_image->page,0,sizeof(RectangleInfo));
+  SetImage(coalesce_image,TransparentOpacity);
+  (void) CompositeImage(coalesce_image,image->matte != MagickFalse ?
+    OverCompositeOp : CopyCompositeOp,image,image->page.x,image->page.y);
   previous_image=coalesce_image;
-  /*
-    Coalesce image.
-  */
   for (next=image->next; next != (Image *) NULL; next=next->next)
   {
     switch (next->dispose)
@@ -281,22 +274,22 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
       case UndefinedDispose:
       case NoneDispose:
       {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
+        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,exception);
         if (coalesce_image->next != (Image *) NULL)
           previous_image=coalesce_image;
         break;
       }
       case BackgroundDispose:
       {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,True,exception);
+        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,exception);
         if (coalesce_image->next != (Image *) NULL)
-          SetImage(coalesce_image->next,OpaqueOpacity);
+          SetImage(coalesce_image->next,TransparentOpacity);
         break;
       }
       case PreviousDispose:
       default:
       {
-        coalesce_image->next=CloneImage(previous_image,0,0,True,exception);
+        coalesce_image->next=CloneImage(previous_image,0,0,MagickTrue,exception);
         break;
       }
     }
@@ -309,12 +302,11 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
     coalesce_image=coalesce_image->next;
     coalesce_image->delay=next->delay;
     coalesce_image->start_loop=next->start_loop;
-    (void) CompositeImage(coalesce_image,next->matte ? OverCompositeOp :
-      CopyCompositeOp,next,next->page.x,next->page.y);
+    (void) CompositeImage(coalesce_image,next->matte != MagickFalse ?
+      OverCompositeOp : CopyCompositeOp,next,next->page.x,next->page.y);
+    coalesce_image->matte=next->matte;
   }
-  while (coalesce_image->previous != (Image *) NULL)
-    coalesce_image=coalesce_image->previous;
-  return(coalesce_image);
+  return(GetFirstImageInList(coalesce_image));
 }
 
 /*
@@ -376,6 +368,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(geometry != (const RectangleInfo *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
@@ -385,8 +379,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
           ((geometry->y+(long) geometry->height) < 0) ||
           (geometry->x >= (long) image->columns) ||
           (geometry->y >= (long) image->rows))
-        ThrowImageException(OptionError,"GeometryDoesNotContainImage",
-          "UnableToCropImage");
+        ThrowImageException(OptionError,"GeometryDoesNotContainImage");
     }
   page=(*geometry);
   if ((page.width != 0) || (page.height != 0))
@@ -414,31 +407,27 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       page=GetImageBoundingBox(image,exception);
       page.width+=geometry->x*2;
       page.height+=geometry->y*2;
-      page.x-=geometry->x;
-      if (page.x < 0)
-        page.x=0;
-      page.y-=geometry->y;
-      if (page.y < 0)
-        page.y=0;
-      if ((((long) page.width+page.x) > (long) image->columns) ||
-          (((long) page.height+page.y) > (long) image->rows))
-        ThrowImageException(OptionError,"GeometryDoesNotContainImage",
-          "UnableToCropImage");
+      page.x+=image->page.x+geometry->x;
+      page.y+=image->page.x+geometry->y;
     }
   if ((page.width == 0) || (page.height == 0))
-    ThrowImageException(OptionError,"GeometryDimensionsAreZero",
-      "UnableToCropImage");
+    ThrowImageException(OptionError,"GeometryDimensionsAreZero");
   if ((page.width == image->columns) && (page.height == image->rows) &&
       (page.x == 0) && (page.y == 0))
-    return(CloneImage(image,0,0,True,exception));
+    return(CloneImage(image,0,0,MagickTrue,exception));
   /*
     Initialize crop image attributes.
   */
-  crop_image=CloneImage(image,page.width,page.height,True,exception);
+  crop_image=CloneImage(image,page.width,page.height,MagickTrue,exception);
   if (crop_image == (Image *) NULL)
     return((Image *) NULL);
   crop_image->page.x=page.x;
   crop_image->page.y=page.y;
+  if (image->previous == (Image *) NULL)
+    {
+      crop_image->page.x=0;
+      crop_image->page.y=0;
+    }
   /*
     Extract crop image.
   */
@@ -448,17 +437,18 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     q=SetImagePixels(crop_image,0,y,crop_image->columns,1);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    (void) memcpy(q,p,crop_image->columns*sizeof(PixelPacket));
+    (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*
+      sizeof(PixelPacket));
     indexes=GetIndexes(image);
     crop_indexes=GetIndexes(crop_image);
     if ((indexes != (IndexPacket *) NULL) &&
         (crop_indexes != (IndexPacket *) NULL))
-      (void) memcpy(crop_indexes,indexes,crop_image->columns*
+      (void) CopyMagickMemory(crop_indexes,indexes,(size_t) crop_image->columns*
         sizeof(IndexPacket));
-    if (!SyncImagePixels(crop_image))
+    if (SyncImagePixels(crop_image) == MagickFalse)
       break;
-    if (QuantumTick(y,crop_image->rows))
-      if (!MagickMonitor(CropImageTag,y,crop_image->rows-1,exception))
+    if (QuantumTick(y,crop_image->rows) != 0)
+      if (MagickMonitor(CropImageTag,y,crop_image->rows-1,exception) == MagickFalse)
         break;
   }
   if (y < (long) crop_image->rows)
@@ -473,6 +463,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %     D e c o n s t r u c t I m a g e s                                       %
 %                                                                             %
 %                                                                             %
@@ -523,28 +514,25 @@ MagickExport Image *DeconstructImages(const Image *image,
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException(ImageError,"ImageSequenceIsRequired",
-      "UnableToDeconstructImageSequence");
   /*
     Ensure the image are the same size.
   */
   for (next=image; next != (Image *) NULL; next=next->next)
   {
     if ((next->columns != image->columns) || (next->rows != image->rows))
-      ThrowImageException(OptionError,"ImagesAreNotTheSameSize",
-        "UnableToDeconstructImageSequence");
+      ThrowImageException(OptionError,"ImagesAreNotTheSameSize");
   }
   /*
     Allocate memory.
   */
-  bounds=(RectangleInfo *)
-    AcquireMemory(GetImageListLength(image)*sizeof(RectangleInfo));
+  bounds=(RectangleInfo *) AcquireMagickMemory((size_t)
+    GetImageListLength(image)*sizeof(RectangleInfo));
   if (bounds == (RectangleInfo *) NULL)
-    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToDeconstructImageSequence");
+    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Compute the bounding box for each next in the sequence.
   */
@@ -562,7 +550,7 @@ MagickExport Image *DeconstructImages(const Image *image,
         break;
       for (y=0; y < (long) next->rows; y++)
       {
-        if (!FuzzyColorCompare(next,p,q))
+        if (FuzzyColorCompare(next,p,q) == MagickFalse)
           break;
         p++;
         q++;
@@ -579,7 +567,7 @@ MagickExport Image *DeconstructImages(const Image *image,
         break;
       for (x=0; x < (long) next->columns; x++)
       {
-        if (!FuzzyColorCompare(next,p,q))
+        if (FuzzyColorCompare(next,p,q) == MagickFalse)
           break;
         p++;
         q++;
@@ -596,7 +584,7 @@ MagickExport Image *DeconstructImages(const Image *image,
         break;
       for (y=0; y < (long) next->rows; y++)
       {
-        if (!FuzzyColorCompare(next,p,q))
+        if (FuzzyColorCompare(next,p,q) == MagickFalse)
           break;
         p++;
         q++;
@@ -604,7 +592,7 @@ MagickExport Image *DeconstructImages(const Image *image,
       if (y < (long) next->rows)
         break;
     }
-    bounds[i].width=x-bounds[i].x+1;
+    bounds[i].width=(unsigned long) (x-bounds[i].x+1);
     for (y=(long) next->rows-1; y >= 0; y--)
     {
       p=AcquireImagePixels(next,0,y,next->columns,1,exception);
@@ -613,7 +601,7 @@ MagickExport Image *DeconstructImages(const Image *image,
         break;
       for (x=0; x < (long) next->columns; x++)
       {
-        if (!FuzzyColorCompare(next,p,q))
+        if (FuzzyColorCompare(next,p,q) == MagickFalse)
           break;
         p++;
         q++;
@@ -621,16 +609,16 @@ MagickExport Image *DeconstructImages(const Image *image,
       if (x < (long) next->columns)
         break;
     }
-    bounds[i].height=y-bounds[i].y+1;
+    bounds[i].height=(unsigned long) (y-bounds[i].y+1);
     i++;
   }
   /*
     Clone first image in sequence.
   */
-  deconstruct_image=CloneImage(image,0,0,True,exception);
+  deconstruct_image=CloneImage(image,0,0,MagickTrue,exception);
   if (deconstruct_image == (Image *) NULL)
     {
-      LiberateMemory((void **) &bounds);
+      bounds=(RectangleInfo *) RelinquishMagickMemory(bounds);
       return((Image *) NULL);
     }
   /*
@@ -639,7 +627,7 @@ MagickExport Image *DeconstructImages(const Image *image,
   i=0;
   for (next=image->next; next != (Image *) NULL; next=next->next)
   {
-    crop_image=CloneImage(next,0,0,True,exception);
+    crop_image=CloneImage(next,0,0,MagickTrue,exception);
     if (crop_image == (Image *) NULL)
       break;
     crop_next=CropImage(crop_image,&bounds[i++],exception);
@@ -650,15 +638,13 @@ MagickExport Image *DeconstructImages(const Image *image,
     crop_next->previous=deconstruct_image;
     deconstruct_image=deconstruct_image->next;
   }
-  LiberateMemory((void **) &bounds);
-  while (deconstruct_image->previous != (Image *) NULL)
-    deconstruct_image=deconstruct_image->previous;
+  bounds=(RectangleInfo *) RelinquishMagickMemory(bounds);
   if (next != (Image *) NULL)
     {
       DestroyImageList(deconstruct_image);
       return((Image *) NULL);
     }
-  return(deconstruct_image);
+  return(GetFirstImageInList(deconstruct_image));
 }
 
 /*
@@ -671,8 +657,8 @@ MagickExport Image *DeconstructImages(const Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method FlattenImage merges a sequence of images.  This is useful for
-%  combining Photoshop layers into a single image.
+%  FlattenImage() merges a sequence of images.  This is useful for combining
+%  Photoshop layers into a single image.
 %
 %  The format of the FlattenImage method is:
 %
@@ -698,15 +684,14 @@ MagickExport Image *FlattenImages(const Image *image,ExceptionInfo *exception)
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  if (image->next == (Image *) NULL)
-    ThrowImageException(ImageError,"ImageSequenceIsRequired",
-      "UnableToFlattenImage");
   /*
     Clone first image in sequence.
   */
-  flatten_image=CloneImage(image,0,0,True,exception);
+  flatten_image=CloneImage(image,0,0,MagickTrue,exception);
   if (flatten_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -764,17 +749,16 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
-
   /*
     Initialize flip image attributes.
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  flip_image=CloneImage(image,image->columns,image->rows,True,exception);
+  flip_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
   if (flip_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -787,17 +771,17 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
       flip_image->columns,1);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    (void) memcpy(q,p,flip_image->columns*sizeof(PixelPacket));
+    (void) CopyMagickMemory(q,p,(size_t) flip_image->columns*sizeof(PixelPacket));
     indexes=GetIndexes(image);
     flip_indexes=GetIndexes(flip_image);
     if ((indexes != (IndexPacket *) NULL) &&
         (flip_indexes != (IndexPacket *) NULL))
-      (void) memcpy(flip_indexes,indexes,image->columns*sizeof(IndexPacket));
-    status=SyncImagePixels(flip_image);
-    if (status == False)
+      (void) CopyMagickMemory(flip_indexes,indexes,(size_t) image->columns*
+        sizeof(IndexPacket));
+    if (SyncImagePixels(flip_image) == MagickFalse)
       break;
-    if (QuantumTick(y,flip_image->rows))
-      if (!MagickMonitor(FlipImageTag,y,flip_image->rows,exception))
+    if (QuantumTick(y,flip_image->rows) != 0)
+      if (MagickMonitor(FlipImageTag,y,flip_image->rows,exception) == MagickFalse)
         break;
   }
   return(flip_image);
@@ -852,17 +836,16 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   register PixelPacket
     *q;
 
-  unsigned int
-    status;
-
   /*
     Initialize flop image attributes.
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  flop_image=CloneImage(image,image->columns,image->rows,True,exception);
+  flop_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
   if (flop_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -886,11 +869,10 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
       *q=(*p);
       p++;
     }
-    status=SyncImagePixels(flop_image);
-    if (status == False)
+    if (SyncImagePixels(flop_image) == MagickFalse)
       break;
-    if (QuantumTick(y,flop_image->rows))
-      if (!MagickMonitor(FlopImageTag,y,flop_image->rows,exception))
+    if (QuantumTick(y,flop_image->rows) != 0)
+      if (MagickMonitor(FlopImageTag,y,flop_image->rows,exception) == MagickFalse)
         break;
   }
   return(flop_image);
@@ -929,21 +911,25 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
   Image
     *mosaic_image;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    scene;
+
   RectangleInfo
     page;
 
   register const Image
     *next;
 
-  unsigned int
-    scene,
-    status;
-
   /*
     Determine mosaic bounding box.
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   page.width=image->columns;
@@ -971,6 +957,7 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
     return((Image *) NULL);
   mosaic_image->columns=page.width;
   mosaic_image->rows=page.height;
+  mosaic_image->background_color=image->background_color;
   SetImage(mosaic_image,OpaqueOpacity);
   /*
     Initialize colormap.
@@ -982,7 +969,7 @@ MagickExport Image *MosaicImages(const Image *image,ExceptionInfo *exception)
       next->page.y);
     status=MagickMonitor(MosaicImageTag,scene++,GetImageListLength(image),
       exception);
-    if (status == False)
+    if (status == MagickFalse)
       break;
   }
   return(mosaic_image);
@@ -1050,9 +1037,11 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  roll_image=CloneImage(image,image->columns,image->rows,True,exception);
+  roll_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
   if (roll_image == (Image *) NULL)
     return((Image *) NULL);
   /*
@@ -1089,11 +1078,11 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
         *roll_indexes=indexes[x];
       *q=(*p);
       p++;
-      if (!SyncImagePixels(roll_image))
+      if (SyncImagePixels(roll_image) == MagickFalse)
         break;
     }
-    if (QuantumTick(y,image->rows))
-      if (!MagickMonitor(RollImageTag,y,image->rows,exception))
+    if (QuantumTick(y,image->rows) != 0)
+      if (MagickMonitor(RollImageTag,y,image->rows,exception) == MagickFalse)
         break;
   }
   return(roll_image);
@@ -1110,9 +1099,9 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ShaveImage shaves pixels from the image edges.  It allocates the
-%  memory necessary for the new Image structure and returns a pointer to the
-%  new image.
+%  ShaveImage() shaves pixels from the image edges.  It allocates the memory
+%  necessary for the new Image structure and returns a pointer to the new
+%  image.
 %
 %  The format of the ShaveImage method is:
 %
@@ -1140,10 +1129,13 @@ MagickExport Image *ShaveImage(const Image *image,
   RectangleInfo
     geometry;
 
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if (((2*shave_info->width) >= image->columns) ||
       ((2*shave_info->height) >= image->rows))
-    ThrowImageException(OptionError,"GeometryDoesNotContainImage",
-      "UnableToShaveImage");
+    ThrowImageException(OptionError,"GeometryDoesNotContainImage");
   SetGeometry(image,&geometry);
   geometry.width-=2*shave_info->width;
   geometry.height-=2*shave_info->height;
@@ -1157,6 +1149,170 @@ MagickExport Image *ShaveImage(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   S p l i c e I m a g e                                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Use SpliceImage() to splice a solid color into the image as defined by
+%  the geometry.
+%
+%  The format of the SpliceImage method is:
+%
+%      Image *SpliceImage(const Image *image,const RectangleInfo *geometry,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o geometry: Define the region of the image to splice with members
+%      x, y, width, and height.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport Image *SpliceImage(const Image *image,
+  const RectangleInfo *geometry,ExceptionInfo *exception)
+{
+#define SpliceImageTag  "Splice/Image"
+
+  Image
+    *splice_image;
+
+  long
+    y;
+
+  register const PixelPacket
+    *p;
+
+  register IndexPacket
+    *splice_indexes,
+    *indexes;
+
+  register long
+    i,
+    x;
+
+  register PixelPacket
+    *q;
+
+  /*
+    Allocate splice image.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  assert(geometry != (const RectangleInfo *) NULL);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  if ((geometry->x < 0) || (geometry->x > (long) image->columns) ||
+      (geometry->y < 0) || (geometry->y > (long) image->rows))
+     ThrowImageException(OptionError,"GeometryDoesNotContainImage");
+  if ((geometry->width == 0) && (geometry->height == 0))
+    return(CloneImage(image,0,0,MagickTrue,exception));
+  splice_image=CloneImage(image,image->columns+geometry->width,image->rows+
+    geometry->height,MagickTrue,exception);
+  if (splice_image == (Image *) NULL)
+    return((Image *) NULL);
+  splice_image->storage_class=DirectClass;
+  /*
+    Splice image.
+  */
+  i=0;
+  for (y=0; y < (long) geometry->y; y++)
+  {
+    p=AcquireImagePixels(image,0,i++,image->columns,1,exception);
+    q=SetImagePixels(splice_image,0,y,splice_image->columns,1);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+      break;
+    indexes=GetIndexes(image);
+    splice_indexes=GetIndexes(splice_image);
+    for (x=0; x < geometry->x; x++)
+    {
+      *q++=(*p++);
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=(*indexes++);
+    }
+    for ( ; x < (long) (geometry->x+geometry->width); x++)
+    {
+      *q++=image->background_color;
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=0;
+    }
+    for ( ; x < (long) splice_image->columns; x++)
+    {
+      *q++=(*p++);
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=(*indexes++);
+    }
+    if (SyncImagePixels(splice_image) == MagickFalse)
+      break;
+    if (QuantumTick(y,splice_image->rows) != 0)
+      if (MagickMonitor(SpliceImageTag,y,splice_image->rows,exception) == MagickFalse)
+        break;
+  }
+  for ( ; y < (long) (geometry->y+geometry->height); y++)
+  {
+    q=SetImagePixels(splice_image,0,y,splice_image->columns,1);
+    if (q == (PixelPacket *) NULL)
+      break;
+    splice_indexes=GetIndexes(splice_image);
+    for (x=0; x < (long) splice_image->columns; x++)
+    {
+      *q++=image->background_color;
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=0;
+    }
+    if (SyncImagePixels(splice_image) == MagickFalse)
+      break;
+    if (QuantumTick(y,splice_image->rows) != 0)
+      if (MagickMonitor(SpliceImageTag,y,splice_image->rows,exception) == MagickFalse)
+        break;
+  }
+  for ( ; y < (long) splice_image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,i++,image->columns,1,exception);
+    q=SetImagePixels(splice_image,0,y,splice_image->columns,1);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+      break;
+    indexes=GetIndexes(image);
+    splice_indexes=GetIndexes(splice_image);
+    for (x=0; x < geometry->x; x++)
+    {
+      *q++=(*p++);
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=(*indexes++);
+    }
+    for ( ; x < (long) (geometry->x+geometry->width); x++)
+    {
+      *q++=image->background_color;
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=0;
+    }
+    for ( ; x < (long) splice_image->columns; x++)
+    {
+      *q++=(*p++);
+      if (splice_image->colorspace == CMYKColorspace)
+        splice_indexes[x]=(*indexes++);
+    }
+    if (SyncImagePixels(splice_image) == MagickFalse)
+      break;
+    if (QuantumTick(y,splice_image->rows) != 0)
+      if (MagickMonitor(SpliceImageTag,y,splice_image->rows,exception) == MagickFalse)
+        break;
+  }
+  return(splice_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   T r a n s f o r m I m a g e                                               %
 %                                                                             %
 %                                                                             %
@@ -1191,14 +1347,16 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
     *resize_image,
     *transform_image;
 
+  MagickStatusType
+    flags;
+
   RectangleInfo
     geometry;
 
-  unsigned int
-    flags;
-
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
+  if ((*image)->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*image)->filename);
   transform_image=(*image);
   if (crop_geometry != (const char *) NULL)
     {
@@ -1214,8 +1372,8 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
       crop_image=(Image *) NULL;
       flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry);
       if ((geometry.width == 0) || (geometry.height == 0) ||
-          ((flags & XiValue) != 0) || ((flags & PsiValue) != 0) ||
-          (flags & PercentValue))
+          ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
+          ((flags & PercentValue) != 0))
         crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
       else
         if ((transform_image->columns > geometry.width) ||
@@ -1288,3 +1446,47 @@ MagickExport void TransformImage(Image **image,const char *crop_geometry,
   transform_image=resize_image;
   *image=transform_image;
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   T r i m I m a g e                                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TrimImage() trims pixels from the image edges.  It allocates the memory
+%  necessary for the new Image structure and returns a pointer to the new
+%  image.
+%
+%  The format of the TrimImage method is:
+%
+%      Image *TrimImage(const Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport Image *TrimImage(const Image *image,ExceptionInfo *exception)
+{
+  RectangleInfo
+    geometry;
+
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  SetGeometry(image,&geometry);
+  geometry.width=0;
+  geometry.height=0;
+  geometry.x=0;
+  geometry.y=0;
+  return(CropImage(image,&geometry,exception));
+}
