@@ -12,10 +12,10 @@ extern "C" {
   Define declarations.
 */
 #define MagickPackageName "ImageMagick"
-#define MagickCopyright  "Copyright (C) 2003 ImageMagick Studio LLC"
-#define MagickLibVersion  0x557
-#define MagickLibVersionText  "5.5.7"
-#define MagickLibVersionNumber  5,5,7,24
+#define MagickCopyright  "Copyright (C) 1999-2004 ImageMagick Studio LLC"
+#define MagickLibVersion  0x602
+#define MagickLibVersionText  "6.0.2"
+#define MagickLibVersionNumber  6,0,2,0
 #if (QuantumDepth == 8)
 #define MagickQuantumDepth  "Q8"
 #elif (QuantumDepth == 16)
@@ -25,16 +25,19 @@ extern "C" {
 #else
 #define MagickQuantumDepth  "Q?"
 #endif
-#define MagickReleaseDate  "08/06/04"
+#define MagickReleaseDate  "06/15/04"
 #define MagickVersion MagickPackageName " " MagickLibVersionText " " \
-  MagickReleaseDate " " MagickQuantumDepth " " MagickWebSite
-#define MagickWebSite  "http://www.imagemagick.org"
+  MagickReleaseDate " " MagickQuantumDepth " " MagickHomeURL
+#define MagickHomeURL  "http://www.imagemagick.org"
+
+extern MagickExport char
+  *GetMagickHomeURL(void);
 
-/*
-  Method declarations.
-*/
 extern MagickExport const char
   *GetMagickCopyright(void),
+  *GetMagickPackageName(void),
+  *GetMagickQuantumDepth(unsigned long *),
+  *GetMagickReleaseDate(void),
   *GetMagickVersion(unsigned long *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
