@@ -12,37 +12,24 @@
 %                                                                             %
 %                     VMS Utility Methods for ImageMagick                     %
 %                                                                             %
-%                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                October 1994                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2003 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -56,7 +43,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/utility.h"
+#include "magick/string_.h"
 #include "magick/vms.h"
 
 #if !defined(_AXP_) && (!defined(__VMS_VER) || (__VMS_VER < 70000000))
@@ -84,9 +71,11 @@
 */
 void closedir(DIR *directory)
 {
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"");
   assert(directory != (DIR *) NULL);
-  LiberateMemory((void **) &directory->pattern);
-  LiberateMemory((void **) &directory);
+  directory->pattern=(char *) RelinquishMagickMemory(directory->pattern);
+  directory=(char *) RelinquishMagickMemory(directory);
 }
 
 /*
@@ -121,7 +110,7 @@ DIR *opendir(char *name)
   /*
     Allocate memory for handle and the pattern.
   */
-  directory=(DIR *) AcquireMemory(sizeof(DIR));
+  directory=(DIR *) AcquireMagickMemory(sizeof(DIR));
   if (directory == (DIR *) NULL)
     {
       errno=ENOMEM;
@@ -129,17 +118,17 @@ DIR *opendir(char *name)
     }
   if (strcmp(".",name) == 0)
     name="";
-  directory->pattern=(char *) AcquireMemory(strlen(name)+sizeof("*.*")+1);
+  directory->pattern=(char *) AcquireMagickMemory(strlen(name)+sizeof("*.*")+1);
   if (directory->pattern == (char *) NULL)
     {
-      LiberateMemory((void **) &directory);
+      directory=(char *) RelinquishMagickMemory(directory);
       errno=ENOMEM;
       return(NULL);
     }
   /*
     Initialize descriptor.
   */
-  FormatString(directory->pattern,"%.1024s*.*",name);
+  (void) FormatMagickString(directory->pattern,MaxTextExtent,"%s*.*",name);
   directory->context=0;
   directory->pat.dsc$a_pointer=directory->pattern;
   directory->pat.dsc$w_length=strlen(directory->pattern);
@@ -204,20 +193,20 @@ struct dirent *readdir(DIR *directory)
   */
   buffer[sizeof(buffer)-1]='\0';
   for (p=buffer; *p; p++)
-    if (isupper(*p))
+    if (isupper((unsigned char) *p))
       *p=tolower(*p);
   /*
     Skip any directory component and just copy the name.
   */
   p=buffer;
-  while (!isspace(*p))
+  while (isspace((unsigned char) *p) == 0)
     p++;
   *p='\0';
   p=strchr(buffer,']');
   if (p)
-    (void) strncpy(directory->entry.d_name,p+1,MaxTextExtent-1);
+    (void) CopyMagickString(directory->entry.d_name,p+1,MaxTextExtent);
   else
-    (void) strncpy(directory->entry.d_name,buffer,MaxTextExtent-1);
+    (void) CopyMagickString(directory->entry.d_name,buffer,MaxTextExtent);
   directory->entry.d_namlen=strlen(directory->entry.d_name);
   return(&directory->entry);
 }
@@ -241,7 +230,7 @@ struct dirent *readdir(DIR *directory)
 %
 %  The format of the VMSIsMagickConflict method is:
 %
-%      status=VMSIsMagickConflict(const char *magick)
+%      MagickBooleanType VMSIsMagickConflict(const char *magick)
 %
 %  A description of each parameter follows:
 %
@@ -252,7 +241,7 @@ struct dirent *readdir(DIR *directory)
 %
 %
 */
-MagickExport unsigned int VMSIsMagickConflict(const char *magick)
+MagickExport MagickBooleanType VMSIsMagickConflict(const char *magick)
 {
   ile3
     item_list[2];
@@ -273,11 +262,11 @@ MagickExport unsigned int VMSIsMagickConflict(const char *magick)
   item_list[0].ile3$w_code=DVI$_DEVCLASS;
   item_list[0].ile3$ps_bufaddr=&device_class;
   item_list[0].ile3$ps_retlen_addr=NULL;
-  (void) memset(&item_list[1],0,sizeof(item_list[1]));
+  (void) ResetMagickMemory(&item_list[1],0,sizeof(item_list[1]));
   status=sys$getdviw(0,0,&device,&item_list,0,0,0,0);
   if ((status == SS$_NONLOCAL) ||
       ((status & 0x01) && (device_class & (DC$_DISK | DC$_TAPE))))
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 #endif /* defined(vms) */
