@@ -1,5 +1,5 @@
 /*
-  X11 User Interface Methods for ImageMagick.
+  ImageMagick X11 Widget Methods.
 */
 #ifndef _MAGICK_WIDGET_H
 #define _MAGICK_WIDGET_H
@@ -8,65 +8,19 @@
 extern "C" {
 #endif
 
-/*
-  Enum declarations.
-*/
-typedef enum
-{
-  ControlState = 0x0001,
-  InactiveWidgetState = 0x0004,
-  JumpListState = 0x0008,
-  RedrawActionState = 0x0010,
-  RedrawListState = 0x0020,
-  RedrawWidgetState = 0x0040,
-  UpdateListState = 0x0100
-} WidgetState;
+#if defined(HasX11)
 
-/*
-  Typedef declarations.
-*/
-typedef struct _XWidgetInfo
-{
-  char
-    *cursor,
-    *text,
-    *marker;
-
-  int
-    id;
-
-  unsigned int
-    bevel_width,
-    width,
-    height;
-
-  int
-    x,
-    y,
-    min_y,
-    max_y;
-
-  unsigned int
-    raised,
-    active,
-    center,
-    trough,
-    highlight;
-} XWidgetInfo;
-
-/*
-  X utilities routines.
-*/
 extern MagickExport int
   XCommandWidget(Display *,XWindows *,const char **,XEvent *),
   XConfirmWidget(Display *,XWindows *,const char *,const char *),
   XDialogWidget(Display *,XWindows *,const char *,const char *,char *),
   XMenuWidget(Display *,XWindows *,const char *,const char **,char *);
 
-extern MagickExport unsigned int
+extern MagickExport MagickBooleanType
   XPreferencesWidget(Display *,XResourceInfo *,XWindows *);
 
 extern MagickExport void
+  DestroyXWidget(void),
   XColorBrowserWidget(Display *,XWindows *,const char *,char *),
   XFileBrowserWidget(Display *,XWindows *,const char *,char *),
   XFontBrowserWidget(Display *,XWindows *,const char *,char *),
@@ -74,10 +28,12 @@ extern MagickExport void
   XListBrowserWidget(Display *,XWindows *,XWindowInfo *,const char **,
     const char *,const char *,char *),
   XMonitorWidget(Display *,XWindows *,const char *,
-    const ExtendedSignedIntegralType,const ExtendedUnsignedIntegralType),
+    const MagickOffsetType,const MagickSizeType),
   XNoticeWidget(Display *,XWindows *,const char *,const char *),
-  XTextViewWidget(Display *,const XResourceInfo *,XWindows *,const unsigned int,
-    const char *,const char **);
+  XTextViewWidget(Display *,const XResourceInfo *,XWindows *,
+    const MagickBooleanType,const char *,const char **);
+
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
